; $Id: dcs_write_binaryoctavefile.pro,v 1.4 2005/02/14 17:50:14 azuah Exp $
; Written by J.R.D. Copley.
;************************************************************************************************
;###############################################################################
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################
pro dcs_write_binaryOctavefile,filename,datafile=datafile,compress=compressed
;************************************************************************************************
;
compile_opt strictarr
;
; This is a procedure that writes a binary Octave file.
;
if (filename eq "") then stop
;
on_ioerror, no_file
get_lun,unit
openw,unit,filename,compress=compressed
;
header="Octave-1-L"
writeu,unit,header,0b
;
dldocgf=bytarr(5)
;
nvars=n_tags(datafile)
tagnames=tag_names(datafile)
;
for ivar=0,nvars-1 do begin
	name=tagnames[ivar]
	nlgth=long(strlen(name))
	writeu,unit,nlgth
	writeu,unit,name
	writeu,unit,dldocgf
	data=datafile.(ivar)
	nsize=size(data)
	ns0=nsize[0]
	nst=nsize[ns0+1]
	if (ns0 eq 0) then begin
		case nst of
			7: dt=5; string
			else: dt=1; scalar
		endcase
	endif
	if (ns0 eq 1) then begin
		case nst of
			7: dt=7; string array
			else: dt=2
		endcase
		if (dt eq 2) then begin & n1=nsize[1] & n2=1 & end
	endif
	if (ns0 eq 2) then begin
		case nst of
			7: dt=0
			else: dt=2
		endcase
		if (dt eq 2) then begin & n1=nsize[1] & n2=nsize[2] & end
	endif
;
	if (dt eq 0) then stop,"No good."
;
	writeu,unit,byte(dt); write data type
	case dt of
		1: begin; write scalar
			writeu,unit,7b
			writeu,unit,double(data)
	end
		2: begin; write matrix
			writeu,unit,long(n1),long(n2)
			writeu,unit,7b
			writeu,unit,double(data)
	end
		5: begin; write string
			nchars=strlen(data)
			if (nchars gt 0) then begin
				writeu,unit,long(nchars)
				str=strarr(nchars)
				for i=0,nchars-1 do str[i]=strmid(data,i,1)
				writeu,unit,str
			endif
		end
		7: begin; write string array
			nstrgs=nsize[1]
			writeu,unit,long(nstrgs)
			strarray=data
			for j=0,nstrgs-1 do begin
				nchars=strlen(data[j])
				if (nchars gt 0) then begin
					writeu,unit,long(nchars)
					str=strarr(nchars)
					for i=0,nchars-1 do str[i]=strmid(data[j],i,1)
					writeu,unit,str
				endif
			endfor
		end
	endcase
endfor
;
free_lun,unit
return
;
no_file: begin
	free_lun,unit
	error="There is no file named "+filename
end
;
end