; $Id: dm_filemenu.pro,v 1.6 2005/07/15 14:51:46 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_filemenu
;
; PURPOSE:
;
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-8562
;  United States
;  yiming.qiu@nist.gov
;  June, 2005
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_filemenu,init=init
    ftypemenu = widget_info(self.tlb,find_by_uname='ftypeMenu')
    ftoolmenu = widget_info(self.tlb,find_by_uname='ftoolMenu')
    if keyword_set(init) then begin  ;initialization
        self.ftypename = ptr_new(['DCS','SPE'])   ;instrument name or file types to be shown in the menu
        self.ftypeext  = ptr_new(['DCS','SPE'])   ;corresponding file type extensions

        for i=0,n_elements(*self.ftypename)-1 do begin
            void = dm_widget_button(ftypemenu,value=(*self.ftypename)[i],uname='ftype_'+dm_to_string(i))
        endfor

        checked = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string(self.ftype))
        for i=0,n_elements(*self.ftypename)-1 do begin
            if i ne self.ftype then begin
               tmp = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string(i))
               unchecked = (n_elements(unchecked) eq 0)?tmp:[unchecked,tmp]
            endif
        endfor
        dm_toggle_menubut,checked,unchecked
    endif

    ;clear all possible background data and mask
    self.bgrate  = !values.f_nan
    self.monrate = 1.0
    ptr_free,self.bgdata,self.bgratePtr,self.mask
    self->my_widget_control,'bgclBut',sensitive=0
    self->my_widget_control,'status_bgrate',_extra=self->getstatus('status_bgrate')
    self->my_widget_control,'status_bgdet',_extra=self->getstatus('status_bgdet')
    self->my_widget_control,'status_bgcan',_extra=self->getstatus('status_bgcan')
    self->my_widget_control,'status_mask',_extra=self->getstatus('status_mask')
    mkcnBut  = widget_info(self.tlb,find_by_uname='mkcnBut')
    mkloBut  = widget_info(self.tlb,find_by_uname='mkloBut')
    mkupBut  = widget_info(self.tlb,find_by_uname='mkupBut')
    mkclBut  = widget_info(self.tlb,find_by_uname='mkclBut')
    intnTyp0 = widget_info(self.tlb,find_by_uname='intnTyp0')
    intnTyp1 = widget_info(self.tlb,find_by_uname='intnTyp1')
    intnTyp2 = widget_info(self.tlb,find_by_uname='intnTyp2')
    intnTyp3 = widget_info(self.tlb,find_by_uname='intnTyp3')
    dm_toggle_menubut,mkcnBut,[mkcnBut,mkloBut,mkupBut]
    widget_control,mkclBut,sensitive=0

    ;construct tool menus
    ;delete all possible menu items in ftoolMenu
    tmp = widget_info(ftoolmenu,/child)
    while(tmp ne 0) do begin
        widget_control, tmp,/destroy
        tmp = widget_info(ftoolmenu,/child)
    endwhile
    ;reconstruct tool menus according to file type
    case self.ftype of  ;warning: don't use a tool uname starts with 'ftype_' !!!
        0:  begin   ;DCS file
            widget_control,ftoolmenu,sensitive=1
            void = widget_button(ftoolmenu,value='Convert DCS Files to SPE Format',uname='cspeBut')
            void = widget_button(ftoolmenu,value='Plot DCS Crystal Alignment Files',uname='algnBut')
            *(self.rebinsize) = [1s,2s,4s,5s,8s,10s,20s,25s,40s,50s]
            self.detPosPtr    = dm_load_detpos(/dcs)                ;load detector position information
            self.intn_typ     = 0                                   ;default S(q,w) intensity type
            dm_toggle_menubut,intnTyp0,[intnTyp1,intnTyp2,intnTyp3]
            end
        1:  begin   ;SPE file   no tools available
            widget_control,ftoolmenu,sensitive=0
            self.intn_typ = 3                                       ;default no kfactor adjustment
            dm_toggle_menubut,intnTyp3,[intnTyp0,intnTyp1,intnTyp2]
            end
        else: begin
            widget_control,ftoolmenu,sensitive=0
            self.intn_typ = 3                                       ;default no kfactor adjustment
            dm_toggle_menubut,intnTyp3,[intnTyp0,intnTyp1,intnTyp2]
            end
    endcase
    self->my_widget_control,'status_int',_extra=self->getstatus('status_int')
    self->my_widget_control,['eAdjust','darkCorr'],sensitive=(self.ftype eq 0)

    ;reset ebin size values
    if (self.samp_typ lt 2) and obj_valid(self.fileSel) then begin
       ebinbut = widget_info(self.tlb,find_by_uname='eBin')
       widget_control,ebinbut,set_value=dm_to_string(*(self.rebinsize))
       tmp = where(*(self.rebinsize) eq self.e_bin[self.samp_typ],count)
       if count eq 0 then self.e_bin[self.samp_typ]=1
       tmp = (0>(tmp[0]))
       dm_set_droplist,ebinbut,select=tmp
    endif
end