; $Id: dm_filetools.pro,v 1.6 2005/06/27 00:24:19 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_filetools
;
; PURPOSE:
;  handle file type menu and file tool menu events and all data file loading related events
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-8562
;  United States
;  yiming.qiu@nist.gov
;  June, 2005
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

; parameter:
;   eventname: event name = widget_info(event.id,/uname)
; keyword:
;   event:     passed event structure

;files need to be included
@dm_load_dcsdata
@dm_load_spe
@dm_load_phx

function dcs_mslice::dm_filetools,eventname,event=event,init=init
    handled = 1b    ;event handled flag
    if strmid(eventname,0,6) eq 'ftype_' then begin ;file type selection event
       type = dm_to_number(strmid(eventname,6,strlen(eventname)-6),/int)
       eventname = 'ftype_'
    endif
    case strlowcase(eventname) of
        'emptbut':begin  ;load empty can file(s)
            self.filesel->getproperty,path=open_path,file=open_file
            if n_elements(open_file) eq 0 then begin
              ok = dialog_message(dialog_parent=self.tlb,'Please select empty can files first.',/error)
              break
            endif
            case self.ftype of
                 0: begin   ;dcs data
                    self->dm_load_dcs_emptycan,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,bgrate=bgrate,qty=qty,dqty=dqty,ang=ang
                    end
                 1: begin   ;spe file
                    self->dm_load_spe,open_path,open_file,parent=event.handler,error=error,$
                        qty=qty,dqty=dqty,ang=ang,t_chan=self.e_bin[self.samp_typ],e_range=self.e_range,$
                        eint_yn=self.eint_yn,avgsum=self.bin_avgsum,diffuse=(self.samp_typ ge 2),$
                        title='loading empty can file...',diffetyp=(self.samp_typ eq 3)
                    end
                 else: error =1b ;do nothing
            endcase
            cd,self.root
            if keyword_set(error) then break
            if n_elements(bgrate) eq 1 then $  ;dark count rate from shutter closed files
               self.bgrate=bgrate
            if n_elements(qty) ne 0 then begin ;background file eg. empty can files
               ptr_free,self.bgdata
               if n_elements(ang) eq 0 then ang=0
               if (self.samp_typ eq 3) and (self.e_bin[4] gt 1) then begin
                  tmp = size(qty,/dimension)
                  dm_grid_ebin,self.e_bin[4],qty,dqty,fltarr(tmp[0]),avgsum=self.bin_avgsum
               endif
               self.bgdata=ptr_new({qty:temporary(qty),err:temporary(dqty),rotation:temporary(ang)})
            endif
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' empty can data'
            self->set_filehistory,comment,open_path,open_file
            self->my_widget_control,'bgclBut',sensitive=1
            self->my_widget_control,'status_bgrate',_extra=self->getstatus('status_bgrate')
            self->my_widget_control,'status_bgcan',_extra=self->getstatus('status_bgcan')
          end
        'bgflbut':begin  ;load detector dependent background file(s)
            self.filesel->getproperty,path=open_path,file=open_file
            if n_elements(open_file) eq 0 then begin
               ok = dialog_message(dialog_parent=self.tlb,'Please select background files first.',/error)
               break
            endif
            case self.ftype of
                 0: begin   ;dcs data
                    self->dm_load_dcs_detbackground,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,bgrate=bgrate,qty=qty,dqty=dqty
                    end
                 1: begin   ;spe file
                    self->dm_load_spe_detbackground,open_path,open_file,parent=event.handler,error=error,$
                        qty=qty,dqty=dqty
                    end
                 else: error =1b ;do nothing
            endcase
            cd,self.root
            if keyword_set(error) then break
            if n_elements(bgrate) eq 1 then $  ;dark count rate from shutter closed files
               self.bgrate=bgrate
            if n_elements(qty) ne 0 then begin ;background file
               ptr_free,self.bgratePtr
               self.bgratePtr=ptr_new(qty)
            endif
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' det. background data'
            self->set_filehistory,comment,open_path,open_file
            self->my_widget_control,'bgclBut',sensitive=1
            self->my_widget_control,'status_bgrate',_extra=self->getstatus('status_bgrate')
            self->my_widget_control,'status_bgdet',_extra=self->getstatus('status_bgdet')
          end
        'vandbut':begin  ;load vanadium file(s)
            self.filesel->getproperty,path=open_path,file=open_file
            if n_elements(open_file) eq 0 then begin
               ok=dialog_message(dialog_parent=self.tlb,'Please select vanadium files first.',/error)
               break
            endif
            case self.ftype of
                 0: begin   ;dcs data
                    self->dm_load_dcs_vanadium,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,qty=qty,ei=ei
                    end
                 1: begin   ;spe file
                    dm_load_spe,open_path,open_file,qty=qty,parent=event.handler,$
                        title='loading vanadium file...',error=error,e_inc=ei
                    end
                 else: error =1b ;do nothing
            endcase
            cd,self.root
            if keyword_set(error) then break
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' vanadium data'
            self->set_filehistory,comment,open_path,open_file
            ptr_free,self.eff
            eff = dm_calc_deteff(qty,/fit)
            ;add absorption correction
            mesg = [ 'To skip the absorption correction for vanadium file,',$
                     'press OK without filling in all the information.',$
                     '',$
                     'L0 is the energy independent mean free path;',$
                     'L1 is the energy dependent mean free path for',$
                     '2200m/s or 25.3 meV neutron.',$
                     'The effective mean free path is:',$
                     'L=1/{1/L0+1/(L1*sqrt[E(meV)/25.3])}' ]
            default = ['2.8','2.7','','']
            parm = dm_dialog_input(['L0(cm):','L1(cm):','Rout(cm):','Rin(cm):'],$
                default=default,/float,info=mesg,$
                title='Absorption correction for vanadium',xsize=140,dialog_parent=self.tlb)
            if total(finite(parm)) eq 4 then begin
               tth    = (*self.detPosPtr).two_theta
               factor = dm_calc_shielding([1,parm],ei,0,tth)
               eff    = eff*factor/mean(factor)
            endif else begin
               ok = dialog_message('Invalid inputs. Absorption correction for vanadium sample is disabled.',dialog_parent=self.tlb)
            endelse
            self.eff = ptr_new(eff,/no_copy)
            self->my_widget_control,'vdclBut',sensitive=1
            self->my_widget_control,'status_van',_extra=self->getstatus('status_van')
          end
        'loadbut':begin  ;load data file(s)
            self.filesel->getproperty,path=open_path,file=open_file
            if n_elements(open_file) eq 0 then begin
               ok=dialog_message(dialog_parent=self.tlb,'Please select data files first.',/error)
               break
            endif
            case self.ftype of
                 0: begin   ;dcs data
                    self->dm_load_dcs,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,legend=legend,qty=qty,dqty=dqty,weight=weight,ang=ang,ei=ei,$
                        info=info,ewid=ewid,emean=emean,monrate=monrate,kfactor=kfactor
                    end
                 1: begin   ;spe file
                    e_range = self.e_range
                    self->dm_load_spe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),$
                        /phxfile,avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,$
                        qty=qty,dqty=dqty,e_inc=ei,ang=ang,emean=emean,info=info,ewid=ewid,$
                        eint_yn=self.eint_yn,kfactor=kfactor,diffetyp=(self.samp_typ eq 3)
                    if (self.samp_typ ge 2) and (n_elements(e_range) ne 0) then self.e_range=e_range
                    end
                 else: error =1b ;do nothing
            endcase
            cd,self.root
            if keyword_set(error) then break
            if n_elements(monrate) eq 0 then monrate = 1.0
            if n_elements(kfactor) eq 0 then kfactor = 1.0
            if n_elements(info)    eq 0 then info = {temperature:300}
            self.monrate=monrate
            ptr_free,self.dataStrPtr,self.projStrPtr
            if self.samp_typ ge 2 then begin
               if n_elements(weight) eq 0 then weight=fltarr(n_elements(ang))+1.0
               if (self.samp_typ eq 3) and (self.e_bin[4] gt 1) then begin
                  dm_grid_ebin,self.e_bin[4],qty,dqty,emean,avgsum=self.bin_avgsum
                  if n_elements(kfactor) gt 1 then $
                     dm_grid_ebin,self.e_bin[4],kfactor $
                  else $
                     kfactor = sqrt(ei/(ei-emean))
                  if n_elements(ewid) ne 0 then dm_grid_ebin,self.e_bin[4],ewid
               endif
               if n_elements(ewid) eq 0 then ewid=1.0
               self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(dqty),weight:temporary(weight),$
                    rotation:temporary(ang),info:temporary(info),ei:temporary(ei),ewid:temporary(ewid),e:temporary(emean),$
                    filename:open_file,filepath:open_path})
               (*self.oifac) = fltarr(n_elements(open_file))+1.0 ;reset the overall intensity factor to 1.0
            endif else begin
               if n_elements(weight) eq 0 then weight=1.0
               if n_elements(ewid) eq 0   then ewid=fltarr(n_elements(emean))+1.0
               self.dataStrPtr = ptr_new({qty:temporary(qty),err:temporary(dqty),weight:temporary(weight),$
                    e:temporary(emean),info:temporary(info),ei:temporary(ei),ewid:temporary(ewid)})
               (*self.oifac) = 1.0 ;reset the overall intensity factor to 1.0
            endelse
            ptr_free,self.kfactor & self.kfactor=ptr_new(kfactor,/no_copy)
            self->my_widget_control,['addBut','clearDat'],sensitive=1     ;allow adding data & clear data
            if (finite(self.orie_psi) eq 0) and (self.samp_typ ge 2) then begin
               self.orie_psi = (*self.dataStrPtr).rotation[0]
               if self.view[0] eq 1 then $
                  self->my_widget_control,'oriePsi',set_value=dm_to_string(self.orie_psi)
            endif
            self->set_display_button,sensitive=0
            if self.view[2] eq 1 then self->my_widget_control,'plotTra',sensitive=0

            if n_elements(legend) eq 0 then legend=''
            if n_elements(comment) eq 0 then comment=''
            self->my_widget_control,'legdLab',set_value=legend & self.legd_lab=legend
            self->my_widget_control,'titlLab',set_value=comment & self.titl_lab=comment

            if strlen(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' data '+open_file[0]+(n_elements(open_file) gt 1?'...':'')
            self->set_filehistory,comment,open_path,open_file
          end
        'addbut':begin   ;add dcs data file to current data pointer
            self.filesel->getproperty,path=open_path,file=open_file
            if n_elements(open_file) eq 0 then begin
               ok=dialog_message(dialog_parent=self.tlb,'Please select data files first.',/error)
               break
            endif
            case self.ftype of
                 0: begin   ;dcs data
                    info = (*self.dataStrPtr).info
                    self->dm_load_dcs,open_path,open_file,parent=event.handler,error=error,$
                        comment=comment,legend=legend,qty=qty,dqty=dqty,weight=weight,ang=ang,$
                        info=info,title='adding data file...'
                    end
                 1: begin   ;spe file
                    info = (*self.dataStrPtr).info
                    e_range = self.e_range
                    self->dm_load_spe,open_path,open_file,parent=event.handler,diffuse=(self.samp_typ ge 2),$
                        avgsum=self.bin_avgsum,e_range=e_range,t_chan=self.e_bin[self.samp_typ],error=error,qty=qty,dqty=dqty,$
                        e_inc=ei,ang=ang,emean=emean,info=info,eint_yn=self.eint_yn,diffetyp=(self.samp_typ eq 3)
                    end
                 else: error =1b ;do nothing
            endcase
            cd,self.root
            if keyword_set(error) then break
            if n_elements(comment) eq 0 then comment=(*self.ftypename)[self.ftype]+' data'
            self->set_filehistory,comment,open_path,open_file
            if self.samp_typ ge 2 then begin
               if n_elements(weight) eq 0 then weight = fltarr(n_elements(ang))+1.0
               if self.samp_typ eq 3 then begin
                  if self.e_bin[4] gt 1 then begin
                     tmp = size(qty,/dimension)
                     dm_grid_ebin,self.e_bin[4],qty,dqty,fltarr(tmp[0]),avgsum=self.bin_avgsum
                  endif
                  qty = [[(*self.dataStrPtr).qty],[temporary(qty)]]
                  err = [[(*self.dataStrPtr).err],[temporary(dqty)]]
               endif else begin
                  qty = [[[(*self.dataStrPtr).qty]],[[temporary(qty)]]]
                  err = [[[(*self.dataStrPtr).err]],[[temporary(dqty)]]]
               endelse
               weight = [(*self.dataStrPtr).weight,temporary(weight)]
               ang    = [(*self.dataStrPtr).rotation,temporary(ang)]
               filename = [(*self.dataStrPtr).filename,open_file]
               e      = (*self.dataStrPtr).e
               ei     = (*self.dataStrPtr).ei
               info   = (*self.dataStrPtr).info
               ewid   = (*self.dataStrPtr).ewid
               path   = (*self.dataStrPtr).filepath
               ptr_free,self.dataStrPtr
               self.dataStrPtr=ptr_new({qty:temporary(qty),err:temporary(err),weight:temporary(weight),$
                   rotation:temporary(ang),info:temporary(info),ei:temporary(ei),ewid:temporary(ewid),$
                   e:temporary(e),filename:temporary(filename),filepath:path})
               (*self.oifac)=[(*self.oifac),fltarr(n_elements(open_file))+1.0]
            endif else begin
               if n_elements(weight) eq 0 then weight = 1.0
               (*self.dataStrPtr).qty = ((*self.dataStrPtr).qty)*((*self.dataStrPtr).weight)+qty*weight
               (*self.dataStrPtr).err = sqrt((((*self.dataStrPtr).err)*((*self.dataStrPtr).weight))^2+(dqty*weight)^2)
               (*self.dataStrPtr).weight = ((*self.dataStrPtr).weight)+weight
               (*self.dataStrPtr).qty = ((*self.dataStrPtr).qty)/((*self.dataStrPtr).weight)
               (*self.dataStrPtr).err = ((*self.dataStrPtr).err)/((*self.dataStrPtr).weight)
            endelse
          end
        'ftype_':   begin
            if self.ftype eq type then break   ;no change
            ptr_free,self.detPosPtr,self.dataStrPtr,self.projStrPtr
            self.ftype = type
            ftypemenu  = widget_info(self.tlb,find_by_uname='ftypeMenu')
            checked    = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string(type))
            for i=0,n_elements(*self.ftypename)-1 do begin
                if i ne type then begin
                   tmp = widget_info(ftypemenu,find_by_uname='ftype_'+dm_to_string(i))
                   unchecked = (n_elements(unchecked) eq 0)?tmp:[unchecked,tmp]
                endif
            endfor
            dm_toggle_menubut,checked,unchecked
            self->dm_filemenu               ;reset file menu
            self.fileSel->set_filter,'.'+(*self.ftypeext)[type]          ;change file filter
            ;change the about ... information in the help menu
            (*self.info)[0] = '--Mslice for '+(*self.ftypename)[type]+' --'
            ;(*self.info)[3] = 'Yiming Qiu(yiming.qiu@nist.gov)  6/2005' ;add your own name here if you like
            widget_control,self.tlb,tlb_set_title=(*self.info)[0]
            self->my_widget_control,'selfBut',set_value='About Mslice for '+(*self.ftypename)[type]+'...'
            if not keyword_set(init) then self->set_display_button,sensitive=0
          end

        ;DCS tools
        'cspebut':  dm_dcs2spe,event        ;convert dcs files to spe files
        'algnbut':  dcs_cryst_align,event,[self.latt_parm,self.view_u1[0:2]]  ;plot dcs single crystal alignment files

        ;put the events of your own instrument specific tools below

        else: handled = 0b      ;event not handled
    endcase

    return,handled
end