; $Id: dm_plot__define.pro,v 1.72 2005/10/24 16:55:21 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_plot
;
; PURPOSE:
;  A versatile object plotting program.
;
; CATEGORY:
;  general
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-8562
;  United States
;  yiming.qiu@nist.gov
;  Oct, 2005
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

;if widget is killed
pro dm_plot_Exit,tlb
    widget_control,tlb,get_uvalue=self
    if obj_valid(self) then begin
       self->getproperty,group_leader=group_leader,tlb=tlb,parenthandler=parenthandler,parentobj=parentobj
       sendevent={plot_kill,ID:tlb,TOP:group_leader,HANDLER:tlb,object:self}
       if obj_valid(parentobj) then $     ;notify parent object
            call_method,parenthandler,parentobj,sendevent $
       else if widget_info(group_leader,/valid_id) ne 0 then $   ;notify the group leader
            widget_control,group_leader,send_event=sendevent,/no_copy
       obj_destroy,self
    endif
end

;event handler
pro dm_plot_event,event
    compile_opt IDL2    ;,strictarrsubs
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return
    end

    widget_control,/hourglass
    widget_control,event.handler,get_uvalue=self
    if self->getproperty(/enotebook) then $
        IF DAVE_SET_FOCUS(EVENT) THEN RETURN
    self->event,event   ;actual event handler
end

;add a plot to the object
pro dm_plot::add_plot,xdat,ydat,zdat,idat,yerr=yerr,zpos=zpos,color=color,psym=psym,linestyle=linestyle,legend=legend,$
    symsize=symsize,thick=thick,coltable=coltable,shading=shading,hideintbar=hideintbar,surfplot=surfplot,$
    usepolygon=usepolygon,layer=layer,no_copy=no_copy
    if float(!version.release) ge 6.0 then v6p0_extra= {register_properties:1}  ;register_properties keyword for IDL6.0 or later
    if n_elements(color) eq 0  then color='white'
    if n_elements(zpos) eq 0 then zpos=!values.f_nan
    mycolor = self->getcolor(color)
    white   = self->getcolor('white')
    ;check keywords
    if keyword_set(surfplot) then cshift=0.5 else cshift=0.0
    if n_params() lt 2 then begin
       ok = dialog_message('Wrong number of parameters for dm_plot::add_plot.',/error,dialog_parent=self.tlb)
       return
    endif
    if total(finite(self.xran[*,0])) ne 2 then begin
       xmax = max(xdat,min=xmin,/nan)
       self.xran[*,0] = [temporary(xmin),temporary(xmax)]
       if self.xran[0,0] eq self.xran[1,0] then begin
         tmp = (self.xran[0,0] eq 0)? 0.5:abs(self.xran[0,0]/2.)
         self.xran[0,0] = self.xran[0,0]-tmp
         self.xran[1,0] = self.xran[1,0]+tmp
       endif
    endif
    if total(finite(self.xran[*,2])) ne 2 then begin   ;set the correct x log range from the data
       poselts = where(xdat gt 0,count)
       if count gt 0 then begin
         xmax = max(xdat[temporary(poselts)],min=xmin)
       endif else begin
         xmin = 1.0 & xmax = 10.0
       endelse
       self.xran[0,2] = (xmin gt self.xran[0,0])?((self.xran[0,0] gt 0)?(self.xran[0,0]):(xmin)):(self.xran[0,0])
       self.xran[1,2] = (xmax gt self.xran[1,0])?((self.xran[1,0] gt 0)?(self.xran[1,0]):(xmax)):(self.xran[1,0])
       if self.xran[1,2] le self.xran[0,2] then self.xran[1,2] = self.xran[0,2]*10.0
    endif
    if total(finite(self.yran[*,0])) ne 2 then begin
       if n_elements(yerr) eq n_elements(ydat) then begin
         ymin = min(ydat-yerr,/nan)
         ymax = max(ydat+yerr,/nan)
       endif else begin
         ymax = max(ydat,min=ymin,/nan)
       endelse
       self.yran[*,0] = [temporary(ymin),temporary(ymax)]
       if self.yran[0,0] eq self.yran[1,0] then begin
         tmp = (self.yran[0,0] eq 0)? 0.5:abs(self.yran[0,0]/2.)
         self.yran[0,0] = self.yran[0,0]-tmp
         self.yran[1,0] = self.yran[1,0]+tmp
       endif
    endif
    if total(finite(self.yran[*,2])) ne 2 then begin   ;set the correct y log range from the data
       poselts = where(ydat gt 0,count)
       if count gt 0 then begin
         ymax = max(ydat[temporary(poselts)],min=ymin)
       endif else begin
         ymin = 1.0 & ymax = 10.0
       endelse
       self.yran[0,2] = (ymin gt self.yran[0,0])?((self.yran[0,0] gt 0)?(self.yran[0,0]):(ymin)):(self.yran[0,0])
       self.yran[1,2] = (ymax gt self.yran[1,0])?((self.yran[1,0] gt 0)?(self.yran[1,0]):(ymax)):(self.yran[1,0])
       if self.yran[1,2] le self.yran[0,2] then self.yran[1,2] = self.yran[0,2]*10.0
    endif
    ;create x & y axis if nonexistent
    if not obj_valid(self.drawxAxis[0]) then begin
       ;x axis
       self.drawxAxis[0] = obj_new('IDLgrAxis',0,range=self.xran[*,0],title=self.xtit,_extra=v6p0_extra,$
         color=white,ticklen=self.default[0]*1.5,/exact,location=[0,-cshift,0],name='axis') ;bottom
       self.drawGroup->add,self.drawxAxis[0]
       if not keyword_set(surfplot) then begin
         self.title->getproperty,strings=title & obj_destroy,self.title
         self.title = obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,_extra=v6p0_extra,$
            color=white,name='titl')
         self.drawxAxis[1] = obj_new('IDLgrAxis',0,range=self.xran[*,0],color=white,_extra=v6p0_extra,$
            ticklen=self.default[0]*1.5,/tickdir,location=[1,1,0],/exact,name='axis',title=self.title,textpos=1,textalignments=[0.5,0])  ;top
         self.drawGroup->add,self.drawxAxis[1]
       endif
       ;y axis
       self.drawyAxis[0] = obj_new('IDLgrAxis',1,range=self.yran[*,0],title=self.ytit,_extra=v6p0_extra,$
         color=white,ticklen=self.default[0],location=[-cshift,0,0],/exact,name='axis') ;left
       self.drawGroup->add,self.drawyAxis[0]
       if not keyword_set(surfplot) then begin
         self.drawyAxis[1] = obj_new('IDLgrAxis',1,range=self.yran[*,0],color=white,_extra=v6p0_extra,$
            ticklen=self.default[0],/tickdir,location=[1,1,0],/exact,/notext,name='axis')    ;right
            self.drawGroup->add,self.drawyAxis[1]
       endif
    endif
    if n_params() ge 3  then begin        ;2d data
       if n_elements(hideintbar) eq 1 then self.hideIntbar = 0>(fix(hideintbar[0]))<1
       if n_elements(shading) ne 0 then  shading  = 0>(fix(shading[0]))<1 else shading=fix(self.default[3]) ;default flat shading
       if n_elements(coltable) ne 0 then coltable = 0>(fix(coltable[0]))<40 else coltable=fix(self.default[4]) ;default rainbow color table
       drawPal=self->getpalette(coltable)
       if total(finite(self.zran[*,0])) ne 2 then begin
         zmax = max(zdat,min=zmin,/nan)
         self.zran[*,0] = [zmin,zmax]
         if self.zran[0,0] eq self.zran[1,0] then begin
            tmp = (self.zran[0,0] eq 0)? 0.5:abs(self.zran[0,0]/2.)
            self.zran[0,0] = self.zran[0,0]-tmp
            self.zran[1,0] = self.zran[1,0]+tmp
         endif
       endif
       if total(finite(self.zran[*,2])) ne 2 then begin  ;set the correct z log range from the data
         finelts = where(finite(zdat),count)
         if count gt 0 then poselts = where(zdat[finelts] gt 0,count)
         if count gt 0 then begin
            zmax = max(zdat[finelts[poselts]],min=zmin)
         endif else begin
            zmin = 1.0 & zmax = 10.0
         endelse
         finelts = -1 & poselts = -1
         self.zran[0,2] = (zmin gt self.zran[0,0])?((self.zran[0,0] gt 0)?(self.zran[0,0]):(zmin)):(self.zran[0,0])
         self.zran[1,2] = (zmax gt self.zran[1,0])?((self.zran[1,0] gt 0)?(self.zran[1,0]):(zmax)):(self.zran[1,0])
         if self.zran[1,2] le self.zran[0,2] then self.zran[1,2] = self.zran[0,2]*10.0
       endif
       if n_elements(idat) ne 0 then begin
         if total(finite(self.iran[*,2])) ne 2 then begin
            finelts = where(finite(idat),count)
            if count gt 0 then poselts = where(idat[finelts] gt 0,count)
            if count gt 0 then begin
                imax = max(idat[finelts[poselts]],min=imin)
            endif else begin
                imin = 1.0 & imax = 10.0
            endelse
            finelts = -1 & poselts = -1
            self.iran[0,2] = (imin gt self.iran[0,0])?((self.iran[0,0] gt 0)?(self.iran[0,0]):(imin)):(self.iran[0,0])
            self.iran[1,2] = (imax gt self.iran[1,0])?((self.iran[1,0] gt 0)?(self.iran[1,0]):(imax)):(self.iran[1,0])
            if self.iran[1,2] le self.iran[0,2] then self.iran[1,2] = self.iran[0,2]*10.0
         endif
       endif else if total(finite(self.iran[*,0])) ne 2 then  $
            self.iran[*,[0,2]]=self.zran[*,[0,2]]
       if not finite(self.zmin) then self.zmin = self.iran[0,2*self.ilog]
       if not finite(self.zmax) then self.zmax = self.iran[1,2*self.ilog]
       if keyword_set(surfplot) then begin
         type = 2   ;surface plot
         ;z axis
         if not obj_valid(self.drawzAxis) then begin
          self.drawzAxis = obj_new('IDLgrAxis',2,range=self.zran[*,0],title=self.ztit,name='axis',_extra=v6p0_extra,$
              color=white,ticklen=self.default[0],location=[-cshift,1-cshift,1],/exact)
          self.drawGroup->add,self.drawzAxis
         endif
         drawPlot = obj_new('idlgrsurface',style=2,color=[255,255,255],shading=shading,palette=drawPal,name='twod',_extra=v6p0_extra)
         self.title->getproperty,strings=title & obj_destroy,self.title
         self.title = obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,_extra=v6p0_extra,$
            color=white,alignment=0.5,location=[0,1.05,1.0],name='titl')
         self.drawModel->add,self.title
       endif else begin
         type = 1   ;contour plot
         if keyword_set(usepolygon) then begin
           ;use idlgrpolygon object instead of idlgrcontour object to speed up plotting
           drawPlot = obj_new('IDLgrPolygon',palette=drawPal,shading=shading,style=2,shade_range=[1,255],name='twod',$
                _extra=v6p0_extra)
         endif else begin
           drawPlot = obj_new('IDLgrContour',/fill,name='twod',c_color=1b+bindgen(255),palette=drawPal,shade_range=[1,255],$
                shading=shading,_extra=v6p0_extra)
         endelse
       endelse
       newplotInfo={dm_plot_info,obj:drawPlot,type:type,coltable:coltable,shading:shading,legend:'',color:mycolor,psym:'',symsize:0e,$
            thick:0e,linestyle:0,xPtr:ptr_new(xdat,no_copy=no_copy),yPtr:ptr_new(ydat,no_copy=no_copy),$
            zPtr:ptr_new(zdat,no_copy=no_copy),iPtr:ptr_new(idat,no_copy=no_copy),zpos:zpos,hide:0b}
       ;colorbar
       ;check if the colorbar axis exists
       if not obj_valid(self.drawxAxis[2]) then begin
         ;x
         self.drawxAxis[2] = obj_new('IDLgrAxis',0,range=[1.1,1.2]-cshift/2.,hide=self.hideIntbar,_extra=v6p0_extra,$
            color=white,ticklen=0.0,/exact,location=[1.1-cshift/2,-cshift/4.,0],/notext,name='cbar')     ;bottom
         self.drawxAxis[3] = obj_new('IDLgrAxis',0,range=[1.1,1.2]-cshift/2.,hide=self.hideIntbar,_extra=v6p0_extra,$
            color=white,ticklen=0.0,location=[1.2-cshift/2.,1-cshift/4.,0],/exact,name='cbar',/notext)   ;top
         self.drawModel->add,self.drawxAxis[2]
         self.drawModel->add,self.drawxAxis[3]
           ;y
         self.drawyAxis[2] = obj_new('IDLgrAxis',1,range=self.iran[*,0],/notext,minor=0,hide=self.hideIntbar,$
            color=white,ticklen=self.default[0],location=[1.1-cshift/2.,-cshift/4.,0],/exact,$
            _extra=v6p0_extra,name='cbar');left
         self.drawyAxis[3] = obj_new('IDLgrAxis',1,range=self.iran[*,0],color=white,$
            hide=self.hideIntbar,ticklen=self.default[0],/tickdir,location=[1.2-cshift/2.,-cshift/4.,0],/exact,$
            name='cbar',minor=0,textpos=1,_extra=v6p0_extra) ;right
         self.drawModel->add,self.drawyAxis[2]
         self.drawModel->add,self.drawyAxis[3]
         i_level = self.iran[0,0]+findgen(256)/255.0*(self.iran[1,0]-self.iran[0,0])
         self.colBar = obj_new('IDLgrContour',transpose([[i_level],[i_level]]),geomy=i_level,geomz=-0.001,$
            geomx=[1.1,1.2]-cshift/2.,/plan,/fill,name='cbar',c_color=1b+bindgen(255),palette=drawPal,shading=shading,$
            hide=self.hideIntbar,_extra=v6p0_extra)
         self.drawModel->add,self.colBar
       endif
    endif else begin               ;1d data
       type = 0    ;1d plot
       if n_elements(symsize) eq 0 then symsize=self.default[1] else symsize=(0.0>(symsize[0])<1.0)
       if n_elements(linestyle) eq 0 then linestyle=0 else linestyle=self->getlinestyle(linestyle[0])
       if n_elements(thick) eq 0 then thick=self.default[2] else thick=1.0>(thick[0])<10.0
       if n_elements(psym) eq 0 then psym='no symbol' else psym=self->getpsym(psym,/name)
       if n_elements(legend) eq 0 then legend=''
       if strlen(legend) eq 0 then legend=' '+(dm_to_string(color))[0]+' '+psym else legend=' '+self->textadjust(legend,/AA)
       if n_elements(layer) eq 0 then begin
           layer = n_elements(*self.plotInfo)
           if layer gt 0 then onedplot=where((*self.plotInfo)[*].type eq 0,layer)
       endif
       drawPsym = self->getpsym(psym,symsize,thick,mycolor)
       drawPlot = obj_new('IDLgrPlot',xdat,ydat,color=mycolor,symbol=drawPsym,linestyle=linestyle,zvalue=-0.0001*(1+layer),$
           /use_zvalue,thick=thick,xrange=self.xran[*,0],yrange=self.yran[*,0],name='plot',_extra=v6p0_extra)
       newplotInfo = {dm_plot_info,obj:drawPlot,type:type,coltable:-1,shading:-1,legend:legend,color:mycolor,psym:psym,$
           symsize:symsize,thick:thick,linestyle:linestyle,xPtr:ptr_new(xdat),yPtr:ptr_new(ydat),zPtr:ptr_new(layer),$
           iPtr:ptr_new(),zpos:zpos,hide:0b}
       if n_elements(yerr) eq n_elements(ydat) then begin
           nx  = n_elements(xdat)
           x   = fltarr(nx*2)
           y   = x
           con = lonarr(nx*3)
           idx = lindgen(nx)
           x[idx*2L]     = xdat
           x[idx*2L+1]   = xdat
           y[idx*2L]     = ydat-yerr
           y[idx*2L+1]   = ydat+yerr
           con[idx*3L]   = 2
           con[idx*3L+1] = 2L*idx
           con[idx*3L+2] = 2L*idx+1 & idx = -1
           errorPlot = obj_new('idlgrpolyline',x,y,fltarr(nx*2)-0.0001*(0.5+layer),color=mycolor,$
                polylines=temporary(con),thick=thick,name='error',_extra=v6p0_extra)
           self.drawGroup->add,errorPlot
           newplotInfo = [newplotInfo,{dm_plot_info,obj:errorPlot,type:-1,coltable:-1,shading:-1,legend:'',color:mycolor,psym:'',$
                symsize:0e,thick:thick,linestyle:0,xPtr:ptr_new(x,/no_copy),yPtr:ptr_new(y,/no_copy),zPtr:ptr_new(layer),iPtr:ptr_new(),$
                zpos:zpos,hide:0b}]
       endif
    endelse
    self.drawGroup->add,drawPlot
    if keyword_set(no_copy) then begin ;destroy xdat and ydat and zdat
       xdat = 0 & tmp = temporary(xdat)
       ydat = 0 & tmp = temporary(ydat)
       zdat = 0 & tmp = temporary(zdat)
    endif
    if n_elements(*self.plotInfo) eq 0 then $
       *self.plotInfo = newplotInfo $
    else $
       *self.plotInfo = [*self.plotInfo,newplotInfo]
    if type eq 2 then begin    ;initial rotation for surface plot
       self.drawGroup->rotate,[1,0,0],-90
       self.drawGroup->rotate,[0,1,0],30
       self.drawGroup->rotate,[1,0,0],30
    endif

    onedplot = where((*self.plotInfo)[*].type eq 0,count0)
    twodplot = where((*self.plotInfo)[*].type gt 0,count1)
    sufplot  = where((*self.plotInfo)[*].type eq 2,count2)
    ;allow(disallow) changing ztitle,zran,colortable, disallow(allow) changing color,psym,and linestyle
    sensitive = (count2 gt 0)
    self->my_widget_control,['ztitBut','zlogBut','zranBut'],sensitive=sensitive
    sensitive = (count1 gt 0)
    self->my_widget_control,['iranBut','ilogBut','cs_coltable','cs_shade'],sensitive=sensitive
    id = widget_info(self.tlb,find_by_uname='cs_hidecbar')
    if id gt 0 then widget_control,id,sensitive=sensitive
    if sensitive then dm_set_button,id,self.hideIntbar,onstring='Show Colorbar',offstring='Hide Colorbar'
    sensitive = (count0 gt 0)
    self->my_widget_control,['pcolBut','psymBut','lineBut','lgndBut'],sensitive=sensitive
    id = widget_info(self.tlb,find_by_uname='cs_hidelegend')
    if id gt 0 then widget_control,id,sensitive=sensitive
    if sensitive then begin
       self.drawLegend->setproperty,hide=self.hideLegend
       dm_set_button,id,self.hideLegend,onstring='Show Legend',offstring='Hide Legend'
    endif else $
       self.drawLegend->setproperty,/hide
end

;draw the data using object graphics
pro dm_plot::draw,view=view
    widget_control,/hourglass
    self->totop ;push the drawwindow to the top
    if keyword_set(view) then begin ;show the view only
        self.drawWin->draw,self.drawView
        return
    endif
    widget_control,self.plotWin,/DRAW_EXPOSE_EVENTS
    nplot = n_elements(*self.plotInfo)
    if nplot eq 0 then begin
        self.drawWin->draw,self.drawView
        return
    endif
    surfplot = where((*self.plotInfo)[*].type eq 2,surfcount)
    onedplot = where((*self.plotInfo)[*].type eq 0,onedcount)
    if surfcount gt 0 then cshift = [-0.5,0] else cshift = [0,0]

    if self.xlog eq 1 then xran = alog10(self.xran[*,2]) else xran = self.xran[*,0]
    if self.ylog eq 1 then yran = alog10(self.yran[*,2]) else yran = self.yran[*,0]
    if self.zlog eq 1 then zran = alog10(self.zran[*,2]) else zran = self.zran[*,0]
    if self.ilog eq 1 then iran = alog10(self.iran[*,2]) else iran = self.iran[*,0]
    if total(finite(xran)) ne 2 then xconv = [0.0,1.0] else xconv = norm_coord(xran)
    if total(finite(yran)) ne 2 then yconv = [0.0,1.0] else yconv = norm_coord(yran)
    if total(finite(zran)) ne 2 then zconv = [0.0,1.0] else zconv = norm_coord(zran)
    if total(finite(iran)) ne 2 then iconv = [0.0,1.0] else iconv = norm_coord(iran)
    if surfcount eq 0 then zconv = [0.0,1.0]

    ;set x,y coordinate conversions
    oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrAxis') then begin   ;axis
         oChildArr[i]->getproperty,direction=dir
         if dir eq 1 then begin
              oChildArr[i]->setproperty,ycoord_conv=iconv+cshift/4.,range=self.iran[*,2*self.ilog],$
                 minor=(self.ilog eq 1)?(-1):0,log=self.ilog
         endif
         oChildArr[i]->getproperty,ticktext=tick
         tick->setproperty,font=self.font[0],recompute_dimensions=2
         oChildArr[i]->setproperty,ticktext=tick,thick=self.axisthick
       endif else if obj_isa(oChildArr[i],'IDLgrContour') then begin ;contour
         i_level = iran[0]+findgen(256)/255.0*(iran[1]-iran[0])
         oChildArr[i]->setproperty,ycoord_conv=iconv+cshift/4.,$
            data_value=transpose([[i_level],[i_level]]),c_value=i_level,geomy=i_level,shading=max((*self.plotInfo)[*].shading)
       endif
    endfor

    oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
    clipplane = [[-1,0,0,xran[0]],[1,0,0,-xran[1]],[0,-1,0,yran[0]],[0,1,0,-yran[1]]]
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrAxis') then begin
         oChildArr[i]->getproperty,direction=dir
         if dir eq 0 then begin
          oChildArr[i]->setproperty,xcoord_conv=xconv+cshift,range=self.xran[*,2*self.xlog],log=self.xlog
         endif else if dir eq 1 then begin
          oChildArr[i]->setproperty,ycoord_conv=yconv+cshift,range=self.yran[*,2*self.ylog],log=self.ylog
         endif else begin
          oChildArr[i]->setproperty,zcoord_conv=zconv,range=self.zran[*,2*self.zlog],log=self.zlog
         endelse
         oChildArr[i]->getproperty,ticktext=tick
         tick->setproperty,recompute_dimensions=2,font=self.font[0]
         oChildArr[i]->setproperty,ticktext=tick,thick=self.axisthick
       endif
    endfor
    for i=0,n_elements(*self.plotInfo)-1 do begin
       case (*self.plotInfo)[i].type of
         ;error bars,IDLgrPolyline object
         -1:  begin
          x     = (*(*self.plotInfo)[i].xPtr)
          y     = (*(*self.plotInfo)[i].yPtr)
          layer = (*(*self.plotInfo)[i].zPtr)
          if (float(!version.release) lt 5.6) then begin
              index_x = where((x ge self.xran[0,2*self.xlog]) and (x le self.xran[1,2*self.xlog]),count_x)
              if count_x gt 0 then begin
                 x = x[index_x]
                 y = y[index_x] & index_x=-1
              endif else begin
                 x = (self.xran[1,2*self.xlog]+1e2)*[1.0,1.0]
                 y = (self.yran[1,2*self.ylog])*[1.0,1.0]
              endelse
          endif
          if (float(!version.release) lt 5.6) then begin
              index_y = where(y gt self.yran[1,2*self.ylog],count_y)
              if count_y ne 0 then y[index_y] = self.yran[1,2*self.ylog]
              index_y = where(y lt self.yran[0,2*self.ylog],count_y)
              if count_y ne 0 then y[index_y] = self.yran[0,2*self.ylog] & index_y = -1
          endif
          if self.xlog then begin
              index_x = where(x gt 0,count_x)
              if count_x gt 0 then begin
                 x = x[index_x]
                 y = y[index_x] & index_x=-1
              endif else begin
                 x = (self.xran[1,2*self.xlog]+1e2)*[1.0,1.0]
                 y = (self.yran[1,2*self.ylog])*[1.0,1.0]
              endelse
          endif
          if self.ylog then begin
              index_y = where(y le 0, count_y)
              if count_y ne 0 then y[index_y] = self.yran[0,2]
              index_y = -1
          endif
          nx   = n_elements(x)/2
          zpos = 0.0
          if finite((*self.plotInfo)[i].zpos) then $
            zpos = (*self.plotInfo)[i].zpos
          if self.zlog then begin
                if zpos gt 0 then zpos = alog10(zpos) $
                else if finite(self.zran[1,2]) then zpos = alog10(self.zran[1,2])+1.0
          endif
          zpos = -0.0001*(1+layer)+zpos
          z    = fltarr(nx*2)+zpos
          con  = lonarr(nx*3)
          idx  = lindgen(nx)
          con[idx*3]   = 2
          con[idx*3+1] = 2*idx
          con[idx*3+2] = 2*idx+1
          if self.xlog eq 1 then x = alog10(x)
          if self.ylog eq 1 then y = alog10(y)
          x = transpose(x) & y = transpose(y) & z = transpose(z)
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=xconv+cshift,polylines=temporary(con),$
              ycoord_conv=yconv+cshift,zcoord_conv=zconv,data=[temporary(x),temporary(y),temporary(z)]
          end
         ;1d plot,IDLgrPlot object
         0:   begin
          x     = (*(*self.plotInfo)[i].xPtr)
          y     = (*(*self.plotInfo)[i].yPtr)
          layer = (*(*self.plotInfo)[i].zPtr)
          if self.xlog then begin
              index_x = where(x gt 0,count_x)
              if count_x gt 0 then begin
                 x = x[index_x]
                 y = y[index_x] & index_x = -1
              endif else begin
                 x = self.xran[1,2]*10 ;fake data
                 y = self.yran[1,2]*10
              endelse
          endif
          if self.ylog then begin
              index_y = where(y gt 0,count_y)
              if count_y gt 0 then begin
                 x = x[index_y]
                 y = y[index_y] & index_y = -1
              endif else begin
                 x = self.xran[1,2]*10 ;fake data
                 y = self.yran[1,2]*10
              endelse
          endif
          if self.xlog eq 1 then x = alog10(x)
          if self.ylog eq 1 then y = alog10(y)
          zpos  = 0.0
          if finite((*self.plotInfo)[i].zpos) then $
            zpos = (*self.plotInfo)[i].zpos
          if self.zlog then begin
                if zpos gt 0 then zpos = alog10(zpos) $
                else if finite(self.zran[1,2]) then zpos = alog10(self.zran[1,2])+1.0
          endif
          zpos  = -0.0001*(1+layer)+zpos
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=xconv+cshift,ycoord_conv=yconv+cshift,$
                zcoord_conv=zconv,xran=xran,yran=yran,datax=temporary(x),datay=temporary(y),zvalue=zpos
          end
         ;2d plot, IDLgrContour or IDLgrPolygon object
         1:   begin
          xdat = (*(*self.plotInfo)[i].xPtr)
          ydat = (*(*self.plotInfo)[i].yPtr)
          zdat = (*(*self.plotInfo)[i].zPtr)
          if (float(!version.release) lt 5.6) or (self.xlog eq 1) or (self.ylog eq 1) then begin
              ind_x = where((xdat ge self.xran[0,2*self.xlog]) and (xdat le self.xran[1,2*self.xlog]),count_x)
              ind_y = where((ydat ge self.yran[0,2*self.ylog]) and (ydat le self.yran[1,2*self.ylog]),count_y)
              if count_x le 1 or count_y le 1 then begin
                 xdat = self.xran[*,self.xlog*2]
                 ydat = self.yran[*,self.ylog*2]
                 zdat = fltarr(2,2)  & zdat[*] = !values.f_nan
              endif else begin
                 xdat = xdat[ind_x]
                 ydat = ydat[ind_y]
                 zdat = zdat[ind_x,*] & zdat = zdat[*,ind_y]
              endelse
          endif
          if self.xlog eq 1 then xdat = alog10(xdat)
          if self.ylog eq 1 then ydat = alog10(ydat)
          if self.ilog eq 1 then begin
              tmp = where(finite(zdat),count)
              if count gt 0 then index = where(zdat[tmp] le 0,count)
              if count gt 0 then zdat[tmp[index]] = !values.f_nan
              zdat = alog10(zdat)
          endif
          zpos  = 0.0
          if finite((*self.plotInfo)[i].zpos) then $
            zpos = (*self.plotInfo)[i].zpos
          if self.zlog then begin
                if zpos gt 0 then zpos = alog10(zpos) $
                else if finite(self.zran[1,2]) then zpos = alog10(self.zran[1,2])+1.0
          endif
          zpos  = -0.001+zpos
          if obj_isa((*self.plotInfo)[i].obj,'IDLgrPolygon') then begin
              nx   = n_elements(xdat)+1
              ny   = n_elements(ydat)+1
              xdat = [xdat[0],(xdat[0:(0>(nx-3L))]+xdat[1:(nx-2L)])/2.,xdat[nx-2L]]
              ydat = [ydat[0],(ydat[0:(0>(ny-3L))]+ydat[1:(ny-2L)])/2.,ydat[ny-2L]]
              conn = lonarr(5L*(nx-1L)*(ny-1L))
              conn_idx = lindgen((nx-1L)*(ny-1L))*5L
              vert_idx = reform((lindgen(nx,ny))[0:(nx-2L),0:(ny-2L)],(nx-1L)*(ny-1L))
              conn[conn_idx]   = 4L
              conn[conn_idx+1] = 0L+vert_idx
              conn[conn_idx+2] = 1L+vert_idx
              conn[conn_idx+3] = 1L+vert_idx+nx
              conn[temporary(conn_idx)+4] = 0L+temporary(vert_idx)+nx
              dattype = max([size(xdat[0],/type),size(ydat[0],/type)])
              if dattype eq size(0d,/type) then begin  ;double
                 xdat = xdat#(dblarr(ny)+1d) & ydat = (dblarr(nx)+1d)#ydat
              endif else begin                  ;float
                 xdat = xdat#(fltarr(ny)+1e) & ydat = (fltarr(nx)+1e)#ydat
              endelse
              data = [[reform(temporary(xdat),nx*ny)],[reform(temporary(ydat),nx*ny)],[fltarr(nx*ny)+zpos]]
              data = transpose(data)
              zdat = [[zdat],[zdat[*,(ny-2L)]]]
              zdat = [zdat,zdat[(nx-2L),*]]
              vc   = intarr(n_elements(zdat))
              ;finite index
              index = where(finite(zdat),count)
              if count ne 0 then begin
                 vc[index] = 1+round((zdat[index]-iran[0])/(iran[1]-iran[0])*254.)
                 index0 = where(zdat[index] ge iran[1],count)
                 if count ne 0 then vc[index[index0]]=255
                 index0 = where(zdat[index] lt iran[0],count)
                 if count ne 0 then vc[index[index0]]=1
                 index = -1 & index0=-1
              endif
              zdat = -1
              (*self.plotInfo)[i].obj->setproperty,xcoord_conv=xconv+cshift,ycoord_conv=yconv+cshift,$
                 zcoord_conv=zconv,vert_colors=temporary(vc),data=temporary(data),polygons=temporary(conn)
          endif else begin    ;IDLgrContour
              nx   = n_elements(xdat)
              ny   = n_elements(ydat)
              xdat = (temporary(xdat))#(fltarr(ny)+1.) & ydat=(fltarr(nx)+1.)#(temporary(ydat))
              i_level = iran[0]+findgen(256)/255.0*(iran[1]-iran[0])
              i_level[255] = (self.ilog eq 1)?(alog10((self.zmax gt 0)?(self.zmax):(self.iran[1,2]))):(self.zmax)
              (*self.plotInfo)[i].obj->setproperty,xcoord_conv=xconv+cshift,ycoord_conv=yconv+cshift,zcoord_conv=zconv,$
                    c_value=i_level,geomx=temporary(xdat),geomy=temporary(ydat),data_values=temporary(zdat),geomz=zpos,/planar
          endelse
          end
         ;2d plot, IDLgrSurface object
         2:   begin
          xdat = (*(*self.plotInfo)[i].xPtr)
          ydat = (*(*self.plotInfo)[i].yPtr)
          zdat = (*(*self.plotInfo)[i].zPtr)
          if n_elements(*(*self.plotInfo)[i].iPtr) ne 0 then $
            idat = (*(*self.plotInfo)[i].iPtr) $
          else $
            idat = zdat
          if (float(!version.release) lt 5.6) or (self.xlog eq 1) or (self.ylog eq 1) then begin
              ind_x = where((xdat ge self.xran[0,2*self.xlog]) and (xdat le self.xran[1,2*self.xlog]),count_x)
              ind_y = where((ydat ge self.yran[0,2*self.ylog]) and (ydat le self.yran[1,2*self.ylog]),count_y)
              if count_x le 1 or count_y le 1 then begin
                 xdat = self.xran[*,self.xlog*2]
                 ydat = self.yran[*,self.ylog*2]
                 zdat = fltarr(2,2) & zdat[*]=!values.f_nan
                 idat = fltarr(2,2) & idat[*]=!values.f_nan
              endif else begin
                 xdat = xdat[ind_x]
                 ydat = ydat[ind_y]
                 zdat = zdat[ind_x,*] &zdat=zdat[*,ind_y]
                 idat = idat[ind_x,*] &idat=idat[*,ind_y]
              endelse
          endif
          if self.xlog eq 1 then xdat = alog10(xdat)
          if self.ylog eq 1 then ydat = alog10(ydat)
          if self.zlog eq 1 then zdat = alog10(zdat)
          if self.ilog eq 1 then idat = alog10(idat)
          vc = intarr(n_elements(idat))
          ;finite index
          index = where(finite(idat),count)
          if count ne 0 then begin
              vc[index] = 1+round((idat[index]-iran[0])/(iran[1]-iran[0])*254.)
              index0    = where(idat[index] ge iran[1],count)
              if count ne 0 then vc[index[index0]] = 255
              index0    = where(idat[index] lt iran[0],count)
              if count ne 0 then vc[index[index0]] = 1
              index = -1 & index0 = -1
          endif
          idat = -1 ;destroy idat
          (*self.plotInfo)[i].obj->setproperty,xcoord_conv=xconv+cshift,ycoord_conv=yconv+cshift,$
              zcoord_conv=zconv,vert_colors=temporary(vc),min_value=zran[0],max_value=zran[1],$
              datax=temporary(xdat),datay=temporary(ydat),dataz=temporary(zdat)
          end
         else:
       endcase
       if float(!version.release) ge 5.6 then $
         (*self.plotInfo)[i].obj->setproperty,clip_planes=clipplane
    endfor

    ;update the drawlegend
    if onedcount gt 0 then begin
        keyw = {item_name:[''],item_linestyle:[6],item_object:[obj_new()]}
        show = where((*self.plotInfo)[onedplot].hide eq 0,count)
        if count gt 0 then begin
            item_name    = (*self.plotInfo)[onedplot[show]].legend
            item_thick   = (*self.plotInfo)[onedplot[show]].thick
            item_linesty = (*self.plotInfo)[onedplot[show]].linestyle
            item_color   = (*self.plotInfo)[onedplot[show]].color
            item_object  = (*self.plotInfo)[onedplot[show]].obj
            for i=0,count-1 do begin
                item_object[i]->getproperty,symbol=tmp
                item_object[i] = tmp
            endfor
            keyw = {item_name:item_name,item_type:intarr(count),item_linestyle:item_linesty,$
                    item_thick:item_thick,item_color:item_color,item_object:item_object}
        endif
        self.drawLegend->setproperty,_extra=keyw
    endif

    ;make grids
    self.drawxAxis[0]->getproperty,tickvalues=xticks
    self.drawyAxis[0]->getproperty,tickvalues=yticks
    if self.xlog then xticks = alog10(xticks)
    if self.ylog then yticks = alog10(yticks)
    nxtick = n_elements(xticks) & nytick = n_elements(yticks)
    if surfcount gt 0 then onedcount = -1
    xdata  = fltarr(3,nxtick*2) & xconn  = intarr(3*nxtick)  & xdata[2,*] = (-1e-4)*(1+onedcount)
    ydata  = fltarr(3,nytick*2) & yconn  = intarr(3*nytick)  & ydata[2,*] = (-1e-4)*(1+onedcount)
    for i=0,nxtick-1 do begin
        xdata[0,2*i:2*i+1] = xticks[i]
        xdata[1,2*i]       = (self.ylog eq 1)?alog10(self.yran[0,2]):self.yran[0,0]
        xdata[1,2*i+1]     = (self.ylog eq 1)?alog10(self.yran[1,2]):self.yran[1,0]
        xconn[3*i:3*i+2]   = [2,2*i,2*i+1]
    endfor
    for i=0,nytick-1 do begin
        ydata[1,2*i:2*i+1] = yticks[i]
        ydata[0,2*i]       = (self.xlog eq 1)?alog10(self.xran[0,2]):self.xran[0,0]
        ydata[0,2*i+1]     = (self.xlog eq 1)?alog10(self.xran[1,2]):self.xran[1,0]
        yconn[3*i:3*i+2]   = [2,2*i,2*i+1]
    endfor
    self.xgrid->setproperty,data=xdata,polyline=xconn,xcoord_conv=norm_coord(xran)+cshift,$
        ycoord_conv=norm_coord(yran)+cshift
    self.ygrid->setproperty,data=ydata,polyline=yconn,xcoord_conv=norm_coord(xran)+cshift,$
        ycoord_conv=norm_coord(yran)+cshift

    ;set boundary
    self->set_view

    ;draw it
    self.drawWin->draw,self.drawView
end

;handle all events that are passed from dm_plot_event and require access to the internal variables
pro dm_plot::event,event
    if float(!version.release) ge 6.0 then v6p0_extra= {register_properties:1}  ;register_properties keyword for IDL6.0 or later
    eventname=widget_info(event.id,/uname)
    case eventname of
       'tlb':        self->resize,event
       'exitBut':    obj_destroy,self
       'readBut':    self->readasciifile,/oned
       'readSpe':    self->readasciifile,/twod
       'save':       self->saveas,'ps'
       'saveps':     self->saveas,'ps'
       'savejpg':    self->saveas,'jpeg'
       'savetif':    self->saveas,'tiff'
       'savebmp':    self->saveas,'bmp'
       'savepng':    self->saveas,'png'
       'saveppm':    self->saveas,'ppm'
       'saveasc':    self->saveas,'ascii'
       'blackbg':    self->setproperty,misscol=0
       'whitebg':    self->setproperty,misscol=1
       'idltick':    self->setproperty,tickstyle=0
       'owntick':    self->setproperty,tickstyle=1
       'printBut':   self->saveas,'printer'
       'keepBut':    begin
          self->setproperty,keep=1b
          sendevent = {plot_keep,ID:self.tlb,TOP:self.group_leader,HANDLER:self.tlb,object:self}
          if obj_valid(self.parentobj) then $
                call_method,self.parenthandler,self.parentobj,sendevent $
          else if self.group_leader ne 0 then $
                widget_control,self.group_leader,send_event=sendevent,/no_copy
         end
       'mkcrBut':  begin
          self->setproperty,keep=0b
          sendevent = {plot_make_current,ID:self.tlb,TOP:self.group_leader,HANDLER:self.tlb,object:self}
          if obj_valid(self.parentobj) then $
                call_method,self.parenthandler,self.parentobj,sendevent $
          else if self.group_leader ne 0 then $    ; send an event there
                widget_control,self.group_leader,send_event=sendevent,/no_copy
         end
       'show_all':   begin
          for i=0,n_elements(*self.plotInfo)-1 do begin
            (*self.plotInfo)[i].obj->setproperty,hide=0
            (*self.plotInfo)[i].hide = 0
          endfor
          if obj_valid(self.title) then self.title->setproperty,hide=0
          ;show all additional text objects
          oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
          for i=0,nKids-1 do begin
            if obj_isa(oChildArr[i],'IDLgrText') then oChildArr[i]->setproperty,hide=0
          endfor
          self->draw
          widget_control,event.id,sensitive=0
         end
       'textBut':    begin
          all_col  = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red',$
            'dark blue','dark magenta','dark green','dark cyan','dark yellow']
          all_ori  = ['Horizontal','Vertical']
          all_font = ['helvetica','courier','times','symbol','monospace symbol','hershey']
          default  = ['','0','0','12','0']
          new = dm_dialog_input(['text:','orientation:','font:','font size:','color:'],title='Add text',default=default,$
                  is_droplist=[0,1,1,0,1],droplist_content=[ptr_new(all_ori),ptr_new(all_font),ptr_new(all_col)],$
                  xsize=130,dialog_parent=self.tlb)
          if strlen(new[0]) eq 0 then break   ;null string
          new[0] = self->textadjust(new[0],/AA)
          xstart = 0.04 & ystart = 0.9;1.01
          if n_elements(*self.plotInfo) gt 0 then $
              if max((*self.plotInfo)[*].type) eq 2 then xstart = -0.6    ;surface plot
          self.drawView->getproperty,dimensions=dim                 ;viewport size
          fsize = (8>(dm_to_number(new[3],/int)))
          nsize = fix((1>min(dim/self.default[5:6]))*fsize)
          ofont = obj_new('idlgrfont',new[2],size=nsize,substitute='Hershey')
          if new[1] eq 'Horizontal' then begin
                baseline = [1.0,0,0]
                updir    = [0,1.0,0]
          endif else begin
                baseline = [0,1.0,0]
                updir    = [-1,0,0]
          endelse
          otext = obj_new('idlgrtext',new[0],font=ofont,color=self->getcolor(new[4]),xcoord_conv=[xstart,1],$
              ycoord_conv=[ystart,1],zcoord_conv=[1.0,1.0],recompute_dimensions=2,/enable,$
              baseline=baseline,updir=updir,_extra=v6p0_extra,name='text')
          self.drawModel->add,otext
          self.drawWin->draw,self.drawView
          help,ofont,output=id & id=strsplit(id,' ',/extract) & id=id[n_elements(id)-1]
          if n_elements(*self.textsize) eq 0 then *self.textsize=[id,dm_to_string(fsize)] $
          else *self.textsize=[*self.textsize,id,dm_to_string(fsize)]
         end
       'isoBut':    self->setproperty,iso=(1b)-self.iso
       'xgridBut':  self->setproperty,showxgrid=(1b)-self.showxgrid
       'ygridBut':  self->setproperty,showygrid=(1b)-self.showygrid
       'titlBut':   begin
          self.title->getproperty,strings=title
          new = dm_dialog_input('title:',title='Set title',xsize=150,default=title,dialog_parent=self.tlb)
          if new ne title then self->setproperty,title=self->textadjust(new,/AA)
         end
       'xranBut':    begin
          new = dm_dialog_input(['from:','to:'],title='Set xrange',xsize=100,default=float(self.xran[*,2*self.xlog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.xran[*,2*self.xlog])) ne 0 then $
                 if new[1] gt new[0] then  self->setproperty,xran=new
         end
       'xlogBut':    self->setproperty,xlog=(1b)-self.xlog
       'xtitBut':    begin
          self.xtit->getproperty,strings=xtit
          new = dm_dialog_input('x tit:',title='Set xtitle',xsize=150,default=xtit,dialog_parent=self.tlb)
          if new ne xtit then self->setproperty,xtit=self->textadjust(new,/AA)
         end
       'yranBut':    begin
          new = dm_dialog_input(['from:','to:'],title='Set yrange',xsize=100,default=float(self.yran[*,2*self.ylog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.yran[*,2*self.ylog])) ne 0 then $
                 if new[1] gt new[0] then self->setproperty,yran=new
         end
       'ylogBut':    self->setproperty,ylog=(1b)-self.ylog
       'ytitBut':    begin
          self.ytit->getproperty,strings=ytit
          new = dm_dialog_input('y tit:',title='Set ytitle',xsize=150,default=ytit,dialog_parent=self.tlb)
          if new ne ytit then self->setproperty,ytit=self->textadjust(new,/AA)
         end
       'zranBut':    begin
          new = dm_dialog_input(['from:','to:'],title='Set zrange',xsize=100,default=float(self.zran[*,2*self.zlog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.zran[*,2*self.zlog])) ne 0 then $
                 if new[1] gt new[0] then self->setproperty,zran=new
         end
       'zlogBut':    self->setproperty,zlog=(1b)-self.zlog
       'iranBut':    begin
          new = dm_dialog_input(['from:','to:'],title='Set irange',xsize=100,default=float(self.iran[*,2*self.ilog]),/float,dialog_parent=self.tlb)
          if total(finite(new)) eq 2 then $
              if total(abs(new-self.iran[*,2*self.ilog])) ne 0 then $
                 if new[1] gt new[0] then self->setproperty,iran=new
         end
       'ilogBut':    self->setproperty,ilog=(1b)-self.ilog
       'psymBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot = where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_psym=['no symbol','circle','bullet','plus','star','dot','diamond','triangle','square','cross','greater','less']
          name    = strarr(count*2) & default=fltarr(count*2) & is_droplist=bytarr(count*2) & old=strarr(count*2)
          for i=0,count-1 do begin
              name[(2*i):(2*i+1)] = ['line '+dm_to_string(i)+': symbol name','       '+'symbol size']
              default[2*i]        = 0>(where(all_psym eq (*self.plotInfo)[onedplot[i]].psym))
              default[2*i+1]      = (*self.plotInfo)[onedplot[i]].symsize
              old[2*i]            = (*self.plotInfo)[onedplot[i]].psym
              old[2*i+1]          = dm_to_string((*self.plotInfo)[onedplot[i]].symsize)
              is_droplist[(2*i):(2*i+1)] = [1,0]
          endfor
          if count eq 1 then name = ['symbol name','symbol size']
          new = dm_dialog_input(name,title='Set plot symbol',default=default,is_droplist=is_droplist,$
              droplist_content=ptr_new(all_psym),xsize=100,dialog_parent=self.tlb)
          index = where(strlen(new) eq 0,count0)
          if count0 gt 0 then new[index]=old[index]
          changeid = -1
          for i=0,count-1 do $
              if (new[2*i] ne old[2*i]) or (new[2*i+1] ne old[2*i+1]) then begin
                 (*self.plotInfo)[onedplot[i]].psym=new[2*i]
                 (*self.plotInfo)[onedplot[i]].symsize=0.0>(dm_to_number(new[2*i+1]))<1.0
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'lineBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot  = where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_lines = ['solid','dotted','dashed','dash dot','dash dot dot dot','long dash','no line']
          name = strarr(count*2) & default = fltarr(count*2)
          is_droplist = bytarr(count*2)+1b & old = strarr(count*2)
          for i=0,count-1 do begin
              name[(2*i):(2*i+1)] = ['line '+dm_to_string(i)+': linestyle','       '+'thickness']
              default[2*i]        = (*self.plotInfo)[onedplot[i]].linestyle
              default[2*i+1]      = (*self.plotInfo)[onedplot[i]].thick
              old[2*i]            = dm_to_string((*self.plotInfo)[onedplot[i]].linestyle)
              old[2*i+1]          = dm_to_string((*self.plotInfo)[onedplot[i]].thick)
              is_droplist[(2*i):(2*i+1)] = [1,0]
          endfor
          if count eq 1 then name = ['linestyle','thickness']
          new = dm_dialog_input(name,title='Set line style',default=default,is_droplist=is_droplist,$
              droplist_content=ptr_new(all_lines),xsize=100,dialog_parent=self.tlb,/return_number)
          index = where(strlen(new) eq 0,count0)
          if count0 gt 0 then new[index]=old[index]
          changeid = -1
          for i=0,count-1 do $
              if (new[2*i] ne old[2*i]) or (new[2*i+1] ne old[2*i+1]) then begin
                 (*self.plotInfo)[onedplot[i]].linestyle = dm_to_number(new[2*i],/int)
                 (*self.plotInfo)[onedplot[i]].thick     = 1.0>(dm_to_number(new[2*i+1]))<10.0
                 changeid = [changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'pcolBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot = where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          all_col  = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red','dark blue',$
              'dark magenta','dark green','dark cyan','dark yellow','user define']
          name = strarr(count) & default = intarr(count) & is_droplist = bytarr(count)+1b
          old  = self->getcolor((*self.plotInfo)[onedplot].color,/name)
          define = where(old eq 'user define',count0)
          if count0 ne 0 then define = (*self.plotInfo)[onedplot[define[0]]].color else define=[255,255,255]
          for i=0,count-1 do begin
              name[i]    = 'line '+dm_to_string(i)
              default[i] = 0>(where(all_col eq old[i]))
          endfor
          if count eq 1 then name=['color']
          new = dm_dialog_input(name,title='Set plot color',default=default,is_droplist=is_droplist,$
              droplist_content=ptr_new(all_col),xsize=100,dialog_parent=self.tlb)
          changeid = -1
          for i=0,count-1 do $
              if new[i] ne old[i] then begin
                 (*self.plotInfo)[onedplot[i]].color=self->getcolor(new[i],define=define)
                 changeid=[changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'lgndBut':    begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          onedplot = where((*self.plotInfo)[*].type eq 0,count)        & if count eq 0 then break
          name = 'line '+dm_to_string(indgen(count)) & old=(*self.plotInfo)[onedplot].legend
          if count eq 1 then name=['label']
          new  = dm_dialog_input(name,title='Set legend labels',default=old,xsize=100,dialog_parent=self.tlb)
          new  = self->textadjust(new,/AA)
          changeid = -1
          for i=0,count-1 do $
              if strtrim(new[i],2) ne strtrim(old[i],2) then begin
                 (*self.plotInfo)[onedplot[i]].legend = ' '+strtrim(new[i],2)
                 changeid = [changeid,onedplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),$
              changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'pnsetBut':   ok = dialog_printersetup(self.printer,dialog_parent=self.tlb)
       'plotWin':    begin
          if event.type eq 4 then self.drawWin->draw,self.drawView else begin
              if n_elements(*self.plotInfo) eq 0 then break    ; no plots
              if event.release eq 4 then begin   ;right mouse button clicked
                 self.select = obj_new()
                 tmp=self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then begin
                   self.select = tmp[0]
                   tmp[0]->getproperty,name=name
                   if name eq 'error' then begin    ;special care to error plot
                    name  = ''
                    index = where((*self.plotInfo).obj eq self.select,count)
                    if count ne 0 then $
                        if (*self.plotInfo)[index[0]-1].type eq 0 then begin ;double check
                           name = 'plot'
                           self.select=(*self.plotInfo)[index[0]-1].obj
                        endif
                   endif
                   if strlen(name) ne 0 then begin
                    if self.select eq self.drawxAxis[1] then begin
                        name = 'titl' & self.select = self.title
                        self.title->getproperty,strings=tmp
                        if strlen(tmp) eq 0 then break
                    endif
                    menuid = widget_info(event.id, find_by_uname='cs_'+name+'Menu')
                    logid  = widget_info(menuid,find_by_uname='cs_logscale')
                    setlog = 0
                    case name of    ;for axis and cbar, need to set properly the log scale button
                        'axis':begin
                           setlog = 1
                           self.select->getproperty,direction=direction
                           case direction of
                             0:     log_yn = self.xlog
                             1:     log_yn = self.ylog
                             else:  log_yn = self.zlog
                           endcase
                          end
                        'cbar':begin
                           setlog = 1
                           log_yn = self.ilog
                            end
                        else:
                    endcase
                    if setlog eq 1 then dm_set_button,logid,log_yn,onstring='Normal Scale',offstring='Logarithmic Scale'
                    widget_displaycontextmenu,event.id,event.x,event.y,menuid
                   endif
                 endif else begin  ;turn off zoom
                   tmp = max((*self.plotInfo)[*].type)
                   if tmp eq 2 then begin
                    self.drawGroup->reset
                    self.drawModel->reset
                    self.drawGroup->rotate,[1,0,0],-90
                    self.drawGroup->rotate,[0,1,0],30
                    self.drawGroup->rotate,[1,0,0],30
                    self.drawWin->draw,self.drawView
                   endif
                   if (total(finite(self.xran[*,2*self.xlog+1])) eq 2) then begin
                        self->setproperty,xran=self.xran[*,2*self.xlog+1],yran=self.yran[*,2*self.ylog+1]
                        self.xran[*,2*self.xlog+1]=[!values.f_nan,!values.f_nan]
                        self.yran[*,2*self.ylog+1]=[!values.f_nan,!values.f_nan]
                   endif
                 endelse
              endif else begin
                 if max((*self.plotInfo)[*].type) eq 2 then $
                   self->rotate,event $
                 else $
                   self->zoom,event
              endelse
          endelse
         end
       'cs_title':   begin
          if obj_isa(self.select,'IDLgrAxis') then begin
              self.select->getproperty,title=title,direction=direction
              case direction of
                 0:    name = 'xtitle'
                 1:    name = 'ytitle'
                 else: name = 'ztitle'
              endcase
              if title eq self.title then name = 'title'
          endif else begin
              title = self.select
              title->getproperty,name=name
              if name eq 'titl' then name = 'title'
          endelse
          if obj_valid(title) then begin
              title->getproperty,strings=old
              new = dm_dialog_input(name+':',title='Set '+name,xsize=150,default=old,dialog_parent=self.tlb)
              if new ne old then begin
                 if name eq 'title' then self->setproperty,title=self->textadjust(new,/AA) $
                 else begin
                   if name eq 'text' and strlen(new) eq 0 then begin    ;remove it
                        title->getproperty,font=ofont
                        obj_destroy,[ofont,title]
                   endif else $
                       title->setproperty,string=self->textadjust(new,/AA)
                   self.drawWin->draw,self.drawView
                 endelse
              end
          endif
         end
       'cs_range':   begin
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              self.select->getproperty,direction=direction
              case direction of
                 0:     type = 'x'
                 1:     type = 'y'
                 2:     type = 'z'
                 else:  type = 'i'
              endcase
          endif else  type = 'i'
          old = call_function("dm_to_string",(type eq 'x')?(self.xran[*,2*self.xlog]):$
            ((type eq 'y')?(self.yran[*,2*self.ylog]):((type eq 'z')?(self.zran[*,2*self.zlog]):$
            (self.iran[*,2*self.ilog]))))
          new = dm_dialog_input(['from:','to:'],title='Set '+type+'range',xsize=100,default=old,dialog_parent=self.tlb)
          if (old[0] ne new[0]) or (old[1] ne new[1]) then begin
              new = dm_to_number(new)
              if total(finite(new)) eq 2 then $
                 if new[1] gt new[0] then begin
                    case type of
                        'x':    self->setproperty,xran=new
                        'y':    self->setproperty,yran=new
                        'z':    self->setproperty,zran=new
                        'i':    self->setproperty,iran=new
                    endcase
                 endif
          endif
         end
       'cs_logscale':begin
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              self.select->getproperty,direction=direction
              case direction of
                 0:    type = 'xlog'
                 1:    type = 'ylog'
                 2:    type = 'zlog'
                 else: type = 'ilog'
              endcase
          endif else  type = 'ilog'
          case type of
             'xlog':  self->setproperty,xlog=1-self.xlog
             'ylog':  self->setproperty,ylog=1-self.ylog
             'zlog':  self->setproperty,zlog=1-self.zlog
             else:    self->setproperty,ilog=1-self.ilog
          endcase
         end
       'cs_axisthick':begin
          old = dm_to_string(self.axisthick)
          new = dm_dialog_input('thick:',title='Set axis thickness',xsize=100,default=old,dialog_parent=self.tlb)
          if old ne new then begin
              new = dm_to_number(new)
              if finite(new) then $
                 self->setproperty,axisthick=new
          end
         end
       'cs_textori':begin
          self.select->getproperty,baseline=baseline
          if baseline[0] eq 1 then old='Horizontal' else old='Vertical'
          ori = ['Horizontal','Vertical']
          new = dm_dialog_input('Orientation:',title='Set Text Orientation',default=(0>((where(ori eq old))[0])),$
                 is_droplist=[1],droplist_content=ptr_new(ori),xsize=100,dialog_parent=self.tlb)
          if new ne old then begin
            if new eq 'Horizontal' then begin
                baseline = [1.0,0,0]
                updir    = [0,1.0,0]
            endif else begin
                baseline = [0,1.0,0]
                updir    = [-1,0,0]
            endelse
            self.select->setproperty,baseline=baseline,updir=updir
            self.drawWin->draw,self.drawView
          endif
         end
       'cs_txtcolor':begin
          if (self.select eq self.xgrid) or (self.select eq self.ygrid) then begin
            is_grid = 1
            title   = 'Set grid color'
            object  = [self.xgrid,self.ygrid]
          endif else begin
            is_grid = 0
            title   = 'Set text color'
            object  = self.select
          endelse
          self.select->getproperty,color=color
          all_col = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red','dark blue',$
              'dark magenta','dark green','dark cyan','dark yellow','user define']
          old = self->getcolor(color,/name)
          new = dm_dialog_input('color:',title=title,default=0>(where(all_col eq old)),$
              is_droplist=[1],droplist_content=ptr_new(all_col),xsize=100,dialog_parent=self.tlb,parentobject=self,$
              object=object,property='color',apply=apply)
          if (apply eq 1) or (new[0] ne old[0]) then begin
              new = self->getcolor(new[0],define=color)
              if is_grid then begin
                self.xgrid->setproperty,color=new
                self.ygrid->setproperty,color=new
              endif else $
                self.select->setproperty,color=new
              self.drawWin->draw,self.drawView
          endif
         end
       'cs_font':begin
          index = 0 ;for ticktext
          self.select->getproperty,name=name
          if name eq 'axis' then begin
              if self.select eq self.drawxAxis[0] then index =  1  $
              else if self.select eq self.drawxAxis[1] then index = 4 $
              else if self.select eq self.drawyAxis[0] then index = 2 $
              else if self.select eq self.drawzAxis then index = 3
          endif else if name eq 'titl' then index = 4 $
          else if name eq 'legend' then index = 5 $
          else if name eq 'text' then index = -1
          if index eq -1 then begin
              self.select->getproperty,font=ofont
              ofont->getproperty,name=old
              help,ofont,output=id & id=strsplit(id,' ',/extract) & id=id[n_elements(id)-1]
              tmpid = where(*self.textsize eq id)
              fsize = float((*self.textsize)[tmpid[0]+1])
          endif else begin
              ofont = self.font[index]
              ofont->getproperty,name=old
              fsize = self.fsize[index]
          endelse
          old       = strlowcase(old)
          all_fonts = ['helvetica','courier','times','symbol','monospace symbol','hershey']
          default   = [0>(where(all_fonts eq old)),fsize]
          old       = [old,dm_to_string(fsize)]
          new       = dm_dialog_input(['font name:','font size:'],title='Set font',default=default,is_droplist=[1,0],$
                        droplist_content=ptr_new(all_fonts),xsize=100,dialog_parent=self.tlb)
          ind       = where(strlen(new) eq 0,count)
          if count gt 0 then new[ind]=old[ind]
          redraw    = 0
          if new[0] ne old[0] then begin
              ofont->setproperty,name=new[0],substitute='Hershey'
              ofont->getproperty,name=tmp
              if (index ge 0) then $
                if strmid(strlowcase(tmp),0,7) eq 'hershey' then self.fscale[index]=1.2 else self.fscale[index]=1.0
              redraw = 1
          endif
          if new[1] ne old[1] then begin
              if index ge 0 then $
                  self.fsize[index]=(8>(dm_to_number(new[1],/int))) $
              else $
                  (*self.textsize)[tmpid[0]+1]=dm_to_string(8>dm_to_number(new[1],/int))
              redraw = 1
          endif
          if redraw then self->draw
         end
       'cs_symbol':begin
          index    = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          all_psym = ['no symbol','circle','bullet','plus','star','dot','diamond','triangle','square','cross','greater','less']
          default  = [0>(where(all_psym eq (*self.plotInfo)[index].psym)),(*self.plotInfo)[index].symsize]
          old      = [(*self.plotInfo)[index].psym,dm_to_string((*self.plotInfo)[index].symsize)]
          new = dm_dialog_input(['symbol name:','symbol size:'],title='Set plot symbol',default=default,$
              is_droplist=[1,0],droplist_content=ptr_new(all_psym),xsize=100,dialog_parent=self.tlb,$
              info='0.0<symbol size<1.0')
          if (new[0] ne old[0]) or (new[1] ne old[1]) then begin
              (*self.plotInfo)[index].psym    = new[0]
              (*self.plotInfo)[index].symsize = 0.0>(dm_to_number(new[1]))<1.0
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_color':   begin
          index   = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          all_col = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red','dark blue',$
              'dark magenta','dark green','dark cyan','dark yellow','user define']
          old     = self->getcolor((*self.plotInfo)[index].color,/name)
          new     = dm_dialog_input('color:',title='Set plot color',default=0>(where(all_col eq old)),$
              is_droplist=[1],droplist_content=ptr_new(all_col),xsize=100,dialog_parent=self.tlb,parentobject=self,$
              object=self.select,property='color',apply=apply)
          if (apply eq 1) or (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].color=self->getcolor(new[0],define=(*self.plotInfo)[index].color)
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_linestyle':begin
          if (self.select eq self.xgrid) or (self.select eq self.ygrid) then begin
            all_lines = ['solid','dotted','dashed','dash dot','dash dot dot dot','long dash']
            is_grid   = 1
            self.select->getproperty,linestyle=old
            object = [self.xgrid,self.ygrid]
            title  = 'Set grid linestyle'
          endif else begin
            all_lines = ['solid','dotted','dashed','dash dot','dash dot dot dot','long dash','no line']
            is_grid   = 0
            index  = where((*self.plotInfo)[*].obj eq self.select,count)
            if count ne 1 then break
            old    = (*self.plotInfo)[index].linestyle
            object = self.select
            title  = 'Set plot linestyle'
          endelse
          new = dm_dialog_input('linestyle:',title=title,default=old,$
              is_droplist=[1],droplist_content=ptr_new(all_lines),xsize=100,dialog_parent=self.tlb,parentobject=self,$
              object=object,property='linestyle',apply=apply,/return_number)
          if (apply eq 1) or (new[0] ne old[0]) then begin
            if is_grid then begin
                self.xgrid->setproperty,linestyle=new[0]
                self.ygrid->setproperty,linestyle=new[0]
                self.drawWin->draw,self.drawView
            endif else begin
                (*self.plotInfo)[index].linestyle=new[0]
                self->setproperty,plotInfo=(*self.plotInfo),changeid=index
            endelse
          endif
         end
       'cs_linethick':begin
          if (self.select eq self.xgrid) or (self.select eq self.ygrid) then begin
            self.select->getproperty,thick=old
            old     = dm_to_string(old)
            is_grid = 1
            title   = 'Set grid thickness'
            object  = [self.xgrid,self.ygrid]
          endif else begin
            is_grid = 0
            index   = where((*self.plotInfo)[*].obj eq self.select,count)
            if count ne 1 then break
            old     = dm_to_string((*self.plotInfo)[index].thick)
            title   = 'Set plot thickness'
            object  = self.select
          endelse
          new = dm_dialog_input('thickness:',title=title,default=old,xsize=70,dialog_parent=self.tlb,$
                  info='1.0<thick<10.0',parentobj=self,object=object,property='thick',apply=apply)
          if (apply eq 1) or (new[0] ne old[0]) then begin
            new =   dm_to_number(new[0])
            if not finite(new) then new=1.0
            if is_grid then begin
                self.xgrid->setproperty,thick=1.0>(new)<10.0
                self.ygrid->setproperty,thick=1.0>(new)<10.0
                self.drawWin->draw,self.drawView
            endif else begin
                (*self.plotInfo)[index].thick=1.0>(new)<10.0
                self->setproperty,plotInfo=(*self.plotInfo),changeid=index
            endelse
          endif
         end
       'cs_legend':  begin
          index = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          old   = strtrim((*self.plotInfo)[index].legend,2)
          new   = dm_dialog_input('legend:',title='Set plot legend',default=old,xsize=100,dialog_parent=self.tlb)
          if (new[0] ne old[0]) then begin
              (*self.plotInfo)[index].legend=' '+self->textadjust(new[0],/AA)
              self->setproperty,plotInfo=(*self.plotInfo),changeid=index
          endif
         end
       'cs_layer':     begin
          index = where((*self.plotInfo)[*].obj eq self.select,count)
          if count ne 1 then break
          old   = (*(*self.plotInfo)[index].zPtr)
          new   = dm_dialog_input('layer:',title='Set plot layer',default=old,xsize=60,dialog_parent=self.tlb)
          if strlen(new) eq 0 then break
          if new ne old then begin
              new = dm_to_number(new,/int)
              old = dm_to_number(old,/int)
              nplots  = n_elements(*self.plotInfo)
              ;check if the new layer is already used, if yes, switch that one to old
              onedplot= where((*self.plotInfo)[*].type eq 0,count)
              index0  = -1
              for i=0,count-1 do if *(*self.plotInfo)[onedplot[i]].zPtr eq new then index0=onedplot[i]
              if index0 ne -1 then begin
                 *(*self.plotInfo)[index0].zPtr=old
                 changeid=[index0,index]
              endif else changeid=index
              *(*self.plotInfo)[index].zPtr=new
              self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid
          endif
         end
       'cs_remove':  begin  ;delete a single 1D plot (with its error bars if exist) or a text object
          self.select->getproperty,name=name
          ans = dialog_message('Are you sure you want to remove this '+name+'?',/question,dialog_parent=self.tlb)
          if strlowcase(ans) eq 'yes' then begin
            index = where((*self.plotInfo)[*].obj eq self.select,count)
            if count ne 1 then begin    ;text object to be removed
                self.select->getproperty,font=ofont
                obj_destroy,[ofont,self.select]
                self.drawWin->draw,self.drawView
                break
            endif
            nplot = n_elements(*self.plotInfo)
            oned  = where((*self.plotInfo)[*].type eq 0,n_oned)
            self.select->getproperty,symbol=tmp1
            if obj_valid(tmp1) then begin
                tmp1[0]->getproperty,data=tmp2
                if obj_valid(tmp2) then obj_destroy,tmp2
                obj_destroy,tmp1
            endif
            obj_destroy,self.select
            ptr_free,(*self.plotInfo)[index].xPtr,(*self.plotInfo)[index].yPtr,(*self.plotInfo)[index].zPtr,(*self.plotInfo)[index].iPtr
            tmp = intarr(nplot)+1 & tmp[index]=0 & index=index+1
            if index lt nplot then $
                if (*self.plotInfo)[index].type eq -1 then begin
                    obj_destroy,(*self.plotInfo)[index].obj
                    ptr_free,(*self.plotInfo)[index].xPtr,(*self.plotInfo)[index].yPtr,(*self.plotInfo)[index].zPtr,(*self.plotInfo)[index].iPtr
                    tmp[index]=0
                endif
            tmp = where(tmp eq 1,count)
            if count ne 0 then $
                (*self.plotInfo)=(*self.plotInfo)[tmp] $
            else begin
                ptr_free,self.plotInfo & self.plotInfo=ptr_new(/allocate_heap)
            endelse
            ;update the drawlegend
            keyword = {item_object:[obj_new()],item_name:[''],item_linestyle:[6]}
            if count ne 0 then onedplot=where((*self.plotInfo)[*].type eq 0,count)
            if count gt 0 then begin
                show = where((*self.plotInfo)[onedplot].hide eq 0,count)
                if count gt 0 then begin
                    item_name    = (*self.plotInfo)[onedplot[show]].legend
                    item_thick   = (*self.plotInfo)[onedplot[show]].thick
                    item_linesty = (*self.plotInfo)[onedplot[show]].linestyle
                    item_color   = (*self.plotInfo)[onedplot[show]].color
                    item_object  = (*self.plotInfo)[onedplot[show]].obj
                    for i=0,count-1 do begin
                        item_object[i]->getproperty,symbol=tmp
                        item_object[i] = tmp
                    endfor
                    keyword = {item_name:item_name,item_type:intarr(count),item_linestyle:item_linesty,$
                               item_thick:item_thick,item_color:item_color,item_object:item_object}
                endif
            endif
            self.drawLegend->setproperty,_extra=keyword
            self.drawWin->draw,self.drawView
          endif
         end
       'cs_coltable':begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          twodplot= where((*self.plotInfo)[*].type gt 0,count) & if count eq 0 then break
          all_ctb = ['B-W LINEAR','BLUE/WHITE','GRN-RED-BLU-WHT','RED TEMPERATURE','BLUE/GREEN/RED/YELLOW','STD GAMMA-II','PRISM',$
              'RED-PURPLE','GREEN/WHITE LINEAR','GRN/WHT EXPONENTIAL','GREEN-PINK','BLUE-RED','16 LEVEL','RAINBOW','STEPS',$
              'STERN SPECIAL','Haze','Blue-Pastel-Red','Pastels','Hue Sat Lightness 1','Hue Sat Lightness 2','Hue Sat Value 1',$
              'Hue Sat Value 2','Purple-Red+Stripes','Beach','Mac Style','Eos A','Eos B','Hardcandy','Nature','Ocean',$
              'Peppermint','plasma','Blue-Red','Rainbow','Blue Waves','Volcano','Waves','Rainbow18','Rainbow+white',$
              'Rainbow+black']
          all_ctb = dm_to_string(indgen(n_elements(all_ctb)))+'  '+all_ctb
          name    = 'plot '+dm_to_string(indgen(count)) & is_droplist=bytarr(count)+1b & old=(*self.plotInfo)[twodplot].coltable
          if count eq 1 then name=['color table']
          new     = dm_dialog_input(name,title='Set color table',default=old,is_droplist=is_droplist,$
              droplist_content=ptr_new(all_ctb),xsize=220,/return_number,/int,dialog_parent=self.tlb,parentobject=self,$
              object=(*self.plotInfo)[twodplot].obj,property='coltable',apply=apply,/coltable)
          changeid = -1
          for i=0,count-1 do begin
              if (apply eq 1) or (new[i] ne old[i]) then begin
                 (*self.plotInfo)[twodplot[i]].coltable=new[i]
                 changeid = [changeid,twodplot[i]]
              endif
          endfor
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'cs_shade':  begin
          if n_elements((*self.plotInfo)) eq 0 then break ; no plots
          twodplot = where((*self.plotInfo)[*].type gt 0,count)        & if count eq 0 then break
          choice   = ['Flat','Gouraud']
          name     = 'plot '+dm_to_string(indgen(count)) & is_droplist=bytarr(count)+1b & old=(*self.plotInfo)[twodplot].shading
          if count eq 1 then name=['shading']
          new      = dm_dialog_input(name,title='Set shading',default=old,is_droplist=is_droplist,droplist_content=ptr_new(choice),$
                    xsize=100,/return_number,/int,dialog_parent=self.tlb)
          changeid = -1
          for i=0,count-1 do $
              if new[i] ne old[i] then begin
                 (*self.plotInfo)[twodplot[i]].shading=new[i]
                 changeid = [changeid,twodplot[i]]
              endif
          if n_elements(changeid) gt 1 then self->setproperty,plotInfo=(*self.plotInfo),changeid=changeid[1:(n_elements(changeid)-1)]
         end
       'cs_hidecbar':   self->setproperty,hideintbar=(1b)-self.hideIntbar
       'cs_hidelegend': self->setproperty,hidelegend=(1b)-self.hideLegend
       'cs_hidegrid':   self->setproperty,showxgrid=0,showygrid=0
       'cs_outline':begin
          widget_control,event.id,get_value=value
          if value eq 'Show Outline' then begin
              value = 'Hide Outline'
              show  = 1
          endif else begin
              value = 'Show Outline'
              show  = 0
          endelse
          widget_control,event.id,set_value=value
          self.select->setproperty,show_outline=show
          self.drawWin->draw,self.drawView
         end
       'cs_lgndgap':begin
          self.select->getproperty,gap=old
          new = dm_dialog_input('legend gap:',title='Set Legend Gap',default=old,/float,dialog_parent=self.tlb,$
              info='border gap=2*gap')
          if finite(new) then $
              if new ne old then begin
                 self.select->setproperty,gap=new,border_gap=new*2
                 self.drawWin->draw,self.drawView
              endif
         end
       'cs_lgndwidth':begin
          self.select->getproperty,glyph_width=old
          new=dm_dialog_input('legend glyph width:',title='Set Legend Glyph Width',default=old,/float,dialog_parent=self.tlb)
          if finite(new) then $
              if new ne old then begin
                 self.select->setproperty,glyph_width=new
                 self.drawWin->draw,self.drawView
              endif
         end
       'cs_propsheet':begin
          redraw = 0
          self.drawLegend->getproperty,item_color=item_color,item_linestyle=item_linestyle,hide=hide
          propsheet=obj_new('dm_propsheet',self.select,parentobject=self,/view) ;a modal interactive dialog widet
          obj_destroy,propsheet
          case strlowcase(obj_class(self.select)) of
            'idlgrplot':begin
                self.select->getproperty,symbol=symbol,color=color,linestyle=linestyle,thick=thick,hide=hide
                redraw = 1  ;need to reset legend
                symbol[0]->getproperty,data=data
                if obj_valid(data) then data->setproperty,color=color
                symbol[0]->setproperty,color=color     ;change symbol color
                ;change legend properties
                onedplot = where((*self.plotInfo)[*].type eq 0)
                show     = where((*self.plotInfo)[onedplot].hide eq 0)
                onedplot = onedplot[show]
                changeid = where((*self.plotInfo)[*].obj eq self.select,count)
                if count eq 0 then break
                (*self.plotInfo)[changeid].color     = color
                (*self.plotInfo)[changeid].linestyle = linestyle
                (*self.plotInfo)[changeid].thick     = thick
                (*self.plotInfo)[changeid].hide      = hide
                nplots = n_elements(*self.plotInfo)
                i_num  = where(onedplot eq changeid[0],count)
                if count ne 0 then begin
                 item_color[*,i_num[0]]   = color
                 item_linestyle[i_num[0]] = linestyle
                endif
                ;change the color and thickness of the error plot if any
                if changeid[0]+1 lt nplots then $
                 if (*self.plotInfo)[changeid[0]+1].type eq -1 then begin
                   (*self.plotInfo)[changeid[0]+1].obj->setproperty,thick=thick,color=color,hide=hide
                   (*self.plotInfo)[changeid[0]+1].color     = color
                   (*self.plotInfo)[changeid[0]+1].linestyle = linestyle
                   (*self.plotInfo)[changeid[0]+1].thick     = thick
                   (*self.plotInfo)[changeid[0]+1].hide      = hide
                 endif
                end
            'idlgrlegend':begin
                ;check if the show status has been changed
                self.select->getproperty,hide=newhide
                if newhide ne hide then self->setproperty,hidelegend=newhide
                end
            'idlgrtext':begin
                self.select->getproperty,hide=hide
                if hide eq 1 then self->my_widget_control,'show_all',sensitive=1
                end
            else:
          endcase
          self.drawLegend->setproperty,item_color=item_color,item_linestyle=item_linestyle
          if redraw then begin
            self->draw
            self->my_widget_control,'show_all',sensitive=1
          endif else $
            self->draw,/view
         end
       else:
    endcase
end

function dm_plot::getcolor,color,scalar=scalar,name=name,define=define
    ncolor=n_elements((keyword_set(name))?(color[0,*]):color)
    if keyword_set(name) then begin
        if ncolor eq 0 then return,'user define'
        result=strarr(ncolor)
        for i=0,ncolor-1 do begin
            index=color[0,i]+256L*color[1,i]+256L*256L*color[2,i]
            case index of
                0L:         name = 'black'
                16777215L:  name = 'white'
                255L:       name = 'red'
                16711680L:  name = 'blue'
                16711935L:  name = 'magenta'
                65280L:     name = 'green'
                16776960L:  name = 'cyan'
                65535L:     name = 'yellow'
                12632256L:  name = 'grey'
                8421504L:   name = 'dark grey'
                128L:       name = 'dark red'
                8388608L:   name = 'dark blue'
                8388736L:   name = 'dark magenta'
                32768L:     name = 'dark green'
                8421376L:   name = 'dark cyan'
                32896L:     name = 'dark yellow'
                else:       name = 'user define'
            endcase
            result[i]=name
        endfor
        return,(ncolor eq 1)?result[0]:result
    endif else begin
        if n_elements(define) ne 3 then define = [255b,255b,255b] else define = byte(define)
        if ncolor eq 0 then $
            if keyword_set(scalar) then return,16777215L else return,define
        if size(color,/type) ne 7 then begin
            if keyword_set(scalar) and n_elements(color) eq 3 then return,color[0]+256L*color[1]+256L*256L*color[2] $
            else return,color
        endif
        if keyword_set(scalar) then result = lonarr(ncolor) else result = bytarr(3,ncolor)
        for i=0,ncolor-1 do begin
            case strlowcase(strtrim(color[i],2)) of
                'white'   :      tmp = [255b,255b,255b]
                'red' :          tmp = [255b,0b,0b]
                'blue'    :      tmp = [0b,0b,255b]
                'magenta':       tmp = [255b,0b,255b]
                'green':         tmp = [0b,255b,0b]
                'cyan':          tmp = [0b,255b,255b]
                'yellow':        tmp = [255b,255b,0b]
                'grey':          tmp = [192b,192b,192b]
                'dark grey':     tmp = [128b,128b,128b]
                'dark red':      tmp = [128b,0b,0b]
                'dark blue':     tmp = [0b,0b,128b]
                'dark magenta':  tmp = [128b,0b,128b]
                'dark green':    tmp = [0b,128b,0b]
                'dark cyan':     tmp = [0b,128b,128b]
                'dark yellow':   tmp = [128b,128b,0b]
                'black':         tmp = [0b,0b,0b]
                'user define':   tmp = define
                else:            tmp = [255b,255b,255b]
            endcase
            if keyword_set(scalar) then result[i]=tmp[0]+256L*tmp[1]+256L*256L*tmp[2] else result[*,i]=tmp
        endfor
    endelse
    if keyword_set(scalar) or keyword_set(name) then return,(ncolor eq 1)?result[0]:result $
    else return, (ncolor eq 1)?result[*,0]:result
end

function dm_plot::getlinestyle,linestyle,name=name
    nline = n_elements(linestyle)
    if nline eq 0 then return,keyword_set(name)?'solid':0
    if keyword_set(name) then begin
        lines=strarr(nline)
        for i=0,nline-1 do begin
          case linestyle[i] of
            0:      lines[i] = 'solid'
            1:      lines[i] = 'dotted'
            2:      lines[i] = 'dashed'
            3:      lines[i] = 'dash dot'
            4:      lines[i] = 'dash dot dot dot'
            5:      lines[i] = 'long dash'
            6:      lines[i] = 'no line'
            else:   lines[i] = 'solid'
          endcase
        endfor
    endif else begin
        if size(linestyle,/type) ne 7 then return,linestyle
        lines  =    lonarr(nline)
        for i=0,nline-1 do begin
          case strlowcase(strtrim(linestyle[i],2)) of
            'solid':              lines[i] = 0L
            'dotted':             lines[i] = 1L
            'dashed':             lines[i] = 2L
            'dash dot':           lines[i] = 3L
            'dash dot dot dot':   lines[i] = 4L
            'long dash':          lines[i] = 5L
            'no line':            lines[i] = 6L
            else:                 lines[i] = 0L
          endcase
        endfor
    endelse
    return,(nline eq 1)?lines[0]:lines
end

function dm_plot::getpalette,coltable,background=background,oldpal=oldpal
    if n_elements(oldpal) ne 0 then drawPal=oldpal $
    else drawPal=obj_new('IDLgrPalette',name='palette')
    ;save the old color table and restore it later to avoid affecting other programs running at the same time
    tvlct,old_r,old_g,old_b,/get
    loadct,coltable,/silent
    stretch,1,255  ;0 left for background color
    tvlct,r,g,b,/get
    if n_elements(background) eq 3 then begin
       r[0] = fix(background[0],type=1)
       g[0] = fix(background[0],type=1)
       b[0] = fix(background[0],type=1)
    endif else begin
       r[0] = 0b
       g[0] = 0b
       b[0] = 0b
    endelse
    drawPal->setproperty,red_values=temporary(r),green_values=temporary(g),blue_values=temporary(b)
    ;restore the old color table
    tvlct,old_r,old_g,old_b
    return,drawPal
end

function dm_plot::getpsym,symbol,symsize,thick,color,oldsymbol=oldsymbol,name=name,landscape=landscape
    if keyword_set(name) then begin
        if size(symbol,/type) eq 7 then return,symbol $
        else begin
            case symbol[0] of
                0:      result = 'no symbol'
                1:      result = 'plus'
                2:      result = 'star'
                3:      result = 'dot'
                4:      result = 'diamond'
                5:      result = 'triangle'
                6:      result = 'square'
                7:      result = 'cross'
                8:      result = 'greater'
                9:      result = 'less'
                else:   result = 'no symbol'
            endcase
        endelse
        return,result
    endif
    if n_elements(oldsymbol) ne 0 then begin
       psym = oldsymbol[0]
       psym->getproperty,data=tmp
       if obj_valid(tmp) then obj_destroy,tmp
    endif else begin
       psym = obj_new('IDLgrSymbol')
    endelse
    if n_elements(symsize) eq 0 then symsize=self.default[1]
    ;modify the symbol size to achieve isotropic effect
    self.drawView->getproperty,dimensions=dim, viewplane_rect=view_rect
    view_rect = view_rect[2:3]
    if keyword_set(landscape) then begin
        dim       = dim[[1,0]]
        view_rect = view_rect[[1,0]]
    endif
    dim = dim/view_rect
    if self.xlog eq 1 then xran=alog10(self.xran[*,2]) else xran=self.xran[*,0]
    if self.ylog eq 1 then yran=alog10(self.yran[*,2]) else yran=self.yran[*,0]
    tmpsize = symsize*[xran[1]-xran[0],yran[1]-yran[0]]*mean(dim)/dim
    if n_elements(thick) eq 0  then thick = self.default[2]
    if n_elements(color) eq 0  then color = 'white'
    if size(symbol,/type) ne 7 then data  = symbol[0] $
    else begin
        case strlowcase(strtrim(symbol,2)) of
            'circle':   data = obj_new('idlgrplot',cos(findgen(41)/40.0*!pi*2),sin(findgen(41)/40.0*!pi*2),$
                            linestyle=0,color=self->getcolor(color),thick=thick)
            'bullet':   data = obj_new('idlgrcontour',fltarr(41)+1.0,geomx=cos(findgen(41)/40.0*!pi*2),geomz=0,/plan,$
                            geomy=sin(findgen(41)/40.0*!pi*2),c_color=0,color=self->getcolor(color),c_value=[0,1],/fill)
            'plus':     data = 1
            'star':     data = 2
            'dot':      data = 3
            'diamond':  data = 4
            'triangle': data = 5
            'square':   data = 6
            'cross':    data = 7
            'greater':  data = 8
            'less':     data = 9
            else:       data = 0  ;no symbol
        endcase
    endelse
    psym->setproperty,data=data,size=tmpsize,thick=thick,color=self->getcolor(color)
    return,psym
end

function dm_plot::getproperty,group_leader=group_leader,keep=keep,parenthandler=parenthandler,parentobj=parentobj,tlb=tlb,$
    enotebook=enotebook,error=error
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(dialog_parrent=self.tlb,!error_state.msg,/error)
       error = 1  ;indicating that an error occurs
       return,0
    end
    error = 0
    if keyword_set(group_leader)  then return, self.group_leader
    if keyword_set(keep)          then return, self.keep
    if keyword_set(parentobj)     then return, self.parentobj
    if keyword_set(parenthandler) then return, self.parenthandler
    if keyword_set(tlb)           then return, self.tlb
    if keyword_set(enotebook)     then return, self.enotebook
    error = 1
end

pro dm_plot::getproperty,group_leader=group_leader,keep=keep,parenthandler=parenthandler,parentobj=parentobj,tlb=tlb,error=error
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(dialog_parrent=self.tlb,!error_state.msg,/error)
       error = 1  ;indicating that an error occurs
       return
    end
    error = 0
    if arg_present(group_leader)  then group_leader  = self.group_leader
    if arg_present(keep)          then keep          = self.keep
    if arg_present(parenthandler) then parenthandler = self.parenthandler
    if arg_present(parentobj)     then parentobj     = self.parentobj
    if arg_present(tlb)           then tlb           = self.tlb
end

pro dm_plot::my_widget_control,uname,base,_extra=extra
    if n_elements(base)  eq 0 then base = self.tlb
    if n_elements(extra) ne 0 then begin
        for i=0,n_elements(uname)-1 do begin
            wid = widget_info(base,find_by_uname=uname[i])
            if wid ne 0 then widget_control,wid,_extra=extra
        endfor
    endif
end

;read in a 1D 2 or 3-column file or a 2D spe file and add it to the plot
pro dm_plot::readasciifile,oned=oned,twod=twod
    widget_control,/hourglass
    type = (keyword_set(twod)?'spe':'*')
    file = dm_choose_file(type,dialog_parent=self.tlb,/read,/must_exist)
    if strlen(file) eq 0 then return
    openr,unit,file,/get_lun,error=error
    if error ne 0 then begin
       ok = dialog_message("Can't read file "+file+'.',dialog_parent=self.tlb)
       return
    endif

    mesg  = widget_base(title='Loading data ...',/col,/base_align_center,group_leader=self.tlb)
    label = widget_label(mesg,value='Loading '+file[0]+'. Please wait ...',/align_center)
    dm_center_kid,mesg,self.tlb
    widget_control,mesg,/realize,map=1

    ;catch and clear possible io errors
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(dialog_parent=self.tlb,!error_state.msg,/error)
       if n_elements(unit) ne 0 then free_lun,unit
       if widget_info(mesg,/valid_id) then widget_control,mesg,/destroy
       return
    end

    if keyword_set(oned) then begin
        all_col = ['white','red','blue','magenta','green','cyan','yellow','grey','dark grey','dark red',$
                   'dark blue','dark magenta','dark green','dark cyan','dark yellow']
        n_col   = n_elements(all_col)
        all_sym = ['circle','bullet','plus','star','dot','diamond','triangle','square','cross','greater','less']
        n_sym   = n_elements(all_sym)
        legend=strsplit(file,':\/',/extract)
        legend=legend[n_elements(legend)-1]
        tmp = '' & line = 0
        while not eof(unit) do begin
            readf,unit,tmp & tmp = strtrim(tmp,2)
            if strlen(tmp) eq 0 then continue
            tmp1 = strsplit(tmp,' ,'+string(9b),/extract)
            tmp1 = dm_to_number(tmp1,/float,count=count)
            if total(finite(tmp1)) eq count then $
                if n_elements(data) eq 0 then data=tmp1 else data=[[data],[tmp1]] $
            else begin
                if n_elements(data) ne 0 then begin
                  color  = all_col[line mod n_col]
                  symbol = all_sym[line mod n_sym]
                  if n_elements(data[*,0]) eq 2 then $
                    self->add_plot,reform(data[0,*]),reform(data[1,*]),legend=legend,psym=symbol,color=color $
                  else if n_elements(data[*,0]) ge 3 then $
                    self->add_plot,reform(data[0,*]),reform(data[1,*]),yerr=reform(data[2,*]),legend=legend,$
                         psym=symbol,color=color
                  tmp2 = temporary(data)    ;destroy data
                  line = line+1
                endif
                legend = tmp
            endelse
        endwhile
        free_lun,unit
        if n_elements(data) ne 0 then begin
            widget_control,label,set_value='drawing ...'
            color  = all_col[line mod n_col]
            symbol = all_sym[line mod n_sym]
            if n_elements(data[*,0]) eq 2 then $
                self->add_plot,reform(data[0,*]),reform(data[1,*]),legend=legend,psym=symbol,color=color $
            else if n_elements(data[*,0]) ge 3 then $
                self->add_plot,reform(data[0,*]),reform(data[1,*]),yerr=reform(data[2,*]),legend=legend,$
                    psym=symbol,color=color
        endif
    endif else if keyword_set(twod) then begin
        dm_load_spe,unit,zdat=data,err=err,xdat=xdat,ydat=ydat
        widget_control,label,set_value='drawing ...'
        if n_elements(data) ne 0 then self->add_plot,xdat,ydat,data,/usepolygon,shading=1
    endif
    self->draw
    widget_control,mesg,/destroy
end

pro dm_plot::rotate,event
    if self.moveLegend eq 0 then begin
       haveTransform=self.trackBall->update(event,transform=qmat)
       if (haveTransform ne 0) then begin
         self.drawGroup->getproperty,transform=t
         self.drawGroup->setproperty,transform=t#qmat
         self.drawWin->draw,self.drawView
       endif
    endif
    case event.type of
       0:    begin  ;button press
         case event.press of
          1:  begin    ;left button down
                 widget_control,self.plotWin,draw_motion_events=1
                 self.drawView->getproperty,dimension=dim
                 xstart = self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
                 ystart = self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
                 tmp = self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then $
                   if obj_isa(tmp[0],'IDLgrLegend') or obj_isa(tmp[0],'IDLgrText') then begin
                    self.select     = tmp[0]
                    self.moveLegend = 1b
                    self.select->getproperty,xcoord_conv=x,ycoord_conv=y
                    self.legdOffset = [x[0]-xstart,y[0]-ystart]
                    break
                   endif
              end
          else:
         endcase
         end
       1:    begin   ;button release
          widget_control,self.plotWin,draw_motion_events=0
          self.moveLegend=0b
         end
       2:    if self.moveLegend then begin
          self.drawView->getproperty,dimension=dim
          xend = self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
          yend = self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
          self.select->getproperty,ycoord_conv=old_yconv,xcoord_conv=old_xconv
          new_yconv       = [yend+self.legdOffset[1],1]
          new_xconv       = [xend+self.legdOffset[0],1]
          if obj_isa(self.select,'IDLgrLegend') then begin
              self.legdpos[0] = self.legdpos[0]+new_xconv[0]-old_xconv[0]
              self.legdpos[1] = self.legdpos[1]+new_yconv[0]-old_yconv[0]
          endif
          self.select->setproperty,xcoord_conv=new_xconv,ycoord_conv=new_yconv
          self.drawWin->draw,self.drawView
         endif
       else:
    endcase
end

pro dm_plot::resize,event
    widget_control,self.tlb,update=0
    mbar_y = 0
    if widget_info(self.mbarSep,/valid_id) then begin
       widget_control,self.mbarSep,xsize=event.x
       mbar_y = widget_info(self.mbarSep,/geometry)
       mbar_y = mbar_y.ysize
    endif
    widget_control,self.plotWin,xsize=event.x,ysize=event.y-mbar_y
    self.drawWin->setproperty,dimension=[event.x,event.y-mbar_y]
    self.drawView->setproperty,dimension=[event.x,event.y-mbar_y]
    self.tranView->setproperty,dimension=[event.x,event.y-mbar_y]
    self->set_view
    widget_control,self.tlb,update=1
    self.drawWin->draw,self.drawView
end

;save plot as images or print
pro dm_plot::saveas,type
    type = strlowcase(strtrim(type,2))
    self.drawView->getproperty,dimension=dim
    self.drawView->setproperty,dimension=self.default[5:6]
    self->set_view,/save   ;reverse black and white color
    resolution = [0.018,0.018]
    if (type ne 'printer') and (type ne 'ps') and (type ne 'ascii') then begin
       buffer  = obj_new('IDLgrBuffer',dimension=self.default[5:6],color_model=0,resolution=resolution)
       buffer->draw,self.drawView
       buffer->getproperty,image_data=image
       obj_destroy,buffer
    endif
    case type of
       'printer':    begin
          ok = dialog_printersetup(self.printer,dialog_parent=self.tlb)
          if not ok then break
          self.printer->getproperty,dimensions=printerDim
          windowAspect  = float(dim[1])/dim[0]
          printerAspect = float(printerDim[1])/printerDim[0]
          margin = 0.1
          if windowAspect le printerAspect then begin
              xstart = margin
              ystart = 0.5-(0.5-margin)*(windowAspect/printerAspect)
              xend   = 1.0-margin
              yend   = 1.0-ystart
          endif else begin
              xstart = 0.5-(0.5-margin)*(printerAspect/windowAspect)
              ystart = margin
              xend   = 1.0-xstart
              yend   = 1.0-margin
          endelse
          self.drawView->setproperty,dimensions=[xend-xstart,yend-ystart],$
              location=[xstart,ystart],units=3
          self.printer->newpage
          self.printer->draw,self.drawView
          self.printer->newdocument
         end
       'ps': begin
          file = dm_choose_file('ps',/write,dialog_parent=self.tlb,deviceKeywords=deviceKeywords)
          widget_control,/hourglass  ;busy signal
          if strlen(file) ne 0 then begin
              if deviceKeywords.inches eq 0 then begin
                 deviceKeywords.inches  = 1
                 deviceKeywords.xsize   = deviceKeywords.xsize/2.54   ;cm->inch
                 deviceKeywords.xoffset = deviceKeywords.xoffset/2.54
                 deviceKeywords.ysize   = deviceKeywords.ysize/2.54   ;cm->inch
                 deviceKeywords.yoffset = deviceKeywords.yoffset/2.54
              endif
              if deviceKeywords.landscape then begin
                    tmp = deviceKeywords.ysize
                    deviceKeywords.ysize   = deviceKeywords.xsize
                    deviceKeywords.xsize   = tmp
                    deviceKeywords.yoffset = deviceKeywords.yoffset-deviceKeywords.ysize
              endif
              if deviceKeywords.encapsulated eq 1 then begin
                 deviceKeywords.xoffset = 0.0
                 deviceKeywords.yoffset = 0.0
              endif
              dimensions = [deviceKeywords.xsize+deviceKeywords.xoffset,$
                      deviceKeywords.ysize+deviceKeywords.yoffset]*2.54/resolution
              border     = [deviceKeywords.xoffset,deviceKeywords.yoffset]*2.54/resolution
              if deviceKeywords.landscape then begin
                transl   = -[0.5,0.5,0]
                self.drawModel->translate,transl[0],transl[1],transl[2]
                self.drawModel->rotate,[0,0,1],-90
                self.drawModel->translate,-transl[0],-transl[1],-transl[2]
              endif
              maxdim = 2048.
              if dimensions[0] gt maxdim then begin
                factor     = maxdim/dimensions[0]
                dimensions = dimensions*factor
                border     = border*factor
              endif
              if dimensions[1] gt maxdim then begin
                factor     = maxdim/dimensions[1]
                dimensions = dimensions*factor
                border     = border*factor
              endif
              self.drawView->setproperty,dimension=dimensions-border,unit=0,location=border
              self->set_view,/save,landscape=deviceKeywords.landscape
              buffer = obj_new('IDLgrClipboard',quality=2,dimensions=dimensions,resolution=resolution)
              buffer->draw,self.drawView,/vector,/postscript,filename=file
              obj_destroy,buffer
              if deviceKeywords.landscape then begin
                self.drawModel->translate,transl[0],transl[1],transl[2]
                self.drawModel->rotate,[0,0,1],90
                self.drawModel->translate,-transl[0],-transl[1],-transl[2]
              endif
          endif
         end
       'ascii': begin ;ascii file
          onedplot = where((*self.plotInfo)[*].type eq 0,count1)
          twodplot = where((*self.plotInfo)[*].type gt 0,count2)
          nplots   = n_elements(*self.plotInfo)
          if count1 ne 0 then begin
            file = dm_choose_file('txt',/write,dialog_parent=self.tlb,title='Save 1D plot data as an ASCII file.')
            if strlen(file) eq 0 then break
            openw,unit,file,/get_lun,error=openerr
            if openerr ne 0 then begin
                ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=self.tlb)
            endif else begin
                widget_control,/hourglass  ;busy signal
                for i=0L,count1-1 do begin
                    if count1 gt 1 then printf,unit,'##line ',dm_to_string(i),': ', (*self.plotInfo)[onedplot[i]].legend
                    x  = *(*self.plotInfo)[onedplot[i]].xPtr
                    y  = *(*self.plotInfo)[onedplot[i]].yPtr
                    nx = n_elements(x)
                    if onedplot[i]+1 lt nplots then $
                        if (*self.plotInfo)[onedplot[i]+1].type eq -1 then $
                            yerr = ((*(*self.plotInfo)[onedplot[i]+1].yPtr)[2*lindgen(nx)+1]-$
                                   (*(*self.plotInfo)[onedplot[i]+1].yPtr)[2*lindgen(nx)])/2.
                    for j=0L,nx-1 do begin
                        if n_elements(yerr) eq nx then $
                            printf,unit,x[j],' ',y[j],' ',yerr[j]  $
                        else $
                            printf,unit,x[j],' ',y[j]
                    endfor
                    yerr = 0 & tmp = temporary(yerr)  ;clear yerr
                endfor
                free_lun,unit
            endelse
          endif
          if count2 eq 1 then begin
            file = dm_choose_file('spe',/write,dialog_parent=self.tlb,title='Save 2D plot data as a SPE file.')
            if strlen(file) eq 0 then break
            openw,unit,file,/get_lun,error=openerr
            if openerr ne 0 then begin
                ok = dialog_message("Can't write in "+file[0],/error,dialog_parent=self.tlb)
            endif else begin
                x  = *((*self.plotInfo)[twodplot[0]]).xPtr
                y  = *((*self.plotInfo)[twodplot[0]]).yPtr
                z  = *((*self.plotInfo)[twodplot[0]]).zPtr
                itmp = where(finite(z) eq 0,count)
                if count ne 0 then z[itmp]=0.0
                dz = z*0.05
                widget_control,/hourglass  ;busy signal
                dm_write_spe,unit,x,y,z,dz
            endelse
          endif
         end
       else: begin
          file = dm_choose_file(type,/write,dialog_parent=self.tlb)
          if strlen(file) eq 0 then break
          id = where(['bmp','jpeg','png','ppm','srf','tiff','dicom'] eq type,count)
          widget_control,/hourglass   ;busy signal
          if count ne 0 then write_image,file,type,image
         end
    endcase
    self.drawView->setproperty,dimension=dim,unit=0,location=[0,0]
    self->set_view
end

;set property
;usage:
;   dm_plot->setproperty,property=value
;     or
;   dm_plot->setproperty,object=object,property=property,value=value
;parameters:
;   NONE
;keyword:
;   axisthick:           axis thickness for all axes
;   changeid:            changed id of plotinfo,internal call
;   color:               plot color
;   coltable:            color table
;   error:               1 if error occurs    0 operation successful
;   font:                font name
;   fsize:               font size
;   hideintnbar:         if set, hide the intensity bar
;   hidelegend:          if set, plot legends are not shown
;   iso:                 1:isotropic plot 0:ratioed to window
;   keep:                1 or 0
;   layer:               layer of 1d plot
;   legend:              legend for the current plot
;   linestyle:           linestyle for the current plot
;   misscol:             missing data color for saving 2d plots 0:black, 1:white
;   no_copy:             to save memory
;   nodraw:              if set, don't draw the view
;   object:              for setting properties from dm_dialog_input apply button
;   plotInfo:            plot inforamtion,internal call
;   psym:                symbol number or symbol object
;   shading:             shading for object graphics
;   surfplot:            if set, 2d data are plot as surface rather than contour
;   symsize:             symbol size
;   thick:               line thickness for plots and symbols
;   title:               title of the plot
;   tickstyle:           0-idl default tick string format 1-use dm_to_string
;   usepolygon:          if set, use idlgrpolygon to draw contour plots instead of idlgrcontour
;   xdat,ydat,zdat,idat: x,y,z,i data
;   xlog,ylog,zlog,ilog: log10 scale x,y,z,i respectively
;   xran,yran,zran,iran: x,y,z,i range
;   xtit,ytit,ztit:      x,y,z axis title string
;   yerr:                error bar of y
;   zmin,zmax:           zmin, zmax for contour plots
;   zoom:                in conjuction with [xyz]ran to show the set range is from zoom
pro dm_plot::setproperty,axisthick=axisthick,changeid=changeid,color=color,coltable=coltable,font=font,fsize=fsize,$
    hideintbar=hideintbar,hideLegend=hideLegend,idat=idat,iran=iran,ilog=ilog,iso=iso,keep=keep,layer=layer,legend=legend,$
    linestyle=linestyle,misscol=misscol,no_copy=no_copy,nodraw=nodraw,object=object,plotInfo=plotInfo,psym=psym,$
    shading=shading,showxgrid=showxgrid,showygrid=showygrid,surfplot=surfplot,symsize=symsize,thick=thick,$
    tickstyle=tickstyle,title=title,usepolygon=usepolygon,xdat=xdat,ydat=ydat,zdat=zdat,yerr=yerr,xlog=xlog,$
    ylog=ylog,zlog=zlog,xran=xran,yran=yran,zran=zran,xtit=xtit,ytit=ytit,ztit=ztit,zmin=zmin,zmax=zmax,zoom=zoom,$
    error=error,_extra=extra

    error  = 0.0
    redraw = 1
    ;catch and ignore all errors in this program
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok = dialog_message(!error_state.msg,/error)
       error = 1
       return
    end
    widget_control,/hourglass
    if (n_elements(object) ne 0) then begin
        if n_elements(coltable) ne 0 then begin
            for i=0,n_elements(object)-1 do begin
                object[i]->getproperty,palette=drawPal
                drawPal = self->getpalette(dm_to_number(coltable[0],/int),oldpal=drawPal)
            endfor
        endif
        if n_elements(color) ne 0 then begin
            for i=0,n_elements(object)-1 do $
                object[i]->setproperty,color=self->getcolor(color)
        endif
        if n_elements(linestyle) ne 0 then begin
            for i=0,n_elements(object)-1 do $
                object[i]->setproperty,linestyle=self->getlinestyle(linestyle)
        endif
        if n_elements(thick) ne 0 then begin
            thick = dm_to_number(thick[0])
            if not finite(thick) then thick=1
            for i=0,n_elements(object)-1 do $
                object[i]->setproperty,thick=thick
        endif
        self.drawWin->draw,self.drawView
        return
    endif
    if n_elements(keep) ne 0 then begin
       self.keep=(0b)>(fix(keep,type=1))<(1b)
       if self.keep then begin
         self->my_widget_control,['keepBut','keepmenu'],sensitive=0
         self->my_widget_control,['mkcrmenu','mkcrBut'],sensitive=1
       endif else begin
         self->my_widget_control,['keepBut','keepmenu'],sensitive=1
         self->my_widget_control,['mkcrmenu','mkcrBut'],sensitive=0
       endelse
       redraw = 0  ;no need to redraw
    end
    if n_elements(misscol) ne 0 then begin
       blackbut = widget_info(self.tlb,find_by_uname='blackbg')
       whitebut = widget_info(self.tlb,find_by_uname='whitebg')
       self.misscol=0>(misscol)<1
       if self.misscol eq 0 then dm_toggle_menubut,blackbut,whitebut $
       else dm_toggle_menubut,whitebut,blackbut
       redraw = 0  ;no need to redraw
    endif
    if n_elements(showxgrid) eq 1 then begin
       self.showxgrid = (0b)>(showxgrid)<(1b)
       self.xgrid->setproperty,hide=1-self.showxgrid
       logid = widget_info(self.tlb,find_by_uname='xgridBut')
       if logid gt 0 then dm_set_button,logid,self.showxgrid,onstring='Hide X Grid',offstring='Show X Grid'
       redraw = 1
    endif
    if n_elements(showygrid) eq 1 then begin
       self.showygrid = (0b)>(showygrid)<(1b)
       self.ygrid->setproperty,hide=1-self.showygrid
       logid = widget_info(self.tlb,find_by_uname='ygridBut')
       if logid gt 0 then dm_set_button,logid,self.showygrid,onstring='Hide Y Grid',offstring='Show Y Grid'
       redraw = 1
    endif
    if n_elements(tickstyle) ne 0 then begin
       idltick = widget_info(self.tlb,find_by_uname='idltick')
       owntick = widget_info(self.tlb,find_by_uname='owntick')
       self.tickstyle=0>(tickstyle)<1
       if self.tickstyle eq 0 then dm_toggle_menubut,idltick,owntick $
       else dm_toggle_menubut,owntick,idltick
       self->set_view
       redraw = 1
    endif
    if n_elements(xlog) ne 0 then begin
       new = (0b>(byte(xlog))<1b)
       if self.xlog ne new then begin
         self.xlog = new
         logid     = widget_info(self.tlb,find_by_uname='xlogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='X Normal Scale',offstring='X Logarithmic Scale'
         redraw    = 2 ;need to reset view
       endif
    endif
    if n_elements(ylog) ne 0 then begin
       new = (0b>(byte(ylog))<1b)
       if self.ylog ne new then begin
         self.ylog = new
         logid     = widget_info(self.tlb,find_by_uname='ylogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='Y Normal Scale',offstring='Y Logarithmic Scale'
         redraw    = 2 ;need to reset view
       endif
    endif
    if n_elements(zlog) ne 0 then begin
       new = (0b>(byte(zlog))<1b)
       if self.zlog ne new then begin
         self.zlog = new
         logid     = widget_info(self.tlb,find_by_uname='zlogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='Z Normal Scale',offstring='Z Logarithmic Scale'
         redraw    = 2 ;need to reset view
       endif
    endif
    if n_elements(ilog) ne 0 then begin
       new = (0b>(byte(ilog))<1b)
       if self.ilog ne new then begin
         self.ilog = new
         logid     = widget_info(self.tlb,find_by_uname='ilogBut')
         if logid gt 0 then dm_set_button,logid,new,onstring='I Normal Scale',offstring='I Logarithmic Scale'
         redraw    = 2 ;need to reset view
       endif
    endif
    if n_elements(plotInfo) ne 0 then begin
       nplots   = n_elements(plotInfo)
       onedplot = where(plotInfo[*].type eq 0)
       if n_elements(changeid) eq 0 then changeid=indgen(nplots)
       self.drawLegend->getproperty,item_color=item_color,item_linestyle=item_linestyle,item_name=item_name,item_thick=item_thick
       for i=0,n_elements(changeid)-1 do begin
         switch plotInfo[changeid[i]].type of
          ;error plot
          -1: begin
              plotInfo[changeid[i]].obj->setproperty,thick=plotInfo[changeid[i]].thick,$
                 color=plotInfo[changeid[i]].color
              break
              end
          ;1d plot
          0:  begin
              zpos = 0
              if finite(plotInfo[changeid[i]].zpos) then $
                zpos = plotInfo[changeid[i]].zpos
              if self.zlog then begin
                if zpos gt 0 then zpos = alog10(zpos) $
                else if finite(self.zran[1,2]) then zpos = 1.+alog10(self.zran[1,2])
              endif
              zpos = -0.0001*(1+(*plotInfo[changeid[i]].zPtr))+zpos
              plotInfo[changeid[i]].obj->getproperty,symbol=drawPsym
              newcolor   = plotInfo[changeid[i]].color
              newlinesty = plotInfo[changeid[i]].linestyle
              drawPsym   = self->getpsym(plotInfo[changeid[i]].psym,plotInfo[changeid[i]].symsize,plotInfo[changeid[i]].thick,$
                 newcolor,old=drawPsym)
              plotInfo[changeid[i]].obj->setproperty,color=newcolor,thick=plotInfo[changeid[i]].thick,linestyle=newlinesty,$
                 zvalue=zpos
              ;change the legend name, linestyle, and color in the legend
              i_num = where(onedplot eq changeid[i],count)
              if count ne 0 then begin
                 item_color[*,i_num]   = newcolor
                 item_linestyle[i_num] = newlinesty
                 item_name[i_num]      = plotInfo[changeid[i]].legend
              endif
              ;change the color and thickness of the error plot if any
              if changeid[i]+1 lt nplots then $
                 if plotInfo[changeid[i]+1].type eq -1 then begin
                   (*plotInfo[changeid[i]+1].zPtr) = (*plotInfo[changeid[i]].zPtr)
                   plotInfo[changeid[i]+1].color   = newcolor
                   plotInfo[changeid[i]+1].thick   = plotInfo[changeid[i]].thick
                   plotInfo[changeid[i]+1].obj->getproperty,data=data
                   data[2,*]=zpos
                   plotInfo[changeid[i]+1].obj->setproperty,thick=plotInfo[changeid[i]+1].thick,color=newcolor,data=data
                 endif
              break
              end
          ;2d plot
          1:
          2:  begin
              plotInfo[changeid[i]].obj->getproperty,palette=drawPal
              drawPal=self->getpalette(plotInfo[changeid[i]].coltable,oldpal=drawPal)
              plotInfo[changeid[i]].obj->setproperty,shading=plotInfo[changeid[i]].shading
              end
         endswitch
       endfor
       self.drawLegend->setproperty,item_color=item_color,item_linestyle=item_linestyle,item_name=item_name,item_thick=item_thick
       (*self.plotInfo) = temporary(plotInfo)
    endif
    if n_elements(iso) ne 0 then begin
       self.iso = 0>(iso)<1
       logid    = widget_info(self.tlb,find_by_uname='isoBut')
       if logid gt 0 then dm_set_button,logid,self.iso,onstring='Disable Isotropic Scaling',offstring='Enable Isotropic Scaling'
       redraw = 2  ;need to reset view
    endif
    if n_elements(hideLegend) ne 0 then begin
       self.hideLegend = 0b>(fix(hideLegend[0],type=1))<1b
       self.drawLegend->setproperty,hide=self.hideLegend
       id = widget_info(self.tlb,find_by_uname='cs_hidelegend')
       dm_set_button,id,self.hideLegend,onstring='Show Legend',offstring='Hide Legend'
    endif
    if n_elements(hideintbar) ne 0 then begin
       redraw = 2  ;need to reset view
       self.hideIntbar=0b>(fix(hideintbar[0],type=1))<1b
       id = widget_info(self.tlb,find_by_uname='cs_hidecbar')
       dm_set_button,id,self.hideIntbar,onstring='Show Colorbar',offstring='Hide Colorbar'
       if obj_valid(self.colBar) then begin
         self.drawxAxis[2]->setproperty,hide=self.hideIntbar
         self.drawxAxis[3]->setproperty,hide=self.hideIntbar
         self.drawyAxis[2]->setproperty,hide=self.hideIntbar
         self.drawyAxis[3]->setproperty,hide=self.hideIntbar
         self.colBar->setproperty,hide=self.hideIntbar
       end
    endif
    if n_elements(xdat) ne 0 then begin
       tmp_xmax = max(xdat,min=tmp_xmin,/nan)
       self.xran[*,0] = [temporary(tmp_xmin),temporary(tmp_xmax)]
       if self.xran[0,0] eq self.xran[1,0] then begin
         tmp = (self.xran[0,0] eq 0)? 0.5:abs(self.xran[0,0]/2.)
         self.xran[0,0] = self.xran[0,0]-tmp
         self.xran[1,0] = self.xran[1,0]+tmp
       endif
       poselts = where(xdat gt 0,count)
       if count gt 0 then begin
         tmp_xmax = max(xdat[temporary(poselts)],min=tmp_xmin)
       endif else begin
         tmp_xmin = 1.0 & tmp_xmax = 10.0
       endelse
       self.xran[0,2] = (tmp_xmin gt self.xran[0,0])?((self.xran[0,0] gt 0)?(self.xran[0,0]):(tmp_xmin)):(self.xran[0,0])
       self.xran[1,2] = (tmp_xmax gt self.xran[1,0])?((self.xran[1,0] gt 0)?(self.xran[1,0]):(tmp_xmax)):(self.xran[1,0])
       if self.xran[1,2] le self.xran[0,2] then self.xran[1,2]=self.xran[0,2]*10.0
    endif
    if n_elements(ydat) ne 0 then begin
       if n_elements(yerr) eq n_elements(ydat) then begin
         tmp_ymin = min(ydat-yerr,/nan)
         tmp_ymax = max(ydat+yerr,/nan)
       endif else $
         tmp_ymax = max(ydat,min=tmp_ymin,/nan)
       self.yran[*,0] = [temporary(tmp_ymin),temporary(tmp_ymax)]
       if self.yran[0,0] eq self.yran[1,0] then begin
         tmp = (self.yran[0,0] eq 0)? 0.5:abs(self.yran[0,0]/2.)
         self.yran[0,0] = self.yran[0,0]-tmp
         self.yran[1,0] = self.yran[1,0]+tmp
       endif
       poselts = where(ydat gt 0,count)
       if count gt 0 then begin
         ymax = max(ydat[temporary(poselts)],min=ymin)
       endif else begin
         ymin = 1.0 & ymax = 10.0
       endelse
       self.yran[0,2] = (ymin gt self.yran[0,0])?((self.yran[0,0] gt 0)?(self.yran[0,0]):(ymin)):(self.yran[0,0])
       self.yran[1,2] = (ymax gt self.yran[1,0])?((self.yran[1,0] gt 0)?(self.yran[1,0]):(ymax)):(self.yran[1,0])
       if self.yran[1,2] le self.yran[0,2] then self.yran[1,2]=self.yran[0,2]*10.0
    endif
    if (n_elements(zdat) eq 0) and (n_elements(idat) ne 0) then zdat=temporary(idat)
    if n_elements(zdat) ne 0 then begin
       tmp_zmax = max(zdat,min=tmp_zmin,/nan)
       self.zran[*,0] = [temporary(tmp_zmin),temporary(tmp_zmax)]
       if self.zran[0,0] eq self.zran[1,0] then begin
         tmp = (self.zran[0,0] eq 0)? 0.5:abs(self.zran[0,0]/2.)
         self.zran[0,0] = self.zran[0,0]-tmp
         self.zran[1,0] = self.zran[1,0]+tmp
       endif
       finelts = where(finite(zdat),count)
       if count gt 0 then poselts = where(zdat[finelts] gt 0,count)
       if count gt 0 then begin
         tmp_zmax = max(zdat[finelts[poselts]],min=tmp_zmin)
       endif else begin
         tmp_zmin = 1 & tmp_zmax = 10.0
       endelse
       finelts = -1 & poselts = -1
       self.zran[0,2] = (tmp_zmin gt self.zran[0,0])?((self.zran[0,0] gt 0)?(self.zran[0,0]):(tmp_zmin)):(self.zran[0,0])
       self.zran[1,2] = (tmp_zmax gt self.zran[1,0])?((self.zran[1,0] gt 0)?(self.zran[1,0]):(tmp_zmax)):(self.zran[1,0])
       if self.zran[1,2] le self.zran[0,2] then self.zran[1,2]=self.zran[0,2]*10.0
       self.zmin = self.zran[0,2*self.zlog]
       self.zmax = self.zran[1,2*self.zlog]
    endif
    if n_elements(idat) ne 0 then begin
       tmp_imax = max(idat,min=tmp_imin,/nan)
       self.iran[*,0] = [temporary(tmp_imin),temporary(tmp_imax)]
       if self.iran[0,0] eq self.iran[1,0] then begin
         tmp = (self.iran[0,0] eq 0)? 0.5:abs(self.iran[0,0]/2.)
         self.iran[0,0] = self.iran[0,0]-tmp
         self.iran[1,0] = self.iran[1,0]+tmp
       endif
       finelts=where(finite(idat),count)
       if count gt 0 then poselts=where(idat[finelts] gt 0,count)
       if count gt 0 then begin
         tmp_imax = max(idat[finelts[poselts]],min=tmp_imin)
       endif else begin
         tmp_imin = 1 & tmp_imax = 10.0
       endelse
       finelts = -1 & poselts = -1
       self.iran[0,2] = (tmp_imin gt self.iran[0,0])?((self.iran[0,0] gt 0)?(self.iran[0,0]):(tmp_imin)):(self.iran[0,0])
       self.iran[1,2] = (tmp_imax gt self.iran[1,0])?((self.iran[1,0] gt 0)?(self.iran[1,0]):(tmp_imax)):(self.iran[1,0])
       if self.iran[1,2] le self.iran[0,2] then self.iran[1,2]=self.iran[0,2]*10.0
       self.zmin = self.iran[0,2*self.ilog]
       self.zmax = self.iran[1,2*self.ilog]
    endif
    if n_elements(xtit) ne 0 then self.xtit->setproperty,strings=xtit[0]
    if n_elements(ytit) ne 0 then self.ytit->setproperty,strings=ytit[0]
    if n_elements(ztit) ne 0 then self.ztit->setproperty,strings=ztit[0]
    if n_elements(title) ne 0 then begin
       self.title->getproperty,strings=old
       self.title->setproperty,strings=title[0]
       if strlen(old) eq 0 or strlen(title[0]) eq 0 then redraw=2
       title = self->textadjust(strtrim(title[0],2),/AA)
       widget_control,self.tlb,base_set_title=(strlen(title) eq 0?'plot':title)
    endif
    if n_elements(xran) eq 2 then $
      if (total(finite(xran)) eq 2) then $
       if  xran[1] gt xran[0] then begin
         redraw = 2
         if keyword_set(zoom) then begin
            if total(finite(self.xran[*,2*self.xlog+1])) ne 2 then self.xran[*,2*self.xlog+1]=self.xran[*,2*self.xlog]
            if total(finite(self.yran[*,2*self.ylog+1])) ne 2 then self.yran[*,2*self.ylog+1]=self.yran[*,2*self.ylog]
         endif
         if self.xlog eq 1 then begin
            if xran[0] gt 0 then self.xran[0,2] = xran[0]
            if xran[1] gt 0 then self.xran[1,2] = xran[1]
         endif else self.xran[*,0] = xran
       endif
    if n_elements(yran) eq 2 then $
      if (total(finite(yran)) eq 2) then $
       if  yran[1] gt yran[0] then begin
         redraw = 2
         if keyword_set(zoom) then begin
            if total(finite(self.xran[*,2*self.xlog+1])) ne 2 then self.xran[*,2*self.xlog+1]=self.xran[*,2*self.xlog]
            if total(finite(self.yran[*,2*self.ylog+1])) ne 2 then self.yran[*,2*self.ylog+1]=self.yran[*,2*self.ylog]
         endif
         if self.ylog eq 1 then begin
            if yran[0] gt 0 then self.yran[0,2] = yran[0]
            if yran[1] gt 0 then self.yran[1,2] = yran[1]
         endif else self.yran[*,0] = yran
       endif
    if n_elements(zran) eq 2 then $
      if (total(finite(zran)) eq 2) then $
       if  zran[1] gt zran[0] then begin
         redraw = 2
         if self.zlog eq 1 then begin
            if zran[0] gt 0 then self.zran[0,2] = zran[0]
            if zran[1] gt 0 then self.zran[1,2] = zran[1]
         endif else self.zran[*,0] = zran
       endif
    if (n_elements(idat) eq 0) and (n_elements(zdat) ne 0) then self.iran[*,[0,2]]=self.zran[*,[0,2]]
    if n_elements(iran) eq 2 then $
      if (total(finite(iran)) eq 2) then $
       if  iran[1] gt iran[0] then begin
         redraw = 2
         if self.ilog eq 1 then begin
          if iran[0] gt 0 then self.iran[0,2] = iran[0]
          if iran[1] gt 0 then self.iran[1,2] = iran[1]
         endif else self.iran[*,0]=iran
       endif
    if n_elements(zmin) ne 0 then self.zmin = zmin[0]
    if n_elements(zmax) ne 0 then self.zmax = zmax[0]
    if n_elements(axisthick) ne 0 then begin
       self.axisthick = 1.0>(axisthick)<10.0
       for i=0,3 do begin
         if obj_valid(self.drawxAxis[i]) then self.drawxAxis[i]->setproperty,thick=self.axisthick
         if obj_valid(self.drawyAxis[i]) then self.drawyAxis[i]->setproperty,thick=self.axisthick
       endfor
       if obj_valid(self.drawzAxis) then self.drawzAxis->setproperty,thick=self.axisthick
    endif
    if (n_elements(xdat) ne 0) and (n_elements(ydat) ne 0) then begin
       redraw=2  ;need to reset view
       self.xran[*,[1,3]] = !values.f_nan
       self.yran[*,[1,3]] = !values.f_nan
       self.zran[*,[1,3]] = !values.f_nan
       self.iran[*,[1,3]] = !values.f_nan
       obj_destroy,self.drawxAxis[0:1]
       obj_destroy,self.drawyAxis[0:1]
       oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
       for i=0,nKids-1 do begin
         if (not obj_isa(oChildArr[i],'IDLgrModel')) and (not obj_isa(oChildArr[i],'IDLgrLegend')) $
          and (not obj_isa(oChildArr[i],'IDLgrText')) then $
            obj_destroy,oChildArr[i]
       endfor
       oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
       for i=0,nKids-1 do begin
         if obj_isa(oChildArr[i],'IDLgrPlot') then begin
            oChildArr[i]->getproperty,symbol=tmp1
            if obj_valid(tmp1) then begin
                tmp1[0]->getproperty,data=tmp2
                if obj_valid(tmp2) then obj_destroy,tmp2
                obj_destroy,tmp1
            endif
         endif else if obj_isa(oChildArr[i],'IDLgrPolygon') or obj_isa(oChildArr[i],'IDLgrContour') or $
         obj_isa(oChildArr[i],'IDLgrSurface') then begin
            oChildArr[i]->getproperty,palette=tmp
            if obj_valid(tmp) then obj_destroy,tmp
         endif
         if (oChildArr[i] ne self.xgrid) and (oChildArr[i] ne self.ygrid) then $
            obj_destroy,oChildArr[i]
       endfor
       if n_elements(*self.plotInfo) ne 0 then ptr_free,(*self.plotInfo)[*].xPtr,(*self.plotInfo)[*].yPtr,$
            (*self.plotInfo)[*].zPtr,(*self.plotInfo)[*].iPtr
       ptr_free,self.plotInfo & self.plotInfo=ptr_new(/allocate_heap)
       self.drawGroup->reset
       self.drawModel->reset
       self.drawLegend->getproperty,item_object=item_object
       obj_destroy,item_object
       self.drawLegend->setproperty,item_object=[obj_new()]
       if n_elements(zdat) ne 0 then $
         self->add_plot,xdat,ydat,zdat,idat,no_copy=no_copy,surfplot=surfplot,coltable=coltable,shading=shading,usepolygon=usepolygon $
       else $
         self->add_plot,xdat,ydat,yerr=yerr,no_copy=no_copy,legend=legend,thick=thick,color=color,psym=psym,linestyle=linestyle,$
          symsize=symsize,layer=layer
    endif
    if keyword_set(nodraw) then return
    if redraw eq 1 then self.drawWin->draw,self.drawView $
    else if redraw eq 2 then self->draw
end

;setup correct viewing plane rectangle
;and suppress xaxis[1]'s tickname
;keyword
;   save:       if set, change white color to black color for saving purpose
;   landscape:  if set, the image is rotated
pro dm_plot::set_view,save=save,landscape=landscape
    if keyword_set(save) then bgcolor=[255b,255b,255b] else bgcolor=[0b,0b,0b]
    if obj_valid(self.drawxAxis[1]) then begin
       self.drawxAxis[1]->getproperty,ticktext=xtick
       xtick->getproperty,strings=tmp
       for i=0,n_elements(tmp)-1 do tmp[i]=''
       xtick->setproperty,strings=tmp
       self.drawxAxis[1]->setproperty,ticktext=xtick
    endif

    self.drawView->getproperty,dimensions=dim                 ;viewport size
    xsize = dim[0]
    ysize = dim[1]
    fsize = self.fsize
    if not keyword_set(save) then $
      fsize = fix((1>min(dim/self.default[5:6]))*fsize[*])
    for i=0,5 do $
      self.font[i]->setproperty,size=fsize[i]     ;modify fontsize

    if keyword_set(landscape) then begin
        yoffset = [0.0,0.0]
        xoffset = [(fsize[0]*self.fscale[0]*2+fsize[1]*self.fscale[1]*2.5+20),self.fscale[0]*fsize[0]*0.5+20]
    endif else begin
        xoffset = [0.0,0.0]
        yoffset = [(fsize[0]*self.fscale[0]*2+fsize[1]*self.fscale[1]*2.5+20),self.fscale[0]*fsize[0]*0.5+20]
    endelse
    ;get the axis tickmarks to figure out the offset of the viewplane
    self.drawxAxis[0]->setproperty,ticktext=obj_new()
    self.drawxAxis[0]->getproperty,ticktext=xtick,tickvalues=xtickvalue
    xtick->getproperty,strings=xticktext
    if self.tickstyle then begin
        xticktext = dm_to_string(xtickvalue)
        xtick->setproperty,strings=xticktext
    endif
    if keyword_set(landscape) then $
        yoffset[0] = (dm_strlen(xticktext[n_elements(xticktext)-1])/2.+1)*fsize[0]*self.fscale[0] $
    else $
        xoffset[1] = (dm_strlen(xticktext[n_elements(xticktext)-1])/2.+1)*fsize[0]*self.fscale[0]
    self.drawyAxis[0]->setproperty,ticktext=obj_new()
    self.drawyAxis[0]->getproperty,ticktext=ytick,tickvalues=ytickvalue
    ytick->getproperty,strings=yticktext
    if self.tickstyle then begin
        yticktext = dm_to_string(ytickvalue)
        ytick->setproperty,strings=yticktext
    endif
    if keyword_set(landscape) then $
        yoffset[1] = (max(dm_strlen(yticktext))+0.75)*fsize[0]*self.fscale[0]+fsize[2]*self.fscale[2]*2.5+20 $
    else $
        xoffset[0] = (max(dm_strlen(yticktext))+0.75)*fsize[0]*self.fscale[0]+fsize[2]*self.fscale[2]*2.5+20
    self.title->getproperty,strings=title
    title = strlowcase(strtrim(title,2))
    if dm_strlen(title) ne 0 then begin
        if keyword_set(landscape) then $
            xoffset[1] = xoffset[1]+1.75*fsize[4]*self.fscale[4]  $
        else $
            yoffset[1] = yoffset[1]+1.75*fsize[4]*self.fscale[4]
    endif

    nplot = n_elements(*self.plotInfo)
    xran  = [0.,1.]
    yran  = [0.,1.]
    if nplot ne 0 then begin
      onedplot = where((*self.plotInfo)[*].type le 0,onedcount)
      twodplot = where((*self.plotInfo)[*].type gt 0,twodcount)
      surfplot = where((*self.plotInfo)[*].type eq 2,surfcount)
      if (self.hideIntbar eq 0b) and (twodcount ne 0) then begin
         if keyword_set(landscape) then $
            yran = [-0.2,1] $
         else $
            xran = [0.,1.2]
         self.drawyAxis[3]->setproperty,ticktext=obj_new()
         self.drawyAxis[3]->getproperty,ticktext=cytick,tickvalues=ytickvalue
         cytick->getproperty,strings=yticktext
         if self.tickstyle then begin
             yticktext = dm_to_string(ytickvalue)
             cytick->setproperty,strings=yticktext
         endif
         if keyword_set(landscape) then $
            yoffset[0] = (0.5+max(dm_strlen(yticktext)))*fsize[0]*self.fscale[0]+20 $
         else $
            xoffset[1] = (0.5+max(dm_strlen(yticktext)))*fsize[0]*self.fscale[0]+20
      endif
      if surfcount ne 0 then begin
         if keyword_set(landscape) then begin
            yran[0] = 1-((yran[1]-yran[0])-0.25)
            yran[1] = 1.7
            xran[0] = -0.3
         endif else begin
            xran[0] = -0.7
            xran[1] = xran[1]-0.25
            yran[0] = -0.3
         endelse
         if keyword_set(save) then begin    ;make sure the clipped off image is visible when saved
            self.drawGroup->getproperty,transform=newt
            newt    = newt[0:2,0:2]
            tmp0    = [-0.5,0.5,1]#newt
            tmp1    = [-0.5,-0.5,0]#newt
            tmp2    = [-0.5,0.5,0]#newt
            tmp3    = [0.5,-0.5,0]#newt
            if keyword_set(landscape) then begin
                yran[1] = 1-fix(min([tmp0[0],tmp1[0],tmp2[0],tmp3[0]])*10.5)/10.0
                xran[0] = fix(min([tmp0[1],tmp1[1],tmp2[1],tmp3[1]])*10.5)/10.0
            endif else begin
                xran[0] = fix(min([tmp0[0],tmp1[0],tmp2[0],tmp3[0]])*10.5)/10.0
                yran[0] = fix(min([tmp0[1],tmp1[1],tmp2[1],tmp3[1]])*10.5)/10.0
            endelse
         endif
         self.drawzAxis->setproperty,ticktext=obj_new()
         self.drawzAxis->getproperty,ticktext=ztick,tickvalues=ztickvalue
         ztick->getproperty,strings=zticktext
         if self.tickstyle then begin
             zticktext =dm_to_string(ztickvalue)
             ztick->setproperty,strings=zticktext
         endif
         if keyword_set(landscape) then begin
            yoffset[1]=(0.5+max(dm_strlen(zticktext)))*fsize[0]*self.fscale[0]+fsize[3]*self.fscale[3]*2.0+40
            xoffset[0]=fsize[0]*self.fscale[0]*0.9+fsize[1]*self.fscale[1]*1.0+20
         endif else begin
            xoffset[0]=(0.5+max(dm_strlen(zticktext)))*fsize[0]*self.fscale[0]+fsize[3]*self.fscale[3]*2.0+20
            yoffset[0]=fsize[0]*self.fscale[0]*0.9+fsize[1]*self.fscale[1]*1.0+20
         endelse
      endif
    endif

    viewplane_rect    = [xran[0],yran[0],xran[1]-xran[0],yran[1]-yran[0]]  ;viewing volume
    xoffset1          = xoffset/(xsize-total(xoffset))*viewplane_rect[2]
    yoffset1          = yoffset/(ysize-total(yoffset))*viewplane_rect[3]
    viewplane_rect[0] = viewplane_rect[0]-xoffset1[0]
    viewplane_rect[1] = viewplane_rect[1]-yoffset1[0]
    viewplane_rect[2] = viewplane_rect[2]+xoffset1[0]+xoffset1[1]
    viewplane_rect[3] = viewplane_rect[3]+yoffset1[0]+yoffset1[1]

    if self.iso then begin ;isotropic scaling
       xoff = 0.0 & yoff = 0.0
       if self.xlog then xran = alog10(self.xran[*,2]) else xran = self.xran[*,0]
       if self.ylog then yran = alog10(self.yran[*,2]) else yran = self.yran[*,0]
       if keyword_set(landscape) then begin
            tmp  = xran
            xran = yran
            yran = tmp
       endif
       aspect = (yran[1]-yran[0])/(xran[1]-xran[0])
       ratio  = float(ysize)/float(xsize)
       ;(y0+dy)/(x0+dx)==ratio/Aspect
       yoff = viewplane_rect[2]*ratio/aspect-viewplane_rect[3]
       xoff = viewplane_rect[3]*aspect/ratio-viewplane_rect[2]
       if xoff lt 0 then xoff = 0.
       if yoff lt 0 then yoff = 0.
       viewplane_rect[0] = viewplane_rect[0]-xoff/2.
       viewplane_rect[1] = viewplane_rect[1]-yoff/2.
       viewplane_rect[2] = viewplane_rect[2]+xoff
       viewplane_rect[3] = viewplane_rect[3]+yoff
    endif

    self.drawView->setproperty,viewplane_rect=viewplane_rect,color=bgcolor
    self.tranView->setproperty,viewplane_rect=viewplane_rect
    self.viewplane  =  viewplane_rect
    ;reverse black and white color of the axis if necessary
    for i=0,3 do begin
       if obj_valid(self.drawxAxis[i]) then begin
         self.drawxAxis[i]->getproperty,color=color
         if total(abs(color-bgcolor)) eq 0 then self.drawxAxis[i]->setproperty,color=[255b,255b,255b]-color
       endif
       if obj_valid(self.drawyAxis[i]) then begin
         self.drawyAxis[i]->getproperty,color=color
         if total(abs(color-bgcolor)) eq 0 then self.drawyAxis[i]->setproperty,color=[255b,255b,255b]-color
       endif
    endfor
    if obj_valid(self.drawzAxis) then begin
       self.drawzAxis->getproperty,color=color
       if total(abs(color-bgcolor)) eq 0 then self.drawzAxis->setproperty,color=[255b,255b,255b]-color
    endif
    ;reverse black and white for the grids if necessary
    if self.showxgrid then begin
        self.xgrid->getproperty,color=color
        if total(abs(color-bgcolor)) eq 0 then self.xgrid->setproperty,color=[255b,255b,255b]-color
    endif
    if self.showygrid then begin
        self.ygrid->getproperty,color=color
        if total(abs(color-bgcolor)) eq 0 then self.ygrid->setproperty,color=[255b,255b,255b]-color
    endif
    ;reverse black and white for text objects and modify their font sizes
    oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
        if obj_isa(oChildArr[i],'IDLgrText') then begin
            oChildArr[i]->getproperty,color=color,font=ofont,name=name
            if total(abs(color-bgcolor)) eq 0 then oChildArr[i]->setproperty,color=[255b,255b,255b]-color
            if name eq 'text' then begin
                help,ofont,output=id & id=strsplit(id,' ',/extract) & id=id[n_elements(id)-1]
                index = where(*self.textsize eq id,count)
                if count eq 1 then begin
                    tmp_fsize  =  fix((1>min(dim/self.default[5:6]))*float((*self.textsize)[index+1]))
                    ofont->setproperty,size=tmp_fsize
                endif
            endif
        endif
    endfor
    ;update symbol size and color
    if nplot ne 0 then begin
      for i=0,onedcount-1 do begin
         (*self.plotInfo)[onedplot[i]].obj->getproperty,symbol=drawPsym,color=color
         if total(abs(color-bgcolor)) eq 0 then (*self.plotInfo)[onedplot[i]].obj->setproperty,color=[255b,255b,255b]-color
         if obj_valid(drawPsym) then begin
           drawPsym=self->getpsym((*self.plotInfo)[onedplot[i]].psym,(*self.plotInfo)[onedplot[i]].symsize,$
            (*self.plotInfo)[onedplot[i]].thick,(*self.plotInfo)[onedplot[i]].color,old=drawPsym,landscape=landscape)
           drawPsym->getproperty,data=tmp,color=color
           if total(abs(color-bgcolor)) eq 0 then drawPsym->setproperty,color=[255b,255b,255b]-color
           if obj_valid(tmp) then begin
             tmp->getproperty,color=color
             if total(abs(color-bgcolor)) eq 0 then  tmp->setproperty,color=[255b,255b,255b]-color
           endif
         endif
      endfor
      if onedcount ne 0 then begin
         ;reverse black and white color if necessary
         self.drawLegend->getproperty,item_color=item_color
         for i=0,n_elements(item_color[0,*])-1 do $
           if total(abs(item_color[*,i]-bgcolor)) eq 0 then item_color[*,i]=[255b,255b,255b]-bgcolor
         self.drawLegend->setproperty,item_color=item_color,text_color=[255b,255b,255b]-bgcolor,outline_color=[255b,255b,255b]-bgcolor
         ;update the drawlegend position
         show = where((*self.plotInfo)[onedplot].hide eq 0,count)
         if count gt 0 then begin
            self.drawLegend->getproperty,gap=gap
            if keyword_set(landscape) then $
                tmp   = self.legdpos[1]-(count+(count-1)*gap+4*gap)*fsize[5]/xsize*self.viewplane[2] $
            else $
                tmp   = self.legdpos[1]-(count+(count-1)*gap+4*gap)*fsize[5]/ysize*self.viewplane[3]
            new_yconv = [tmp,1]
            new_xconv = [self.legdpos[0]+((surfcount gt 0)?(-0.5):(0.0)),1]
            self.drawLegend->setproperty,ycoord_conv=new_yconv,xcoord_conv=new_xconv
         endif
      endif

      if keyword_set(save) then tmpcolor=[255b,255b,255b]*self.misscol else tmpcolor=bgcolor
      if twodcount gt 0 then $
         for i=0,twodcount-1 do begin
           if not obj_isa((*self.plotInfo)[twodplot[i]].obj,'IDLgrImage') then begin
              (*self.plotInfo)[twodplot[i]].obj->getproperty,color=color
              if total(abs(color-bgcolor)) eq 0 then $
                (*self.plotInfo)[twodplot[i]].obj->setproperty,color=[255b,255b,255b]-bgcolor
           endif
           (*self.plotInfo)[twodplot[i]].obj->getproperty,palette=drawPal
           drawPal->getproperty,red_values=r,green_values=g,blue_values=b
           r[0]=tmpcolor[0] & g[0]=tmpcolor[1] & b[0]=tmpcolor[2]
           drawPal->setproperty,red_values=temporary(r),green_values=temporary(g),blue_values=temporary(b)
         endfor
    endif
end

;special treatment of strings
;keywords:
;   AA  :if set AA->
function dm_plot::textadjust,text, AA=AA
    ntext = n_elements(text)
    if ntext eq 0 then return,''
    tmp0  = text

    ;replace AA with 
    if keyword_set(AA) then begin
        for i=0,ntext-1 do begin
            tmp1 = strpos(tmp0[i],'AA')
            while(tmp1 ne -1) do begin
                tmp0[i] = strmid(tmp0[i],0,tmp1)+''+strmid(tmp0[i],tmp1+2,strlen(tmp0[i])-tmp1-2)
                tmp1    = strpos(tmp0[i],'AA')
            endwhile
        endfor
    endif

    return, (ntext eq 1)?tmp0[0]:tmp0
end

;push the plot window on top of other applications
pro dm_plot::totop
    widget_control,self.tlb,/show,iconify=0
    widget_control,self.plotWin,/input_focus
end

;mouse events in the plotwin
pro dm_plot::zoom,event
    if self.xlog eq 1 then xran = alog10(self.xran[*,2]) else xran = self.xran[*,0]
    if self.ylog eq 1 then yran = alog10(self.yran[*,2]) else yran = self.yran[*,0]
    case event.type of
       0:    begin  ;mouse button down
         case event.press of
          1:  begin    ;left button down
                 widget_control,self.plotWin,draw_motion_events=1
                 ;initialize the zoom box
                 self.drawView->getproperty,dimension=dim
                 xstart = self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
                 ystart = self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
                 tmp = self.drawWin->select(self.drawView,[event.x,event.y])
                 if obj_valid(tmp[0]) then $
                   if obj_isa(tmp[0],'IDLgrLegend') or obj_isa(tmp[0],'IDLgrText') then begin
                    self.moveLegend = 1b
                    self.select = tmp[0]
                    self.select->getproperty,xcoord_conv=x,ycoord_conv=y
                    self.legdOffset = [x[0]-xstart,y[0]-ystart]
                    break
                   endif
                 xstart = xran[0]+(0.0>(xstart)<1.0)*(xran[1]-xran[0])
                 ystart = yran[0]+(0.0>(ystart)<1.0)*(yran[1]-yran[0])
                 self.zoomBox = obj_new('IDLgrPolyline',replicate(xstart,5),replicate(ystart,5),replicate(0.02,5),$
                   color=self->getcolor('green'),xcoord_conv=norm_coord(xran),ycoord_conv=norm_coord(yran))
                 self.tranModel = obj_new('IDLgrModel')
                 self.tranModel->add,self.zoomBox
                 ;take a snap short of current image
                 self.drawWin->draw,self.drawView,create_instance=2
                 self.tranView->add,self.tranModel
              end
          else:
         endcase
         end
       1:    begin  ;mouse button up
          widget_control,self.plotWin,draw_motion_events=0    ;disable motion event
          widget_control,self.plotWin,/clear_events
          self.moveLegend = 0b
          if obj_valid(self.zoomBox) then begin
              self.zoomBox->getproperty,data=data
              xran = [min(data[0,[0,1]]),max(data[0,[0,1]])]
              yran = [min(data[1,[1,2]]),max(data[1,[1,2]])]
              obj_destroy,self.zoomBox
              obj_destroy,self.tranModel
              self.drawWin->draw,self.tranView,/draw_instance
              if (xran[1]-xran[0] ne 0) and (yran[1]-yran[0] ne 0) then $
                 self->setproperty,yran=(self.ylog eq 1)?(10.0^yran):yran,xran=(self.xlog eq 1)?(10.0^xran):xran,/zoom
          end
         end
       2:    begin ;mouse motion
          self.drawView->getproperty,dimension=dim
          xend = self.viewplane[0]+(0>(event.x/dim[0])<1)*self.viewplane[2]
          yend = self.viewplane[1]+(0>(event.y/dim[1])<1)*self.viewplane[3]
          if self.moveLegend eq 1b then begin
              self.select->getproperty,ycoord_conv=old_yconv,xcoord_conv=old_xconv
              new_yconv       = [yend+self.legdOffset[1],1]
              new_xconv       = [xend+self.legdOffset[0],1]
              if obj_isa(self.select,'IDLgrLegend') then begin
                  self.legdpos[0] = self.legdpos[0]+new_xconv[0]-old_xconv[0]
                  self.legdpos[1] = self.legdpos[1]+new_yconv[0]-old_yconv[0]
              endif
              self.select->setproperty,xcoord_conv=new_xconv,ycoord_conv=new_yconv
              self.drawWin->draw,self.drawView
          endif else begin
              xend = xran[0]+(0.0>(xend)<1.0)*(xran[1]-xran[0])
              yend = yran[0]+(0.0>(yend)<1.0)*(yran[1]-yran[0])
              self.zoomBox->getproperty,data=data
              data[0,[1,2]] = [xend,xend]
              data[1,[2,3]] = [yend,yend]
              self.zoomBox->setproperty,data=data
              self.drawWin->draw,self.tranView,/draw_instance
          endelse
         end
       else:
    endcase
end

;cleanup, life cycle method
pro dm_plot::Cleanup
    if widget_info(self.tlb,/valid_id) then widget_control,self.tlb,/destroy
    ;destroy all possible text objects
    oChildArr = self.drawModel->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
        if obj_isa(oChildArr[i],'IDLgrText') then begin
            oChildArr[i]->getproperty,font=ofont
            obj_destroy,ofont
        endif
    endfor
    ;destroy all the plot symbols
    oChildArr = self.drawGroup->IDL_Container::Get(/ALL, COUNT=nKids)
    for i=0,nKids-1 do begin
       if obj_isa(oChildArr[i],'IDLgrPlot') then begin
         oChildArr[i]->getproperty,symbol=tmp1
         if obj_valid(tmp1) then begin
          tmp1[0]->getproperty,data=tmp2
          if obj_valid(tmp2) then obj_destroy,tmp2
          obj_destroy,tmp1
         endif
       endif
    endfor
    if n_elements(*self.plotInfo) ne 0 then begin
        ;destroy the draw palettes
        twodplot=where((*self.plotInfo)[*].type gt 0,count1)
        for i=0,count1-1 do begin
            (*self.plotInfo)[twodplot[i]].obj->getproperty, palette=tmp
            if obj_valid(tmp) then obj_destroy,tmp
        endfor
        ptr_free,(*self.plotInfo)[*].xPtr,(*self.plotInfo)[*].yPtr,(*self.plotInfo)[*].zPtr,(*self.plotInfo)[*].iPtr
    endif
    ptr_free,self.plotInfo,self.textsize
    obj_destroy,[self.drawView,self.tranView]
    obj_destroy,[self.font,self.printer,self.trackBall]
    obj_destroy,[self.xtit,self.ytit,self.ztit,self.title]
end

;initialization, life cycle method
;usage:
;   newplot=obj_new('dm_plot')
;parameters:
;   NONE
;keyword:
;   see the definition
function dm_plot::Init,xdat,ydat,zdat,idat,yerr=yerr,axisthick=axisthick,coltable=coltable,color=color,font=font,fsize=fsize,$
    group_leader=group_leader,hideintbar=hideintbar,hidelegend=hidelegend,iso=iso,iran=iran,ilog=ilog,legend=legend,$
    linestyle=linestyle,misscol=misscol,no_copy=no_copy,parentobj=parentobj,parenthandler=parenthandler,psym=psym,$
    shading=shading,showxgrid=showxgrid,showygrid=showygrid,surfplot=surfplot,symsize=symsize,thick=thick,title=title,$
    usepolygon=usepolygon,xlog=xlog,xran=xran,xtit=xtit,ylog=ylog,yran=yran,ytit=ytit,zlog=zlog,zmax=zmax,zmin=zmin,$
    zran=zran,ztit=ztit,layer=layer

    if float(!version.release) ge 6.0 then v6p0_extra= {register_properties:1}  ;register_properties keyword for IDL6.0 or later
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(!error_state.msg,/error)
       return,0  ;indicating that an error occurs
    end

    ;default values for [ticklength,symsize,thick,shading,coltable,xsize,ysize,legend gap]
    self.default = [0.02,0.008,1.5,0.0,13.0,600,400,0.2]
    self.xran[*] = !values.f_nan
    self.yran[*] = !values.f_nan
    self.zran[*] = !values.f_nan
    self.iran[*] = !values.f_nan
    self.zmin    = !values.f_nan & self.zmax=!values.f_nan
    self.tickstyle = 0b
    self.legdpos = [0.65,0.85]
    ;check keywords
    if n_elements(parentobj) ne 0 then $
        if obj_valid(parentobj) then begin
            if n_elements(group_leader) eq 0 then parentobj->getproperty,tlb=group_leader
            if n_elements(parenthandler) eq 0 then parenthandler='event'
            self.parentobj     = parentobj
            self.parenthandler = parenthandler
        endif
    if n_elements(group_leader) ne 0 then self.group_leader=group_leader else self.group_leader=0
    if n_elements(xtit)  ne 0 then xtit = self->textadjust(strtrim(xtit[0],2),/AA) else xtit = 'X'
    if n_elements(ytit)  ne 0 then ytit = self->textadjust(strtrim(ytit[0],2),/AA) else ytit = 'Y'
    if n_elements(ztit)  ne 0 then ztit = self->textadjust(strtrim(ztit[0],2),/AA) else ztit = 'Z'
    if n_elements(zmin)  ne 0 then self.zmin = zmin[0]
    if n_elements(zmax)  ne 0 then self.zmax = zmax[0]
    if n_elements(title) ne 0 then title = self->textadjust(strtrim(title[0],2),/AA) else title = ''
    if n_elements(fsize) ne 0 then self.fsize[*]=(8>(fix(fsize[0]))) else self.fsize[*]=12
    if n_elements(axisthick) ne 0 then self.axisthick=1.0>(axisthick[0])<10.0 else self.axisthick=self.default[2]
    self.xlog = keyword_set(xlog)
    self.ylog = keyword_set(ylog)
    self.zlog = keyword_set(zlog)
    self.ilog = keyword_set(ilog)
    self.iso  = keyword_set(iso)
    self.showxgrid  = keyword_set(showxgrid)
    self.showygrid  = keyword_set(showygrid)
    self.hideIntbar = keyword_set(hideintbar)
    self.hideLegend = keyword_set(hidelegend)
    self.misscol    = keyword_set(misscol)
    if not keyword_set(font) then font='Helvetica'
    self.textsize = ptr_new(/allocate_heap)
    for i=0,5 do $
       self.font[i] = obj_new('IDLgrFont',font,size=self.fsize[i],substitute='Hershey')
    self.font[0]->getproperty,name=tmp
    if strmid(strlowcase(tmp),0,7) eq 'hershey' then self.fscale[*]=1.2 else self.fscale[*]=1.0

    self.xtit  = obj_new('IDLgrText',xtit,recompute_dimensions=2,font=self.font[1],/enable,_extra=v6p0_extra,name='xtitle')
    self.ytit  = obj_new('IDLgrText',ytit,recompute_dimensions=2,font=self.font[2],/enable,_extra=v6p0_extra,name='ytitle')
    self.ztit  = obj_new('IDLgrText',ztit,recompute_dimensions=2,font=self.font[3],/enable,_extra=v6p0_extra,name='ztitle')
    self.title = obj_new('IDLgrText',title,recompute_dimensions=2,font=self.font[4],/enable,color=self->getcolor('white'),$
                  name='titl',_extra=v6p0_extra)

    ;create the widgets
    self.tlb = widget_base(uname='tlb',mbar=bar,group_leader=self.group_leader,/tlb_size_events,/col,$
                  ypad=0,xpad=0,/align_center,map=0,space=0,title=(strlen(title) eq 0?'Plot':title))
    filemenu = widget_button(bar,value='File',/menu)
    editmenu = widget_button(bar,value='Edit',/menu)
    if self.group_leader ne 0 then begin
       keepmenu = widget_button(bar,value='Keep',/menu,uname='keepmenu')
       mkcrmenu = widget_button(bar,value='Make Current',/menu,uname='mkcrmenu',sensitive=0)
       ;keep menu
       keepBut  = widget_button(keepmenu,value='Keep',uname='keepBut')
       mkcrBut  = widget_button(mkcrmenu,value='Make Current',uname='mkcrBut',sensitive=0)
    endif

    ;file menu
    saveBut  = widget_button(filemenu,value='Save',uname='save')
    savemenu = widget_button(filemenu,value='Save As...',uname='savemenu',/menu)
    bgclmenu = widget_button(filemenu,value='Color of Missing Data',/menu)
    readBut  = widget_button(filemenu,value='Read 2 or 3-Column 1D Data File...',uname='readBut',/separator)
    readBut  = widget_button(filemenu,value='Read 2D SPE File...',uname='readSpe')
    printBut = widget_button(filemenu,value='Print',uname='printBut',/separator)
    pnsetBut = widget_button(filemenu,value='Printer Setup...',uname='pnsetBut')
    exitBut  = widget_button(filemenu,value='Exit',uname='exitBut',/separator)
    saveps   = widget_button(savemenu,value='ps file',uname='saveps')
    savejpeg = widget_button(savemenu,value='jpeg file',uname='savejpg')
    savebmp  = widget_button(savemenu,value='bmp file',uname='savebmp')
    savepng  = widget_button(savemenu,value='png file',uname='savepng')
    saveppm  = widget_button(savemenu,value='ppm file',uname='saveppm')
    savetiff = widget_button(savemenu,value='tiff file',uname='savetif')
    saveascii= widget_button(savemenu,value='ascii file',uname='saveasc',/separator)
    blackbg  = dm_widget_button(bgclmenu,value='Black',uname='blackbg')
    whitebg  = dm_widget_button(bgclmenu,value='White',uname='whitebg')
    if self.misscol eq 0 then dm_toggle_menubut,blackbg,whitebg $
    else dm_toggle_menubut,whitebg,blackbg
    ;edit menu
    textBut  = widget_button(editmenu,value='Add Text',uname='textBut')
    isotBut  = dm_widget_button(editmenu,value='Isotropic Scaling',uname='isoBut',set_button=self.iso,$
                    onstring='Disable Isotropic Scaling',offstring='Enable Isotropic Scaling',/separator)
    gridBut  = dm_widget_button(editmenu,value='Show X Grid',uname='xgridBut',set_button=self.showxgrid,$
                    onstring='Hide X Grid',offstring='Show X Grid')
    gridBut  = dm_widget_button(editmenu,value='Show Y Grid',uname='ygridBut',set_button=self.showygrid,$
                    onstring='Hide Y Grid',offstring='Show Y Grid')
    if float(!version.release) ge 6.0 then $
    vallBut  = widget_button(editmenu,value='Show All Plots',uname='show_all',sensitive=0)
    titlBut  = widget_button(editmenu,value='Title',uname='titlBut',/separator)
    xtitBut  = widget_button(editmenu,value='X Title',uname='xtitBut')
    xranBut  = widget_button(editmenu,value='X Range',uname='xranBut')
    ylogBut  = dm_widget_button(editmenu,value='X Logarithmic Scale',uname='xlogBut',set_button=self.xlog,$
                    onstring='X Normal Scale',offstring='X Logarithmic Scale')
    ytitBut  = widget_button(editmenu,value='Y Title',uname='ytitBut')
    yranBut  = widget_button(editmenu,value='Y Range',uname='yranBut')
    ylogBut  = dm_widget_button(editmenu,value='Y Logarithmic Scale',uname='ylogBut',set_button=self.ylog,$
                    onstring='Y Normal Scale',offstring='Y Logarithmic Scale')
    ztitBut  = widget_button(editmenu,value='Z Title',uname='ztitBut',sensitive=0)
    zranBut  = widget_button(editmenu,value='Z Range',uname='zranBut',sensitive=0)
    zlogBut  = dm_widget_button(editmenu,value='Z Logarithmic Scale',uname='zlogBut',sensitive=0,set_button=self.zlog,$
                    onstring='Z Normal Scale',offstring='Z Logarithmic Scale')
    iranBut  = widget_button(editmenu,value='I Range',uname='iranBut',sensitive=0)
    ilogBut  = dm_widget_button(editmenu,value='I Logarithmic Scale',uname='ilogBut',sensitive=0,set_button=self.zlog,$
                    onstring='I Normal Scale',offstring='I Logarithmic Scale')
    tickmenu = widget_button(editmenu,value='Tick Format',/menu)
    psymBut  = widget_button(editmenu,value='Plot Symbol',uname='psymBut',sensitive=0,/separator)
    lineBut  = widget_button(editmenu,value='Plot Linestyle',uname='lineBut',sensitive=0)
    pcolBut  = widget_button(editmenu,value='Plot Color',uname='pcolBut',sensitive=0)
    lgndBut  = widget_button(editmenu,value='Legend Label',uname='lgndBut',sensitive=0)
    hideBut  = dm_widget_button(editmenu,value='Hide Legend',uname='cs_hidelegend',sensitive=0,set_button=self.hideLegend,$
                    onstring='Show Legend',offstring='Hide Legend')
    ctabBut  = widget_button(editmenu,value='Color Table',uname='cs_coltable',sensitive=0,/separator)
    shadBut  = widget_button(editmenu,value='Shading',uname='cs_shade',sensitive=0)
    hideBut  = dm_widget_button(editmenu,value='Hide Colorbar',uname='cs_hidecbar',sensitive=0,set_button=self.hideIntbar,$
                    onstring='Show Colorbar',offstring='Hide Colorbar')
    idltick  = dm_widget_button(tickmenu,value='Default',uname='idltick')
    owntick  = dm_widget_button(tickmenu,value='Abbreviated',uname='owntick')
    if self.tickstyle eq 0 then dm_toggle_menubut,idltick,owntick $
    else dm_toggle_menubut,owntick,idltick

    ;menu bar separator for Windows operating system
    if !version.os_family eq 'Windows' then $
       self.mbarSep = widget_label(self.tlb,sensitive=0,/dynamic_resize,xsize=1)
    ;draw widget
    self.plotWin = widget_draw(self.tlb,/button_events,uname='plotWin',graphics_level=2,scr_xsize=self.default[5],$
                    scr_ysize=self.default[6],renderer=1,retain=0,/no_copy)
                    ;tooltip='Press right button for context-sensitive menu.')
                    ;/no_copy,retain=0)
                    ;renderer=0,/no_copy,expose_events=0)

    ;context senstive menu
    cs_axisMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_axisMenu')  ;axis object
       titlbut   = widget_button(cs_axisMenu,value='Title',uname='cs_title')
       rangebut  = widget_button(cs_axisMenu,value='Range',uname='cs_range')
       thickbut  = widget_button(cs_axisMenu,value='Thick',uname='cs_axisthick')
       fontbut   = widget_button(cs_axisMenu,value='Font', uname='cs_font')
       logbut    = dm_widget_button(cs_axisMenu,value='Logarithmic Scale',uname='cs_logscale')
    cs_titlMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_titlMenu')  ;title text object
       titlbut   = widget_button(cs_titlMenu,value='Title',uname='cs_title')
       fontbut   = widget_button(cs_titlMenu,value='Font', uname='cs_font')
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_titlMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_plotMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_plotMenu')  ;1-d plot object
       symbbut   = widget_button(cs_plotMenu,value='Symbol',uname='cs_symbol')
       colbut    = widget_button(cs_plotMenu,value='Color', uname='cs_color')
       linebut   = widget_button(cs_plotMenu,value='Linestyle',uname='cs_linestyle')
       thickbut  = widget_button(cs_plotMenu,value='Thick', uname='cs_linethick')
       legndbut  = widget_button(cs_plotMenu,value='Legend',uname='cs_legend')
       layerbut  = widget_button(cs_plotMenu,value='Layer',uname='cs_layer',/separator)
       removebut = widget_button(cs_plotMenu,value='Remove',uname='cs_remove')
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_plotMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_twodMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_twodMenu')  ;2-d plot object
       coltbut   = widget_button(cs_twodMenu,value='Color table',uname='cs_coltable')
       shadbut   = widget_button(cs_twodMenu,value='Shading',uname='cs_shade')
    cs_legendMenu= widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_legendMenu')  ;legend object
       lablbut   = widget_button(cs_legendMenu,value='Labels',uname='lgndBut')
       fontbut   = widget_button(cs_legendMenu,value='Font',uname='cs_font')
       lgapbut   = widget_button(cs_legendMenu,value='Gap',uname='cs_lgndgap')
       gwidbut   = widget_button(cs_legendMenu,value='Glyph Width',uname='cs_lgndwidth')
       oulnbut   = widget_button(cs_legendMenu,value='Show Outline',uname='cs_outline')
       hidebut   = widget_button(cs_legendMenu,value='Hide',uname='cs_hidelegend',/separator)
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_legendMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_cbarMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_cbarMenu')  ;color bar object
       zranbut   = widget_button(cs_cbarMenu,value='Range',uname='cs_range')
       coltbut   = widget_button(cs_cbarMenu,value='Color table',uname='cs_coltable')
       shadbut   = widget_button(cs_cbarMenu,value='Shading',uname='cs_shade')
       fontbut   = widget_button(cs_cbarMenu,value='Font', uname='cs_font')
       logbut    = dm_widget_button(cs_cbarMenu,value='Logarithmic Scale',uname='cs_logscale')
       hidebut   = widget_button(cs_cbarMenu,value='Hide',uname='cs_hidecbar',/separator)
    cs_textMenu  = widget_base(self.plotWin,/CONTEXT_MENU,UNAME='cs_textMenu')  ;title text object
       titlbut   = widget_button(cs_textMenu,value='String',uname='cs_title')
       fontbut   = widget_button(cs_textMenu,value='Font', uname='cs_font')
       colbut    = widget_button(cs_textMenu,value='Color',uname='cs_txtcolor')
       oribut    = widget_button(cs_textMenu,value='Orientation',uname='cs_textori')
       removebut = widget_button(cs_textMenu,value='Remove',uname='cs_remove',/separator)
       if float(!version.release) ge 6.0 then $
       propsheet = widget_button(cs_textMenu,value='Property Sheet',uname='cs_propsheet',/separator)
    cs_gridMenu  = widget_base(self.plotWin,/CONTEXT_MENU,uname='cs_gridMenu')  ;grids
       colbut    = widget_button(cs_gridMenu,value='Color',uname='cs_txtcolor')
       linebut   = widget_button(cs_gridMenu,value='Linestyle',uname='cs_linestyle')
       thickbut  = widget_button(cs_gridMenu,value='Thick', uname='cs_linethick')
       hideBut   = widget_button(cs_gridMenu,value='Hide',uname='cs_hidegrid',/separator)

    dm_center_kid,self.tlb
    widget_control,self.tlb,/realize
    widget_control,self.plotWin,get_value=oWindow
    self.drawWin   = oWindow
    self.drawView  = obj_new('IDLgrView',location=[0,0],dimensions=self.default[5:6],viewplane_rect=[0,0,1,1],$
                        color=self->getcolor('black'),zclip=[1.1,-1.1])
    self.tranView  = obj_new('IDLgrView',/transparent,location=[0,0],dimensions=self.default[5:6])
    self.drawModel = obj_new('IDLgrModel',lighting=0)
    self.drawGroup = obj_new('IDLgrModel',lighting=0)
    self.drawModel->add,self.drawGroup
    self.drawView->add,self.drawModel
    self.xgrid      = obj_new('IDLgrPolyline',color=self->getcolor('grey'),hide=1-self.showxgrid,name='grid')
    self.ygrid      = obj_new('IDLgrPolyline',color=self->getcolor('grey'),hide=1-self.showygrid,name='grid')
    self.drawGroup->add,self.xgrid
    self.drawGroup->add,self.ygrid
    self.drawLegend= obj_new('IDLgrLegend',text_color=self->getcolor('white'),xcoord_conv=[self.legdpos[0],1],$
       ycoord_conv=[self.legdpos[1],1],zcoord_conv=[1.0,1.0],font=self.font[5],name='legend',gap=self.default[7],$
       border_gap=2*self.default[7],outline_color=self->getcolor('white'),glyph_width=1.5,_extra=v6p0_extra)
    if float(!version.release) ge 6.0 then begin    ;register some properties
       self.drawLegend->registerproperty,'border_gap',3,    name='border gap'
       self.drawLegend->registerproperty,'fill_color',5,    name='fill color'
       self.drawLegend->registerproperty,'gap',3
       self.drawLegend->registerproperty,'glyph_width',3,   name='glyph width'
       self.drawLegend->registerproperty,'outline_color',5, name='outline color'
       self.drawLegend->registerproperty,'outline_thick',3, name='outline thick'
       self.drawLegend->registerproperty,'show_fill',1,     name='show fill'
       self.drawLegend->registerproperty,'show_outline',1,  name='show outline'
       self.drawLegend->registerproperty,'text_color',5,    name='text color'
    endif
    self.drawModel->add,self.drawLegend
    self.trackBall = obj_new('Trackball',(self.default[5:6])/2,self.default[6]/2)
    self.printer   = obj_new('IDLgrPrinter',units=1,print_quality=2)
    self.plotInfo  = ptr_new(/allocate_heap)

    if n_params() eq 2  then $
       self->setproperty,xdat=xdat,ydat=ydat,yerr=yerr,no_copy=no_copy,legend=legend,symsize=symsize,linestyle=linestyle,$
         thick=thick,psym=psym,color=color,xran=xran,yran=yran,layer=layer,/nodraw $
    else if n_params() ge 3 then $
       self->setproperty,xdat=xdat,ydat=ydat,zdat=zdat,idat=idat,no_copy=no_copy,surfplot=surfplot,shading=shading,coltable=coltable,$
         usepolygon=usepolygon,xran=xran,yran=yran,zran=zran,iran=iran,/nodraw
    ;show the horizontal bar style menu bar separator
    if widget_info(self.mbarsep,/valid_id) then begin
       sep=bytarr(1000) & sep[*]=byte(175) & sep=string(sep)
       widget_control,self.mbarSep,set_value=temporary(sep),xsize=self.default[5]
    endif


    widget_control,self.tlb,set_uvalue=self

    if xregistered('dave',/noshow) and (self.group_leader ne 0) then begin
        self.enotebook = 1b
        RET   = DAVE_SET_FOCUS(self.tlb)
    endif

    xmanager,'dm_plot',self.tlb,cleanup='dm_plot_Exit',/no_block
    return,1
end

;class definition
pro dm_plot__define
    struct={dm_plot,        $   ;class name
      group_leader:     0L, $   ;group leader of this object
      parentobj: obj_new(), $   ;parent object
      parenthandler:    '', $   ;event handler of parent object,default is 'event'
      tlb:              0L, $   ;identifier of top level base
      mbarSep:          0L, $   ;menu bar separator
                            $   ;ranges: [*,0]-current range [*,1]-original range  [*,2]-current log range [*,3]-original log range
      xran:    fltarr(2,4), $   ;xrange for the axis
      yran:    fltarr(2,4), $   ;yrange
      zran:    fltarr(2,4), $   ;zrange
      iran:    fltarr(2,4), $   ;irange for 2d plots, shown in color bar
      zmin:             0e, $   ;zmin for contour plot and surface plot
      zmax:             0e, $   ;zmax
      plotWin:          0L, $   ;draw widget
      font:      objarr(6), $   ;font  0:for axis ticks 1:for xtitle 2:for ytitle 3:ztitle 4:title 5:legend
      fsize:     intarr(6), $   ;font size
      fscale:    fltarr(6), $   ;scaling factor
      axisthick:      1.5e, $   ;axis thickness
      title:     obj_new(), $   ;title
      xtit:      obj_new(), $   ;xtitle
      ytit:      obj_new(), $   ;ytitle
      ztit:      obj_new(), $   ;ztitle
      xlog:             0b, $   ;log10 scale x flag
      ylog:             0b, $   ;log10 scale y flag
      zlog:             0b, $   ;log10 scale z flag
      ilog:             0b, $   ;log 10 scale i flag    shown in colorbar
      drawWin:   obj_new(), $   ;draw window object
      drawView:  obj_new(), $   ;draw view object
      drawModel: obj_new(), $   ;draw model object
      drawGroup: obj_new(), $   ;contains all moveable graphic atoms, for surface rotation
      xgrid:     obj_new(), $   ;xgrid lines
      ygrid:     obj_new(), $   ;ygrid lines
      drawLegend:obj_new(), $   ;legends
      moveLegend:       0b, $   ;flag for legend movement
      hideLegend:       0b, $   ;hide legend flag
      legdOffset:fltarr(2), $   ;offset for legend movement
      legdpos:   fltarr(2), $   ;position of the legend
      hideIntbar:       0b, $   ;flag for showing the intensity bar in 2d contour plot
      plotInfo:  ptr_new(), $   ;plot informations
      tranView:  obj_new(), $   ;transparent view for zoom box
      tranModel: obj_new(), $   ;model object for zoom box
      drawxAxis: objarr(4), $   ;x axis
      drawyAxis: objarr(4), $   ;y axis
      drawzAxis: obj_new(), $   ;z axis
      colBar:    obj_new(), $   ;color bar
      zoomBox:   obj_new(), $   ;zoom box, a polyline object
      trackBall: obj_new(), $   ;trackall object for rotation
      iso:              0b, $   ;1-isotropic 0-aspect
      showxgrid:        0b, $   ;xgrid
      showygrid:        0b, $   ;ygrid
      keep:             0b, $   ;1: keep this plot  0:open to new plots
      viewplane: fltarr(4), $   ;view plane rectangle
      printer:   obj_new(), $   ;printer
      select:    obj_new(), $   ;selected object
      default:   fltarr(8), $   ;default values for [ticklength,symsize,thick,shading,coltable,xsize,ysize,legend gap]
      misscol:          0b, $   ;flag for missing data color in saved file, default is black:0
      textsize:  ptr_new(), $   ;to store the actual size of the text fonts, point to a string array
      tickstyle:        0b, $   ;0-idl default tick string format 1-use dm_to_string
      enotebook:        0b  $   ;use Rob's enotebook
    }
end


;examples: test,/oned & test,/usepolygon & test,/surfplot
pro test,oned=oned,surfplot=surfplot,_extra=_extra
    if keyword_set(surfplot) then begin
;        ;;sphere
;        r     = 100
;        nth   = 180
;        nphi  = 360
;        theta = findgen(nth)*!dtor*180./nth
;        phi   = findgen(nphi)*!dtor*360./nphi
;        x     = fltarr(nth,nphi)
;        y     = fltarr(nth,nphi)
;        z     = fltarr(nth,nphi)
;        for i=0,nth-1 do begin
;            for j=0,nphi-1 do begin
;                x[i,j] = r*sin(theta[i])*cos(phi[j])
;                y[i,j] = r*sin(theta[i])*sin(phi[j])
;            endfor
;            z[i,*] = r*cos(theta[i])
;        endfor
;        int = sin(!dtor*(x*5+y*y))
;        r   = r*1.05
;        zr  = [-r,r]
;        xr  = [-r,r]
;        yr  = [-r,r]

        ;;donut
        rx    = 100
        rz    = 20
        nth   = 180
        nphi  = 360
        theta = findgen(nth+1)*!dtor*360./nth
        phi   = findgen(nphi+1)*!dtor*360./nphi
        x     = fltarr(nth+1,nphi+1)
        y     = x
        z     = x
        for i=0,nth do begin
            for j=0,nphi do begin
                tmp    = rx+rz*cos(phi[j])
                x[i,j] = tmp*cos(theta[i])
                y[i,j] = tmp*sin(theta[i])
                z[i,j] = rz*sin(phi[j])
            endfor
        endfor
        int = (sin(x*!dtor)+sin(y*!dtor*2)+cos(z*!dtor*3))
        int = int + abs(min(int))*1.2
        xr  = [-rx-rz,rx+rz]*1.2
        yr  = xr
        zr  = [-rz,rz]*3
    endif else begin
        nx  = 201
        r   = (nx-1.)/2.    & rsq = (r)^2
        x   = findgen(nx)-r
        y   = findgen(nx)-r
        z   = fltarr(nx,nx)
        z[*]= !values.f_nan
        for i=0,nx-1 do begin
            xsq = x[i]*x[i]
            for j=0,nx-1 do begin
                tmp = r^2-xsq-y[j]*y[j]
                if tmp ge 0 then z[i,j] = sqrt(tmp)
            endfor
        endfor
        r   = r*1.05
        zr  = [-r,r]
        xr  = [-r,r]
        yr  = [-r,r]
    endelse

    current = systime(/sec)

    if keyword_set(oned)then begin
       nx  = 1e2
       tmp = obj_new('dm_plot',sin(findgen(nx))*10,cos(findgen(nx))*10,title='1d plot',$
         color='white',linestyle='solid',psym='circle',_extra=_extra)
       tmp->draw & wait,1
       tmp->add_plot,[-5,5,0,5.5],[0,0,6,5],color=[190,150,110],psym='triangle',legend='hi'
       tmp->draw & wait,1
       tmp->add_plot,[-5,5],[5,5],yerr=[0.8,0.8],color='white',psym=6,legend='    '    ;symbol 6='square'
    endif else begin
       ;z[0:100,70:75]=!values.f_nan  ;generate some missing data
       ;z=z[*,0:80] & y=y[0:80]
       tmp = obj_new('dm_plot',x,y,z,int,title='2d plot',zr=zr,xr=xr,yr=yr,surfplot=surfplot,_extra=_extra)
       ;tmp = obj_new('dm_plot')
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2.,zdat=z,title='2d plot',zr=zr,xr=[-nx/5*6,nx/5*6],_extra=_extra
       ;tmp->draw & wait,1
       ;tmp->setproperty,xdat=[0,20],ydat=[10,10],linestyle='solid',psym='circle',color='red'
       ;tmp->draw & wait,1
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2,zdat=z,title='2d plot',zr=zr,/surfplot
       ;tmp->draw & wait,1
       ;tmp->setproperty,xdat=x-nx/2.,ydat=y-nx/2,zdat=z,title='2d plot',zr=zr,usepolygon=usepolygon,surfplot=surfplot,/no_copy
       ;tmp->draw & wait,1
       ;tmp->add_plot,[0,20],[10,10],yerr=[0.8,0.8],zpos=50,linestyle='solid',psym='circle',color='red'
    endelse
    tmp->draw
    print,systime(/sec)-current,'seconds'
end
