; $Id: dm_proj_view.pro,v 1.7 2005/06/10 20:31:21 ymqiu Exp $
;#######################################################################
;
; NAME:
;  dm_proj_view
;
; PURPOSE:
;  project Q[3,ne,ndet] onto viewing axis U1[4], U2[4], U3[4]
;  U1, U2, U3 can only have one with Ui[3] ne 0
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-8562
;  United States
;  yiming.qiu@nist.gov
;  June, 2005
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

function dm_proj_view,Q,en,U1,U2,U3,debug=debug,diffuse=diffuse
    if keyword_set(debug) then begin
       print,'$$proj view...'
       current=systime(/sec)
    endif
    abc = [[U1[0:2]],[U2[0:2]],[U3[0:2]]]
    for i=0,2 do begin
        if total(abs(abc[*,i])) eq 0 then begin
           abc[*,i]=crossp(abc[*,(i+1) mod 3],abc[*,(i+2) mod 3])
           break
        endif
    endfor
    V  = dm_q2hkl(Q,abc,debug=debug)
    if not keyword_set(diffuse) then begin          ;Q=[3,nen,ndet]
       tmp = size(Q,/dimension)
       E   = en#(fltarr(tmp[2])+1.0)
       if U1[3] ne 0 then V[0,*,*]=E
       if U2[3] ne 0 then V[1,*,*]=E
       if U3[3] ne 0 then V[2,*,*]=E
    endif
    if keyword_set(debug) then begin
       print,'$$proj view finished in ',systime(/sec)-current,' sec.'
    endif
    return,V
end