; $Id: dm_to_fit.pro,v 1.11 2005/02/07 20:03:59 qiuym Exp $
;#######################################################################
;
; NAME:
;  dm_to_fit
;
; PURPOSE:
;  send the cut data to PAN for fitting
;
; CATEGORY:
;  dcs_mslice
;
; AUTHOR:
;  Yiming Qiu
;  NIST Center for Neutron Research
;  100 Bureau Drive, Gaithersburg, MD 20899-8562
;  United States
;  yiming.qiu@nist.gov
;  Feb, 2005
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;#######################################################################

pro dcs_mslice::dm_to_fit
    if (not ptr_valid(self.projStrPtr)) or (not ptr_valid(self.dataStrPtr)) or self.error then begin
       ok=dialog_message('Please load data and calculate projection first.',dialog_parent=self.tlb)
       self.error=1
       return
    endif
    opan=0
    ;error handling
    catch, myerror
    if myerror ne 0 then begin
       catch,/cancel
       ok=dialog_message(dialog_parent=self.tlb,!error_state.msg,/error)
       if obj_valid(opan) then obj_destroy,opan
       return
    end
    ;select data
    self->dm_choose_cutdata,xdat=xdat,ydat=ydat,yerr=yerr,xtit=xtit,ytit=ytit
    ;save cut
    if strlen(self.cut_name) ne 0 then begin
       type = strsplit(self.filetype[self.cut_type],'.',/extract) & type = strlowcase(type[n_elements(type)-1])
       tmp  = strsplit(self.cut_name,'.',/extract)
       if strlowcase(tmp[n_elements(tmp)-1]) ne type then self.cut_name=self.cut_name+'.'+type
       ok   = file_test(self.cut_name)
       if ok then begin
         ans=dialog_message(self.cut_name+' exists. Do you want to overwrite it?',/question,dialog_parent=self.tlb)
         if ans eq 'No' then begin
          defsysv,'!dave_defaults',exists=exists
          if exists then $
            if ptr_valid(!dave_defaults) then cd,(*!dave_defaults).workDir
          self.cut_name=dm_choose_file(type,dialog_parent=self.tlb,/write,file='mslice_cut.'+type)
          if strlen(self.cut_name) eq 0 then $
              self.cut_name='mslice_cut.'+type
          self->my_widget_control,'cutName',self.dispBas,set_value=self.cut_name
         endif
       endif
       openw,unit,self.cut_name,/get_lun,error=openerr
       if openerr ne 0 then begin
         ok=dialog_message("Can't write in "+self.cut_name+'.',/error,dialog_parent=self.tlb)
       endif else begin
         for i=0,n_elements(xdat)-1 do begin
         if n_elements(yerr) ne 0 then printf,unit,xdat[i],ydat[i],yerr[i] $
         else printf,unit,xdat[i],ydat[i]
         endfor
         free_lun,unit
       endelse
    endif
    ;to PAN
    if n_elements(yerr) ne n_elements(ydat) then begin
       ok  =dialog_message('No yerr. 0.1*y is used as yerr.',dialog_parent=self.tlb)
       yerr=0.1*ydat
    endif
    ;make sure that there's no 0's in yerr
    index = where(yerr eq 0.0,count,complement=index1,ncomplement=count1)
    if count ne 0 then begin
        if count1 eq 0 then err=0.01 $
        else err=min(yerr[index1])
        yerr[index] = err
    endif
    opan=obj_new('opan',mydata=ydat,error=yerr,xvalues=xdat,yvalues=0,group_leader=self.tlb)
end