;+
; NAME:
;       Graphical_Masking
;PD Dr. Philip Tregenna-Piggott,
;Laboratory for Neutron Scattering,
;ETHZ and Paul-Scherrer Institute,
;CH-5232 Villigen PSI,
;Switzerland.
;
;Tel. :+41 56 310 54 05
;Fax. :+41 56 310 29 39
;Email:philip.tregenna@psi.ch
;
;Based on Programs written by:
;       Robert M. Dimeo, Ph.D.
;     NIST Center for Neutron Research
;       100 Bureau Drive
;     Gaithersburg, MD 20899
;       Phone: (301) 975-8135
;       E-mail: robert.dimeo@nist.gov
;       http://www.ncnr.nist.gov/staff/dimeo
;
; CATEGORY:
;
;       Focus Utilities
;
; CALLING SEQUENCE:
;
;      Graphical_Masking
;
;
; INPUT FIELDS:
;
;
; EXAMPLES FOR POTENTIALS (must use IDL syntax as shown below):
;
;
; DISCLAIMER
;
;     This software is provided as is without any warranty whatsoever.
;     Permission to use, copy, modify, and distribute modified or
;     unmodified copies is granted, provided this disclaimer
;     is included unchanged.
;
; MODIFICATION HISTORY:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;pro gmCleanup,tlb
;widget_control,tlb,get_uvalue = pState
;ptr_free,(*pState).evalsPtr,(*pState).probPtr,(*pState).xPtr,(*pState).vPtr
;wdelete,(*pState).winPix
;ptr_free,pState
;end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro GMCancel,event
widget_control,event.top,get_uvalue = pState
detarr=(*pState).detarr
wherezero=where(detarr eq 0,count)
if count ne 0 then begin
detarr[wherezero]=1
(*pState).detarr=detarr
endif
widget_control,event.top,/destroy
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro GMAccept,event
widget_control,event.top,/destroy
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro GMReset,event
widget_control,event.top,get_uvalue = pState
detarr=(*pState).detarr
wherezero=where(detarr eq 0,count)
if count eq 0 then return
detarr[wherezero]=1
(*pState).detarr=detarr
widget_control,(*pState).omit,set_value = ''
widget_control,(*pState).remain,set_value = n_elements(detarr)
      wset,(*pState).winPix
      gmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro gmPlot,event
widget_control,event.top,get_uvalue = pState
x = (*pState).x
y = (*pState).y
detarr=(*pState).detarr
x=x[where(detarr eq 1)]
y=y[where(detarr eq 1)]
if (*pState).autoscale eq 1 then begin
  xlo = min(x) & xhi = max(x)
  ylo = min(y) & yhi = max(y)
  (*pState).xrange = [xlo,xhi]
  (*pState).yrange = [ylo,yhi]
endif

widget_control,(*pState).XAxis,get_value = val
if val eq 0 then begin
xtitle = '2 Theta / degrees'
title = 'Intensity vs. 2 Theta'
endif
if val eq 1 then begin
xtitle = 'Sin(Theta)/Lambda'
title = 'Intensity vs. Sin(Theta)/Lambda'
endif
if val eq 2 then begin
xtitle = 'Detector Number'
title = 'Intensity vs. Detector Number'
endif


plot,x,y,xrange = (*pState).xrange,yrange = (*pState).yrange, $
     xstyle = 1,ystyle = 1,xtitle = xtitle, $
     ytitle = 'Intensity', title = title,linestyle = 0, $
     thick = 3.0, psym=-4, symsize=1.0

end
;widget_control,(*pState).info,set_value = strout
;
;wset,(*pState).winPix
;sePlot,event
;wset,(*pState).winVis
;device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro gmdraw,event
widget_control,event.top,get_uvalue = pState
widget_control,(*pState).action,get_value = val
if val eq 0 then begin
case event.type of
0:  begin       ; button press
      (*pState).mouse = event.press
      if (*pState).mouse eq 4 then begin
        (*pState).autoscale = 1

        wset,(*pState).winPix
        gmPlot,event
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      endif
      if (*pState).mouse eq 1 then begin
        (*pState).xbox[0] = event.x
        (*pState).ybox[0] = event.y
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
        empty
        (*pState).autoscale = 0
        widget_control,(*pState).win,/draw_motion_events
      endif
    end
1:  begin    ; button release
     if (*pState).mouse eq 1 then begin
      xll = (*pState).xbox[0] < (*pState).xbox[1]
      yll = (*pState).ybox[0] < (*pState).ybox[1]
      w = abs((*pState).xbox[1] - (*pState).xbox[0])
      h = abs((*pState).ybox[1] - (*pState).ybox[0])
      xur = xll + w
      yur = yll + h
      ll = convert_coord(xll,yll,/device,/to_data)
      ur = convert_coord(xur,yur,/device,/to_data)
      (*pState).xrange = [ll[0],ur[0]]
      (*pState).yrange = [ll[1],ur[1]]
      wset,(*pState).winPix
      gmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
     if (*pState).mouse eq 4 then begin
      wset,(*pState).winPix
      gmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
    end
2:  begin    ; mouse motion
      if (*pState).mouse eq 1 then begin
       (*pState).xbox[1] = event.x
       (*pState).ybox[1] = event.y
       xc = [(*pState).xbox[0],event.x,event.x,$
             (*pState).xbox[0],$
             (*pState).xbox[0]]
       yc = [(*pState).ybox[0],(*pState).ybox[0],$
             event.y,event.y,$
             (*pState).ybox[0]]
       wset,(*pState).winVis
       device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
       plots,xc,yc,/device
       empty
      endif
    end
else:
endcase
return
endif
if val eq 1 then begin
case event.type of
0:  begin       ; button press
      (*pState).mouse = event.press
      if (*pState).mouse eq 4 then begin
        (*pState).autoscale = 1

        wset,(*pState).winPix
        gmPlot,event
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      endif
      if (*pState).mouse eq 1 then begin
        (*pState).xbox[0] = event.x
        (*pState).ybox[0] = event.y
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
        empty
        (*pState).autoscale = 0
        widget_control,(*pState).win,/draw_motion_events
      endif
    end
1:  begin    ; button release
     if (*pState).mouse eq 1 then begin
      xll = (*pState).xbox[0] < (*pState).xbox[1]
      yll = (*pState).ybox[0] < (*pState).ybox[1]
      w = abs((*pState).xbox[1] - (*pState).xbox[0])
      h = abs((*pState).ybox[1] - (*pState).ybox[0])
      xur = xll + w
      yur = yll + h
      ll = convert_coord(xll,yll,/device,/to_data)
      ur = convert_coord(xur,yur,/device,/to_data)
      ;
      ;Here we add some new code
      (*pState).xrangedel = [ll[0],ur[0]]
      (*pState).yrangedel = [ll[1],ur[1]]
      ;print, 'xrange:',(*pState).xrange
      ;print, 'yrange:',(*pState).yrange
      ;print,''
      ;print, 'xrangedel:',(*pState).xrangedel
      ;print, 'yrangedel:',(*pState).yrangedel
      GMremove_detectors,event

      wset,(*pState).winPix
      gmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
     if (*pState).mouse eq 4 then begin
      wset,(*pState).winPix
      gmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
    end
2:  begin    ; mouse motion
      if (*pState).mouse eq 1 then begin
       (*pState).xbox[1] = event.x
       (*pState).ybox[1] = event.y
       xc = [(*pState).xbox[0],event.x,event.x,$
             (*pState).xbox[0],$
             (*pState).xbox[0]]
       yc = [(*pState).ybox[0],(*pState).ybox[0],$
             event.y,event.y,$
             (*pState).ybox[0]]
       wset,(*pState).winVis
       device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
       plots,xc,yc,/device
       empty
      endif
    end
else:
endcase
return
endif
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro GMremove_detectors,event
widget_control,event.top,get_uvalue = pState
xrangedel=(*pState).xrangedel
yrangedel=(*pState).yrangedel
detarr=(*pState).detarr
x=(*pState).x
y=(*pState).y
;print,'x =',x
;filter the data first by the x range
;print,''
;print,'xrangedel[0]',xrangedel[0]
;print,'xrangedel[1]',xrangedel[1]
;print,''
;print,'yrangedel[0]',yrangedel[0]
;print,'yrangedel[1]',yrangedel[1]
whereinxrange=where(x ge xrangedel[0],count)
if count eq 0 then return
subscriptref=whereinxrange
x_1stcull=x[whereinxrange]
y_1stcull=y[whereinxrange]
;print,'y_1stcull',y_1stcull
whereinxrange=where(x_1stcull lt xrangedel[1],count)
if count eq 0 then return
subscriptref=subscriptref[0]+whereinxrange
x_2ndcull=x_1stcull[whereinxrange]
y_2ndcull=y_1stcull[whereinxrange]
;print,'x_2ndcull',x_2ndcull
;print,'y_2ndcull',y_2ndcull
;
;now filter the data by the y range
whereinyrange=where(y_2ndcull ge yrangedel[0],count)
if count eq 0 then return
subscriptref1=subscriptref[0]+whereinyrange
;print,'subscriptref1 = ',subscriptref1
whereinyrange=where(y_2ndcull le yrangedel[1],count)
if count eq 0 then return
subscriptref2=subscriptref[0]+whereinyrange
;print,'subscriptref2 = ',subscriptref2
uni1=1+bytarr(n_elements(subscriptref1))
uni2=1+bytarr(n_elements(subscriptref2))
mat1=uni1#subscriptref2
mat2=subscriptref1#uni2
mat3=mat1-mat2
;print,'mat3 =',mat3
wherezero=where(mat3 eq 0,count)
if count eq 0 then return
;print,'mat1[wherezero] =',mat1[wherezero]
;print,'mat2[wherezero] =',mat2[wherezero]
;subscriptref3=subscriptref1 eq subscriptref2
;print,'subscriptref3 = ',subscriptref3
;print,'y[subscriptref3] =',y[subscriptref3]
remdet=mat1[wherezero]
;print,'remdet =',remdet
detarr[remdet]=0
(*pState).detarr=detarr

wherezero=where(detarr eq 0,count)
num=n_elements(y)-count
if num eq 0 then begin
GMReset,event
return
endif
if count ne 0 then begin
;print,wherezero
wherezero+=1
;print,wherezero
n=n_elements(wherezero)
omit=string(wherezero[0])
for i=1,n-1 do begin
omit=[omit,','+strtrim(string(wherezero[i]),2)]
endfor
;help,omit
widget_control,(*pState).omit,set_value = strtrim(omit,2)
endif
widget_control,(*pState).remain,set_value = num
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro gmEvents,event
widget_control,event.top,get_uvalue = pState
if event.id eq (*pState).Action then return
if event.id eq (*pState).XAxis then begin
widget_control,(*pState).XAxis,get_value = val
if val eq 0 then (*pState).x = (*pState).twotheta
if val eq 1 then (*pState).x = (*pState).sintholam
if val eq 2 then (*pState).x = (*pState).xdet
x=(*pState).x
y= (*pState).y
  xlo = min(x) & xhi = max(x)
  ylo = min(y) & yhi = max(y)
  (*pState).xrange = [xlo,xhi]
  (*pState).yrange = [ylo,yhi]
         wset,(*pState).winPix
   gmplot,event
   wset,(*pState).winVis
   device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
   return
endif
; Handle the resizing events
ctrlgeom = widget_info((*pState).base,/geometry)
tlbgeom = widget_info(event.top,/geometry)
xsize = event.x
ysize = event.y

newxsize = xsize-ctrlgeom.xsize
newysize = ysize

widget_control,(*pState).win,draw_xsize = newxsize, $
               draw_ysize = newysize
wdelete,(*pState).winPix
window,/free,/pixmap,xsize = newxsize,ysize = newysize
(*pState).winPix = !d.window

wset,(*pState).winPix
gmplot,event
wset,(*pState).winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function Graphical_Masking,data,twotheta,lambda,group_leader=group_leader
loadct,0,/silent
if n_elements(group_leader) eq 0 then begin
  tlb = widget_base(/row,title = 'Graphical Masking Utility')
endif else begin
  tlb = widget_base(group_leader = group_leader, /row,/modal, $
        title = 'Graphical Masking Utility')
endelse

;loadct,0,/silent
; Widget definition module
;tlb = widget_base(/row,title = 'Graphical Masking Utility', $
;      /tlb_size_events,mbar = bar)


;sum over the time channels
SumData=total(Data,1)
;x is a vector of the phi values
x=twotheta
sintholam=sin((!dtor*twotheta/2.0)/lambda[0])
;print,sintholam
xdet=1+findgen(n_elements(x))
xlo=min(x)
xhi=max(x)
nx=n_elements(SumData)
ylo=min(SumData)
yhi=max(SumData)
y=SumData
detarr=1+intarr(n_elements(y))
;ny=n_elements(y)
;remdet=[0]

;if !d.name eq 'WIN' then begin
;  thisFont = "Comic Sans MS*16"
;  bigFont = "Comic Sans MS*20*Bold"
;;endif else begin
;  thisFont = -1
;  bigFont = -1
;endelse

base = widget_base(tlb,/row)
;newBase = widget_base(base,/row,/align_center)
thisbase = widget_base(base,/col)


Action_Types = ['Zoom',$
              'Omit']
Action = cw_bgroup(thisbase,Action_Types,/col,/exclusive,label_top='Mouse Action', $
           set_value = 0,/return_index,/no_release, /frame)

XAxis_Types = ['2 Theta',$
              'Sin(theta)/Lambda', $
              'Detector Number']
XAxis = cw_bgroup(thisbase,XAxis_Types,/col,/exclusive,label_top='X Axis', $
           set_value = 0,/return_index,/no_release, /frame)

omit = cw_field(thisbase,/col,title = 'Detectors to be omitted',/string, $
      /noedit)
remain = cw_field(thisbase,/col,value=n_elements(y),title = 'Number of Detectors Remaining',/string, $
      /noedit)
thisbase1 = widget_base(thisbase,/align_center,/row)

undo = widget_button(thisbase1,value = 'Reset',event_pro = 'GMReset')
;thisbase2 = widget_base(thisbase,/row)

void = widget_button(thisbase1,value = 'Cancel',event_pro = 'GMCancel')
void = widget_button(thisbase1,value = 'Accept',event_pro = 'GMAccept')

xsize = 500 & ysize = 400
win = widget_draw(base,xsize = xsize,ysize = ysize,/button_events, $
      event_pro = 'gmdraw')

geom = widget_info(base,/geometry)
newysize = geom.ysize
widget_control,win,draw_ysize = newysize
widget_control,tlb,/realize


widget_control,win,get_value = winVis
window,/free,/pixmap,xsize = xsize,ysize = newysize
winPix = !d.window



state = {winVis:winVis, $
         winPix:winPix, $
         autoscale:1, $
         xrange:[xlo,xhi], $
         yrange:[ylo,yhi], $
         xrangedel:[0.0,1.0], $
         yrangedel:[0.0,1.0], $
         xbox:[xlo,xhi], $
         ybox:[ylo,yhi], $
         mouse:0B, $
         action:action, $
         win:win, $
         xlo:xlo, $
         xhi:xhi, $
         ylo:ylo, $
         yhi:yhi, $
         XAxis:XAxis, $
         x:x, $
         twotheta:twotheta, $
         sintholam:sintholam, $
         xdet:xdet, $
         y:y, $
         detarr:detarr, $
         base:base, $
         omit:omit, $
         remain:remain $
         }


pState = ptr_new(state,/no_copy)
widget_control,tlb,set_uvalue = pState

plot,x,y,xrange = (*pState).xrange,yrange = (*pState).yrange, $
     xstyle = 1,ystyle = 1,xtitle = '2 Theta / degrees', $
     ytitle = 'Intensity', title = 'Intensity vs. 2 Theta',linestyle = 0, $
     thick = 3.0, psym=-4, symsize=1.0
wset,(*pState).winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]


xmanager,'Graphical_Masking',tlb,event_handler = 'gmEvents'

detarr=(*pState).detarr
wdelete,(*pState).winPix
ptr_free,pState
return,detarr
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;