; NAME:      scroll_spectra_Masking
;PD Dr. Philip Tregenna-Piggott,
;Laboratory for Neutron Scattering,
;ETHZ and Paul-Scherrer Institute,
;CH-5232 Villigen PSI,
;Switzerland.
;
;Tel. :+41 56 310 54 05
;Fax. :+41 56 310 29 39
;Email:philip.tregenna@psi.ch
;
;Based on Programs written by:
;       Robert M. Dimeo, Ph.D.
;     NIST Center for Neutron Research
;       100 Bureau Drive
;     Gaithersburg, MD 20899
;       Phone: (301) 975-8135
;       E-mail: robert.dimeo@nist.gov
;       http://www.ncnr.nist.gov/staff/dimeo
;
; CATEGORY:
;
;       Focus Utilities
;
; CALLING SEQUENCE:
;
;      Scroll_Spectra_Masking
;
;
; INPUT FIELDS:
;
;
; EXAMPLES FOR POTENTIALS (must use IDL syntax as shown below):
;
;
; DISCLAIMER
;
;     This software is provided as is without any warranty whatsoever.
;     Permission to use, copy, modify, and distribute modified or
;     unmodified copies is granted, provided this disclaimer
;     is included unchanged.
;
; MODIFICATION HISTORY:
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro SSMCancel,event
widget_control,event.top,get_uvalue = pState
detarr=(*pState).detarr
wherezero=where(detarr eq 0,count)
if count ne 0 then begin
detarr[wherezero]=1
(*pState).detarr=detarr
endif
widget_control,event.top,/destroy
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro SSMAccept,event
widget_control,event.top,/destroy
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro SSMReset,event
widget_control,event.top,get_uvalue = pState
detarr=(*pState).detarr
wherezero=where(detarr eq 0,count)
if count eq 0 then return
detarr[wherezero]=1
(*pState).detarr=detarr
widget_control,(*pState).omit,set_value = ''
widget_control,(*pState).remain,set_value = n_elements(detarr)
      wset,(*pState).winPix
      ssmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ssmdisplay,event
widget_control,event.top,get_uvalue = pState
; This method plots the data to a pixmap then to the screen.
wset,(*pState).winPix
ssmPlot,event
wset,(*pState).winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
widget_control,(*pState).win,draw_motion_events = 0
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ssmPlot,event
widget_control,event.top,get_uvalue = pState
data=(*pState).data
error=(*pState).error
x = (*pState).x
y = (*pState).y
detnum = (*pState).detnum
detarr = (*pState).detarr
widget_control,(*pState).grpSlider,get_value = val
widget_control,(*pState).twotheta,set_value = detnum[val-1]
;val=fix(val[0])
;help,detarr
;help,val
if detarr[val-1] eq 1 then widget_control,(*pState).Remove,set_value = 0
if detarr[val-1] eq 0 then widget_control,(*pState).Remove,set_value = 1
;if (size(*self.dataPtr))[0] eq 1 then val = 1
;if val gt (size(*self.dataPtr))[2] then return
;print,'val =',val
;print,'(size(*self.dataPtr))[2] =',(size(*self.dataPtr))[2]
;val = fix(val[0])
z = reform(data[*,val-1])
zerr = reform(error[*,val-1])
;print,'x =',x
;print,''
;print,'y =',y
;print,''
;print,'z =',z
;print,''
if (*pState).autoscale eq 1 then begin
  (*pState).xrange = [min(x),max(x)]
  dz = 0.1*(max(z)-min(z))
  (*pState).yrange = [min(z)-max(zerr)-dz, $
                 max(z)+max(zerr)+dz]
endif

plot,x,z,xrange = (*pState).xrange,yrange = (*pState).yrange, $
     xstyle = 1,ystyle = 1,xtitle = 'Time / microseconds', $
     ytitle = 'Intensity', title = 'Intensity vs. Time',linestyle = 0, $
     thick = 3.0
errplot,x,z-zerr,z+zerr,width = 0.0

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ssmdraw,event
widget_control,event.top,get_uvalue = pState
case event.type of
0:  begin       ; button press
      (*pState).mouse = event.press
      if (*pState).mouse eq 4 then begin
        (*pState).autoscale = 1

        wset,(*pState).winPix
        ssmPlot,event
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      endif
      if (*pState).mouse eq 1 then begin
        (*pState).xbox[0] = event.x
        (*pState).ybox[0] = event.y
        wset,(*pState).winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
        empty
        (*pState).autoscale = 0
        widget_control,(*pState).win,/draw_motion_events
      endif
    end
1:  begin    ; button release
     if (*pState).mouse eq 1 then begin
      xll = (*pState).xbox[0] < (*pState).xbox[1]
      yll = (*pState).ybox[0] < (*pState).ybox[1]
      w = abs((*pState).xbox[1] - (*pState).xbox[0])
      h = abs((*pState).ybox[1] - (*pState).ybox[0])
      xur = xll + w
      yur = yll + h
      ll = convert_coord(xll,yll,/device,/to_data)
      ur = convert_coord(xur,yur,/device,/to_data)
      (*pState).xrange = [ll[0],ur[0]]
      (*pState).yrange = [ll[1],ur[1]]
      wset,(*pState).winPix
      ssmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
     if (*pState).mouse eq 4 then begin
      wset,(*pState).winPix
      ssmplot,event
      wset,(*pState).winVis
      device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
      (*pState).mouse = 0B
      widget_control,(*pState).win,draw_motion_events = 0
     endif
    end
2:  begin    ; mouse motion
      if (*pState).mouse eq 1 then begin
       (*pState).xbox[1] = event.x
       (*pState).ybox[1] = event.y
       xc = [(*pState).xbox[0],event.x,event.x,$
             (*pState).xbox[0],$
             (*pState).xbox[0]]
       yc = [(*pState).ybox[0],(*pState).ybox[0],$
             event.y,event.y,$
             (*pState).ybox[0]]
       wset,(*pState).winVis
       device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]
       plots,xc,yc,/device
       empty
      endif
    end
else:
endcase
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro SSMremove_detectors,event
widget_control,event.top,get_uvalue = pState
detarr=(*pState).detarr
widget_control,(*pState).grpSlider,get_value = grpSlider
grpSlider-=1
widget_control,(*pState).Remove,get_value = Remove
;print,'remove =',Remove
Remove=Remove[0]
if Remove eq 1 then detarr[grpSlider]=0
if Remove eq 0 then detarr[grpSlider]=1
(*pState).detarr=detarr
wherezero=where(detarr eq 0,count)
num=n_elements(detarr)-count
if num eq 0 then begin
SSMReset,event
return
endif
if count ne 0 then begin
;print,wherezero
wherezero+=1
;print,wherezero
n=n_elements(wherezero)
omit=string(wherezero[0])
for i=1,n-1 do begin
omit=[omit,','+strtrim(string(wherezero[i]),2)]
endfor
;help,omit
widget_control,(*pState).omit,set_value = strtrim(omit,2)
endif
widget_control,(*pState).remain,set_value = num
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro ssmEvents,event
uname=widget_info(event.id,/uname)
case uname of
'Remove_Types':    $
begin
SSMremove_detectors,event
end
else:
endcase
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function Scroll_Spectra_Masking,data,error,detnum,x,group_leader=group_leader
loadct,0,/silent
if n_elements(group_leader) eq 0 then begin
  tlb = widget_base(/row,title = 'Scroll Spectra Masking Utility')
endif else begin
  tlb = widget_base(group_leader = group_leader, /row,/modal, $
        title = 'Scroll Spectra Masking Utility')
endelse

nx=n_elements(x)
ndet=n_elements(detnum)

z = data[*,0]
zerr = error[*,0]
xlo=min(x)
xhi=max(x)
dz = 0.1*(max(z)-min(z))
ylo=min(z)-max(zerr)-dz
yhi= max(z)+max(zerr)+dz
y=detnum[0]

  xrange = [xlo,xhi]
  yrange = [ylo,yhi]

detarr=1+intarr(n_elements(detnum))

base = widget_base(tlb,/row)
;newBase = widget_base(base,/row,/align_center)
thisbase = widget_base(base,/col)

Remove_Types = ['Mark For Deletion']
;range=1
Remove = cw_bgroup(thisbase,Remove_Types,/col,/nonexclusive,$
           set_value = 0,/return_index, /frame,$
           uname = 'Remove_Types')


omit = cw_field(thisbase,/col,title = 'Detectors to be Omitted',/string, $
      /noedit)
remain = cw_field(thisbase,/col,value=ndet,title = 'Number of Detectors Remaining',/string, $
      /noedit)
twotheta = cw_field(thisbase,/col,value=detnum[0],title = 'Two*Theta For Current Group',/string, $
      /noedit)
;thisbase1 = widget_base(thisbase,/align_center,/col)
;thisbase1 = widget_base(thisbase,/align_center)
void = widget_button(thisbase,value = 'Reset',event_pro = 'SSMReset')
;thisbase2 = widget_base(thisbase,/align_center)
void = widget_button(thisbase,value = 'Cancel',event_pro = 'SSMCancel')
;thisbase3 = widget_base(thisbase,/align_center)
void = widget_button(thisbase,value = 'Accept',event_pro = 'SSMAccept')

;void = widget_button(thisbase1,value = 'Remove',event_pro = 'SSMremove_detectors')

;thisbase2 = widget_base(thisbase,/align_center,/row)


;xsize = 500 & ysize = 400
xsize = 500 & ysize = 400

plotBase = widget_base(base,/col)

win = widget_draw(plotBase,xsize = xsize,ysize = ysize, $
           /button_events,event_pro = 'ssmdraw')
grpSlider = widget_slider(plotBase,minimum = 1,maximum = ndet, $
          event_pro = 'ssmdisplay')



geom = widget_info(base,/geometry)
newysize = geom.ysize
widget_control,win,draw_ysize = newysize
widget_control,tlb,/realize


widget_control,win,get_value = winVis
window,/free,/pixmap,xsize = xsize,ysize = newysize
winPix = !d.window



state = {winVis:winVis, $
         winPix:winPix, $
         autoscale:1, $
         xbox:[xlo,xhi], $
         ybox:[ylo,yhi], $
         mouse:0B, $
         win:win, $
         xrange:[xlo,xhi], $
         yrange:[ylo,yhi], $
         data:data, $
         error:error, $
         x:x, $
         y:y, $
         detarr:detarr, $
         twotheta:twotheta, $
         detnum:detnum, $
         base:base, $
         omit:omit, $
         remain:remain, $
         grpSlider:grpSlider, $
         Remove:Remove $
         }


pState = ptr_new(state,/no_copy)
widget_control,tlb,set_uvalue = pState

plot,x,z,xrange = (*pState).xrange,yrange = (*pState).yrange, $
     xstyle = 1,ystyle = 1,xtitle = 'Time / microseconds', $
     ytitle = 'Intensity', title = 'Intensity vs. Time',linestyle = 0, $
     thick = 3.0
errplot,x,z-zerr,z+zerr,width = 0.0
wset,(*pState).winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,(*pState).winPix]


xmanager,'Scroll_Spectra_Masking',tlb,event_handler = 'ssmEvents'

detarr=(*pState).detarr
wdelete,(*pState).winPix
ptr_free,pState
return,detarr
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;