@napi45

; $Id: focusreduction__define.pro,v 1.7 2005/10/11 19:04:06 tregenna Exp $
;###############################################################################
;
; NAME:
;  FOCUSREDUCTION__DEFINE
;
; PURPOSE:
;  See description below.
;
; CATEGORY:
;  DAVE, focus, data reduction
;
; AUTHOR:
;
;PD Dr. Philip Tregenna-Piggott,
;Laboratory for Neutron Scattering,
;ETHZ and Paul-Scherrer Institute,
;CH-5232 Villigen PSI,
;Switzerland.
;
;Tel. :+41 56 310 54 05
;Fax. :+41 56 310 29 39
;Email:philip.tregenna@psi.ch
;
;Based on the Program HFBSREDUCTION__DEFINE by:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
;+
; NAME:
;       FOCUSREDUCTION__DEFINE
;
; PURPOSE:
;
;       This object widget program performs the data reduction steps for
;   data sets collected on the Direct Geometry TOF Spectrometer FOCUS, at SINQ, PSI.
;   This program is meant to run in the DAVE environment.
;
; AUTHOR:
;
;PD Dr. Philip Tregenna-Piggott,
;Laboratory for Neutron Scattering,
;ETHZ and Paul-Scherrer Institute,
;CH-5232 Villigen PSI,
;Switzerland.
;
;Tel. :+41 56 310 54 05
;Fax. :+41 56 310 29 39
;Email:philip.tregenna@psi.ch
;
; CATEGORY:
;
;       Objects, widgets, data reduction, DAVE software
;
; CALLING SEQUENCE:
;
;       object = obj_new('focusreduction')
;
;
; INPUT PARAMETERS:
;
;       NONE
;
; INPUT KEYWORDS:
;
;   GROUP_LEADER   - Parent widget of FOCUSREDUCTION object widget
;   NOTIFYID    - Vector of TOP and ID of calling widget
;   WORKDIR      - working directory, where DAVE and/or CON files will be put
;   DATDIR      - data directory, directory of raw data
;
; REQUIRED PROGRAMS:
;
;        DREBIN.PRO
;    OPAN_Q_REBIN_WIDGET.PRO
;   opan_rebin_FOCUS_Energy_widget.pro
;   opan_time_rebin_widget.pro
;   focusDetGroupWidget.pro
;   Graphical_Masking.pro
;   Scroll_Spectra_Masking.pro
;   PlotDiffractionData.pro
;
;
; COMMON BLOCKS:
;
;       NONE
;
; RESTRICTIONS
;
;       NONE
;
; OBJECT METHODS:
;
; There are no explicitly private object methods in IDL but the
; methods used in this class are divided into PUBLIC and PRIVATE
; to indicate which ones should be used by users who wish to run
; the program from the command line, for instance.  Execution of
; the program's controls from the command line is described in the
; example below.
;
; PUBLIC OBJECT PROCEDURE METHODS:
;
;
;
; PRIVATE OBJECT PROCEDURE METHODS:
;
;
; EXAMPLE
;
;
; MODIFICATION HISTORY:
;
;       Written by Rob Dimeo, October 23, 2002.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusReductionCleanup,tlb
widget_control,tlb,get_uvalue = self
if obj_valid(self) then begin
  s = size(self->getNotifyIds())
  if s[0] ne 0 then begin
    if s[0] eq 1 then count = 0 else count = s[2]-1
    for j = 0,count do begin
      focusInfo = {focusReductionEvent,$
                            ID:(self->getNotifyIds())[0,j],$
                            Top:(self->getNotifyIds())[1,j],$
                            Handler:0l,$
                            daveFiles:self->getDaveFiles()}
      if widget_info((self->getNotifyIds())[0,j],/valid_id) then begin $
        widget_control,(self->getNotifyIds())[0,j],send_event = focusInfo
      endif
    endfor
  endif else begin
    ;obj_destroy,self
  endelse
endif
obj_destroy,self
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function focusreduction::getDaveFiles
return,self.daveFiles
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::cleanup
; First let's restore the colors
tvlct,*self.rPtr,*self.gPtr,*self.bPtr
; Now free up the pointers
ptr_free,self.odataPtr,self.oerrorPtr
ptr_free,self.oxvalsPtr,self.oyvalsPtr
ptr_free,self.dataPtr,self.errorPtr
ptr_free,self.xvalsPtr,self.yvalsPtr
ptr_free,self.notifyIdPtr;,self.daveFiles
ptr_free,self.rPtr,self.bPtr,self.selDetPtr
ptr_free,self.pgroup_leader,self.sigFilePtr
ptr_free,self.detPtr,self.vanFilePtr,self.bgFilePtr
ptr_free,self.tempPtr,self.treatmentPtr,self.lambda_fieldPtr
ptr_free,self.Mon_tcPtr,self.Mon_cumPtr,self.headerPtr,self.instrumentPtr
ptr_free,self.titlePtr,self.gPtr,self.tof_corr_structurePtr,self.good_detPtr
ptr_free,self.Sample_NamePtr

; Delete any pixmap windows
wdelete,self.winPix
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::quit,event = event
; Widget, destroy thyself
widget_control,self.tlb,/destroy
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::selSigFiles,event = event
files = dialog_pickfile(dialog_parent = event.top,/read, $
                               filter = ['*.hdf'],$
                               title = 'Select raw signal data file(s)',$
                               path = self.datDir, $
                               /multiple_files)
nfiles = n_elements(files)
if (1.0*total(file_test(files,/regular)) ne nfiles) then begin
  void = dialog_message(dialog_parent = event.top, $
   'At least one of the selected files is invalid')
  return
endif
*self.sigFilePtr = files
nfiles = n_elements(files)
dispFiles = strarr(nfiles)
for i = 0,nfiles-1 do begin
  text = files[i]
  textLen = strlen(text)
  pos = textLen-16
  dispFiles[i] = strmid(text,pos)
endfor
widget_control,self.fileText,set_value = dispFiles
;
;
;
  filename_result = strsplit(files[0],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)

  widget_control, self.runGroup, get_value=runGroup
;if only one file selected then switch to single sample runs
if rungroup eq 1 and nfiles eq 1 then begin
rungroup=0
  widget_control, self.runGroup, set_value=runGroup
endif
      if runGroup eq 1 then filename+='++'   ;set to 'sum runs'
widget_control, self.stem1, set_value=filename
widget_control, self.diffstem1, set_value=filename

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::selBgFiles,event = event
files = dialog_pickfile(dialog_parent = event.top,/read, $
                               filter = ['*.hdf'],$
                               title = 'Select background data file(s)',$
                               path = self.datDir,$
                               /multiple_files)
nfiles = n_elements(files)
if (1.0*total(file_test(files,/regular)) ne nfiles) then begin
  void = dialog_message(dialog_parent = event.top, $
   'At least one of the selected files is invalid')
  return
endif
*self.bgFilePtr = files
nfiles = n_elements(files)
dispFiles = strarr(nfiles)
for i = 0,nfiles-1 do begin
  text = files[i]
  textLen = strlen(text)
  pos = textLen-16
  dispFiles[i] = strmid(text,pos)
endfor
widget_control,self.bgText,set_value = dispFiles
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::selVanFile,event = event
files = dialog_pickfile(dialog_parent = event.top,/read, $
                               filter = ['*.hdf'],$
                               title = 'Select vanadium data file',$
                               path = self.datDir)
nfiles = n_elements(files)
if (1.0*total(file_test(files,/regular)) ne nfiles) then begin
  void = dialog_message(dialog_parent = event.top,'The selected file is invalid')
  return
endif
*self.vanFilePtr = files
nfiles = n_elements(files)
dispFiles = strarr(nfiles)
for i = 0,nfiles-1 do begin
  text = files[i]
  textLen = strlen(text)
  pos = textLen-16
  dispFiles[i] = strmid(text,pos)
endfor
widget_control,self.vanText,set_value = dispFiles
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function focusreduction::getnotifyIds
return,*self.notifyIdPtr
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusReductionEvents,event
if dave_set_focus(event) then return
if tag_names(event,/structure_name) eq 'WIDGET_BASE' then begin
  widget_control,event.top,get_uvalue = self
  self->resize,event = event
  return
endif
widget_control,event.id,get_uvalue = cmd
call_method,cmd.method,cmd.object,event = event
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::plotData,event = event
if n_elements(*self.dataPtr) eq 0 then return
widget_control,self.grpSlider,get_value = val
if (size(*self.dataPtr))[0] eq 1 then val = 1
if val gt (size(*self.dataPtr))[2] then return
;print,'val =',val
;print,'(size(*self.dataPtr))[2] =',(size(*self.dataPtr))[2]
;val = fix(val[0])
z = reform((*self.dataPtr)[*,val-1])
zerr = reform((*self.errorPtr)[*,val-1])
x = (*self.xvalsPtr)
y = (*self.yvalsPtr)[val-1]
;print,'x =',x
;print,''
;print,'y =',y
;print,''
;print,'z =',z
;print,''
if self.autoscale eq 1 then begin
  self.xrange = [min(x),max(x)]
  dz = 0.1*(max(z)-min(z))
  self.yrange = [min(z)-max(zerr)-dz, $
                 max(z)+max(zerr)+dz]
endif
;print,self.xrange
;print,''
;print,self.yrange
;
plot,x,z,psym = 4,xrange = self.xrange,yrange = self.yrange, $
     xstyle = 1,ystyle = 1,xtitle = self.xlabel, ytitle = self.zlabel
     ;, $
     ;title = (*self.titlePtr)[val-1]
errplot,x,z-zerr,z+zerr,width = 0.0

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::doNothing,event = event
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::ebinning,event = event
widget_control,self.DataOutputTypesGroup,get_value = DataOutputType
; DataOutputType 0 S(Q,w)
; DataOutputType 1 S(theta,w)
; DataOutputType 2 S(theta,t)
if DataOutputType ne 2 then begin
self.xlabel = 'Energy / meV'
endif else begin
self.xlabel = 'Time / microseconds'
endelse
self.units=0
;   is energy binning requested?
widget_control,self.ebin_Group,get_value = thisValue
if thisValue[0] eq 0 then return
if n_elements(*self.dataPtr) eq 0 then return
data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
;help,data
;help,error
widget_control,self.ebin_Group1,get_value = ebin_command
;   modify default(0), use previous(1), modify previous(2)
IF DataOutputType ne 2 then begin
;   modify default(0), use previous(1), modify previous(2)
if ebin_command eq 0 then begin
;
total_bins=n_elements(x)
;print,'total_bins =',total_bins
;if min(x) lt -50.0 then begin
;nb_xlo=-50.0
;xtemp=x(where(x ge min(x)) and where(x lt -50.0))
;total_bins-=n_elements(xtemp)
;x=x[n_elements(xtemp):*]
;print,n_elements(x)
;endif else begin
;nb_xlo=min(x)
;endelse
nb_xlo=min(x)
;print,'total_bins =',total_bins
nb_xhi=-max(x)
b_xlo=nb_xhi
b_xhi=max(x)
xtemp=x[where(x lt nb_xhi)]
;print,nb_xlo
;print,nb_xhi
;
;help,xtemp
;print,xtemp
nb_nbins=n_elements(xtemp)
;
;step=x[nb_nbins]-x[nb_nbins-1]
;b_nbins=long(((b_xhi-b_xlo)/step)+1)
;
b_nbins=fix(total_bins-nb_nbins)
step=(b_xhi-b_xlo)/(b_nbins-1)

;   find interpolated range
    where_x_lt_b_xhi=where(x lt b_xhi-step/100.0,count)
;    print,count
;    print,(nb_nbins+1)
;    print,n_elements(x)
    ix=x[nb_nbins-1:count]
;ix=x[(nb_nbins+1):*]
ix_plus=ix[1:*]
ix_minus=ix[0:(n_elements(ix)-1)]
ix_diff=ix_plus-ix_minus
;print,ix_diff
u=1+bytarr(n_elements(ix_diff))
ustep=step*u
where_int=where(ix_diff gt ustep,count)
;print,'where_int =',where_int
;print,'where_int + 1 =',where_int+1
if count ge 1 then begin
;help,ix
ix=[ix[0],ix[(where_int+1)]]
;help,ix
i_nbins=count+1
i_xlo=b_xlo
i_xhi=max(ix)
endif else begin
i_nbins=0
i_xlo=b_xlo
i_xhi=b_xlo
endelse
range=1
unit=0
;help,step
;ebin = opan_rebin_widget(x,group_leader = event.top)

;help,x
;print,x

ebin = opan_rebin_FOCUS_Energy_widget(x,nb_xlo=nb_xlo,nb_xhi=nb_xhi,nb_nbins=nb_nbins, $
                    b_xlo=b_xlo,b_xhi=b_xhi,b_nbins=b_nbins,step=step, $
                    i_xlo=i_xlo,i_xhi=i_xhi,i_nbins=i_nbins, $
                    range=range,unit=unit,group_leader = event.top)
;ebin = {nb_xlo:nb_xlo,nb_xhi:nb_xhi,nb_nbins:nb_nbins,b_xlo:b_xlo,b_xhi:b_xhi,b_nbins:b_nbins, $
;       i_xlo:i_xlo,i_xhi:i_xhi,i_nbins:i_nbins,step:step,  $
;       range:range, unit:unit, cancel:cancel}
nb_xlo=ebin.nb_xlo
nb_xhi=ebin.nb_xhi
nb_nbins=ebin.nb_nbins
b_xlo=ebin.b_xlo
b_xhi=ebin.b_xhi
b_nbins=ebin.b_nbins
step=ebin.step
i_xlo=ebin.i_xlo
i_xhi=ebin.i_xhi
i_nbins=ebin.i_nbins
range=ebin.range
unit=ebin.unit
cancel=ebin.cancel
;Now write the contents to a file
filename = 'ebinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,nb_xlo
printf,lun,nb_xhi
printf,lun,nb_nbins
printf,lun,b_xlo
printf,lun,b_xhi
printf,lun,b_nbins
printf,lun,step
printf,lun,i_xlo
printf,lun,i_xhi
printf,lun,i_nbins
printf,lun,range
printf,lun,unit
free_lun,lun,/force
;
;print,unit
    if cancel eq 1 then return
       if unit eq 1 then begin
x*=8.06554097
self.units=1
self.xlabel = 'Energy / Wavenumbers'
       endif
if unit eq 2 then begin
x*=241.7989052
self.units=2
self.xlabel = 'Energy / GHz'
    endif
endif
;
;   option: modify previous values
if ebin_command eq 1 then begin
;check to see whether file exists
filename = 'ebinParameters.txt'
if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,nb_xlo
readf,lun,nb_xhi
readf,lun,nb_nbins
readf,lun,b_xlo
readf,lun,b_xhi
readf,lun,b_nbins
readf,lun,step
readf,lun,i_xlo
readf,lun,i_xhi
readf,lun,i_nbins
readf,lun,range
readf,lun,unit
free_lun,lun,/force
;help,step
;help,x
;print,x
xcall=x
if unit eq 1 then xcall*=8.06554097
if unit eq 2 then xcall*=241.7989052
ebin = opan_rebin_FOCUS_Energy_widget(xcall,nb_xlo=nb_xlo,nb_xhi=nb_xhi,nb_nbins=nb_nbins, $
                    b_xlo=b_xlo,b_xhi=b_xhi,b_nbins=b_nbins,step=step, $
                    i_xlo=i_xlo,i_xhi=i_xhi,i_nbins=i_nbins, $
                    range=range,unit=unit,group_leader = event.top)
;ebin = {nb_xlo:nb_xlo,nb_xhi:nb_xhi,nb_nbins:nb_nbins,b_xlo:b_xlo,b_xhi:b_xhi,b_nbins:b_nbins, $
;       i_xlo:i_xlo,i_xhi:i_xhi,i_nbins:i_nbins,  $
;       range:range, unit:unit, cancel:cancel}
nb_xlo=ebin.nb_xlo
nb_xhi=ebin.nb_xhi
nb_nbins=ebin.nb_nbins
b_xlo=ebin.b_xlo
b_xhi=ebin.b_xhi
b_nbins=ebin.b_nbins
;help,ebin.b_nbins

step=ebin.step
i_xlo=ebin.i_xlo
i_xhi=ebin.i_xhi
i_nbins=ebin.i_nbins
range=ebin.range
unit=ebin.unit
cancel=ebin.cancel
;Now write the contents to a file
filename = 'ebinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,nb_xlo
printf,lun,nb_xhi
printf,lun,nb_nbins
printf,lun,b_xlo
printf,lun,b_xhi
printf,lun,b_nbins
printf,lun,step
printf,lun,i_xlo
printf,lun,i_xhi
printf,lun,i_nbins
printf,lun,range
printf,lun,unit
free_lun,lun,/force
;
;print,ebin.unit
    if cancel eq 1 then return
       if unit eq 1 then begin
x*=8.06554097
self.units=1
self.xlabel = 'Energy / Wavenumbers'
       endif
if unit eq 2 then begin
x*=241.7989052
self.units=2
self.xlabel = 'Energy / GHz'
    endif
endif else begin
message=string('THE FILE ',self.workDir,'ebinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
;help,msg
;msg = 'THE FILE "ebinParameters.txt" HAS NOT BEEN FOUND.  REVERTING TO DEFAULT '
;help,msg
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb
;if the file does not exist then revert to default
total_bins=n_elements(x)
nb_xlo=min(x)
nb_xhi=-max(x)
b_xlo=nb_xhi
b_xhi=max(x)
xtemp=x[where(x lt nb_xhi)]
nb_nbins=n_elements(xtemp)
b_nbins=fix(total_bins-nb_nbins)
step=(b_xhi-b_xlo)/(b_nbins-1)

;   find interpolated range
    where_x_lt_b_xhi=where(x lt b_xhi-step/100.0,count)
    ix=x[nb_nbins-1:count]
ix_plus=ix[1:*]
ix_minus=ix[0:(n_elements(ix)-1)]
ix_diff=ix_plus-ix_minus
u=1+bytarr(n_elements(ix_diff))
ustep=step*u
where_int=where(ix_diff gt ustep,count)
if count ge 1 then begin
ix=[ix[0],ix[(where_int+1)]]
i_nbins=count+1
i_xlo=b_xlo
i_xhi=max(ix)
endif else begin
i_nbins=0
i_xlo=b_xlo
i_xhi=b_xlo
endelse
range=1
unit=0
ebin = opan_rebin_FOCUS_Energy_widget(x,nb_xlo=nb_xlo,nb_xhi=nb_xhi,nb_nbins=nb_nbins, $
                    b_xlo=b_xlo,b_xhi=b_xhi,b_nbins=b_nbins,step=step, $
                    i_xlo=i_xlo,i_xhi=i_xhi,i_nbins=i_nbins, $
                    range=range,unit=unit,group_leader = event.top)
nb_xlo=ebin.nb_xlo
nb_xhi=ebin.nb_xhi
nb_nbins=ebin.nb_nbins
b_xlo=ebin.b_xlo
b_xhi=ebin.b_xhi
b_nbins=ebin.b_nbins
step=ebin.step
i_xlo=ebin.i_xlo
i_xhi=ebin.i_xhi
i_nbins=ebin.i_nbins
range=ebin.range
unit=ebin.unit
cancel=ebin.cancel
;Now write the contents to a file
filename = 'ebinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,nb_xlo
printf,lun,nb_xhi
printf,lun,nb_nbins
printf,lun,b_xlo
printf,lun,b_xhi
printf,lun,b_nbins
printf,lun,step
printf,lun,i_xlo
printf,lun,i_xhi
printf,lun,i_nbins
printf,lun,range
printf,lun,unit
free_lun,lun,/force
;
    if cancel eq 1 then return
       if unit eq 1 then begin
x*=8.06554097
self.units=1
self.xlabel = 'Energy / Wavenumbers'
       endif
if unit eq 2 then begin
x*=241.7989052
self.units=2
self.xlabel = 'Energy / GHz'
    endif
endelse
endif

;
if ebin_command eq 2 then begin

;check to see whether file exists
filename = 'ebinParameters.txt'
if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,nb_xlo
readf,lun,nb_xhi
readf,lun,nb_nbins
readf,lun,b_xlo
readf,lun,b_xhi
readf,lun,b_nbins
readf,lun,step
readf,lun,i_xlo
readf,lun,i_xhi
readf,lun,i_nbins
readf,lun,range
readf,lun,unit
free_lun,lun,/force
;
nb_nbins=fix(nb_nbins)
b_nbins=fix(b_nbins)
i_nbins=fix(i_nbins)
range=fix(range)
unit=fix(unit)
;
       if unit eq 1 then begin
x*=8.06554097
self.units=1
self.xlabel = 'Energy / Wavenumbers'
       endif
if unit eq 2 then begin
x*=241.7989052
self.units=2
self.xlabel = 'Energy / GHz'
    endif
endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'ebinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
;help,msg
;msg = 'THE FILE "ebinParameters.txt" HAS NOT BEEN FOUND.  REVERTING TO DEFAULT '
;help,msg
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb
;if the file does not exist then revert to default
total_bins=n_elements(x)
nb_xlo=min(x)
nb_xhi=-max(x)
b_xlo=nb_xhi
b_xhi=max(x)
xtemp=x[where(x lt nb_xhi)]
nb_nbins=n_elements(xtemp)
b_nbins=fix(total_bins-nb_nbins)
step=(b_xhi-b_xlo)/(b_nbins-1)

;   find interpolated range
    where_x_lt_b_xhi=where(x lt b_xhi-step/100.0,count)
    ix=x[nb_nbins-1:count]
ix_plus=ix[1:*]
ix_minus=ix[0:(n_elements(ix)-1)]
ix_diff=ix_plus-ix_minus
u=1+bytarr(n_elements(ix_diff))
ustep=step*u
where_int=where(ix_diff gt ustep,count)
if count ge 1 then begin
ix=[ix[0],ix[(where_int+1)]]
i_nbins=count+1
i_xlo=b_xlo
i_xhi=max(ix)
endif else begin
i_nbins=0
i_xlo=b_xlo
i_xhi=b_xlo
endelse
range=1
unit=0
ebin = opan_rebin_FOCUS_Energy_widget(x,nb_xlo=nb_xlo,nb_xhi=nb_xhi,nb_nbins=nb_nbins, $
                    b_xlo=b_xlo,b_xhi=b_xhi,b_nbins=b_nbins,step=step, $
                    i_xlo=i_xlo,i_xhi=i_xhi,i_nbins=i_nbins, $
                    range=range,unit=unit,group_leader = event.top)
nb_xlo=ebin.nb_xlo
nb_xhi=ebin.nb_xhi
nb_nbins=ebin.nb_nbins
b_xlo=ebin.b_xlo
b_xhi=ebin.b_xhi
b_nbins=ebin.b_nbins
step=ebin.step
i_xlo=ebin.i_xlo
i_xhi=ebin.i_xhi
i_nbins=ebin.i_nbins
range=ebin.range
unit=ebin.unit
cancel=ebin.cancel
;Now write the contents to a file
filename = 'ebinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,nb_xlo
printf,lun,nb_xhi
printf,lun,nb_nbins
printf,lun,b_xlo
printf,lun,b_xhi
printf,lun,b_nbins
printf,lun,step
printf,lun,i_xlo
printf,lun,i_xhi
printf,lun,i_nbins
printf,lun,range
printf,lun,unit
free_lun,lun,/force
;
    if cancel eq 1 then return
       if unit eq 1 then begin
x*=8.06554097
self.units=1
self.xlabel = 'Energy / Wavenumbers'
       endif
if unit eq 2 then begin
x*=241.7989052
self.units=2
self.xlabel = 'Energy / GHz'
    endif
endelse
endif
;print,ebin
;print,tag_names(ebin)
;datSize = size(data)
;nx = datSize[1]
;if ebin_structure.nbins gt nx then begin
;  strout = 'Number of desired bins cannot exceed the number of input bins'
;  void = dialog_message(dialog_parent = event.top,strout)
;  return
;endif

case range of
0: begin
;   include all data, only a part of which is to be binned
;   find data not to be binned
        where_x_lt_nb_xhi=where(x lt nb_xhi,count)
;       print,where_x_lt_nb_xhi
;       print,n_elements(where_x_lt_nb_xhi)
        if count ge 1 then begin
        xnb=x[where_x_lt_nb_xhi]
        datanb=data[0:n_elements(xnb)-1,*]
        errornb=error[0:n_elements(xnb)-1,*]
        endif
       xlo = b_xlo & xhi = b_xhi
       nbins = b_nbins
       dx = (xhi-xlo)/(nbins-1.0)
       x_out = xlo+dx*dindgen(nbins)
    end
1: begin
;   include only binned data,
       xlo = b_xlo & xhi = b_xhi
       nbins = b_nbins
       dx = (xhi-xlo)/(nbins-1.0)
       x_out = xlo+dx*dindgen(nbins)
    end
2: begin
;   include only binned data, exclude interpolated region
       xlo = i_xhi & xhi = b_xhi
       nbins = b_nbins-i_nbins
       dx = (xhi-xlo)/(nbins-1.0)
       x_out = xlo+dx*dindgen(nbins)
 ;      print,xlo
 ;      print,xhi
end
else:
endcase
;
;
;
;
oxrange = '('+strtrim(string(min(x)),2)+','+strtrim(string(max(x)),2)+')'
xrange = '('+strtrim(string(xlo),2)+','+strtrim(string(xhi),2)+')'
if self.units eq 0 then ustring='Energy Units = meV'
if self.units eq 1 then ustring='Energy Units = wavenumbers'
if self.units eq 2 then ustring='Energy Units = GHz'
treat = ['Data has been rebinned in Energy.', $
        'Old bins:'+strtrim(string(n_elements(x)),2), $
       ustring, $
        'Old Energy-range: '+oxrange, $
        'New bins:'+strtrim(string(nbins),2), $
        'New Energy-range: ',xrange]
*self.treatmentPtr = [*self.treatmentPtr,treat]
;print, nb_nbins
;print, n_elements(xnb)
;print,''
;print, b_nbins
;print, n_elements(xb)

;oxrange = '('+strtrim(string(min(x)),2)+','+strtrim(string(max(x)),2)+')'
;xrange = '('+strtrim(string(xlo),2)+','+strtrim(string(xhi),2)+')'
;treat = ['Data has been rebinned in energy.', $
;        'Old bins:'+strtrim(string(nx),2), $
;        'Old x-range: '+oxrange, $
;        'New bins:'+strtrim(string(nbins),2), $
;        'New x-range: ',xrange]
;*self.treatmentPtr = [*self.treatmentPtr,treat]

; Ok, we've done the error checking...now do the rebinning
;
drebin,x,data,error,x_out,z_out,dz_out, $
    /points,/to_points,err=err,emsg=emsg
;
print,emsg
if err ne 0 then return

if n_elements(datanb) ne 0 then begin
z=[datanb,z_out]
dz=[errornb,dz_out]
x=[xnb,x_out]
endif else begin
z=z_out
dz=dz_out
x=x_out
endelse
;
;
*self.dataPtr=z
*self.errorPtr=dz
*self.xvalsPtr=x

;print,z[800,0]
ENDIF ELSE BEGIN
; the user wishes S(theta,t)
self.units=3
nx=n_elements(x)

if ebin_command eq 0 then begin

tmin=min(x)
tmax=max(x)
ntbins=nx
step=(tmax-tmin)/(ntbins)
time_structure = opan_time_rebin_widget(tmin,tmax,ntbins,step,x,group_leader = event.top)
;out = {tmin:thmin,tmax:thmax,step:step,ntbins:ntbins,cancel:1}
  if time_structure.cancel eq 1 then return
tmin = time_structure.tmin & tmax = time_structure.tmax
ntbins = time_structure.ntbins
step=time_structure.step
;write the contents to a file
filename = 'tbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,tmin
printf,lun,tmax
printf,lun,ntbins
printf,lun,step
free_lun,lun,/force
;
endif

if ebin_command eq 1 then begin

filename = 'tbinParameters.txt'
    if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,tmin
readf,lun,tmax
readf,lun,ntbins
readf,lun,step
free_lun,lun,/force
ntbins=fix(ntbins)


    endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'tbinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb

tmin=min(x)
tmax=max(x)
ntbins=nx
step=(tmax-tmin)/(ntbins)
    endelse
time_structure = opan_time_rebin_widget(tmin,tmax,ntbins,step,x,group_leader = event.top)
;out = {tmin:thmin,tmax:thmax,step:step,ntbins:ntbins,cancel:1}
  if time_structure.cancel eq 1 then return
tmin = time_structure.tmin & tmax = time_structure.tmax
ntbins = time_structure.ntbins
step=time_structure.step
;write the contents to a file
filename = 'tbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,tmin
printf,lun,tmax
printf,lun,ntbins
printf,lun,step
free_lun,lun,/force
;
endif


if ebin_command eq 2 then begin

filename = 'tbinParameters.txt'
    if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,tmin
readf,lun,tmax
readf,lun,ntbins
readf,lun,step
free_lun,lun,/force
ntbins=fix(ntbins)


    endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'tbinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb

tmin=min(x)
tmax=max(x)
ntbins=nx
step=(tmax-tmin)/(ntbins)
time_structure = opan_time_rebin_widget(tmin,tmax,ntbins,step,x,group_leader = event.top)
;out = {tmin:thmin,tmax:thmax,step:step,ntbins:ntbins,cancel:1}
  if time_structure.cancel eq 1 then return
tmin = time_structure.tmin & tmax = time_structure.tmax
ntbins = time_structure.ntbins
step=time_structure.step
;write the contents to a file
filename = 'tbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,tmin
printf,lun,tmax
printf,lun,ntbins
printf,lun,step
free_lun,lun,/force
    endelse
;
endif


oxrange = '('+strtrim(string(min(x)),2)+','+strtrim(string(max(x)),2)+')'
xrange = '('+strtrim(string(tmin),2)+','+strtrim(string(tmax),2)+')'
treat = ['Data has been rebinned in time.', $
        'Old bins:'+strtrim(string(nx),2), $
        'Old time-range: '+oxrange, $
        'New bins:'+strtrim(string(ntbins),2), $
        'New time-range: ',xrange]
*self.treatmentPtr = [*self.treatmentPtr,treat]



 tbins=tmin+step/2.0
 tbins_arr=tbins
    while tbins lt (tmax-step*1.49999) do begin
    tbins+=step
    tbins_arr=[tbins_arr,tbins]
    endwhile
;print,'tbins_arr =',tbins_arr
;print,ntbins
;ntbins=n_elements(tbins_arr) ; just in case


  drebin,x,data,error,tbins_arr,z_out,dz_out,/points,/to_points, $
     err = err,emsg = emsg
;
;
*self.dataPtr=z_out
*self.errorPtr=dz_out
*self.xvalsPtr=tbins_arr

ENDELSE


;print,n_elements(x)
;print,b_nbins+nb_nbins
;help,data
;help,error
;help,z_out
;help,dz_out
;help,x_out

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::qbinning,event = event
z = *self.dataPtr
dz = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
; is the data to be output in spe format
widget_control,self.FormatTypesGroup,get_value = vals
if vals eq 3 then begin
msg=string('SPE Output Format Requested.  No Grouping Shall Be Performed')
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb

mindz=min(dz[where(dz gt 0)])
;print,'mindz =',mindz
wherezero=where(dz eq 0,count)
if count ne 0 then dz(wherezero)=mindz
*self.dataPtr=z
*self.errorPtr=dz
return
endif


;print, 'self.ndet passed =',self.ndet
;   sum all or re-bin
widget_control,self.Qbin_Group,get_value = thisValue
    if thisValue[0] eq 0 then begin
    ;sum all
;print,'lambda0 = ',lambda0
;print,'min(*self.yvalsPtr) =',min(*self.yvalsPtr)
;print,'max(*self.yvalsPtr) =',max(*self.yvalsPtr)
 z=total(z,2)
;help,sigdata
whereneg=where(z lt 0,count)
if count ne 0 then z(whereneg)=0.0
;
err=sqrt(total(dz^2,2))
dz=err
;print,'total(dz) quick =',total(dz)

;
;print,'count =',count
mindz=min(dz[where(dz gt 0)])
wherezero=where(dz eq 0,count)
if count ne 0 then dz(wherezero)=mindz
;print,'count =',count
y=0.0
signdet=1
*self.dataPtr=z
*self.errorPtr=dz
*self.yvalsPtr=y
;*self.xvalsPtr*=241.7989052
self.ndet=signdet
;print,'(size(*self.dataPtr))[2] =',(size(*self.dataPtr))[2]
;print,'(size(sigData))[2] =',(size(sigData))[2]
;print,'(size(sigData))[2] =',(size(sigData))[2]
return
 endif

widget_control,self.qbin_Group1,get_value = qbin_command
;   modify default(0), use previous(1), modify previous(2)
;should the output be S(Q,w) , S(theta,w) or  S(theta,t)
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
IF DataOutput eq 0 then begin ; convert to Q and re-bin
;wherezero=where(dz eq 0,count)
;if count ne 0 then dz(wherezero)=1.0
;*self.errorPtr=dz
;*self.dataPtr=z
;*self.xvalsPtr=x
;*self.yvalsPtr=y
;return
;print,min(y) & print,max(y)
signdet=self.ndet
tof_corr_structure=*self.tof_corr_structurePtr
lambda0=tof_corr_structure.lambdatof
;print,'lambda0 =',lambda0

ny=n_elements(y)
nx=n_elements(x)
uy = 1+bytarr(ny)
ux = 1+bytarr(nx)
phimat = ux#y
;converts wavelength to energy in meV
k0=2.0*!dpi/lambda0
;print,'k0 =',k0
;1 meV = 2.072099427*k^2
alpha=2.072099427
E0=81.80320651/lambda0^2
if self.units eq 1 then begin
E0*=8.06554097
alpha*=8.06554097
endif
if self.units eq 2 then begin
E0*=241.7989052
alpha*=241.7989052
endif

;as given by Craig
;qmat = sqrt((2.0*eo-xmat-2.0*sqrt(eo*(eo-xmat))*cos(phimat*!dtor))/alpha)
;
;as adapted by Philip
;
xmat = x#uy
qmat = sqrt((2.0*E0-xmat-2.0*sqrt(E0*(E0-xmat))*cos(phimat*!dtor))/alpha)



    if qbin_command eq 0 then begin; modify default
qmin = min(qmat,max = qmax)
nqbins=5
step=(qmax-qmin)/(nqbins)
qbin_structure = opan_Q_rebin_widget(qmin,qmax,nqbins,step,group_leader = event.top)
;qbin_structure = {qmin:qmin,qmax:qmax,step:step,nqbins:nqbins,cancel:1}

         if qbin_structure.cancel eq 1 then begin
; if cancel then some over detectors
 z=total(z,2)
whereneg=where(z lt 0,count)
if count ne 0 then z(whereneg)=0.0
;
err=sqrt(total(dz^2,2))
dz=err
;
;print,'count =',count
mindz=min(dz[where(dz gt 0)])
wherezero=where(dz eq 0,count)
if count ne 0 then dz(wherezero)=mindz
y=0.0
signdet=1
*self.dataPtr=z
*self.errorPtr=dz
*self.yvalsPtr=y
return
       endif
;print,qbin_structure
;
qmin=qbin_structure.qmin
qmax=qbin_structure.qmax
step=qbin_structure.step
nqbins=qbin_structure.nqbins
;   write the contents to a file
filename = 'QbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,qmin
printf,lun,qmax
printf,lun,nqbins
printf,lun,step
free_lun,lun,/force
    endif
;
;
;
    if qbin_command eq 1 then begin; modify previous
filename = 'QbinParameters.txt'
    if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,qmin
readf,lun,qmax
readf,lun,nqbins
readf,lun,step
free_lun,lun,/force
nqbins=fix(nqbins)
    endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'QbinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb
qmin = min(qmat,max = qmax)
nqbins=5
step=(qmax-qmin)/(nqbins)
    endelse
qbin_structure = opan_Q_rebin_widget(qmin,qmax,nqbins,step,group_leader = event.top)
;qbin_structure = {qmin:qmin,qmax:qmax,step:step,nqbins:nqbins,cancel:1}
         if qbin_structure.cancel eq 1 then begin
; if cancel then some over detectors
 z=total(z,2)
whereneg=where(z lt 0,count)
if count ne 0 then z(whereneg)=0.0
;
err=sqrt(total(dz^2,2))
dz=err
;
;print,'count =',count
mindz=min(dz[where(dz gt 0)])
wherezero=where(dz eq 0,count)
if count ne 0 then dz(wherezero)=mindz
y=0.0
signdet=1
*self.dataPtr=z
*self.errorPtr=dz
*self.yvalsPtr=y
return
       endif
;
qmin=qbin_structure.qmin
qmax=qbin_structure.qmax
step=qbin_structure.step
nqbins=qbin_structure.nqbins
;   write the contents to a file
filename = 'QbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,qmin
printf,lun,qmax
printf,lun,nqbins
printf,lun,step
free_lun,lun,/force
    endif



    if qbin_command eq 2 then begin; use previous
filename = 'QbinParameters.txt'
    if file_test(self.workDir+filename) eq 1 then begin
filename = filepath(filename,root_dir = self.workDir)
openr,lun,filename,/get_lun
readf,lun,qmin
readf,lun,qmax
readf,lun,nqbins
readf,lun,step
free_lun,lun,/force
nqbins=fix(nqbins)
    endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'QbinParameters.txt', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb
qmin = min(qmat,max = qmax)
nqbins=5
step=(qmax-qmin)/(nqbins)
qbin_structure = opan_Q_rebin_widget(qmin,qmax,nqbins,step,group_leader = event.top)
qmin=qbin_structure.qmin
qmax=qbin_structure.qmax
step=qbin_structure.step
nqbins=qbin_structure.nqbins
filename = 'QbinParameters.txt'
filename = filepath(filename,root_dir = self.workDir)
openw,lun,filename,/get_lun
printf,lun,qmin
printf,lun,qmax
printf,lun,nqbins
printf,lun,step
free_lun,lun,/force
;qbin_structure = {qmin:qmin,qmax:qmax,step:step,nqbins:nqbins,cancel:1}
         if qbin_structure.cancel eq 1 then begin
; if cancel then some over detectors
 z=total(z,2)
;
whereneg=where(z lt 0,count)
if count ne 0 then z(whereneg)=0.0
;
err=sqrt(total(dz^2,2))
dz=err
;
;print,'count =',count
mindz=min(dz[where(dz gt 0)])
;print,'mindz =',mindz
wherezero=where(dz eq 0,count)
if count ne 0 then dz(wherezero)=mindz
;print,'count =',count
;
y=0.0
signdet=1
*self.dataPtr=z
*self.errorPtr=dz
*self.yvalsPtr=y
return
       endif
    endelse
;
;   write the contents to a file
    endif
;
;
;
 qbins=qmin+step/2.0
 qbins_arr=qbins
    while qbins lt (qmax-step*1.49999) do begin
    qbins+=step
    qbins_arr=[qbins_arr,qbins]
    endwhile
;print,'qbins_arr =',qbins_arr
;print,nqbins
nqbins=n_elements(qbins_arr) ; just in case
;print,nqbins
;qmin=qbin_structure.qmin
;qmax=qbin_structure.qmax
;nqbins=qbin_structure.nqbins
;
;print,'qmin =',qmin
;print,'qmax =',qmax
;print,'nqbins =',nqbins
;uq = 1+bytarr(nqbins)
;dq = (qmax-qmin)/(nqbins)
;qbins = qmin+dq/2.0+dq*findgen(nqbins)
;print,qbins
;
dataReb = fltarr(nx,nqbins)
errReb = fltarr(nx,nqbins)
;
;do the same for the actual values....
;
; Now do the rebinning in Q
for i = 0,nx-1 do begin
  z_in = reform(z[i,*])
  dz_in = reform(dz[i,*])
  x_in = reform(qmat[i,*])
  drebin,x_in,z_in,dz_in, qbins_arr,z_out,dz_out,/points,/to_points, $
     err = err,emsg = emsg
  dataReb[i,*] = z_out
  errReb[i,*] = dz_out
endfor
ndet=n_elements(qbins_arr)

;help,dataReb

;whereLTone=where(dataReb lt 1,count)
;if count ne 0 then begin
;dataReb(whereLTone)=0.0
;errReb(whereLTone)=1.0
;print,whereLTone
;endif



whereneg=where(dataReb lt 0,count)
if count ne 0 then dataReb(whereneg)=0.0
;print,'count =',count
mindz=min(errReb[where(errReb gt 0)])
wherezero=where(errReb eq 0,count)
if count ne 0 then errReb(wherezero)=mindz
;print,'count =',count

self.ndet=ndet
*self.dataPtr=dataReb
*self.errorPtr=errReb
*self.yvalsPtr=qbins_arr

;help,*self.dataPtr
;help,*self.errorPtr
;help,*self.xvalsPtr
;help,*self.yvalsPtr
;return
oyrange = '('+strtrim(string(min(qmat)),2)+','+strtrim(string(max(qmat)),2)+')'
yrange = '('+strtrim(string(min(qbins_arr)),2)+','+strtrim(string(max(qbins_arr)),2)+')'
treat = ['Data has been rebinned in Q.', $
        'Old Q-range: '+oyrange, $
        'New Q-range: ',yrange ,$
        'New bins:'+strtrim(string(ndet),2)]
*self.treatmentPtr = [*self.treatmentPtr,treat]


ENDIF
;
;
;
if DataOutput eq 1 or DataOutput eq 2 then begin ; group in phi


file='PhiBinParameters.sav'
   fileOut = filepath(file,root_dir = self.workDir)

    if qbin_command eq 0 then begin

array = intarr(self.ndet,self.ndet)
;help,array
;print,n_elements(self.ndet)
for i = 0,self.ndet-1 do begin
  array[i,i] = 1
endfor
*self.selDetPtr = array

*self.selDetPtr = focusDetGroupWidget(group_leader = event.top,self.selDetPtr,y)
DetPtr=*self.selDetPtr
save, filename=fileout, DetPtr

    endif

    if qbin_command eq 1 then begin
        if file_test(self.workDir+file) eq 1 then begin
    restore, fileout
;    help,Detptr
    *self.selDetPtr=Detptr
       ;print,'Modify Previous'
        ;print,(size(Detptr))[1]
        ;print,self.ndet
    *self.selDetPtr = focusDetGroupWidget(group_leader = event.top,self.selDetPtr,y)
     DetPtr=*self.selDetPtr
save, filename=fileout, DetPtr
  endif else begin
;revert to default
message=string('THE FILE ',self.workDir,'PhiBinParameters.sav', ' HAS NOT BEEN FOUND;  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb

       array = intarr(self.ndet,self.ndet)
for i = 0,self.ndet-1 do begin
  array[i,i] = 1
endfor
*self.selDetPtr = array

*self.selDetPtr = focusDetGroupWidget(group_leader = event.top,self.selDetPtr,y)
DetPtr=*self.selDetPtr
save, filename=fileout, DetPtr

       endelse
    endif


    if qbin_command eq 2 then begin
        if file_test(self.workDir+file) eq 1 then begin
    restore, fileout
        *self.selDetPtr=Detptr
       ;print,'Use Previous'
        ;print,(size(Detptr))[1]
        ;print,self.ndet
       endif else begin
       ;revert to default
message=string('THE FILE ',self.workDir,'PhiBinParameters.sav', ' HAS NOT BEEN FOUND.  REVERTING TO DEFAULT ')
msg=message[0]+message[1]+message[2]
self->flashMessage_create,msg,tlb
wait,5
self->flashMessage_destroy,tlb
       array = intarr(self.ndet,self.ndet)
       ;help,array
       ;print,'self.ndet =',self.ndet
       ;help,y
for i = 0,self.ndet-1 do begin
  array[i,i] = 1
endfor
*self.selDetPtr = array

*self.selDetPtr = focusDetGroupWidget(group_leader = event.top,self.selDetPtr,y)
;print,'woof'
DetPtr=*self.selDetPtr
save, filename=fileout, DetPtr

       endelse
    endif


wheregtzero=where(total((*self.selDetPtr),2) gt 0,num)
;print,'num =',num
yout=(*self.selDetPtr)[wheregtzero,*]
znew=fltarr((size(z))[1],num)
dznew=fltarr((size(z))[1],num)
ynew=fltarr(num)
treat = ['Data has been binned in groups of phi.', $
        'Number of groups: '+strtrim(string(num),2),  $
        'Phi values for detectors in each group are as follows:']

;help,znew
;help,yout
;print,yout
for i = 0, num-1 do begin
whereone=where(yout[i,*] eq 1,count)
if count eq 0 then return
;print, 'group number =', i
;print, 'Detectors =', whereone
;print,''
;print,count
if count gt 1 then begin
znew[*,i]=total(z[*,whereone],2)
ynew[i]=(moment(y[whereone]))[0] ; mean average of detectors in group
dznew[*,i]=sqrt(total((dz[*,whereone])^2,2))
;print, 'whereone =',whereone
endif else begin
znew[*,i]=z[*,whereone]
dznew[*,i]=sqrt((dz[*,whereone])^2)
ynew[i]=y[whereone] ; mean average of detectors in group
endelse
treat=[treat, $
       'Group '+strtrim(string(i+1),2)+': ', $
       strtrim(string(y[whereone]),2), $
       '']
;       print,'y[whereone] =',y[whereone]
endfor
treat=[treat,'The y vector corresponds to the average value of phi in a given group', $
         '']

;help,znew
;help,dznew
;help,ynew
;print,ynew
whereneg=where(znew lt 0,count)
if count ne 0 then znew(whereneg)=0.0
;dznew=sqrt(znew)
;print,'count =',count
mindznew=min(dznew[where(dznew gt 0)])
wherezero=where(dznew eq 0,count)
if count ne 0 then dznew(wherezero)=mindznew
;print,'count =',count


*self.dataPtr=znew
*self.errorPtr=dznew
*self.yvalsPtr=ynew

*self.treatmentPtr = [*self.treatmentPtr,treat]

;help,yout
;print,yout(1,*)
endif

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::normalize_to_vanadium,event = event
if n_elements(*self.dataPtr) eq 0 then return
if n_elements(*self.vanFilePtr) eq 0 then return
; First pull out the current data values
sigData = *self.dataPtr
sigError = *self.errorPtr
sigX = *self.xvalsPtr
sigY = *self.yvalsPtr
sigMon_tc = *self.Mon_tcPtr
sigMon_cum = *self.Mon_cumPtr
sigTemp = *self.tempPtr
signdet=self.ndet
vanFile = *self.vanFilePtr
tof_corr_structure=*self.tof_corr_structurePtr
self->Nexus_read_raw_focus,vanFile,event = event
vdata = *self.dataPtr
vtemperature = *self.tempPtr
vtemperature = vtemperature[0]
;help,vtemperature
;print,'vtemperature =',vtemperature
detnum=*self.yvalsPtr
;print,'*self.xvalsPtr =',*self.xvalsPtr
;
;Apply the same detector filtering as before
;
;
good_det=*self.good_detPtr
pnewdata=ptr_new(fltarr((size(vdata))(1),n_elements(good_det)))
pnewdetnum=ptr_new(fltarr(n_elements(good_det)))
;
*pnewdata=vdata(*,good_det)
*pnewdetnum=detnum(good_det)
vData=*pnewdata
detnum=*pnewdetnum
ptr_free,pnewdata
ptr_free,pnewdetnum
;
;print,'detnum =',detnum
;print,'good_det =',good_det
;print,'n_elements(good_det) =',n_elements(good_det)


;sort the data in ascending order of detnum
;
sortorder=sort(detnum)
detnum=detnum(sortorder)
vdata=vdata(*,sortorder)
;print,detnum
;
pnewdata=ptr_new(fltarr((size(vdata))(1),(size(vdata))(2)))
pnewdetnum=ptr_new(fltarr((size(detnum))(1)))
;remove degeneracies
;
(*pnewdetnum)[0]=detnum[0]
(*pnewdata)[*,0]=vdata[*,0]
j=0
for i=1,n_elements(detnum)-1 do begin
    if detnum(i) ne detnum(i-1) then begin
j+=1
(*pnewdetnum)[j]=detnum[i]
(*pnewdata)[*,j]=vdata[*,i]
endif else begin
(*pnewdata)[*,j]+=vdata[*,i]
endelse
endfor
vData=(*pnewdata)[*,0:j]
detnum=(*pnewdetnum)[0:j]
ptr_free,pnewdata
ptr_free,pnewdetnum

;*self.treatmentPtr = [*self.treatmentPtr,treat]
*self.dataPtr = vData
vError = sqrt(vData)
*self.errorPtr = vError
*self.yvalsPtr = detnum
;
;
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
;convert to energy unless the desired quantity is S(theta,t)
if DataOutput ne 2 then self->convert_to_energy,event = event
vanData = *self.dataPtr
vanx = *self.xvalsPtr
;print,total(vandata)
;print,'*self.xvalsPtr =',*self.xvalsPtr
;
;print,'vanData(*,1) =',vanData(*,1)
;print,''
for i = 0,n_elements(detnum)-1 do begin
   ; Get a good estimate of the starting parameters for the fits


if DataOutput ne 2 then begin
 y = reform(vanData[*,i])
 zeropos=(vanx[where(y eq max(y))])[0]
  coeff=[max(y),zeropos,0.05,0]
  region = where((vanX ge zeropos-0.5) and (vanX le zeropos+0.5))
x=vanX[region]
;print,'x = ',x
 y = reform(vanData[region,i])
 endif else begin
  y = reform(vanData[*,i])
 zeropos=(vanx[where(y eq max(y))])[0]
  coeff=[max(y),zeropos,20.0,0]
  region = where((vanX ge zeropos-150.0) and (vanX le zeropos+150.0))
x=vanX[region]
;print,'x = ',x
 y = reform(vanData[region,i])

endelse
 ;print,'y = ',y

 ;print,'coeff =',coeff
;print,i
;print,vanx
  ; Fit it...
  result = gaussfit(x,y,p,chisq=gof,estimates = coeff,nterms = 4)
  ; the next line should be the area
  ;print, 'Chisq =',gof
  ;print, 'fwhh =',p[2]
  ;print, 'max(y) =',p[0]
  ;print, 'x position =',p[1]
  ;print,''
  a = p[0]*sqrt(2.0*!pi*p[2]^2)
  if i eq 0 then area = a else area = [area,a]
endfor
  ;print,'area =',area

;print,'vanData(*,245) =',vanData(*,245)
;print, 'vanx =',vanx
stats = moment(area[0:n_elements(detnum)-1])
;stats[0] is then the mean average area.  Now divide each area by the mean average
area = temporary(area)/stats[0]
ux = 1+bytarr(n_elements(sigX))
lambda0=tof_corr_structure.lambdatof
q = (4.0*!pi/lambda0)*sin(0.5*!dtor*detnum)
;help,q
;print,'min(Q) =',min(Q)
;print,'max(Q) =',max(Q)

;the next line creates n_elements(sigX) columns of exp(0.066*sigY^2) data
;dwf = ux#(exp(-0.0067^(vtemperature/300.0)*q^2))  ; Debye-Waller factor for vanadium
dwf = ux#(exp(-0.0067*vtemperature/300.0*q^2))  ; Debye-Waller factor for vanadium
;print,'min(dwf) =',min(dwf)
;print,'max(dwf) =',max(dwf)
;dwf/=dwf
;help,dwf
;the next line creates an array of n_elements(sigX) columns
;and 1+n_elements(area) rows
;help,area
;print,'area =',area

a = ux#(area)
;help,a
;print,'a =',a
;help,sigdata
;print,'total(sigdata) =',total(sigdata)
sigData = sigData*dwf/a
;print,'total(sigdata) =',total(sigdata)
;help,sigdata
sigError = sigError*dwf/a
*self.dataPtr = sigData
*self.errorPtr = sigError
*self.xvalsPtr = sigX
*self.yvalsPtr = sigY
*self.Mon_tcPtr = sigMon_tc
*self.Mon_cumPtr = sigMon_cum
*self.tempPtr = sigTemp
self.ndet=signdet
;print,'v ndet =',self.ndet
treat = 'Normalised to vanadium file: '+strtrim(string(vanFile),2)
*self.treatmentPtr = [*self.treatmentPtr,treat]
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::Nexus_read_raw_focus,filename,event = event,err = err
err = 0
hdlerror=0
Lbank='merged'
;IF N_ELEMENTS(Lbank) EQ 0 THEN Lbank='bank1'
handle=0L
;dummy = nxopen(filename,'read',handle)   ;for nidl
dummy = nxopen(filename,'NXACC_READ',handle) ;for napi45
if n_elements(filename) eq 0 then Begin
hdlerror=-1
return
endif
self.curFile = filename
widget_control,self.DetectorBankTypesGroup,get_value = DBT
if DBT[0] eq 0 then Lbank='merged'
if DBT[0] eq 1 then Lbank='lowerbank'
if DBT[0] eq 2 then Lbank='bank1'
if DBT[0] eq 3 then Lbank='upperbank'
;print,Lbank
;
;
;-------- get some global attributes
    dummy1 = nxgetattr(handle,'file_time',ftime,typ)
    dummy2 = nxgetattr(handle,'file_name',foriname, typ)
    dummy3 = nxgetattr(handle,'owner',owner,typ)

    ;------ open data vGroup and get counts, two_theta
    dummy = nxopengroup(handle,'entry1','NXentry')
    dummy = nxopengroup(handle,Lbank,'NXdata')
    IF NOT dummy THEN BEGIN
       dummy=DIALOG_MESSAGE('Old file, only one bank! Opening bank1', /information, DIALOG_PARENT=tof1)
        dummy = nxopengroup(handle,'bank1','NXdata')
    END

    dummy = nxopendata(handle,'counts')
    dummy = nxgetdata(handle,cnts)
    dummy = nxclosedata(handle)

    dummy = nxopendata(handle,'theta')
    dummy = nxgetdata(handle,detnum)
    dummy = nxclosedata(handle)

    dummy = nxopendata(handle,'time_binning')
    dummy = nxgetdata(handle,tof)
    dummy = nxclosedata(handle)
    dummy = nxclosegroup(handle)

    ;------ get title Information
    dummy = nxopendata(handle,'title')
    dummy = nxgetdata(handle,title)
    dummy = nxclosedata(handle)

    ; -------- get sample information
    dummy = nxopengroup(handle,'sample','NXsample')
    dummy = nxopendata(handle,'name')
    dummy = nxgetdata(handle,sample)
    dummy = nxclosedata(handle)
    dummy = nxopendata(handle,'distance')
    dummy = nxgetdata(handle,sadist)
    dummy = nxclosedata(handle)
    dummy = nxopendata(handle, 'temperature')
    dummy = nxgetdata(handle, temp)
    dummy = nxclosegroup(handle)

    ; ------- enter instrument group
    dummy = nxopengroup(handle,'FOCUS','NXinstrument')
    dummy = nxopendata(handle,'tof_monitor')
    IF dummy THEN BEGIN
       dummy = nxgetdata(handle,mcnts)
        dummy = nxclosedata(handle)
    END ELSE mcnts=MAKE_ARRAY(1, /double, value=0.0)

    ; ------- enter monochromator group
    dummy = nxopengroup(handle,'monochromator','NXmonochromator')

    ;------ get lambda
    dummy = nxopendata(handle,'lambda')
    dummy = nxgetdata(handle,lambda)
    dummy = nxclosedata(handle)
    dummy = nxclosegroup(handle)


    ;------- get monitor
    dummy = nxopengroup(handle,'counter','NXmonitor')
    dummy = nxopendata(handle,'monitor')
    dummy = nxgetdata(handle,monitor)
    dummy = nxclosedata(handle)

    ;------ get measurement time
    dummy = nxopendata(handle,'time')
    dummy = nxgetdata(handle,time)
    dummy = nxclosedata(handle)
    dummy = nxclosegroup(handle)

    ;------- fermi-chopper
    dummy = nxopengroup(handle,'fermi_chopper','NXchopper')
    dummy = nxopendata(handle,'rotation_speed')
    dummy = nxgetdata(handle,fermispeed)
    dummy = nxclosedata(handle)

    ;-------- get phase
    dummy = nxopendata(handle,'phase')
    dummy = nxgetdata(handle,fermiphase)
    dummy = nxclosedata(handle)

    ;------ fermi-distance
    dummy = nxopendata(handle,'distance')
    dummy = nxgetdata(handle,fermidist)
    dummy = nxclosedata(handle)
    dummy = nxclosegroup(handle)

    ;-------- disk chopper
    dummy = nxopengroup(handle,'disk_chopper','NXchopper')

    ;------ get speed
    dummy = nxopendata(handle,'rotation_speed')
    dummy = nxgetdata(handle,diskspeed)
    dummy = nxclosedata(handle)
    dummy = nxclosegroup(handle)

    ;------ close the file
    dummy = nxclosegroup(handle)
    dummy = nxclose(handle)

    IF hdlerror NE 0 THEN BEGIN
       IF hdlerror EQ 1 THEN mes='There was '+STRCOMPRESS(STRING(hdlerror), /REMOVE_ALL)+' error in reading the NEXUS file!' $
       ELSE mes='There were '+STRCOMPRESS(STRING(hdlerror), /REMOVE_ALL)+' errors in reading the NEXUS file!'
       dummy=DIALOG_MESSAGE(mes, /error, DIALOG_PARENT=tof1)
    END
;
; code added by Philip
;
;cd,current=dir
;filename=dir+path_sep()+'FOCUSFile_contents.txt'
;openw,lun,filename,/get_lun
;printf,lun,'file_time  ftime ='
;printf,lun,ftime
;printf,lun,''
;printf,lun,'file_name foriname = '
;printf,lun,foriname
;printf,lun,''
;printf,lun,'owner owner ='
;printf,lun,owner
;printf,lun,''
;printf,lun,'counts cnts  ='
;printf,lun,cnts
;printf,lun,''
;printf,lun,'theta detnum ='
;printf,lun,detnum
;printf,lun,''
;printf,lun,'time_binning tof ='
;printf,lun,tof
;printf,lun,''
;printf,lun,'title  title ='
;printf,lun,title
;printf,lun,''
;printf,lun,'name  sample ='
;printf,lun,sample
;printf,lun,''
;printf,lun,'distance  sadist='
;printf,lun,sadist
;printf,lun,''
;printf,lun,'temperature temp='
;printf,lun,temp
;printf,lun,''
;printf,lun,'tof_monitor mcnts='
;printf,lun,mcnts
;printf,lun,''
;printf,lun,'lambda  lambda= '
;printf,lun,lambda
;printf,lun,''
;printf,lun,'NXmonitor monitor'
;printf,lun,monitor
;printf,lun,''
;printf,lun,'time time ='
;printf,lun,time
;printf,lun,''
;printf,lun,'fermi_chopper fermispeed ='
;printf,lun,fermispeed
;printf,lun,''
;printf,lun,'phase fermiphase ='
;printf,lun,fermiphase
;printf,lun,''
;printf,lun,'distance fermidist ='
;printf,lun,fermidist
;printf,lun,''
;printf,lun,'disk_chopper diskspeed'
;printf,lun,diskspeed
;
;free_lun,lun,/force
;;
;help, mcnts
;help, tof
;
;
data=float(cnts)
error=sqrt(data)
wherezero=where(error eq 0,thisnum)
if thisnum gt 0 then begin
    error[wherezero]=1.0
endif
;
;

;print,'detnum =',detnum
;print,'sort(detnum) =',sort(detnum)
;print,total(data)
;
*self.odataPtr =data
*self.oerrorPtr=error
*self.oxvalsPtr=tof
*self.oyvalsPtr=detnum
;convert to point data
tof+=(tof[1]-tof[0])/2.0
;

;
;print,'n_elements(detnum) =',n_elements(detnum)
;print,'n_elements(newdetnum) =',n_elements(newdetnum)
;help,j
;help,data
;help,error
;help,detnum
;print,detnum
;print,'total(data) =',total(data)
;
*self.dataPtr = data
*self.errorPtr=error
*self.xvalsPtr=tof
*self.yvalsPtr=detnum

;print,'n_elements(detnum) =',n_elements(detnum)
;help,data

;newdata=total(data,2)
;help,newdata
;print,total(newdata)
;*self.yvalsPtr=0
;*self.dataPtr = newdata
;*self.errorPtr=sqrt(newdata)
;
;
;
;
*self.Mon_cumPtr=monitor
*self.Mon_tcPtr=mcnts
*self.tempPtr=temp
*self.detPtr=detnum
;print,*self.detPtr
;
self.ylabel = 'Group'
self.zlabel = 'Intensity'
self.ndet = n_elements(detnum)

;print,'self.ndet =',self.ndet
;
;help,title
;print,title
;help,ftime
;help,diskspeed
;help,fermidist
;print,sample
;help,sample
;print,owner
;help,owner
;sample=string(sample)
;
;   create structure with instrumental details
instrument_details={instrument_details, $
              ftime:ftime,   $
              foriname:foriname,    $
              owner:owner,     $
              title:title,     $
              sample:sample,    $
              sadist:sadist,  $
              temp:temp,   $
              lambda:lambda,$
              fermispeed:fermispeed,    $
              fermiphase:fermiphase,    $
              fermidist:fermidist,$
              diskspeed:diskspeed, $
              MONtoDET:2.784, $
              FCtoDET:3.0, $
              FCtoSAM:0.5, $
              SAMtoDET:2.5}
;
;print,instrument_details

;MONtoDET=monitor to detector distance
;FCtoDET=Fermi chopper to detector
;FCtoSAM=Fermi chopper to sample
;SAMtoDET=Sample to detector


*self.InstrumentPtr=instrument_details
;
;
;   All the information not required for the data reduction are stored in the header
nheader=12
header = strarr(nheader)
;help,ftime
;help,diskspeed
;help,fermidist
;print,sample
;help,sample
;print,owner
;help,owner
;sample=string(sample)

header[0]=string('file_time = ',ftime)
header[1]=string('Original file name = ',foriname)
header[2]=string('Owner = ',owner)
header[3]=string('Title = ',title)
header[4]='Sample name = '
header[4]+=sample
header[5]=string('Distance to sample = ',sadist)
header[6]=string('Temperature of sample = ',temp)
header[7]=string('Wavelength / A  = ',lambda)
header[8]=string('Speed of Fermi chopper = ',fermispeed)
header[9]=string('Phase of Fermi chopper = ',fermiphase)
header[10]=string('Fermi Distance = ',fermidist)
header[11]=string('Disk Chopper Speed = ',diskspeed)
*self.headerPtr = header
;for  i=0,11 do begin
;print,header[i]
;endfor
;
;
;
END
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::TOF_corr,event=event
;
if total(*self.Mon_tcPtr) eq 0 then begin
self->TOF_corr_oldfile,event=event
return
endif

;
widget_control,self.elasticGroup,get_value = elasticGroup
instrument_details=*self.InstrumentPtr
;
if elasticGroup[0] eq 0 then begin
;
if n_elements(*self.VanFilePtr) ne 0 then begin

;Store All the Data Locally
;
sigData = *self.dataPtr
sigError = *self.errorPtr
sigx = *self.xvalsPtr
sigy = *self.yvalsPtr
sigTemp = *self.tempPtr
sigMon_cum = *self.Mon_cumPtr
sigMon_tc = *self.Mon_tcPtr
signdet=self.ndet


vanfile=*self.VanFilePtr
self->Nexus_read_raw_focus,vanfile,event = event,err = err



;get the data
mcnts=*self.Mon_tcPtr
cnts=*self.dataPtr
tof=*self.xvalsPtr
;print,'instrument_details.fermispeed',instrument_details.fermispeed

Fermi_period=60000000.0/(2.0*(instrument_details.fermispeed))
;
;
;print,'Fermi_period = ',Fermi_period
;
;Extract distances
MONtoDET=instrument_details.MONtoDET
FCtoDET=instrument_details.FCtoDET
;
;Define Plank's constant
h=6.6260755e-34
mn=1.67495e-27
;

;
;   sum the detectors
cnts=total(cnts,2)
;
;omit unwanted peaks
;mcnts-=50.0 ;converted to float
;whereltzero=where(mcnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    mcnts[whereltzero]=0.0
;
;endif
;
;whereltzero=where(cnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    cnts[whereltzero]=0.0
;endif
;
;
coeff=[max(cnts),(tof[where(cnts eq max(cnts))])[0],20.0,0,0,0]
dummy=gaussfit(tof,cnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,p[i]
;endfor
det_time=p[1]
;print,'det time = ',det_time
;
;
coeff=[max(mcnts),(tof[where(mcnts eq max(mcnts))])[0],20.0,0,0,0]
;print,''
dummy=gaussfit(tof,mcnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,p[i]
;endfor
;
mon_time=p[1]-Fermi_period
While mon_time-Fermi_period gt 0 do begin
mon_time-=Fermi_period
endwhile

;print,'mon_time = ',mon_time

t=(det_time-mon_time)*1e-6
;print,'t = ',t
v=montodet/t
lambdaTOF=(h/(v*mn))*1e10
;print,'lambda =',lambda
;print,'instrument_details.lambda',instrument_details.lambda
; next calculate the tof shift
;   First calculate the time from FC to detector based upon new wavelength (velocity)
t=(FCtoDET/v)*1e6
;compare to det_time
tof_shift=t-det_time
;print, 'tof_shift = ',tof_shift
;tof should be added to the time bins
;print,tof
;print, 'tof = ',tof
;print,''
;print, 'tof = ',tof
;print,*self.xvalsPtr
;
;print, 'Elastic Line From Vanadium'
treat = 'Elastic Line Calculated From Vanadium File, TOF Data'
*self.treatmentPtr = [*self.treatmentPtr,treat]


;Restore All the Data
;
 *self.dataPtr=sigData
 *self.errorPtr=sigError
 *self.xvalsPtr=sigx
 *self.yvalsPtr=sigy
 *self.tempPtr=sigTemp
 *self.Mon_cumPtr=sigMon_cum
 *self.Mon_tcPtr=sigMon_tc
self.ndet=signdet
endif else begin
elasticgroup=1


widget_control,self.elasticGroup,set_value = elasticgroup
endelse
endif
;
if elasticGroup[0] eq 1 or elasticGroup[0] eq 2 then begin   ;   Elastic Line From Sample

;print, 'Elastic Line From Sample'
if elasticGroup[0] eq 1 then treat = 'Elastic Line Calculated From Sample File(s), TOF Data'
if elasticGroup[0] eq 2 then treat = 'Elastic Line Calculated From Sample File(s), Monochromator'
*self.treatmentPtr = [*self.treatmentPtr,treat]

;get the data
mcnts=*self.Mon_tcPtr
cnts=*self.dataPtr
tof=*self.xvalsPtr
;print,'instrument_details.fermispeed',instrument_details.fermispeed

Fermi_period=60000000.0/(2.0*(instrument_details.fermispeed))
;
;
;print,'Fermi_period = ',Fermi_period
;
;Extract distances
MONtoDET=instrument_details.MONtoDET
FCtoDET=instrument_details.FCtoDET
;
;Define Plank's constant and neutron mass
h=6.6260755e-34
mn=1.67495e-27
;
;
;
;   sum the detectors
cnts=total(cnts,2)
;
;omit unwanted peaks
;mcnts-=50.0 ;converted to float
;whereltzero=where(mcnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    mcnts[whereltzero]=0.0
;endif
;
;whereltzero=where(cnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    cnts[whereltzero]=0.0
;endif
;
;
coeff=[max(cnts),(tof[where(cnts eq max(cnts))])[0],20.0,0,0,0]
;print,'coeff =',coeff
dummy=gaussfit(tof,cnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,'p[i] =',p[i]
;endfor
;
det_time=p[1]
;print,'det time = ',det_time
;
;
coeff=[max(mcnts),(tof[where(mcnts eq max(mcnts))])[0],20.0,0,0,0]
;print,'tof[where(mcnts eq max(mcnts))]',tof[where(mcnts eq max(mcnts))]
dummy=gaussfit(tof,mcnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,'p[i] =',p[i]
;endfor
;
mon_time=p[1]-Fermi_period
While mon_time-Fermi_period gt 0 do begin
mon_time-=Fermi_period
endwhile

;print,'mon_time = ',mon_time

t=(det_time-mon_time)*1e-6
;print,'t = ',t
v=montodet/t
lambdaTOF=(h/(v*mn))*1e10
;print,'lambda =',lambda
;print,'instrument_details.lambda',instrument_details.lambda
; next calculate the tof shift
;   First calculate the time from FC to detector based upon new wavelength (velocity)
t=(FCtoDET/v)*1e6
;compare to det_time
tof_shift=t-det_time
;print, 'tof_shift = ',tof_shift
;tof should be added to the time bins
;print,tof
;print, 'tof = ',tof
;print,''
;tof+=tof_shift[0]
;print, 'tof = ',tof
;print,*self.xvalsPtr
endif
if elasticGroup[0] eq 2 then begin   ;   Elastic Line From Monochromator
lambda=instrument_details.lambda
tof_shift=0.0
treat = 'Elastic Line Calculated From Monochromator Value'
*self.treatmentPtr = [*self.treatmentPtr,treat]
endif
if elasticGroup[0] eq 3 then begin   ;   Elastic Line From Input Value
treat = 'Elastic Line Calculated From Input Value'
*self.treatmentPtr = [*self.treatmentPtr,treat]
widget_control,self.lambda_own_value,get_value = lambda
tof_shift=0.0
endif
;
;
if elasticGroup[0] eq 0 or elasticGroup[0] eq 1 then begin
tof_corr_structure={tof_corr_structure,tof_shift:tof_shift[0],lambdatof:lambdaTOF[0]}
*self.tof_corr_structurePtr=tof_corr_structure
endif else begin
tof_corr_structure={tof_corr_structure,tof_shift:tof_shift[0],lambdatof:lambda[0]}
*self.tof_corr_structurePtr=tof_corr_structure
endelse
;
widget_control,(*self.lambda_fieldPtr).lambda_monochromator,set_value = string(instrument_details.lambda)
if elasticgroup lt 3 then begin
widget_control,(*self.lambda_fieldPtr).lambda_tof,set_value = string(lambdaTOF[0])
endif else begin
widget_control,(*self.lambda_fieldPtr).lambda_tof,set_value = ''
endelse
;
;
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::TOF_corr_oldfile,event=event
;
;print,'TOF_corr_oldfile'
;
widget_control,self.elasticGroup,get_value = elasticGroup
instrument_details=*self.InstrumentPtr
;
if elasticGroup[0] eq 0 then begin
;
if n_elements(*self.VanFilePtr) ne 0 then begin

;Store All the Data Locally
;
sigData = *self.dataPtr
sigError = *self.errorPtr
sigx = *self.xvalsPtr
sigy = *self.yvalsPtr
sigTemp = *self.tempPtr
sigMon_cum = *self.Mon_cumPtr
sigMon_tc = *self.Mon_tcPtr
signdet=self.ndet


vanfile=*self.VanFilePtr
self->Nexus_read_raw_focus,vanfile,event = event,err = err



;get the data
cnts=*self.dataPtr
tof=*self.xvalsPtr
;print,'instrument_details.fermispeed',instrument_details.fermispeed

Fermi_period=60000000.0/(2.0*(instrument_details.fermispeed))
;
;
;print,'Fermi_period = ',Fermi_period
;
;Extract distances
;MONtoDET=instrument_details.MONtoDET
FCtoDET=instrument_details.FCtoDET
;
;Define Plank's constant
h=6.6260755e-34
mn=1.67495e-27
;
;
;
;   sum the detectors
cnts=total(cnts,2)
;
;omit unwanted peaks
;mcnts-=50.0 ;converted to float
;whereltzero=where(mcnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    mcnts[whereltzero]=0.0
;
;endif
;
;whereltzero=where(cnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    cnts[whereltzero]=0.0
;endif
;
;
coeff=[max(cnts),(tof[where(cnts eq max(cnts))])[0],20.0,0,0,0]
dummy=gaussfit(tof,cnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,p[i]
;endfor
det_time=p[1]
;print,'det time = ',det_time
;
;
;coeff=[max(mcnts),(tof[where(mcnts eq max(mcnts))])[0],20.0,0,0,0]
;print,''
;dummy=gaussfit(tof,mcnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,p[i]
;endfor
;
;mon_time=p[1]-Fermi_period
;While mon_time-Fermi_period gt 0 do begin
;mon_time-=Fermi_period
;endwhile

;print,'mon_time = ',mon_time

;t=(det_time-mon_time)*1e-6
t=det_time*1e-6
;print,'t = ',t
;v=montodet/t
v=FCtoDET/t

lambdaTOF=(h/(v*mn))*1e10
;print,'lambda =',lambda
;print,'instrument_details.lambda',instrument_details.lambda
; next calculate the tof shift
;   First calculate the time from FC to detector based upon new wavelength (velocity)
;t=(FCtoDET/v)*1e6
;compare to det_time
;tof_shift=t-det_time
tof_shift=0.0
;print, 'tof_shift = ',tof_shift
;tof should be added to the time bins
;print,tof
;print, 'tof = ',tof
;print,''
;print, 'tof = ',tof
;print,*self.xvalsPtr
;
;print, 'Elastic Line From Vanadium'
treat = 'Elastic Line Calculated From Vanadium File, TOF Data'
*self.treatmentPtr = [*self.treatmentPtr,treat]


;Restore All the Data
;
 *self.dataPtr=sigData
 *self.errorPtr=sigError
 *self.xvalsPtr=sigx
 *self.yvalsPtr=sigy
 *self.tempPtr=sigTemp
 *self.Mon_cumPtr=sigMon_cum
 *self.Mon_tcPtr=sigMon_tc
self.ndet=signdet
endif else begin
elasticgroup=1


widget_control,self.elasticGroup,set_value = elasticgroup
endelse
endif
;
if elasticGroup[0] eq 1 or elasticGroup[0] eq 2 then begin   ;   Elastic Line From Sample

;print, 'Elastic Line From Sample'
if elasticGroup[0] eq 1 then treat = 'Elastic Line Calculated From Sample File(s), TOF Data'
if elasticGroup[0] eq 2 then treat = 'Elastic Line Calculated From Sample File(s), Monochromator'
*self.treatmentPtr = [*self.treatmentPtr,treat]

;get the data
;mcnts=*self.Mon_tcPtr
cnts=*self.dataPtr
tof=*self.xvalsPtr
;print,'instrument_details.fermispeed',instrument_details.fermispeed

Fermi_period=60000000.0/(2.0*(instrument_details.fermispeed))
;
;
;print,'Fermi_period = ',Fermi_period
;
;Extract distances
;MONtoDET=instrument_details.MONtoDET
FCtoDET=instrument_details.FCtoDET
;
;Define Plank's constant and neutron mass
h=6.6260755e-34
mn=1.67495e-27
;
;
;
;   sum the detectors
cnts=total(cnts,2)
;
;omit unwanted peaks
;mcnts-=50.0 ;converted to float
;whereltzero=where(mcnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    mcnts[whereltzero]=0.0
;endif
;
;whereltzero=where(cnts lt 0.0,thisnum)
;if thisnum gt 0 then begin
;    cnts[whereltzero]=0.0
;endif
;
;
coeff=[max(cnts),(tof[where(cnts eq max(cnts))])[0],20.0,0,0,0]
;print,'coeff =',coeff
dummy=gaussfit(tof,cnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,'p[i] =',p[i]
;endfor
;
det_time=p[1]
;print,'det time = ',det_time
;
;
;coeff=[max(mcnts),(tof[where(mcnts eq max(mcnts))])[0],20.0,0,0,0]
;print,'tof[where(mcnts eq max(mcnts))]',tof[where(mcnts eq max(mcnts))]
;dummy=gaussfit(tof,mcnts,p,estimates=coeff,nterms=6)
;for i=0,5 do begin
;print,'p[i] =',p[i]
;endfor
;
;mon_time=p[1]-Fermi_period
;While mon_time-Fermi_period gt 0 do begin
;mon_time-=Fermi_period
;endwhile

;print,'mon_time = ',mon_time

;t=(det_time-mon_time)*1e-6
t=det_time*1e-6
;print,'t = ',t
;v=montodet/t
v=FCtoDET/t
;
lambdaTOF=(h/(v*mn))*1e10
;print,'lambda =',lambda
;print,'instrument_details.lambda',instrument_details.lambda
; next calculate the tof shift
;   First calculate the time from FC to detector based upon new wavelength (velocity)
;t=(FCtoDET/v)*1e6
;compare to det_time
tof_shift=0.0
;tof_shift=t-det_time
;print, 'tof_shift = ',tof_shift
;tof should be added to the time bins
;print,tof
;print, 'tof = ',tof
;print,''
;tof+=tof_shift[0]
;print, 'tof = ',tof
;print,*self.xvalsPtr
endif
if elasticGroup[0] eq 2 then begin   ;   Elastic Line From Monochromator
lambda=instrument_details.lambda
tof_shift=0.0
treat = 'Elastic Line Calculated From Monochromator Value'
*self.treatmentPtr = [*self.treatmentPtr,treat]
endif
if elasticGroup[0] eq 3 then begin   ;   Elastic Line From Input Value
treat = 'Elastic Line Calculated From Input Value'
*self.treatmentPtr = [*self.treatmentPtr,treat]
widget_control,self.lambda_own_value,get_value = lambda
tof_shift=0.0
endif
;
;
if elasticGroup[0] eq 0 or elasticGroup[0] eq 1 then begin
tof_corr_structure={tof_corr_structure,tof_shift:tof_shift[0],lambdatof:lambdaTOF[0]}
*self.tof_corr_structurePtr=tof_corr_structure
endif else begin
tof_corr_structure={tof_corr_structure,tof_shift:tof_shift[0],lambdatof:lambda[0]}
*self.tof_corr_structurePtr=tof_corr_structure
endelse
;
widget_control,(*self.lambda_fieldPtr).lambda_monochromator,set_value = string(instrument_details.lambda)
if elasticgroup lt 3 then begin
widget_control,(*self.lambda_fieldPtr).lambda_tof,set_value = string(lambdaTOF[0])
endif else begin
widget_control,(*self.lambda_fieldPtr).lambda_tof,set_value = ''
endelse
;
;
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::subtract_bad_detectors,event = event,err = err
;extract data
Data = *self.dataPtr
Error = *self.errorPtr
detnum = *self.yvalsPtr
x = *self.xvalsPtr
ndet=self.ndet
file='BadDetParameters.sav'
   fileOut = filepath(file,root_dir = self.workDir)

;woof=total(Data,1)
;help,woof

widget_control,self.Bad_DetectorGroup,get_value = treat
;Bad_DetectorTypes = ['Automatic',$
;              'Graphical Masking', $
;              'Scroll Spectra Masking',$
;              'Previous Selection',  $
;              'Use All Detectors']
;print,'treat =',treat

IF treat eq 0 then begin ;automatic
;remove bad detectors and then sort
;Calculate the mean height over all detectors
detarr=1+intarr(n_elements(detnum))
;
height=0.0
cumheight=0.0
for i=0,ndet-1 do begin
height=(max(Data(*,i))-min(Data(*,i)))
cumheight+=height
if i eq 0 then begin
heightdet=height
endif else begin
heightdet=[heightdet,height]
endelse
;print,'height =',height
endfor
avheight=cumheight/ndet
;
;print,'heightdet =',heightdet
;print,'n_elements(heightdet) =',n_elements(heightdet)
;print,'avheight =',avheight
;
;Keep only those detectors with intensity greater than 20% of the average
;First identify those detectors
for i=0,ndet-1 do begin
    if heightdet[i] gt 0.2*avheight then begin
       if n_elements(good_det) eq 0 then begin
       good_det=i
       endif else begin
       good_det=[good_det,i]
       endelse
    endif else begin
        if n_elements(omitted) ne 0 then omitted=[omitted,i+1]
    if n_elements(omitted) eq 0 then omitted=i+1
    detarr(i)=0
    endelse
endfor
*self.good_detPtr=good_det
pnewdata=ptr_new(fltarr((size(data))(1),n_elements(good_det)))
pnewdetnum=ptr_new(fltarr(n_elements(good_det)))
;
*pnewdata=data(*,good_det)
*pnewdetnum=detnum(good_det)
data=*pnewdata
error=sqrt(data)
detnum=*pnewdetnum
ptr_free,pnewdata
ptr_free,pnewdetnum

save, filename=fileout, detarr

;print, detarr

ENDIF

IF treat eq 1 then begin ;graphical masking
;sort, remove degeneracies and then

detarr=Graphical_Masking(data,detnum,(*self.InstrumentPtr).lambda,group_leader = event.top)
;print, detarr
whereone=where(detarr eq 1,count)
if count ne 0 then begin
data=data[*,whereone]
error=error[*,whereone]
detnum=detnum[whereone]
*self.good_detPtr=whereone
omitted=where(detarr eq 0)+1

;endif else begin
;  void = dialog_message(dialog_parent = event.top, $
;   'All Detectors Have Been Marked for Omission!  The Reduction Shall Now Proceed Employing All Detectors ')
;return
;endelse
endif
;help,data
;help,error
;help,detnum
save, filename=fileout, detarr
ENDIF

IF treat eq 2 then begin ;scroll spectra masking
detarr=Scroll_Spectra_Masking(data,error,detnum,x,group_leader = event.top)
whereone=where(detarr eq 1,count)
if count ne 0 then begin
data=data[*,whereone]
error=error[*,whereone]
detnum=detnum[whereone]
*self.good_detPtr=whereone
omitted=where(detarr eq 0)+1

;endif else begin
;  void = dialog_message(dialog_parent = event.top, $
;   'All Detectors Have Been Marked for Omission!  The Reduction Shall Now Proceed Employing All Detectors ')
;return
;endelse
endif
;help,data
;help,error
;help,detnum
save, filename=fileout, detarr
ENDIF


IF treat eq 3 then begin ;use previous
    restore, fileout
    n=n_elements(detarr)
    if n eq ndet then begin
     whereone=where(detarr eq 1,count)
       if count ne 0 then begin
       data=data[*,whereone]
       error=error[*,whereone]
       detnum=detnum[whereone]
       *self.good_detPtr=whereone
       omitted=where(detarr eq 0)+1
       endif
    endif else begin
      void = dialog_message(dialog_parent = event.top, $
   'Incompatible Array Dimensions.  Switching to Graphical Masking Tool ')
widget_control,self.Bad_DetectorGroup,set_value = 1
detarr=Graphical_Masking(data,detnum,(*self.InstrumentPtr).lambda,group_leader = event.top)
whereone=where(detarr eq 1,count)
if count ne 0 then begin
data=data[*,whereone]
error=error[*,whereone]
detnum=detnum[whereone]
*self.good_detPtr=whereone
omitted=where(detarr eq 0)+1
widget_control,self.qbin_group1,set_value = 0
endif
save, filename=fileout, detarr
    endelse
ENDIF



IF treat eq 4 then begin ;no bad detector removal
;just write the file
detarr=1+intarr(n_elements(detnum))
save, filename=fileout, detarr
*self.good_detPtr=indgen(ndet)
ENDIF

totaldet=n_elements(*self.yvalsPtr)
remaining=n_elements(detnum)
removed=totaldet-remaining



;
;print,'detnum =',detnum
;print,'good_det =',good_det
;print,'n_elements(good_det) =',n_elements(good_det)


;sort the data in ascending order of detnum
;
sortorder=sort(detnum)
detnum=detnum(sortorder)
data=data(*,sortorder)
;print,detnum
;
pnewdata=ptr_new(fltarr((size(data))(1),(size(data))(2)))
pnewdetnum=ptr_new(fltarr((size(detnum))(1)))
;remove degeneracies
;
(*pnewdetnum)[0]=detnum[0]
(*pnewdata)[*,0]=data[*,0]
j=0
for i=1,n_elements(detnum)-1 do begin
    if detnum(i) ne detnum(i-1) then begin
j+=1
(*pnewdetnum)[j]=detnum[i]
(*pnewdata)[*,j]=data[*,i]
endif else begin
(*pnewdata)[*,j]+=data[*,i]
endelse
endfor
data=(*pnewdata)[*,0:j]
error=sqrt(data)
detnum=(*pnewdetnum)[0:j]
ptr_free,pnewdata
ptr_free,pnewdetnum
;
*self.dataPtr=data
*self.errorPtr=error
*self.yvalsPtr=detnum
self.ndet=n_elements(detnum)


;print,omitted
n_omitted=n_elements(omitted)
;print,n_omitted
if n_omitted gt 0 then begin
    for i = 0, n_omitted-1 do begin
    if i eq 0 then begin
    omit_string =strtrim(string(omitted[0]),2)
    endif else begin
    omit_string =omit_string+','+strtrim(string(omitted[i]),2)
    endelse
    endfor
    omit_string='Detector numbers of Those Excluded: '+omit_string
    treat = ['Total Number of Detectors Read From File: '+strtrim(string(totaldet),2)+'; ', $
       'Number of Detectors Excluded in the Reduction: '+strtrim(string(removed),2)+'; ', $
       omit_string+'; ', $
       strtrim(string(remaining),2)+' Detectors Reduced to '+strtrim(string(self.ndet),2)+ $
       ' With Unique Two Theta Angles'+'. ']
endif else begin
treat = ['Total Number of Detectors Read From File: '+strtrim(string(totaldet),2)+'; ', $
       'Number of Detectors Excluded in the Reduction: '+strtrim(string(removed),2)+'; ', $
       strtrim(string(remaining),2)+' Detectors Reduced to '+strtrim(string(self.ndet),2)+ $
       ' With Unique Two Theta Angles'+'. ']
endelse


*self.treatmentPtr = [*self.treatmentPtr,treat]


;print,'sam ndet =',self.ndet
;print,*self.good_detPtr

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::sum_runs,event = event,err = err
err = 0
if n_elements(*self.sigFilePtr) eq 0 then begin
  err = -1
  return
endif

files = *self.sigFilePtr
nfiles = n_elements(files)
;print,'nfiles = ',nfiles
treat = 'Files used in sum: '
for i = 0,nfiles-1 do begin
  treat = [treat,files[i]]
  self->Nexus_read_raw_focus,files[i],event = event,err = err
  if err eq 1 then return
  if i eq 0 then begin
    data = *self.dataPtr
    temperature = *self.tempPtr
    Mon_cum = *self.Mon_cumPtr
    Mon_tc = *self.Mon_tcPtr
  endif else begin
  ;print,'data[544,17] = ',data[544,17]
    data = data+*self.dataPtr
      ;print,'data[544,17] = ',data[544,17]
    temperature = [temperature,*self.tempPtr]
    Mon_cum = [Mon_cum,*self.Mon_cumPtr]
    ;Mon_tc = [Mon_tc,*self.Mon_tcPtr]
    ;Mon_cum += *self.Mon_cumPtr
    Mon_tc += *self.Mon_tcPtr

  endelse
endfor

*self.treatmentPtr = [*self.treatmentPtr,treat]
error = sqrt(data)
where_zero = where(data eq 0,count_zero)
if count_zero gt 0 then error[where_zero] = 1.0
*self.errorPtr = error
*self.dataPtr = data
*self.Mon_cumPtr = Mon_cum
*self.Mon_tcPtr = Mon_tc
*self.tempPtr = temperature

return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::subtract_background,event = event
;
; In order to subtract the background we must do a few things first.....
;
; 1.  Pull out the signal data and store it in local variables.  We are assuming
;     that the data has been converted to energy.
; 2.  Read in and sum up all of the background files.
; 3   Remove bad detectors, sort, and remove degeneracies the same way as applied for the signal data
; 4.  Convert to energy and normalize to the monitor.
; 5.  Rebin the background onto the same energy grid as the signal file.
; 6.  Perform the subtraction scaling the background up to the signal file.
;
if n_elements(*self.dataPtr) eq 0 then return
;
if n_elements(*self.bgFilePtr) eq 0 then return
nbgFiles = n_elements(*self.bgFilePtr)
bgFiles = *self.bgFilePtr

; Store the signal data locally while determining the background signal
sigData = *self.dataPtr
sigError = *self.errorPtr
sigx = *self.xvalsPtr
sigy = *self.yvalsPtr
sigTemp = *self.tempPtr
sigMon_cum = *self.Mon_cumPtr
sigMon_tc = *self.Mon_tcPtr
signdet=self.ndet
treat = 'Background file(s) used in subtraction:'
; Sum the background files if there are more than one
for i = 0,nbgfiles-1 do begin
  treat = [treat,bgfiles[i]]
    self->Nexus_read_raw_focus,bgfiles[i],event = event
  if i eq 0 then begin
    bgdata = *self.dataPtr
;    help,bgdata
    bgtemperature = *self.tempPtr
    bgMon_cum = *self.Mon_cumPtr
    bgMon_tc = *self.Mon_tcPtr
  endif else begin
    bgdata = bgdata+*self.dataPtr
    bgtemperature = [bgtemperature,*self.tempPtr]
    bgMon_cum = [bgMon_cum,*self.Mon_cumPtr]
    ;bgMon_tc = [bgMon_tc,*self.Mon_tcPtr]
    bgMon_tc += *self.Mon_tcPtr

  endelse
endfor
detnum=*self.yvalsPtr
good_det=*self.good_detPtr
pnewdata=ptr_new(fltarr((size(bgdata))(1),n_elements(good_det)))
pnewdetnum=ptr_new(fltarr(n_elements(good_det)))
;
*pnewdata=bgdata(*,good_det)
*pnewdetnum=detnum(good_det)
bgData=*pnewdata
detnum=*pnewdetnum
ptr_free,pnewdata
ptr_free,pnewdetnum
;
;print,'detnum =',detnum
;print,'good_det =',good_det
;print,'n_elements(good_det) =',n_elements(good_det)


;sort the data in ascending order of detnum
;
sortorder=sort(detnum)
detnum=detnum(sortorder)
bgdata=bgdata(*,sortorder)
;print,detnum
;
pnewdata=ptr_new(fltarr((size(bgdata))(1),(size(bgdata))(2)))
pnewdetnum=ptr_new(fltarr((size(detnum))(1)))
;remove degeneracies
;
(*pnewdetnum)[0]=detnum[0]
(*pnewdata)[*,0]=bgdata[*,0]
j=0
for i=1,n_elements(detnum)-1 do begin
    if detnum(i) ne detnum(i-1) then begin
j+=1
(*pnewdetnum)[j]=detnum[i]
(*pnewdata)[*,j]=bgdata[*,i]
endif else begin
(*pnewdata)[*,j]+=bgdata[*,i]
endelse
endfor
bgData=(*pnewdata)[*,0:j]
detnum=(*pnewdetnum)[0:j]
ptr_free,pnewdata
ptr_free,pnewdetnum
;
*self.treatmentPtr = [*self.treatmentPtr,treat]
*self.dataPtr = bgData
bgError = sqrt(bgData)
*self.errorPtr = bgError
*self.yvalsPtr = detnum
*self.tempPtr = bgtemperature
*self.Mon_cumPtr = bgMon_cum
*self.Mon_tcPtr = bgMon_tc
; Convert the background files to energy
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
;convert to energy unless the desired quqntity is S(theta,t)
if DataOutput ne 2 then self->convert_to_energy,event = event
bgx = *self.xvalsPtr
bgData = *self.dataPtr
bgError = *self.errorPtr
sigMon = total(sigMon_cum)
bgMon = total(bgMon_cum)
sf = (1.0*sigMon/bgMon)
newData = sigData
newError = sigError

;help, newdata
;help,bgdata
; Cast the scale factor into an appropriate form
datSize = size(sigData)
nchan = datSize[1]
ndet = datSize[2]

sfm = (1+bytarr(nchan))#(1+fltarr(ndet))
newData[*,0:(ndet-1)] = 1.0*sigData[*,0:(ndet-1)] - $
                           sf*((bgData[*,0:(ndet-1)])*sfm)
newError[*,0:(ndet-1)] = sqrt(sigError[*,0:(ndet-1)]^2+ $
                                 (sf*(bgError[*,0:(ndet-1)])*sfm)^2)
;help,newerror
;print,'sigerror[860,8] =',sigerror[860,8]
;print,'newerror[860,8] =', newerror[860,8]
;print,'total(sigerror) =',total(sigerror)
;print,'total(newerror) =',total(newerror)

;print,'sfm =',sfm
;print,'sf =',sf
;print,sigData[fix(nchan/2.0),10]
;print,bgData[fix(nchan/2.0),10]
;print,newData[fix(nchan/2.0),10]

*self.xvalsPtr = sigx
*self.dataPtr = newData
*self.errorPtr = newError
*self.tempPtr = [sigTemp];[sigTemp,bgtemperature]
*self.Mon_cumPtr = [sigMon_cum,bgMon_cum]
*self.Mon_tcPtr = [sigMon_tc,bgMon_tc]
;print,'*self.Mon_cumPtr =',*self.Mon_cumPtr
self.ndet=signdet
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::flat_baseline,event = event
widget_control,self.flat_baseline_range,get_value = range

; Store the signal data locally
z = *self.dataPtr
;dz = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
nx=n_elements(x)
ny=n_elements(y)
;
if range lt 2 or range gt nx then begin
range = 100
help,range
widget_control,self.flat_baseline_range,set_value = range
endif
range=fix(range[0])
;
for i=0,ny-1 do begin
;print,'initial min =',min(z[*,i])
j=0
mean=0
tempmean=(moment(z[j:(j+range-1),i]))[0]
mean=tempmean
j+=range
while (j+range-1) le nx-1 do begin
tempmean=(moment(z[j:(j+range-1),i]))[0]
    if tempmean lt mean then mean = tempmean
j+=range
endwhile
       if j le nx-1 then begin
tempmean=(moment(z[j:*,i]))[0]
if tempmean lt mean then mean = tempmean
z[*,i]-=mean
;print,'mean =',mean
       endif
;print,'final min =',min(z[*,i])
endfor
;whereltzero=where(z lt 0,count)
;if count ne 0 then begin
;z[whereltzero]=0.0
;endif

treat = ['Flat Baseline Correction Has Been Applied to the Data.  Number of Channels: '+strtrim(string(range),2)]
    *self.treatmentPtr = [*self.treatmentPtr,treat]


;dz=sqrt(z)
*self.dataPtr=z
;*self.errorPtr=dz
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::convert_to_energy,event = event

;return
;
; Store the relevant signal data locally
sigData = *self.dataPtr
sigError = *self.errorPtr
sigx = *self.xvalsPtr
sigy = *self.yvalsPtr
ny=n_elements(sigy)
nx=n_elements(sigx)
instrument_details = *self.InstrumentPtr
tof_corr_structure=*self.tof_corr_structurePtr
;Define Plank's constant and neutron mass
h=6.6260755e-34
mn=1.67495e-27
;joules to meV conversion factor
jtomeV=6.241504661e21
;Extract lambda zero
lambda0=tof_corr_structure.lambdatof
;Extract TOF shift
tof_shift=tof_corr_structure.tof_shift
;Add to tof values
;print, 'lambda0 =',lambda0
;help,lambda0

;print, 'sigx =',sigx
;help,sigx

sigx +=tof_shift
;print, 'sigxcorr =',sigx
;help,sigx
;Extract Distances
FCtoSAM=instrument_details.FCtoSAM
FCtoDET=instrument_details.FCtoDET
SAMtoDET=instrument_details.SAMtoDET
;print,'instrument_Details =',instrument_details
;
;Calculate t0, the time for all neutrons to travel from the FC to the Sample
t0=FCtoSAM/(h*1e10/(lambda0*mn))
;help,t0
;subtract this value from sigx to get tf (flight time in the secondary instrument)
;convert to seconds
sigx/=1e6
tf=sigx-t0
;help,tf
nx = n_elements(tf)
;print,'nx =',nx
;  calculate v0, k0 and E0
;print,'lambda0 =',lambda0
;
v0=(h*1e10/(lambda0*mn))
;print,'v0 =',v0
k0=2.0*!dpi*1e10/lambda0
;print,'k0 =',k0
E0=mn*v0^2/2.0
;that should give E0 in joules, now convert to meV
E0*=jtomeV
vf=SAMtoDET/tf
lambdaf=(h/(vf*mn))*1e10
kf=2.0*!dpi*1e10/lambdaf
Ef=mn*vf^2/2.0
Ef*=jtomeV
E_trans=E0-Ef

norm=(total(tf^4))/nx

;help,norm
;print,norm

;print, '(size(sigdata))[2] =',(size(sigdata))[2]
for i = 0, (size(sigdata))[2]-1 do begin
sigdata(*,i) = sigData(*,i)*(tf^4)/norm
sigerror(*,i) = sigerror(*,i)*(tf^4)/norm
endfor


whereltzero=where(sigdata lt 0,count1)
IF count1 ne 0 then begin
for i=0,ny-1 do begin
whereltzero=where(sigdata[*,i] lt 0,count)
if count ne 0 then begin
;print,'wooooof',count,n_elements(sigdata)
minz=min(sigdata[*,i])
;print,minz
sigdata[*,i]-=minz
endif
endfor
ENDIF
wherezero=where(sigerror eq 0,count)
if count ne 0 then begin
mindz=min(sigerror)
sigerror[wherezero]=mindz
endif
;   Is Detailed balance requested:
widget_control,self.Misc1Group,get_value = val
if val[0] eq 1 then begin
;print,E_trans
c=2.0*instrument_details.temp*0.08617385692

for j = 0, ny-1 do begin
for i = 0, nx-1 do begin
sigdata[i,j]*=exp(-E_trans(i)/c)
sigerror[i,j]*=exp(-E_trans(i)/c)
endfor
endfor

treat = ['Detailed Balance Correction Has Been Applied to the Data.']
*self.treatmentPtr = [*self.treatmentPtr,treat]


endif

;help,sigdata
*self.dataPtr=sigdata
*self.errorPtr=sigerror
*self.xvalsPtr=E_trans
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::display,event = event
; This method plots the data (if present) to a pixmap then to the screen.
wset,self.winPix
self->plotData,event = event
wset,self.winVis
device,copy = [0,0,!d.x_size,!d.y_size,0,0,self.winPix]
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::resize,event = event
; This method allows the user to resize the application interface while changing
; the size of the plot window accordingly.

ctrlgeom = widget_info(self.ctrlbase,/geometry)
tlbgeom = widget_info(self.tlb,/geometry)
xsize = event.x
ysize = event.y

; New data window dimensions
newxsize = xsize-ctrlgeom.xsize
newysize = ysize > ctrlgeom.ysize

widget_control,self.win,draw_xsize = newxsize, $
               draw_ysize = newysize
wdelete,self.winPix
window,/free,/pixmap,xsize = newxsize,ysize = newysize
self.winPix = !d.window
self->display,event = event
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::treatmentHandler,event = event
; This method responds to changes in the state of the
; treatment control by sensitizing or desensitizing appropriate
; buttons as necessary.
widget_control,self.treatGroup,get_value = vals
widget_control,self.vanButton,sensitive = vals[0]
widget_control,self.bgButton,sensitive = vals[1]
;
widget_control,self.DiffONGroup,get_value = DiffON
widget_control,self.Diffraction_Base1,sensitive = vals[0]
if vals[0] eq 0 then widget_control,self.Diffraction_Base2,sensitive = vals[0]
if vals[0] eq 1 and DiffON eq 1 then widget_control,self.Diffraction_Base2,sensitive = vals[0]
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::LambdaHandler,event = event
; This method responds to changes in the state of the
; lambda own field by sensitizing or desensitizing according to whether
;   the own value button is selected
widget_control,self.elasticGroup,get_value = val
if val eq 0 then widget_control,self.elastic_base_sensitive,sensitive = 0
if val eq 1 then widget_control,self.elastic_base_sensitive,sensitive = 0
if val eq 2 then widget_control,self.elastic_base_sensitive,sensitive = 0
if val eq 3 then widget_control,self.elastic_base_sensitive,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::E_TOF_handler,event = event
; This method responds to changes in the state of the
; lambda own field by sensitizing or desensitizing according to whether
;   the own value button is selected
widget_control,self.ebin_group,get_value = val
if val eq 0 then widget_control,self.E_TOF_base1,sensitive = 0
if val eq 1 then widget_control,self.E_TOF_base1,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::q_theta_handler,event = event
widget_control,self.qbin_group,get_value = val
if val eq 0 then widget_control,self.q_theta_base1,sensitive = 0
if val eq 1 then widget_control,self.q_theta_base1,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::OutputName_handler,event = event
widget_control,self.Output_nameGroup,get_value = val
if val eq 0 then begin
widget_control,self.Output_name_subbase_sensitive2,sensitive = 0
widget_control,self.stem2,set_value=''
endif
if val eq 1 then begin
widget_control,self.Output_name_subbase_sensitive2,sensitive = 0
widget_control,self.stem2,set_value=''
endif
if val eq 2 then widget_control,self.Output_name_subbase_sensitive2,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::rungrouphandler,event = event
widget_control,self.runGroup,get_value = rungroup
if rungroup eq 1 and n_elements(*self.sigFilePtr) eq 1 then begin
rungroup=0
  widget_control, self.runGroup, set_value=runGroup
endif
widget_control,self.stem1,get_value = stem1
stem1_length=strlen(stem1)
if stem1_length eq 0 then return
stem1_end=strmid(stem1,(stem1_length-2),2)
;print,stem1_end
if rungroup eq 1 and stem1_end ne '++' then stem1+='++'
if rungroup eq 0 and stem1_end eq '++' then stem1=strmid(stem1,0,(stem1_length-2))


widget_control,self.stem1,set_value = stem1
widget_control,self.diffstem1,set_value = stem1

;stem1_length
;print,'woof'



;if val eq 0 then widget_control,self.Output_name_subbase_sensitive2,sensitive = 0
;if val eq 1 then widget_control,self.Output_name_subbase_sensitive2,sensitive = 0
;if val eq 2 then widget_control,self.Output_name_subbase_sensitive2,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::Misc1Handler,event = event
widget_control,self.Misc1Group,get_value = val
;print,'val =',val
if val[0] eq 1 then widget_control,self.DataOutputTypesGroup,set_value = 0
if val[1] eq 0 then widget_control,self.Misc_Base2,sensitive = 0
if val[1] eq 1 then widget_control,self.Misc_Base2,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::Data_Output_Handler,event = event
widget_control,self.DataOutputTypesGroup,get_value = val
widget_control,self.FormatTypesGroup,get_value = format
;print,'val =',val
if format eq 3 then widget_control,self.DataOutputTypesGroup,set_value=1
;
widget_control,self.Misc1Group,get_value = Misc1Group
if Misc1Group[0] eq 1 then widget_control,self.DataOutputTypesGroup,set_value=0
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::qbin_Types1_handler,event = event
widget_control,self.qbin_group1,get_value = val
;print,'val =',val
if val eq 1 then widget_control,self.Bad_DetectorGroup,set_value=3
if val eq 2 then widget_control,self.Bad_DetectorGroup,set_value=3
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::Bad_DetectorGroup_Handler,event = event
widget_control,self.Bad_DetectorGroup,get_value=val
;print,'val =',val
if val ne 3 then widget_control,self.qbin_group1,set_value = 0
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::FormatTypes,event = event
widget_control,self.FormatTypesGroup,get_value = vals
if vals eq 0 then text='dyn.dave'
if vals eq 1 then text='.itx'
if vals eq 2 then text='.txt'
if vals eq 3 then text='.spe'
widget_control, self.stem3, set_value=text
if vals eq 3 then widget_control,self.DataOutputTypesGroup,set_value = 1
;
;
;
;if vals[0] eq 0 then begin
;vals[0] = 1
;widget_control,self.FormatTypesGroup,set_value = vals
;endif
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::DiffONHandler,event = event
widget_control,self.DiffONGroup,get_value=val
;print,'val =',val
if val eq 0 then widget_control,self.Diffraction_Base2,sensitive = 0
if val eq 1 then widget_control,self.Diffraction_Base2,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::DiffractionCalcHandler,event = event
widget_control,self.DiffractionCalcGroup,get_value=val
;print,'val =',val
if val eq 0 then widget_control,self.Diffraction_Base3,sensitive = 0
if val eq 1 then widget_control,self.Diffraction_Base3,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::DiffOutputName_handler,event = event
widget_control,self.DiffOutput_nameGroup,get_value = val
if val eq 0 then begin
widget_control,self.DiffOutput_name_subbase_sensitive2,sensitive = 0
widget_control,self.Diffstem2,set_value=''
endif
if val eq 1 then begin
widget_control,self.DiffOutput_name_subbase_sensitive2,sensitive = 0
widget_control,self.Diffstem2,set_value=''
endif
if val eq 2 then widget_control,self.DiffOutput_name_subbase_sensitive2,sensitive = 1
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::flashMessage_create,msg,tlb
; This method simply flashes a widget with a message that remains present
; until the flashMessage_destroy method is invoked.
;
; Center it.
geom = widget_info(self.tlb, /geometry)
xpos = geom.xoffset + geom.xsize/2 - 100
ypos = geom.yoffset + geom.ysize/2 - 50

tlb = widget_base(title='Reduction status:',/row,xoffset=xpos,yoffset=ypos, $
      tlb_frame_attr = 3)
void = widget_text(tlb,value = msg,xsize = strlen(msg),/editable)
widget_control,tlb,/realize
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::flashMessage_destroy,tlb
widget_control,tlb,/destroy
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::reduce,event = event
if n_elements(*self.sigFilePtr) eq 0 then return
files = *self.sigFilePtr
if n_elements(*self.treatmentPtr) ne 0 then begin
  ptr_free,self.treatmentPtr
  self.treatmentPtr = ptr_new(/allocate_heap)
;*self.treatmentPtr = 'Data reduction/treatment details:'
endif
; How do we treat the data?
widget_control,self.treatGroup,get_value = treatVal
;What is the output format of the data?
widget_control,self.FormatTypesGroup,get_value = FormatType

; Do we treat these individually or sum them?
widget_control,self.runGroup,get_value = thisValue
;
;   What quantity do we wish to calculate?
;
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
;
; if DataOutput eq 2 then S(theta,t)
;
;   Do we specify a file name(1) or is the file name automatically assigned(0)
widget_control,self.Output_nameGroup,get_value = Output_name
;
;   detailed balance and flat baseline options are stored in  Misc1Group
    widget_control,self.Misc1Group,get_value = Misc1Group
;
;
;   Is the diffraction pattern requested?
    widget_control,self.DiffONGroup,get_value = DiffONGroup
;
;
;
if FormatType eq 0    then begin
filter = ['*.dave']
title = 'DAVE output filename'
endif
;
if FormatType eq 1    then begin
filter = ['*.itx']
title = 'IGOR output filename'
endif
;
if FormatType eq 2    then begin
filter = ['*.txt']
title = 'ASCII output filename'
endif
;
if FormatType eq 3    then begin
filter = ['*.spe']
title = 'MATLAB output filename'
endif
;
;
;
;
;
;
    if thisValue[0] eq 0 then begin   ; treat individually

  nfiles = n_elements(files)

  for i = 0,nfiles-1 do begin
   *self.treatmentPtr = 'Data reduction/treatment details:'
   msg = 'Reading in a raw focus file'
   self->flashMessage_create,msg,tlb
   self->Nexus_read_raw_focus,files[i],event = event,err = err ; always
   self->flashMessage_destroy,tlb
   if err eq 1 then return
;
Sample_Name={sample_name,  $
         ftime:(*self.InstrumentPtr).ftime,  $
         sample:(*self.InstrumentPtr).sample,  $
         foriname:(*self.InstrumentPtr).foriname,  $
         title:(*self.InstrumentPtr).title}

*self.Sample_NamePtr=Sample_Name



   msg = 'Subtracting Bad Detectors'
   self->flashMessage_create,msg,tlb
   self->subtract_bad_detectors,event = event    ; always
   self->flashMessage_destroy,tlb
;
;
;
;
   msg = 'Calculating Wavelength '
   self->flashMessage_create,msg,tlb
   self->TOF_corr,event=event   ;always
   self->flashMessage_destroy,tlb
;
;   We now have all we need to calculate the diffraction pattern
    if DiffONGroup eq 1 then begin
       msg = 'Calculating Diffraction Pattern '
   self->flashMessage_create,msg,tlb
   self->Diffraction,event=event
   self->flashMessage_destroy,tlb
    endif
;
;

     if n_elements(*self.bgFilePtr) eq 0 and Misc1Group[1] eq 1 then begin  ; subtract flat baseline
    msg = 'Flat Baseline Routine    '
    self->flashMessage_create,msg,tlb
    self->flat_baseline,event = event
    self->flashMessage_destroy,tlb
    endif
;
if DataOutput ne 2 then begin
   msg = 'Converting to Energy'
   self->flashMessage_create,msg,tlb
   self->convert_to_energy,event = event
   self->flashMessage_destroy,tlb
endif else begin
self.xlabel = 'Time / usec'
endelse


  if n_elements(*self.bgFilePtr) ne 0 then begin  ; subtract background
   msg = 'Subtracting Background File(s)'
   self->flashMessage_create,msg,tlb
   self->subtract_background,event = event
   self->flashMessage_destroy,tlb
  endif



    if treatVal[0] eq 1 then begin
     msg = 'Normalising to Vanadium Run'
     self->flashMessage_create,msg,tlb
     self->normalize_to_vanadium,event = event   ; normalize to vanadium
     self->flashMessage_destroy,tlb
    endif

    ; Do the rebinning at this point
   msg = 'Energy/TOF Rebinning    '
   self->flashMessage_create,msg,tlb
   self->ebinning,event=event
   self->flashMessage_destroy,tlb

   msg = 'Q/Theta Rebinning     '
   self->flashMessage_create,msg,tlb
   self->qbinning,event=event
   self->flashMessage_destroy,tlb

    if  Output_name eq 0 then begin
  filename_result = strsplit(files[i],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)


   fileOut = filepath(filename,root_dir = self.workDir)
   endif


       if Output_name eq 1 then begin
;
; First extract the filename
  filename_result = strsplit(files[i],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)
  ;extPos = strpos(files[0],'.focus')
  ;filename = strmid(files[0],extPos-11,11)

  fileExample = filename
  fileOut = dialog_pickfile(dialog_parent = event.top,/write, $
  ;                          file = filename, $
                            file = filename, $ ; following IGOR
                            filter = filter,$
                            title = title,$
                            path = self.workDir)
;   fileOut = filepath(filename+'++',root_dir = self.workDir)

;
       endif

       if  Output_name eq 2 then begin
  filename_result = strsplit(files[i],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)

   fileOut = filepath(filename,root_dir = self.workDir)
   widget_control, self.stem2, get_value=stem_add
    if strlen(stem_add) gt 0 then    fileOut+=stem_add

   endif


    ;print,fileout
FilePar=fileout+'.par'
FilePhx=fileout+'.phx'

;
if FormatType eq 0    then begin
fileOut+= 'dyn.dave' ;Dave File
  msg = 'Writing to a DAVE file...'
  self->flashMessage_create,msg,tlb
  self->writeDave,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif
;
if FormatType eq 1    then begin
fileOut+= '.itx' ; IGOR Text File
  msg = 'Writing to an IGOR Text file...'
  self->flashMessage_create,msg,tlb
  self->writeIGOR,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif
;
if FormatType eq 2  then begin
fileOut+= '.txt' ; ASCII File
  msg = 'Writing to an ASCII file...'
  self->flashMessage_create,msg,tlb
  self->writeASCII,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif
;
if FormatType eq 3  then begin
fileOut+= '.spe' ; Matlab File
  msg = 'Writing to a Matlab file...'
  self->flashMessage_create,msg,tlb
  self->writeMatlab,event = event,fileout = fileout,FilePar = FilePar,FilePhx = FilePhx
  self->flashMessage_destroy,tlb
endif

   treat = 'Filename: '+strtrim(files[i],2)
   *self.treatmentPtr = [*self.treatmentPtr,treat]

   if n_elements(*self.treatmentPtr) ne 0 then begin
      ptr_free,self.treatmentPtr
      self.treatmentPtr = ptr_new(/allocate_heap)
   endif
  endfor

    endif else begin         ; sum runs together
   *self.treatmentPtr = 'Data reduction/treatment details:'
   msg = 'Reading in files to sum'
   self->flashMessage_create,msg,tlb
   self->sum_runs, event = event, err = err
   self->flashMessage_destroy,tlb
   if err eq 1 then return
;
Sample_Name={sample_name,  $
         ftime:(*self.InstrumentPtr).ftime,  $
         sample:(*self.InstrumentPtr).sample,  $
         foriname:(*self.InstrumentPtr).foriname,  $
         title:(*self.InstrumentPtr).title}

*self.Sample_NamePtr=Sample_Name


   msg = 'Subtracting Bad Detectors'
   self->flashMessage_create,msg,tlb
   self->subtract_bad_detectors,event = event    ; always
   self->flashMessage_destroy,tlb
;
;
;
   msg = 'Calculating Wavelength '
   self->flashMessage_create,msg,tlb
   self->TOF_corr,event=event   ;always
   self->flashMessage_destroy,tlb
;
;   We now have all we need to calculate the diffraction patter
    if DiffONGroup eq 1 then begin
       msg = 'Calculating Diffraction Pattern '
   self->flashMessage_create,msg,tlb
   self->Diffraction,event=event
   self->flashMessage_destroy,tlb
    endif
;
;
     if n_elements(*self.bgFilePtr) eq 0 and Misc1Group[1] eq 1 then begin  ; subtract flat baseline
    msg = 'Flat Baseline Routine    '
    self->flashMessage_create,msg,tlb
    self->flat_baseline,event = event
    self->flashMessage_destroy,tlb
    endif
;
;
if DataOutput ne 2 then begin
   msg = 'Converting to energy'
   self->flashMessage_create,msg,tlb
   self->convert_to_energy,event = event
   self->flashMessage_destroy,tlb
endif else begin
self.xlabel = 'Time / usec'
endelse



; Ok, we've performed all of the essential reduction treatment.  Now
; let's do any further steps that the user wishes.

  if n_elements(*self.bgFilePtr) ne 0 then begin  ; subtract background
   msg = 'Subtracting Background File(s)'
   self->flashMessage_create,msg,tlb
   self->subtract_background,event = event
   self->flashMessage_destroy,tlb
  endif

;
  if treatVal[0] eq 1 then begin
   msg = 'Normalising to Vanadium Run'
   self->flashMessage_create,msg,tlb
   self->normalize_to_vanadium,event = event    ; normalize to vanadium
   self->flashMessage_destroy,tlb
  endif


    ; Do the rebinning at this point
   msg = 'Energy/TOF Rebinning    '
   self->flashMessage_create,msg,tlb
   self->ebinning,event=event
   self->flashMessage_destroy,tlb

   msg = 'Q/Theta Rebinning         '
   self->flashMessage_create,msg,tlb
   self->qbinning,event=event
   self->flashMessage_destroy,tlb



   if  Output_name eq 0 then begin
  filename_result = strsplit(files[0],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)


   fileOut = filepath(filename,root_dir = self.workDir)
   fileout+='++'
   endif



       if  Output_name eq 1 then begin
;
; First extract the filename
  filename_result = strsplit(files[0],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)
  ;extPos = strpos(files[0],'.focus')
  ;filename = strmid(files[0],extPos-11,11)

  fileExample = filename
  fileOut = dialog_pickfile(dialog_parent = event.top,/write, $
  ;                          file = filename, $
                            file = filename+'++', $ ; following IGOR
                            filter = filter,$
                            title = title,$
                            path = self.workDir)
;   fileOut = filepath(filename+'++',root_dir = self.workDir)

  endif
  ;
  ;
       if  Output_name eq 2 then begin
  filename_result = strsplit(files[0],path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)

   fileOut = filepath(filename,root_dir = self.workDir)
      fileout+='++'
   widget_control, self.stem2, get_value=stem_add
    if strlen(stem_add) gt 0 then    fileOut+=stem_add
   endif

;
FilePar=fileout+'.par'
FilePhx=fileout+'.phx'

if FormatType eq 0    then begin
fileOut+= 'dyn.dave' ;Dave File
  msg = 'Writing to a DAVE file...'
  self->flashMessage_create,msg,tlb
  self->writeDave,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif
;
if FormatType eq 1    then begin
fileOut+= '.itx' ; IGOR Text File
  msg = 'Writing to an IGOR Text file...'
  self->flashMessage_create,msg,tlb
  self->writeIGOR,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif
;
if FormatType eq 2  then begin
fileOut+= '.txt' ; ASCII File
  msg = 'Writing to an ASCII file...'
  self->flashMessage_create,msg,tlb
  self->writeASCII,event = event,fileout = fileout,FilePar = FilePar
  self->flashMessage_destroy,tlb
endif

if FormatType eq 3  then begin
fileOut+= '.spe' ; Matlab File
  msg = 'Writing to a Matlab file...'
  self->flashMessage_create,msg,tlb
  self->writeMatlab,event = event,fileout = fileout,FilePar = FilePar,FilePhx = FilePhx
  self->flashMessage_destroy,tlb
endif


    ;print,fileout
;

    endelse



self->display,event = event
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::writeDave,event = event,fileout = fileout,FilePar = FilePar
; This method populates the DAVE pointer in memory and saves the data into
; a "*.DAVE" file.
treat = 'Converted to DAVE format'
*self.treatmentPtr = [*self.treatmentPtr,treat]

n = n_elements(*self.treatmentPtr)
;for i = 0,n-1 do print,(*self.treatmentPtr)[i]
davePtr = self.davePtr

data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
;print,'Number of groups:',self.ndet
; Ok, start filling up the dave pointer...
; First, fill up the data...



;help,data
datSize = size(data)
nchan = datSize[1]
if datSize[0] eq 2 then begin
ndet = datSize[2]
endif else begin
ndet=1
endelse
Sample_Name=*self.Sample_NamePtr

Instrument_Details=*self.InstrumentPtr

;Sample_Name={sample_name,  $
;         ftime:(*self.InstrumentPtr).ftime,  $
;         sample:(*self.InstrumentPtr).sample,  $
;         foriname:(*self.InstrumentPtr).foriname,  $
;         title:(*self.InstrumentPtr).title}
Instrument_Details.ftime=Sample_Name.ftime
Instrument_Details.sample=Sample_Name.sample
Instrument_Details.foriname=Sample_Name.foriname
Instrument_Details.title=Sample_Name.title



header=*self.headerPtr
nh = n_elements(*self.headerPtr)
detSize = (size(data))[2]
temperature=*self.tempPtr
;
specific = {nchan:nchan,$
            ndet:ndet,$
            header:header, $
            phi:*self.detPtr,$
            instrument:'focus',$
            wavelength:(*self.tof_corr_structurePtr).lambdatof,$
            date:systime(),$
            temp_sample:temperature,$
            sumMon_tc:total(*self.Mon_tcPtr),$
            sumMon_cum:total(*self.Mon_cumPtr),$
            Instrument_details:Instrument_Details $
            }


;print,'lambda =',(*self.tof_corr_structurePtr).lambdatof

;*(*(*davePtr).dataStrPtr).specificPtr = specific

hbar_omega = '!6!sh!r!e/!n !7x!6'
if self.units ne 3 then begin
xlabel = hbar_omega
endif else begin
xlabel = 't'
endelse

if self.units eq 0 then xunits = 'Energy / meV'
if self.units eq 1 then xunits = 'Energy / Wavenumbers'
if self.units eq 2 then xunits = 'Energy / GHz'
if self.units eq 3 then xunits = 'Time / Microseconds'
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
if DataOutput[0] eq 0 then begin
yunits = 'wavevector:A-1'
ylabel = 'Q '
endif
if DataOutput[0] eq 1 or DataOutput[0] eq 2 then begin
yunits = 'Phi:degrees'
ylabel = 'Phi '
endif

histlabel = 'I (arbitrary units)'
histunits = ''

Mon_cumTreat = 'Total incident beam monitor counts: '+ $
           strtrim(string(total(*self.Mon_cumPtr)),2)
*self.treatmentPtr = [*self.treatmentPtr,Mon_cumTreat]
msg = 'Number of channels: '+strtrim(string(nchan),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]
msg = 'Number of groups: '+strtrim(string(ndet),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]

;*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr = *self.treatmentPtr
if n_elements(temperature) gt 1 then temperature = (moment(temperature))[0]
retval = create_dave_pointer(self.davePtr,   $
     qty = data,   $
     err = error,    $
     xvals = x,            $
     yvals = y,      $
     specificstr = specific,      $
     instrument = 'FOCUS',      $
     xtype = 'POINTS',        $
     ytype = 'POINTS',        $
     xlabel = xlabel,         $
     ylabel = ylabel,         $
     qtlabel = histlabel,      $
     xunits = xunits,         $
     yunits = yunits,         $
     qtunits = histunits,      $
     dname = 'T',          $
     dunits = 'temperature:K',     $
     dlegend = 'Temperature',    $
     dqty = temperature,         $
     derr = 0.0,            $
     treatment = *self.treatmentptr,   $
     ermsg = ermsg           )



if n_elements(fileout) eq 0 then return
if (*self.daveFiles)[0] eq '' then begin
  *self.daveFiles = fileOut
endif else begin
  *self.daveFiles = [*self.daveFiles,fileOut]
endelse
save, davePtr, filename = fileout

openw,lun,FilePar,/get_lun
printf,lun, 'Instrument: FOCUS'
printf,lun, 'Sample: '+strtrim(string(Sample_Name.sample),2)
printf,lun, 'Original File Name: '+strtrim(string(Sample_Name.foriname),2)
printf,lun, 'Title: '+strtrim(string(Sample_Name.title),2)
printf,lun, 'User: '+strtrim(string(Instrument_Details.owner),2)
printf,lun, 'Temperature / K: '+strtrim(string(temperature),2)
printf,lun, 'XType: Points'
printf,lun, 'Group Type: Points'
printf,lun, 'X Units: '+strtrim(string(xunits),2)
printf,lun, 'Y Units: '+strtrim(string(histlabel),2)
printf,lun, 'Group Units: '+strtrim(string(yunits),2)
printf,lun, 'Group Label: '+strtrim(string(ylabel),2)
printf,lun, 'Number of Channels: '+strtrim(string(nchan),2)
printf,lun, 'Number of Groups: '+strtrim(string(ndet),2)
printf,lun, 'Wavelength / A: '+strtrim(string((*self.tof_corr_structurePtr).lambdatof),2)
printf,lun, 'Date Data File Created: '+strtrim(string(Sample_Name.ftime),2)
printf,lun, 'Date This File Created: '+strtrim(string(systime()),2)
printf,lun, 'Fermispeed: '+strtrim(string(Instrument_Details.fermispeed),2)
printf,lun, 'Fermiphase: '+strtrim(string(Instrument_Details.fermiphase),2)
printf,lun, 'Diskspeed: '+strtrim(string(Instrument_Details.diskspeed),2)
printf,lun, *self.treatmentPtr
free_lun,lun,/force

end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::writeIGOR,event = event,fileout = fileout,FilePar = FilePar
;
if n_elements(fileout) eq 0 then return
;
treat = 'Converted to IGOR Text format'
*self.treatmentPtr = [*self.treatmentPtr,treat]

n = n_elements(*self.treatmentPtr)
;for i = 0,n-1 do print,(*self.treatmentPtr)[i]

data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
;print,'Number of groups:',self.ndet

;help,data
datSize = size(data)
nchan = datSize[1]
if datSize[0] eq 2 then begin
ndet = datSize[2]
endif else begin
ndet=1
endelse

header=*self.headerPtr
nh = n_elements(*self.headerPtr)
detSize = (size(data))[2]
temperature=*self.tempPtr
Instrument_Details=*self.InstrumentPtr

;Sample_Name={sample_name,  $
;         ftime:(*self.InstrumentPtr).ftime,  $
;         sample:(*self.InstrumentPtr).sample,  $
;         foriname:(*self.InstrumentPtr).foriname,  $
;         title:(*self.InstrumentPtr).title}

Sample_Name=*self.Sample_NamePtr


if self.units ne 3 then begin
xlabel = 'Energy'
endif else begin
xlabel = 't'
endelse

if self.units eq 0 then xunits = 'Energy / meV'
if self.units eq 1 then xunits = 'Energy / Wavenumbers'
if self.units eq 2 then xunits = 'Energy / GHz'
if self.units eq 3 then xunits = 'Time / Microseconds'
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
if DataOutput[0] eq 0 then begin
yunits = 'wavevector:A-1'
ylabel = 'Q '
endif
if DataOutput[0] eq 1 or DataOutput[0] eq 2 then begin
yunits = 'Phi:degrees'
ylabel = 'Phi '
endif

histlabel = 'I (arbitrary units)'
histunits = ''

Mon_cumTreat = 'Total incident beam monitor counts: '+ $
           strtrim(string(total(*self.Mon_cumPtr)),2)
*self.treatmentPtr = [*self.treatmentPtr,Mon_cumTreat]
msg = 'Number of channels: '+strtrim(string(nchan),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]
msg = 'Number of groups: '+strtrim(string(ndet),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]

;*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr = *self.treatmentPtr
if n_elements(temperature) gt 1 then temperature = (moment(temperature))[0]


openw,lun,fileout,/get_lun
printf,lun, 'IGOR'
for i = 0, ndet-1 do begin
;printf,lun, '|Group Number: '+strtrim(string(i+1),2)
;printf,lun, '|Group Value: '+strtrim(string(y[i]),2)
xval='''XValue_'+strtrim(string(y[i]),2)+''''+'  '
yval='''Intensity_'+strtrim(string(y[i]),2)+''''+'  '
yval_err='''dIntensity_'+strtrim(string(y[i]),2)+''''+'  '
printf,lun, 'WAVES/D ',xval,yval,yval_err
printf,lun, 'Begin'
    for j = 0, nchan-1 do begin
printf,lun, strtrim(string(x[j]),1)+'  ',strtrim(string(data[j,i]),1)+'  ',strtrim(string(error[j,i]),1)
        endfor
printf,lun, 'End'
printf, lun, 'X SetScale d 0,0,'+'"'+strtrim(string(xunits),2)+'"'+', ' +strtrim(string(xval),2)
printf, lun, 'X SetScale d 0,0,'+'"'+strtrim(string(histlabel),2)+' '+strtrim(string(ylabel),2)+  $
          ' '+strtrim(string(yunits),2)+  $
                   ' '+strtrim(string(y[i]),2)+' '+'"'+', '+strtrim(string(yval),2)
endfor

free_lun,lun,/force


openw,lun,FilePar,/get_lun
printf,lun, 'Instrument: FOCUS'
printf,lun, 'Sample: '+strtrim(string(Sample_Name.sample),2)
printf,lun, 'Original File Name: '+strtrim(string(Sample_Name.foriname),2)
printf,lun, 'Title: '+strtrim(string(Sample_Name.title),2)
printf,lun, 'User: '+strtrim(string(Instrument_Details.owner),2)
printf,lun, 'Temperature / K: '+strtrim(string(temperature),2)
printf,lun, 'XType: Points'
printf,lun, 'Group Type: Points'
printf,lun, 'X Units: '+strtrim(string(xunits),2)
printf,lun, 'Y Units: '+strtrim(string(histlabel),2)
printf,lun, 'Group Units: '+strtrim(string(yunits),2)
printf,lun, 'Group Label: '+strtrim(string(ylabel),2)
printf,lun, 'Number of Channels: '+strtrim(string(nchan),2)
printf,lun, 'Number of Groups: '+strtrim(string(ndet),2)
printf,lun, 'Wavelength / A: '+strtrim(string((*self.tof_corr_structurePtr).lambdatof),2)
printf,lun, 'Date Data File Created: '+strtrim(string(Sample_Name.ftime),2)
printf,lun, 'Date This File Created: '+strtrim(string(systime()),2)
printf,lun, 'Fermispeed: '+strtrim(string(Instrument_Details.fermispeed),2)
printf,lun, 'Fermiphase: '+strtrim(string(Instrument_Details.fermiphase),2)
printf,lun, 'Diskspeed: '+strtrim(string(Instrument_Details.diskspeed),2)
printf,lun, *self.treatmentPtr
free_lun,lun,/force


end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::writeASCII,event = event,fileout = fileout,FilePar = FilePar
;
if n_elements(fileout) eq 0 then return
;
treat = 'Converted to ASCII format'
*self.treatmentPtr = [*self.treatmentPtr,treat]

n = n_elements(*self.treatmentPtr)
;for i = 0,n-1 do print,(*self.treatmentPtr)[i]

data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
;print,'Number of groups:',self.ndet

;help,data
datSize = size(data)
nchan = datSize[1]
if datSize[0] eq 2 then begin
ndet = datSize[2]
endif else begin
ndet=1
endelse

header=*self.headerPtr
nh = n_elements(*self.headerPtr)
detSize = (size(data))[2]
temperature=*self.tempPtr
Instrument_Details=*self.InstrumentPtr

;Sample_Name={sample_name,  $
;         ftime:(*self.InstrumentPtr).ftime,  $
;         sample:(*self.InstrumentPtr).sample,  $
;         foriname:(*self.InstrumentPtr).foriname,  $
;         title:(*self.InstrumentPtr).title}

Sample_Name=*self.Sample_NamePtr


if self.units ne 3 then begin
xlabel = 'Energy'
endif else begin
xlabel = 't'
endelse

if self.units eq 0 then xunits = 'Energy / meV'
if self.units eq 1 then xunits = 'Energy / Wavenumbers'
if self.units eq 2 then xunits = 'Energy / GHz'
if self.units eq 3 then xunits = 'Time / Microseconds'
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
if DataOutput[0] eq 0 then begin
yunits = 'wavevector:A-1'
ylabel = 'Q '
endif
if DataOutput[0] eq 1 or DataOutput[0] eq 2 then begin
yunits = 'Phi:degrees'
ylabel = 'Phi '
endif

histlabel = 'I (arbitrary units)'
histunits = ''

Mon_cumTreat = 'Total incident beam monitor counts: '+ $
           strtrim(string(total(*self.Mon_cumPtr)),2)
*self.treatmentPtr = [*self.treatmentPtr,Mon_cumTreat]
msg = 'Number of channels: '+strtrim(string(nchan),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]
msg = 'Number of groups: '+strtrim(string(ndet),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]

;*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr = *self.treatmentPtr
if n_elements(temperature) gt 1 then temperature = (moment(temperature))[0]


openw,lun,fileout,/get_lun
printf,lun, 'Begin'
for i = 0, ndet-1 do begin
printf,lun, '#Group Number: '+strtrim(string(i+1),2)
printf,lun, '#Group Value: '+strtrim(string(y[i]),2)
printf,lun, '#X Value    ','Intensity ','dIntensity'
    for j = 0, nchan-1 do begin
printf,lun, strtrim(string(x[j]),1)+'  ',strtrim(string(data[j,i]),1)+'  ',strtrim(string(error[j,i]),1)
    endfor
endfor
print,lun, 'end'
free_lun,lun,/force
;
openw,lun,FilePar,/get_lun
printf,lun, 'Instrument: FOCUS'
printf,lun, 'Sample: '+strtrim(string(Sample_Name.sample),2)
printf,lun, 'Original File Name: '+strtrim(string(Sample_Name.foriname),2)
printf,lun, 'Title: '+strtrim(string(Sample_Name.title),2)
printf,lun, 'User: '+strtrim(string(Instrument_Details.owner),2)
printf,lun, 'Temperature / K: '+strtrim(string(temperature),2)
printf,lun, 'XType: Points'
printf,lun, 'Group Type: Points'
printf,lun, 'X Units: '+strtrim(string(xunits),2)
printf,lun, 'Y Units: '+strtrim(string(histlabel),2)
printf,lun, 'Group Units: '+strtrim(string(yunits),2)
printf,lun, 'Group Label: '+strtrim(string(ylabel),2)
printf,lun, 'Number of Channels: '+strtrim(string(nchan),2)
printf,lun, 'Number of Groups: '+strtrim(string(ndet),2)
printf,lun, 'Wavelength / A: '+strtrim(string((*self.tof_corr_structurePtr).lambdatof),2)
printf,lun, 'Date Data File Created: '+strtrim(string(Sample_Name.ftime),2)
printf,lun, 'Date This File Created: '+strtrim(string(systime()),2)
printf,lun, 'Fermispeed: '+strtrim(string(Instrument_Details.fermispeed),2)
printf,lun, 'Fermiphase: '+strtrim(string(Instrument_Details.fermiphase),2)
printf,lun, 'Diskspeed: '+strtrim(string(Instrument_Details.diskspeed),2)
printf,lun, *self.treatmentPtr
free_lun,lun,/force

;
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::writeMatlab,event = event,fileout = fileout,FilePar = FilePar,FilePhx = FilePhx
;
if n_elements(fileout) eq 0 then return
;

widget_control,self.DataOutputTypesGroup,get_value = DataOutput

if DataOutput eq 0 then output='S(Q,w)
if DataOutput eq 1 then output='S(phi,w)
if DataOutput eq 2 then output='S(phi,t)

treat = 'Converted to Matlab Format'
*self.treatmentPtr = [*self.treatmentPtr,treat]

n = n_elements(*self.treatmentPtr)
;for i = 0,n-1 do print,(*self.treatmentPtr)[i]

data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
;print,'Number of groups:',self.ndet

;help,data
datSize = size(data)
nchan = datSize[1]
if datSize[0] eq 2 then begin
ndet = datSize[2]
endif else begin
ndet=1
endelse

header=*self.headerPtr
nh = n_elements(*self.headerPtr)
detSize = (size(data))[2]
temperature=*self.tempPtr
Instrument_Details=*self.InstrumentPtr

;Sample_Name={sample_name,  $
;         ftime:(*self.InstrumentPtr).ftime,  $
;         sample:(*self.InstrumentPtr).sample,  $
;         foriname:(*self.InstrumentPtr).foriname,  $
;         title:(*self.InstrumentPtr).title}

Sample_Name=*self.Sample_NamePtr


if self.units ne 3 then begin
xlabel = 'Energy'
endif else begin
xlabel = 't'
endelse

if self.units eq 0 then xunits = 'Energy / meV'
if self.units eq 1 then xunits = 'Energy / Wavenumbers'
if self.units eq 2 then xunits = 'Energy / GHz'
if self.units eq 3 then xunits = 'Time / Microseconds'
widget_control,self.DataOutputTypesGroup,get_value = DataOutput
if DataOutput[0] eq 0 then begin
yunits = 'Wavevector:A-1'
ylabel = 'Q '
endif
if DataOutput[0] eq 1 or DataOutput[0] eq 2 then begin
yunits = 'Phi:degrees'
ylabel = 'Phi '
endif

histlabel = 'I (arbitrary units)'
histunits = ''

Mon_cumTreat = 'Total incident beam monitor counts: '+ $
           strtrim(string(total(*self.Mon_cumPtr)),2)
*self.treatmentPtr = [*self.treatmentPtr,Mon_cumTreat]
msg = 'Number of channels: '+strtrim(string(nchan),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]
msg = 'Number of groups: '+strtrim(string(ndet),2)
*self.treatmentPtr = [*self.treatmentPtr,msg]

;*(*(*davePtr).dataStrPtr).commonStr.treatmentPtr = *self.treatmentPtr
if n_elements(temperature) gt 1 then temperature = (moment(temperature))[0]

thisformat='(8(f10.2))'
openw,lun,fileout,/get_lun
printf,lun, ndet,nchan
printf,lun, '###  '+ylabel+' '+yunits
printf,lun, y,Format=thisformat
printf,lun, '###  '+xunits
printf,lun, x,Format=thisformat
for i = 0, ndet-1 do begin
printf,lun, '###  '+output
printf,lun, data[*,i],Format=thisformat
printf,lun, '###  error'
printf,lun,error[*,i],Format=thisformat
endfor
free_lun,lun,/force
;
openw,lun,FilePar,/get_lun
printf,lun, 'Instrument: FOCUS'
printf,lun, 'Sample: '+strtrim(string(Sample_Name.sample),2)
printf,lun, 'Original File Name: '+strtrim(string(Sample_Name.foriname),2)
printf,lun, 'Title: '+strtrim(string(Sample_Name.title),2)
printf,lun, 'User: '+strtrim(string(Instrument_Details.owner),2)
printf,lun, 'Temperature / K: '+strtrim(string(temperature),2)
printf,lun, 'XType: Points'
printf,lun, 'Group Type: Points'
printf,lun, 'X Units: '+strtrim(string(xunits),2)
printf,lun, 'Y Units: '+strtrim(string(histlabel),2)
printf,lun, 'Group Units: '+strtrim(string(yunits),2)
printf,lun, 'Group Label: '+strtrim(string(ylabel),2)
printf,lun, 'Number of Channels: '+strtrim(string(nchan),2)
printf,lun, 'Number of Groups: '+strtrim(string(ndet),2)
printf,lun, 'Wavelength / A: '+strtrim(string((*self.tof_corr_structurePtr).lambdatof),2)
printf,lun, 'Date Data File Created: '+strtrim(string(Sample_Name.ftime),2)
printf,lun, 'Date This File Created: '+strtrim(string(systime()),2)
printf,lun, 'Fermispeed: '+strtrim(string(Instrument_Details.fermispeed),2)
printf,lun, 'Fermiphase: '+strtrim(string(Instrument_Details.fermiphase),2)
printf,lun, 'Diskspeed: '+strtrim(string(Instrument_Details.diskspeed),2)
printf,lun, *self.treatmentPtr
free_lun,lun,/force
;
form='(8(f10.2))'
;
dy=fltarr(n_elements(y))+0.01
openw,lun, FilePhx,/get_lun
printf,lun, ndet
for i = 0, ndet-1 do begin
printf,lun, 10.0,0.0,y[i],0.0,dy[i],5.0,1,format=form
endfor
free_lun,lun,/force

;
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::Diffraction,event=event
files = *self.sigFilePtr
files=files[0]
if n_elements(*self.vanFilePtr) eq 0 then return
data = *self.dataPtr
error = *self.errorPtr
x = *self.xvalsPtr
y = *self.yvalsPtr
ndet=self.ndet
lambda=(*self.tof_corr_structurePtr).lambdatof
widget_control,self.DiffractionCalcGroup,get_value = DiffractionCalc ; 0 sum all time channels; 1 sum over range
;
IF DiffractionCalc eq 0 then begin
DiffInt=total(Data,1)
self->DiffVanCal,DiffInt,event = event
DiffInt_error=sqrt(DiffInt)
void=PlotDiffractionData(DiffInt,DiffInt_error,y,lambda,group_leader = event.top)
treat=['Diffraction Pattern Calculated From Sum Over All Time Channels.']
ENDIF
;
IF DiffractionCalc eq 1 then begin
widget_control,self.DiffMin,get_value = DiffMin
widget_control,self.DiffMax,get_value = DiffMax
DiffMin = float(DiffMin[0])
DiffMax = float(DiffMax[0])
xrangesubs1=where(x ge DiffMin,count)
if count eq 0 then return
x_1stcull=x[xrangesubs1]
xrangesubs2=where(x_1stcull le DiffMax,count)
if count eq 0 then return
xrangesubs=xrangesubs1[0]+xrangesubs2
;print,''
;print,x[xrangesubs]
;help,Data
Data=Data[xrangesubs,*]
;help,Data
DiffInt=total(Data,1)
self->DiffVanCal,DiffInt,event = event
DiffInt_error=sqrt(DiffInt)
void=PlotDiffractionData(DiffInt,DiffInt_error,y,lambda,group_leader = event.top)
treat=['Diffraction Pattern Calculated from Time Channels in the Range: ' $
       +strtrim(string(DiffMin),2)+' to '+strtrim(string(DiffMax),2)+' Microseconds.']
ENDIF
*self.treatmentPtr = [*self.treatmentPtr,treat]
;
sintholam=sin((!dtor*y/2.0)/lambda)
;
; what is the filename?
widget_control,self.runGroup,get_value = rungroup
rungroup=rungroup[0]
widget_control,self.DiffOutput_nameGroup,get_value = DiffOutput_name
DiffOutput_name=DiffOutput_name[0]
;
    if  DiffOutput_name eq 0 then begin
  filename_result = strsplit(files,path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)


   fileOut = filepath(filename,root_dir = self.workDir)
    if rungroup eq 1 then  fileout+='++'
endif


       if DiffOutput_name eq 1 then begin
;
; First extract the filename
  filename_result = strsplit(files,path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)
  ;extPos = strpos(files[0],'.focus')
  ;filename = strmid(files[0],extPos-11,11)

  fileExample = filename
  fileOut = dialog_pickfile(dialog_parent = event.top,/write, $
  ;                          file = filename, $
                            file = filename, $ ; following IGOR
                          ;  if rungroup eq 1 then  file+='++', $
                          filter = ['*diff.dat'],$
                            title = 'Diffraction Output Filename',$
                            path = self.workDir)
;   fileOut = filepath(filename+'++',root_dir = self.workDir)
;
       endif

       if  DiffOutput_name eq 2 then begin
  filename_result = strsplit(files,path_sep(),/extract)
  n_comp = n_elements(filename_result)
  filename = filename_result[n_comp-1] ; filename with .hdf attached
  extPos = strpos(filename,'.hdf')
  filename = strmid(filename,0,extPos)

   fileOut = filepath(filename,root_dir = self.workDir)
    if rungroup eq 1 then  fileout+='++'
  widget_control, self.Diffstem2, get_value=stem_add
    if strlen(stem_add) gt 0 then    fileOut+=stem_add

   endif
   fileout+='diff.dat'

;print the data to a file
;
thisformat='(4(f15.6,2x))'
openw,lun,fileout,/get_lun
printf,lun, *self.treatmentPtr
printf,lun, ''
printf,lun, '      Two Theta    Sin(Theta)/lambda   Intensity          Error'
printf,lun, 'Begin'
for i = 0, ndet-1 do begin
printf,lun, y[i],sintholam[i],DiffInt[i],DiffInt_Error[i],Format=thisformat
endfor
free_lun,lun,/force


end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::DiffVanCal,DiffInt,event = event
; First pull out the current data values
sigData = *self.dataPtr
sigError = *self.errorPtr
sigX = *self.xvalsPtr
sigY = *self.yvalsPtr
sigMon_tc = *self.Mon_tcPtr
sigMon_cum = *self.Mon_cumPtr
sigTemp = *self.tempPtr
signdet=self.ndet
vanFile = *self.vanFilePtr
tof_corr_structure=*self.tof_corr_structurePtr
self->Nexus_read_raw_focus,vanFile,event = event
vdata = *self.dataPtr
vtemperature = *self.tempPtr
vtemperature = vtemperature[0]
;help,vtemperature
;print,'vtemperature =',vtemperature
detnum=*self.yvalsPtr
;print,'*self.xvalsPtr =',*self.xvalsPtr
;
;Apply the same detector filtering as before
;
;
good_det=*self.good_detPtr
pnewdata=ptr_new(fltarr((size(vdata))(1),n_elements(good_det)))
pnewdetnum=ptr_new(fltarr(n_elements(good_det)))
;
*pnewdata=vdata(*,good_det)
*pnewdetnum=detnum(good_det)
vData=*pnewdata
detnum=*pnewdetnum
ptr_free,pnewdata
ptr_free,pnewdetnum
;
;print,'detnum =',detnum
;print,'good_det =',good_det
;print,'n_elements(good_det) =',n_elements(good_det)


;sort the data in ascending order of detnum
;
sortorder=sort(detnum)
detnum=detnum(sortorder)
vdata=vdata(*,sortorder)
;print,detnum
;
pnewdata=ptr_new(fltarr((size(vdata))(1),(size(vdata))(2)))
pnewdetnum=ptr_new(fltarr((size(detnum))(1)))
;remove degeneracies
;
(*pnewdetnum)[0]=detnum[0]
(*pnewdata)[*,0]=vdata[*,0]
j=0
for i=1,n_elements(detnum)-1 do begin
    if detnum(i) ne detnum(i-1) then begin
j+=1
(*pnewdetnum)[j]=detnum[i]
(*pnewdata)[*,j]=vdata[*,i]
endif else begin
(*pnewdata)[*,j]+=vdata[*,i]
endelse
endfor
vanData=(*pnewdata)[*,0:j]
detnum=(*pnewdetnum)[0:j]
ptr_free,pnewdata
ptr_free,pnewdetnum

;*self.treatmentPtr = [*self.treatmentPtr,treat]
;*self.dataPtr = vData
;vError = sqrt(vData)
;*self.errorPtr = vError
;*self.yvalsPtr = detnum
;
;
vanx = *self.xvalsPtr
;print,total(vandata)
;print,'*self.xvalsPtr =',*self.xvalsPtr
;
;print,'vanData(*,1) =',vanData(*,1)
;print,''
for i = 0,n_elements(detnum)-1 do begin
   ; Get a good estimate of the starting parameters for the fits


  y = reform(vanData[*,i])
 zeropos=(vanx[where(y eq max(y))])[0]
  coeff=[max(y),zeropos,20.0,0]
  region = where((vanX ge zeropos-150.0) and (vanX le zeropos+150.0))
x=vanX[region]
;print,'x = ',x
 y = reform(vanData[region,i])

 ;print,'y = ',y

 ;print,'coeff =',coeff
;print,i
;print,vanx
  ; Fit it...
  result = gaussfit(x,y,p,chisq=gof,estimates = coeff,nterms = 4)
  ; the next line should be the area
  ;print, 'Chisq =',gof
  ;print, 'fwhh =',p[2]
  ;print, 'max(y) =',p[0]
  ;print, 'x position =',p[1]
  ;print,''
  a = p[0]*sqrt(2.0*!pi*p[2]^2)
  if i eq 0 then area = a else area = [area,a]
endfor
  ;print,'area =',area

;print,'vanData(*,245) =',vanData(*,245)
;print, 'vanx =',vanx
stats = moment(area[0:n_elements(detnum)-1])
;stats[0] is then the mean average area.  Now divide each area by the mean average
area = temporary(area)/stats[0]
;ux = 1+bytarr(n_elements(sigX))
lambda0=tof_corr_structure.lambdatof
q = (4.0*!pi/lambda0)*sin(0.5*!dtor*detnum)
dwf = (exp(-0.0067*vtemperature/300.0*q^2))  ; Debye-Waller factor for vanadium
DiffInt = DiffInt*dwf/area
;restore the data
*self.dataPtr = sigData
*self.errorPtr = sigError
*self.xvalsPtr = sigX
*self.yvalsPtr = sigY
*self.Mon_tcPtr = sigMon_tc
*self.Mon_cumPtr = sigMon_cum
*self.tempPtr = sigTemp
self.ndet=signdet
;print,'v ndet =',self.ndet
return



end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::zoomEvents,event = event
if n_elements(*self.dataPtr) eq 0 then return
case event.type of
0: begin    ; button press
     self.mouse = event.press
     if self.mouse eq 4 then begin
       self.autoscale = 1
        self->display,event = event
     endif
     if self.mouse eq 1 then begin
       self.xbox[0] = event.x
       self.ybox[0] = event.y
        self->display,event = event
       empty
       self.autoscale = 0
       widget_control,self.win,/draw_motion_events
     endif
   end
1: begin ; button release
    if self.mouse eq 1 then begin
     xll = self.xbox[0] < self.xbox[1]
     yll = self.ybox[0] < self.ybox[1]
     w = abs(self.xbox[1] - self.xbox[0])
     h = abs(self.ybox[1] - self.ybox[0])
     xur = xll + w
     yur = yll + h
     ll = convert_coord(xll,yll,/device,/to_data)
     ur = convert_coord(xur,yur,/device,/to_data)
     self.xrange = [ll[0],ur[0]]
     self.yrange = [ll[1],ur[1]]
      self->display,event = event
     self.mouse = 0B
     widget_control,self.win,draw_motion_events = 0
    endif
    if self.mouse eq 4 then begin
     self->display,event = event
     self.mouse = 0B
     widget_control,self.win,draw_motion_events = 0
    endif
   end
2: begin ; mouse motion
     if self.mouse eq 1 then begin
        self.xbox[1] = event.x
        self.ybox[1] = event.y
        xc = [self.xbox[0],event.x,event.x,$
              self.xbox[0],$
              self.xbox[0]]
        yc = [self.ybox[0],self.ybox[0],$
              event.y,event.y,$
              self.ybox[0]]
        wset,self.winVis
        device,copy = [0,0,!d.x_size,!d.y_size,0,0,self.winPix]
        plots,xc,yc,/device
        empty
     endif
   end
else:
endcase
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::reduce_help,event = event
;pdf_file = file_which('focus_data_reduction.pdf',/include_current_dir)
pdf_file = !DAVE_AUXILIARY_DIR+'focus_data_reduction.pdf'
void = launch_help(pdf_file,tlb = event.top)
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction::createwidgets
; Widget definition method
if n_elements(*self.pgroup_leader) eq 0 then begin
  tlb = widget_base(/row, $
        title = 'Focus Data Reduction',mbar = bar,/tlb_size_events)
endif else begin
  tlb = widget_base(group_leader = *self.pgroup_leader,/row, $
        title = 'Focus Data Reduction',mbar = bar,/tlb_size_events)
endelse
self.tlb = tlb
self.ctrlBase = widget_base(self.tlb,/row)
master_button_base = widget_base(self.ctrlbase,/col,/frame)

ubase_master = widget_base(master_button_base,/col)
void=widget_label(ubase_master,/sunken_frame,value='BASIC INPUT')
ubase = widget_base(ubase_master,/row)
lbase = widget_base(master_button_base,/col)
void=widget_label(lbase,/sunken_frame,value='ADVANCED OPTIONS')
lubase_col = widget_base(ubase,/col)
rubase_col = widget_base(ubase,/col)


treatTypes = ['Vanadium Normalisation',$
              'Background Subtraction'];, $
;              'Energy binning']
treatGroup = cw_bgroup(lubase_col,treatTypes,/col,/nonexclusive,$
           set_value = [0,0],/return_index, /frame,$
           uvalue = {object:self,method:'treatmentHandler'})
self.treatGroup = treatGroup
runTypes = ['Single Sample Runs','Sum All Runs']
runGroup = cw_bgroup(lubase_col,runTypes,/col,/exclusive,$
           /no_release,set_value = 0,/return_index, /frame,$
           uvalue = {object:self,method:'rungroupHandler'})
self.runGroup = runGroup
;help,runGroup
;print,runGroup
;
;
void = widget_button(lubase_col,value = 'Select Signal File(s)', $
      uvalue = {object:self,method:'selSigFiles'})
self.vanButton = widget_button(lubase_col,value = 'Select Vanadium File', $
                sensitive = 0, $
               uvalue = {object:self,method:'selVanFile'})
self.bgButton = widget_button(lubase_col,value = 'Select Background File(s)', $
                sensitive = 0, $
               uvalue = {object:self,method:'selBgFiles'})
void = widget_label(rubase_col,value = 'Signal File(s)')
xtextSize = 20
self.fileText = widget_text(rubase_col,ysize = 5,xsize = xtextSize,/scroll,/editable)
void = widget_label(rubase_col,value = 'Vanadium File')
self.vanText = widget_text(rubase_col,xsize = xtextSize,/scroll,/editable)
void = widget_label(rubase_col,value = 'Background File(s)')
self.bgText = widget_text(rubase_col,xsize = xtextSize,ysize = 5,/scroll,/editable)
void = widget_button(lubase_col,value = '?Help?', $
               uvalue = {object:self,method:'reduce_help'})
void = widget_button(lubase_col,value = 'Quit', $
       uvalue = {object:self,method:'quit'})
void = widget_button(lubase_col,value = 'REDUCE DATA', $
               uvalue = {object:self,method:'reduce'})

Opt_Tab=widget_Tab(lbase,uvalue={object:self,method:'doNothing'})
Output_master = widget_base(Opt_Tab,/row,title='Output')
Bin_master = widget_base(Opt_Tab,/row,title='Binning')
ElasticLine_master = widget_base(Opt_Tab,/row,title='Elastic Line')
Detector_master = widget_base(Opt_Tab,/row,title='Detector')
Diffraction_master = widget_base(Opt_Tab,/row,title='Diffraction')
Misc_master = widget_base(Opt_Tab,/row,title='Misc.')

Output_name_Lbase=widget_base(Output_master,/col)

Output_name_base=widget_base(Output_name_Lbase,/col)
void=widget_label(Output_name_base,value='Output File Name')
Output_name_subbase=widget_base(Output_name_base,/col,/frame)

Output_nameTypes= ['Automatically Assigned',  $
          'Specify Output File Name', $
              'Add to Stem:']

Output_nameGroup = cw_bgroup(Output_name_subbase,Output_nameTypes,/col,/exclusive, $
           /no_release,set_value = 0,/return_index, $
           uvalue = {object:self,method:'OutputName_handler'})
self.Output_nameGroup = Output_nameGroup

self.Output_name_subbase_sensitive=widget_base(Output_name_subbase,/row)

self.Output_name_subbase_sensitive1=widget_base(self.Output_name_subbase_sensitive,sensitive=0)
self.stem1 = cw_field(self.Output_name_subbase_sensitive1,title='', $
              uvalue = {object:self,method:'doNothing'},/row,value = '',/noedit)

self.Output_name_subbase_sensitive2=widget_base(self.Output_name_subbase_sensitive,sensitive=0)
self.stem2 = cw_field(self.Output_name_subbase_sensitive2,title='', xsize=5, $
              uvalue = {object:self,method:'doNothing'},/row,value = '')

self.Output_name_subbase_sensitive3=widget_base(self.Output_name_subbase_sensitive,sensitive=0)
self.stem3 = cw_field(self.Output_name_subbase_sensitive3,title='', xsize=8, $
          uvalue = {object:self,method:'doNothing'},/row,value = 'dyn.dave',/noedit)

Output_name_Ubase=widget_base(Output_name_Lbase,/row)

FormatTypes = ['Dave',$
              'IGOR Text', $
              'ASCII',  $
              'Matlab']
FormatTypesGroup = cw_bgroup(Output_name_Ubase,FormatTypes,/col,/exclusive,label_top='Output Formats',$
           set_value = 0,/return_index, /frame,$
           uvalue = {object:self,method:'FormatTypes'})
self.FormatTypesGroup = FormatTypesGroup



;Output_name_Rbase=widget_base(Output_master,/col)


DataOutputTypes = ['S(Q,w)',$
              'S(Phi,w)', $
              'S(Phi,t)']

DataOutputTypesGroup = cw_bgroup(Output_name_Ubase,DataOutputTypes,/col,/exclusive,label_top='Data Output', $
           set_value = 0,/return_index,/frame, $
           uvalue = {object:self,method:'Data_Output_Handler'})
self.DataOutputTypesGroup = DataOutputTypesGroup


EBin_submaster=widget_base(Bin_master,/col)
void=widget_label(EBin_submaster,value='Energy/TOF Binning')

E_TOF_base=widget_base(EBin_submaster,/col,/frame)


ebin_Types = ['No Binning','Binning and Units']
ebin_Group = cw_bgroup(E_TOF_base,ebin_Types,row=2,/exclusive,$
           /no_release,set_value = 0,/return_index,$
           uvalue = {object:self,method:'E_TOF_handler'})
self.ebin_group = ebin_Group

self.E_TOF_base1=widget_base(E_TOF_base,/col,sensitive=0)

ebin_Types1 = ['Modify Default','Modify Previous','Use Previous']
ebin_Group1 = cw_bgroup(self.E_TOF_base1,ebin_Types1,row=3,/exclusive, $
           /no_release,set_value = 0,/return_index,$
           uvalue = {object:self,method:'doNothing'})
self.ebin_group1 = ebin_Group1



QBin_submaster=widget_base(Bin_master,/col)
void=widget_label(QBin_submaster,value='Q/Phi Binning/Grouping')

Q_Theta_base=widget_base(QBin_submaster,/col,/frame)


Qbin_Types = ['Sum All','Binning/Grouping']
qbin_Group = cw_bgroup(Q_Theta_base,qbin_Types,row=2,/exclusive,$
           /no_release,set_value = 0,/return_index,$
           uvalue = {object:self,method:'Q_Theta_handler'})
self.qbin_group = qbin_Group


self.q_theta_base1=widget_base(q_theta_base,/col,sensitive=0)

qbin_Types1 = ['Modify Default','Modify Previous','Use Previous']
qbin_Group1 = cw_bgroup(self.q_theta_base1,qbin_Types1,row=3,/exclusive, $
           /no_release,set_value = 0,/return_index,$
           uvalue = {object:self,method:'qbin_Types1_handler'})
self.qbin_group1 = qbin_Group1





elasticbase=widget_base(ElasticLine_master,/frame,/col)

elasticTypes = ['Vanadium File, TOF Data',$
          'Sample File(s), TOF Data',$
          'Sample File(s), Monochromator',$
          'Own Value']
elasticGroup = cw_bgroup(elasticbase,elasticTypes,row=4,/exclusive,label_top='Elastic Line From:',$
           /no_release,set_value = 0,/return_index,$
           uvalue = {object:self,method:'LambdaHandler'})
self.elasticGroup = elasticGroup

lambda_tof = cw_field(elasticbase,/row,uvalue = {object:self,method:'doNothing'},$
                 title = 'Lambda TOF',/noedit)

lambda_monochromator = cw_field(elasticbase,/row,uvalue = {object:self,method:'doNothing'},$
                title = 'Lambda Mon.',/noedit)

self.elastic_base_sensitive=widget_base(elasticbase,/col,sensitive=0)
 self.lambda_own_value = cw_field(self.elastic_base_sensitive,/row,uvalue = {object:self,method:'doNothing'},$
                 title = 'Lambda Own')



lambda_field={lambda_field,lambda_monochromator:lambda_monochromator, $
          lambda_tof:lambda_tof}

*self.lambda_fieldPtr=lambda_field
;print,(*self.lambda_fieldPtr).lambda_tof



Bad_DetectorTypes = ['Automatic',$
              'Graphical Masking', $
              'Scroll Spectra Masking',$
              'Previous Selection',  $
              'Use All Detectors']
Bad_DetectorGroup = cw_bgroup(Detector_master,Bad_DetectorTypes,row=5,/exclusive,label_top='Bad Detector Removal',$
           /no_release,set_value = 0,/return_index, /frame,$
           uvalue = {object:self,method:'Bad_DetectorGroup_Handler'})
self.Bad_DetectorGroup = Bad_DetectorGroup

DetectorBankTypes = ['Merged',$
              'Lower', $
              'Middle', $
              'Upper']
DetectorBankTypesGroup = cw_bgroup(Detector_master,DetectorBankTypes,/col,/exclusive,label_top='Detector Bank',$
           set_value = 0,/return_index, /frame,$
           uvalue = {object:self,method:'doNothing'})
self.DetectorBankTypesGroup = DetectorBankTypesGroup

;Diffraction Tab
Diffraction_Base_submaster=widget_base(Diffraction_master,/col)

self.Diffraction_Base1=widget_base(Diffraction_Base_submaster,/col,sensitive=0)

DiffONType = ['Generate Diffraction Pattern (Vanadium Run Required)']

DiffONGroup = cw_bgroup(self.Diffraction_Base1,DiffONType,/col,/nonexclusive,$
           set_value = 0,/return_index,$
           uvalue = {object:self,method:'DiffONHandler'})

self.DiffONGroup=DiffONGroup




self.Diffraction_Base2=widget_base(Diffraction_Base_submaster,/col,sensitive=0)


DiffractionCalcTypes = ['Sum Over All Channels',$
                     'Sum Between Channels (Microseconds):']

DiffractionCalcGroup = cw_bgroup(self.Diffraction_Base2,DiffractionCalcTypes,/col,/exclusive,$
           set_value = 0,/return_index,$
           uvalue = {object:self,method:'DiffractionCalcHandler'})

self.DiffractionCalcGroup=DiffractionCalcGroup

self.Diffraction_Base3=widget_base(self.Diffraction_Base2,/row,sensitive=0)


self.DiffMin = cw_field(self.Diffraction_Base3,title='Min', xsize=5, $
              uvalue = {object:self,method:'doNothing'},/row,value = '')

self.DiffMax = cw_field(self.Diffraction_Base3,title='Max', xsize=5, $
              uvalue = {object:self,method:'doNothing'},/row,value = '')





DiffOutput_name_base=widget_base(self.Diffraction_Base2,/col)
void=widget_label(DiffOutput_name_base,value='Output File Name')
DiffOutput_name_subbase=widget_base(DiffOutput_name_base,/col,/frame)

DiffOutput_nameTypes= ['Automatically Assigned',  $
          'Specify Output File Name', $
              'Add to Stem:']

DiffOutput_nameGroup = cw_bgroup(DiffOutput_name_subbase,DiffOutput_nameTypes,/col,/exclusive, $
           /no_release,set_value = 0,/return_index, $
           uvalue = {object:self,method:'DiffOutputName_handler'})
self.DiffOutput_nameGroup = DiffOutput_nameGroup

self.DiffOutput_name_subbase_sensitive=widget_base(DiffOutput_name_subbase,/row)

self.DiffOutput_name_subbase_sensitive1=widget_base(self.DiffOutput_name_subbase_sensitive,sensitive=0)
self.Diffstem1 = cw_field(self.DiffOutput_name_subbase_sensitive1,title='', $
              uvalue = {object:self,method:'doNothing'},/row,value = '',/noedit)

self.DiffOutput_name_subbase_sensitive2=widget_base(self.DiffOutput_name_subbase_sensitive,sensitive=0)
self.Diffstem2 = cw_field(self.DiffOutput_name_subbase_sensitive2,title='', xsize=5, $
              uvalue = {object:self,method:'doNothing'},/row,value = '')

self.DiffOutput_name_subbase_sensitive3=widget_base(self.DiffOutput_name_subbase_sensitive,sensitive=0)
self.Diffstem3 = cw_field(self.DiffOutput_name_subbase_sensitive3,title='', xsize=8, $
          uvalue = {object:self,method:'doNothing'},/row,value = 'diff.dat',/noedit)




;Misc Tab

Misc_Base1=widget_base(Misc_master,/col)

Misc1Types = ['Detailed Balance',$
              'Flat Baseline']

Misc1Group = cw_bgroup(Misc_Base1,Misc1Types,/col,/nonexclusive,$
           set_value = [0,0],/return_index,$
           uvalue = {object:self,method:'Misc1Handler'})

self.Misc1Group=Misc1Group


self.Misc_Base2=widget_base(Misc_Base1,/col,sensitive=0)
self.flat_baseline_range = cw_field(self.Misc_Base2,title='Channels:', $
              uvalue = {object:self,method:'doNothing'},/row,value = '100',xsize=5)


plotBase = widget_base(self.tlb,/col)
winxsize = 400 & winysize = 200
self.win = widget_draw(plotBase,xsize = winxsize,ysize = winysize, $
           /button_events,uvalue = {object:self,method:'zoomEvents'})
self.grpSlider = widget_slider(plotBase,minimum = 1,maximum = 269, $
          uvalue = {object:self,method:'display'})

; Get the vertical size right
cgeom = widget_info(self.ctrlBase,/geometry)
winysize = cgeom.ysize > winysize
winxsize = winysize
widget_control,self.win,ysize = winysize,xsize = winxsize

; Center the widget
geom = widget_info(self.tlb,/geometry)
device,get_screen_size = sz
sx = sz[0] & sy = sz[1]
xoff = fix(0.5*(sx-geom.xsize))
yoff = fix(0.5*(sy-geom.ysize))
widget_control,self.tlb,xoffset = xoff,yoffset = yoff

widget_control,self.tlb,/realize

window,/free,/pixmap,xsize = winxsize,ysize = winysize
self.winPix = !d.window
widget_control,self.win,get_value = winVis
self.winVis = winVis

widget_control,tlb,set_uvalue = self
ret = dave_set_focus(self.tlb)
xmanager,'focusreduction::createwidgets',self.tlb, $
         event_handler = 'focusReductionEvents',$
         cleanup = 'focusReductionCleanup', /no_block
return
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function focusreduction::init, group_leader = group_leader, $
                 notifyId = notifyId, $
                 workDir = workDir, $
                 datDir = datDir

tvlct,r,g,b,/get
self.rPtr = ptr_new(r)
self.gPtr = ptr_new(g)
self.bPtr = ptr_new(b)
loadct,0,/silent

self.notifyIdPtr = ptr_new(/allocate_heap)
self.pgroup_leader = ptr_new(/allocate_heap)

if n_elements(notifyId) ne 0 then $
  *self.notifyIdPtr = notifyId

if n_elements(group_leader) ne 0 then begin
  *self.pgroup_leader = group_leader
  ; Get the DAVE pointer information from group leader
  ; Note that the DAVE pointer is not valid if there is no
  ; group leader specified here!
  widget_control,group_leader,get_uvalue = statePtr
  self.davePtr = (*statePtr).davePtr
  ; DO NOT FREE THE STATEPTR HERE!!!! IT BELONGS TO THE
  ; PARENT (CALLING) WIDGET!!!!
endif

cd,current = curDir
if n_elements(workDir) eq 0 then self.workDir = curDir else self.workDir = workDir
if n_elements(datDir) eq 0 then self.datDir = '' else self.datDir = datDir

self.lambda_fieldPtr = ptr_new(/allocate_heap)

self->createwidgets
self.autoscale = 1
self.xrange = [0.0,1.0]
self.yrange = [0.0,1.0]
self.xbox = self.xrange
self.ybox = self.yrange
self.mouse = 0B

; Initialize all of the pointers
self.dataPtr = ptr_new(/allocate_heap)
self.errorPtr = ptr_new(/allocate_heap)
self.xvalsPtr = ptr_new(/allocate_heap)
self.yvalsPtr = ptr_new(/allocate_heap)
self.odataPtr = ptr_new(/allocate_heap)
self.oerrorPtr = ptr_new(/allocate_heap)
self.oxvalsPtr = ptr_new(/allocate_heap)
self.oyvalsPtr = ptr_new(/allocate_heap)
self.sigFilePtr = ptr_new(/allocate_heap)
self.bgFilePtr = ptr_new(/allocate_heap)
self.vanFilePtr = ptr_new(/allocate_heap)
self.Mon_tcPtr = ptr_new(/allocate_heap)
self.Mon_cumPtr = ptr_new(/allocate_heap)
self.tempPtr = ptr_new(/allocate_heap)
self.InstrumentPtr = ptr_new(/allocate_heap)
self.tof_corr_structurePtr = ptr_new(/allocate_heap)
self.treatmentPtr = ptr_new(/allocate_heap)
self.headerPtr = ptr_new(/allocate_heap)
self.daveFiles = ptr_new(/allocate_heap)
self.good_detPtr = ptr_new(/allocate_heap)
self.selDetPtr = ptr_new(/allocate_heap)
*self.daveFiles = ''
self.Sample_NamePtr=ptr_new(/allocate_heap)




self.xlabel = ''
self.ylabel = ''
self.zlabel = ''
self.curFile = ''

self.detPtr = ptr_new(/allocate_heap)

self.titlePtr = ptr_new(/allocate_heap)
unit = !dave_invAngstromSym
titleArray='woof'
*self.titlePtr = titleArray

return,1
end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
pro focusreduction__define
define = {focusreduction,      $

          tlb:0L,          $
          ctrlBase:0L,        $
          grpSlider:0L,       $
          runGroup:0L,        $
          elasticGroup:0L,        $
          Bad_DetectorGroup:0L,  $
          treatGroup:0L,      $
          FormatTypesGroup:0L,      $
          DataOutputTypesGroup:0L,      $
          DetectorBankTypesGroup:0L,      $
          vanButton:0L,       $
          lambda_own_value:0L,       $
          elastic_base_sensitive:0L,       $
          bgButton:0L,        $
          grpButton:0L,       $
          fileText:0L,        $
          vanText:0L,         $
          bgText:0L,       $
          ebin_group:0L,       $
          ebin_group1:0L,       $
          qbin_group1:0L,       $
          E_TOF_base1:0L,       $
          q_theta_base1:0L,       $
          Qbin_group:0L,       $
          units:0L,       $
          Output_nameGroup:0L,       $
          Output_name_subbase_sensitive:0L,       $
          Output_name_subbase_sensitive1:0L,       $
          Output_name_subbase_sensitive2:0L,       $
          Output_name_subbase_sensitive3:0L,       $
          stem1:0L,       $
          stem2:0L,       $
          stem3:0L,       $
          Misc1Group:0L,       $
          Misc_Base2:0L, $
          flat_baseline_range:0L,   $
          DiffractionCalcGroup:0L,   $
          Diffraction_Base3:0L,   $
          DiffMin:0L,   $
          DiffMax:0L,   $
          DiffONGroup:0L,   $
          Diffraction_Base2:0L,   $
          DiffOutput_nameGroup:0L,   $
          DiffOutput_name_subbase_sensitive:0L,   $
          DiffOutput_name_subbase_sensitive1:0L,   $
          Diffstem1:0L,   $
          DiffOutput_name_subbase_sensitive2:0L,   $
          Diffstem2:0L,   $
          DiffOutput_name_subbase_sensitive3:0L,   $
          Diffstem3:0L,   $
          Diffraction_Base1:0L,   $

          ; Window/display variables
          win:0L,          $
          winVis:0L,       $
          winPix:0L,       $
          xrange:fltarr(2),      $
          yrange:fltarr(2),      $
          xbox:fltarr(2),      $
          ybox:fltarr(2),      $
          autoscale:1,        $
          mouse:0B,         $
          xlabel:'',       $
          ylabel:'',       $
          zlabel:'',       $
          titlePtr:ptr_new(),    $
          daveFiles:ptr_new(),     $

          ; Directory variables
          workDir:'',         $
          datDir:'',       $
          ; Detector pointer
          selDetPtr:ptr_new(),     $

          ; Raw data file variables
          Mon_tcPtr:ptr_new(),      $
          Mon_cumPtr:ptr_new(),      $
          headerPtr:ptr_new(),     $
          detPtr:ptr_new(),      $
          tempPtr:ptr_new(),   $
          instrumentPtr:ptr_new(),   $
          Sample_NamePtr:ptr_new(),   $
          tof_corr_structurePtr:ptr_new(),   $
          ndet:0,          $
          sf:fltarr(375),      $
          curFile:'',         $

          ; Original data set
          odataPtr:ptr_new(),    $
          oerrorPtr:ptr_new(),     $
          oxvalsPtr:ptr_new(),     $
          oyvalsPtr:ptr_new(),     $

       ; Working data set
          dataPtr:ptr_new(),   $
          errorPtr:ptr_new(),    $
          xvalsPtr:ptr_new(),    $
          yvalsPtr:ptr_new(),    $

       ; Pointer to the lambda fields
       lambda_fieldPtr:ptr_new(),  $
       ; Good Detector Pointer
        good_detPtr:ptr_new(),   $

       ; Colors for restoration upon quitting
          rPtr:ptr_new(),      $
          gPtr:ptr_new(),      $
          bPtr:ptr_new(),      $
          ; Detector pointer
          ; Treatment pointer
          treatmentPtr:ptr_new(),   $

          ; File pointers
          sigFilePtr:ptr_new(),   $
          bgFilePtr:ptr_new(),     $
          vanFilePtr:ptr_new(),   $

          ; Dave pointer
          davePtr:ptr_new(),   $

          ; Info about parent
          notifyIdPtr:ptr_new(), $
          pgroup_leader:ptr_new()   $

          }


return
end
