; $Id: centerwindow.pro,v 1.3 2005/02/09 16:58:01 dimeo Exp $
;###############################################################################
;
; NAME:
;  CENTERWINDOW
;
; PURPOSE:
;  Application launcher for fixed window scan data analysis.
;
; CATEGORY:
;  DAVE, HFBS, general utility
;
; AUTHOR:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
; centerwindow.pro
; Written by R.M.Dimeo (11/11/01)
;
; NAME:
;       CENTERWINDOW
;
; PURPOSE:
;       This procedure creates a window of the desired
;		size centered in the screen
;
; CATEGORY:
;       Graphics.
;
; CALLING SEQUENCE:
;       CENTERWINDOW,WIN = WIN, XSIZE = XSIZE, YSIZE = YSIZE
;
; INPUTS:
;
;	NONE
;
; KEYWORDS:
;       WIN:			window index (default = free)
;
;       XSIZE:		   horizontal window size (default = 350)
;
;		YSIZE:		   vertical window size (default = 300)
;
; EXAMPLES:
;
; 					winxsize = 450 & winysize = 400
; 					centerwindow,win = 0,xsize = winxsize,ysize = winysize
; 					plot,findgen(10),findgen(10)
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
;
;-
pro centerwindow,win = win,xsize = xsize,ysize = ysize
if n_elements(xsize) eq 0 then xsize = 350
if n_elements(ysize) eq 0 then ysize = 300
device,get_screen_size = ss
sx = ss[0] & sy = ss[1]
xpos = fix(0.5*(sx-xsize))
ypos = fix(0.5*(sy-ysize))
if n_elements(win) eq 0 then begin
  window,/free,xsize = xsize,ysize = ysize,xpos = xpos,ypos = ypos
endif else begin
  window,win,xsize = xsize,ysize = ysize,xpos = xpos,ypos = ypos
endelse
return
end