; $Id: davedat1d__get.pro,v 1.8 2005/02/08 17:17:34 azuah Exp $
;###############################################################################
;
; CLASS NAME:
;   DaveDat1D
;
; PURPOSE:
;   This class encapsulates a DAVE formated dataset containing 1D
;   data.
;
; CATEGORY:
;   Data Browser
;
; SUPERCLASSES:
;   DaveDat
;
; PUBLIC METHODS:
;   Clone
;   Get
;   Print
;   Rebin
;   ResetData
;   Restore
;   Set
;
; AUTHOR:
;   Richard Tumanjong Azuah
;   NIST Center for Neutron Research
;   100 Bureau Drive, Gaithersburg, MD 20899
;   United States
;   azuah@nist.gov; (301) 9755604
;   Oct, 2002
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################



;===============================================================================
; DaveDat1D::get
;
; PURPOSE:
;   Main get method for class. Used to retrieve class variables (all
;   class variables are 'protected' in IDL). 
;   This method extends DaveDat::get and relies heavily on it.
;
; PARAMETERS:
;
; KEYWORDS:
;   offset [out] - Return the offset as a float. The data offset is
;                  used when creating a multiplot containing other datasets.
;
;   updDavePtr [out] - Return a _pointer_ to a _copy_ of the object's
;                      updated davePtr if present. It includes any
;                      monifications that might have been performed on
;                      the dataset so far (eg a rebinning)
;
;   zVal [out] - returns a float array containing the z-axis values.
;
;   _REF_EXTRA - allows keywords to be passed to the base class (see
;                DaveDat::get).
;
; RETURN VALUE: 
;   0 => failure
;   1 => sucess.
;
function DaveDat1D::get, _REF_EXTRA=etc, updDavePtr=updDavePtr, offset=offset, zVal=zVal

retVal = 1

; data offset
if (arg_present(offset)) then begin
    offset = self.offset
endif


; get davePtr with data updated to reflect current state of this object
if (arg_present(updDavePtr)) then begin
    if (ptr_valid(updDavePtr)) then heap_free, updDavePtr ; delete the heap variable if defined already
    if (ptr_valid(self.davePtr)) then dave_beast_ops,'dup',self.davePtr,updDavePtr
    ;; copy object's data to new davePtr
    if (ptr_valid(self.treatPtr)) then begin 
        (*(*(*updDavePtr).dataStrPtr).commonStr.treatmentPtr) = (*self.treatPtr)
    endif
    void = self->get(xval=xval,zval=zval,zerr=zerr)
    ; If the xaxis was reversed, undo it
    if (self.xDir eq 1) then begin
        xval = reverse(xval)
        zval = reverse(zval)
        zerr = reverse(zerr)
    endif
    ;if (n_elements(xval) le 0) xval = 0.0
    ;if (n_elements(zval) le 0) zval = 0.0
    ;if (n_elements(zerr) le 0) zerr = 0.0
    yval = 0.0
    (*(*(*updDavePtr).dataStrPtr).commonStr.histPtr) = {qty:zval,err:zerr,x:xval,y:yval}
    (*(*updDavePtr).descriPtr) = (*self.descriPtr)
endif

; zVal
if (arg_present(zval)) then begin
    if (not self->DaveDat::get(zval=zval)) then return, 0
    zval = zval + self.offset
endif

; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::get(_EXTRA=etc)
endif

return, retVal
end
;_______________________________________________________________________________
