; $Id: davedat1d__set.pro,v 1.3 2005/02/08 17:17:34 azuah Exp $
;###############################################################################
;
; CLASS NAME:
;   DaveDat1D
;
; PURPOSE:
;   This class encapsulates a DAVE formated dataset containing 1D
;   data.
;
; CATEGORY:
;   Data Browser
;
; SUPERCLASSES:
;   DaveDat
;
; PUBLIC METHODS:
;   Clone
;   Get
;   Print
;   Rebin
;   ResetData
;   Restore
;   Set
;
; AUTHOR:
;   Richard Tumanjong Azuah
;   NIST Center for Neutron Research
;   100 Bureau Drive, Gaithersburg, MD 20899
;   United States
;   azuah@nist.gov; (301) 9755604
;   Oct, 2002
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################



;===============================================================================
; DaveDat1D::set
; 
; PURPOSE:
;   Main set method for class.
;   Extends DaveDat::set method.
;
; PARAMETERS:
;
; KEYWORDS:
;   offset [in] - assign value (expects float) to the object's offset
;                 variable.
;
;   _EXTRA - allows keywords to be passed to the base class (see
;            DaveDat::set).
;
; RETURN VALUE: 
;   0 => failure
;   1 => success
;
function DaveDat1D::set, _EXTRA=etc, offset=offset

; reset !error_state structure, ie clear all errors
message,/reset

retVal = 1

; offset
if (n_elements(offset) gt 0) then begin
    ;; make sure value is a number
    offsetType = vartype(offset)
    if ( (offsetType eq 'FLOAT') or $
         (offsetType eq 'DOUBLE') or $
         (offsetType eq 'INT') or $
         (offsetType eq 'LONG') ) then begin
        self.offset = float(offset) 
    endif else begin
        !error_state.msg = 'DaveDat1D::set: offset keyword value is not a float/double'
        !error_state.code = -1
        retVal = 0
    endelse
endif


; Anything else is handled by the base class
if (n_elements(etc) gt 0) then begin
    retVal = self->DaveDat::set(_EXTRA=etc)
endif


return, retVal
end
;_______________________________________________________________________________
