; $Id: davedat__print.pro,v 1.3 2005/02/08 17:17:34 azuah Exp $
;###############################################################################
;
; CLASS NAME:
;   DaveDat
;
; PURPOSE:
;   Private utility method which can be used during development to
;   print summary info of a DaveDat object.
;
; CATEGORY:
;   Data Browser
;
; SUPERCLASSES:
;
; PUBLIC METHODS:
;   Get
;   Set
;   ResetData
;
; AUTHOR:
;   Richard Tumanjong Azuah
;   NIST Center for Neutron Research
;   100 Bureau Drive, Gaithersburg, MD 20899
;   United States
;   azuah@nist.gov; (301) 9755604
;   Oct, 2002
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################



;===============================================================================
; DaveDat::print
;
; PURPOSE:
;   Private utility method which can be used during development to
;   print a summary of a DaveDat object.
;
; PARAMETERS:
;
; KEYWORDS:
;
pro DaveDat::print

print,'______________________________'
print,'### DaveDat object: brief info'
print,''
if (ptr_valid(self.davePtr)) then begin
    print,'********** davePtr is valid **********'
endif else print,'********** davePtr is invalid **********'
print,''
if ptr_valid(self.descriPtr) then begin
    print,'********** descriPtr structure **********'
    help,(*self.descriPtr),/structure
    print,''
endif else begin
    print,'********** descriPtr is invalid **********'
endelse
fstr1 = '(A,T30,A)'
fstr2 = '(A,T30,G10.4)'
fstr3 = '(A,T30,I5)' 
print,format=fstr1,'filename: ',self.filename
print,format=fstr2,'scaleFactor: ',self.scaleFactor
print,''
if ptr_valid(self.xvPtr) then print,format=fstr3,'Xaxis -- length: ',n_elements(*self.xvPtr)
print,format=fstr1,'Xaxis -- label: ',self.xlabel
print,format=fstr1,'Xaxis -- type: ',self.xtype
print,format=fstr1,'Xaxis -- units: ',self.xunits
print,''
if ptr_valid(self.zvPtr) then begin
    zinfo = size(*self.zvPtr)
    print,format=fstr3,'Zaxis -- dimensions: ',zinfo[0]
    print,format=fstr3,'Zaxis -- length (dim 1): ',zinfo[1]
    if (zinfo[0] eq 2) then  print,format=fstr3,'Zaxis -- length (dim 2): ',zinfo[2]
endif
print,format=fstr1,'Zaxis -- label: ',self.zlabel
print,format=fstr1,'Zaxis -- units: ',self.zunits
print,'_________________________________________________________________________'

end
