; $Id: davegr2d__print.pro,v 1.4 2005/02/08 17:17:34 azuah Exp $
;###############################################################################
;
; CLASS NAME:
;   DaveGr2D
;
; PURPOSE:
;   The DaveGr2D class extends the DaveGr class to better handle
;   plotting of 2D data ie area and surface plots.
;
; CATEGORY:
;   Data Browser
;
; SUPERCLASSES:
;   DaveGr
;
; PUBLIC METHODS:
;   Display
;   Get
;   ReserMiscGrAttribs
;   Set
;
; AUTHOR:
;   Richard Tumanjong Azuah
;   NIST Center for Neutron Research
;   100 Bureau Drive, Gaithersburg, MD 20899
;   United States
;   azuah@nist.gov; (301) 9755604
;   Dec, 2002
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################



;===============================================================================
; DaveGr2D::print
;
; PURPOSE:
;   Private utility method which can be used during development to
;   print a summary of a DaveGr2D object.
;
; PARAMETERS:
;
;
pro DaveGr2D::print

print,'_________________________________________________'
print,'### DaveGr2D object: brief info'
print,''
print,'Plot Type: ',self.plotType
if (ptr_valid(self.dPlotAttribPtr)) then begin
    print,'********** Default Plot Attribute Structure'
    names = tag_names(*self.dPlotAttribPtr)
    for i=0,n_tags(*self.dPlotAttribPtr)-1 do begin
        if (vartype((*self.dPlotAttribPtr).(i)) eq 'POINTER') then begin
            if (ptr_valid((*self.dPlotAttribPtr).(i))) then begin
                print,format='(A20,A2,G10.4)',names[i],' =',*((*self.dPlotAttribPtr).(i))
            endif else begin
                print,format='(A20,A20)',names[i],' =      NULL POINTER'
            endelse
        endif else if (size((*self.dPlotAttribPtr).(i),/tname) eq 'OBJREF') then begin
            print,format='(A20,A2,A20)',names[i],' = ',vartype((*self.dPlotAttribPtr).(i))
        endif else begin
            print,format='(A20,A2,G10.4)',names[i],' =',(*self.dPlotAttribPtr).(i)
        endelse
    endfor 
    print,''
endif
if (ptr_valid(self.plotAttribPtr)) then begin
    print,'********** Plot Attribute Structure'
    names = tag_names(*self.plotAttribPtr)
    for i=0,n_tags(*self.plotAttribPtr)-1 do begin
        if (vartype((*self.plotAttribPtr).(i)) eq 'POINTER') then begin
            if (ptr_valid((*self.plotAttribPtr).(i))) then begin
                print,format='(A20,A2,G10.4)',names[i],' =',*((*self.plotAttribPtr).(i))
            endif else begin
                print,format='(A20,A20)',names[i],' =      NULL POINTER'
            endelse
        endif else if (size((*self.plotAttribPtr).(i),/tname) eq 'OBJREF') then begin
            print,format='(A20,A2,A20)',names[i],' = ',vartype((*self.plotAttribPtr).(i))
        endif else begin
            print,format='(A20,A2,G10.4)',names[i],' =',(*self.plotAttribPtr).(i)
        endelse
    endfor 
    print,''
endif

; display base class component
self->DaveGr::print

end
