; $Id: davegr__add.pro,v 1.3 2005/02/08 17:17:34 azuah Exp $
;###############################################################################
;
; CLASS NAME:
;   DaveGr
;
; PURPOSE:
;   This DaveGr class defines an entity which consists of one or more
;   datasets and all the necessary properties or attributes to display
;   or plot the dataset(s). A dataset is contained in a DaveDat
;   object and can be manipulated using that class' methods.
;
; CATEGORY:
;   Data Browser
;
; SUPERCLASSES:
;   IDL_CONTAINER
;
; PUBLIC METHODS:
;   Add
;   Get
;   Remove
;   Set
;
; AUTHOR:
;   Richard Tumanjong Azuah
;   NIST Center for Neutron Research
;   100 Bureau Drive, Gaithersburg, MD 20899
;   United States
;   azuah@nist.gov; (301) 9755604
;   Mar, 2003
;
; LICENSE:
;  The software in this file is written by an employee of 
;  National Institute of Standards and Technology 
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used or if the code in this file is
;  included in another product.
;
;###############################################################################



;===============================================================================
; DaveGr::Add
;
; PURPOSE:
;   Overide IDL_CONTAINER::Add method which adds an object to
;   this object's internal container. However we need to record
;   this addition by modified the object being added.
;
; PARAMETERS:
;   objRef [in] - the object reference to be added
;
;   _EXTRA [in] - allows keywords to be passed to IDL_CONTAINER::Add
;
pro DaveGr::add, objRef, _EXTRA=etc

; Increment the (data) object's internal reference counter by 1
nObj = n_elements(objRef)
if (nObj gt 0) then begin
    for i=0,nObj-1 do begin
        void = (objRef[i])->set(/addRef)
    endfor
endif

self->IDL_CONTAINER::add, objRef, _EXTRA=etc

; update graph attribute limits 
self->resetMiscGrAttribs, /limits

end
