; $Id: rains_lorentzian.pro,v 1.2 2005/02/08 18:30:58 dimeo Exp $
;
;###############################################################################
;
; NAME:
;  RAINS_LORENTZIAN
;
; PURPOSE:
;  Implementation of the Lorentzian distribution function for use in fitting
;  data in RAINS.
;
; CATEGORY:
;  DAVE, Data Analysis, RAINS, surface fitting
;
; AUTHOR:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
function rains_lorentzian,x,	area = area,	$
								center = center,	$
								fwhm = fwhm
csize = size(center)
nx = csize[1] & ny = csize[2]
ux = 1+bytarr(nx) & uy = 1+bytarr(ny)
gam = 0.5*fwhm
y = (ux#(area*gam/!dpi)#uy)/((x#uy-center)^2+(ux#gam#uy)^2)
return,y
end