; $Id: pan_delta.pro,v 1.5 2005/02/08 17:57:15 dimeo Exp $
;###############################################################################
;
; NAME:
;  PAN_DELTA
;
; PURPOSE:
;  Delta function for use in fitting data in PAN.
;
; CATEGORY:
;  DAVE, Data Analysis, PAN
;
; AUTHOR:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function pan_delta,x,parms,$
                     parmnames = parmnames, $
                     resPtr = resPtr,$
                     rlimit = rlimit, $
                     canDraw = canDraw, $
                     drawMessage = drawMessage, $
                     _Extra = extra

canDraw = 1
if n_elements(extra) eq 0 then extra = {dummy:0L}
if n_params() eq 0 then begin
  parmnames = ['area','center']
  return,-1
endif

drawMessage = strarr(4)
drawMessage[0:1] = ['Hold left mouse button down','and drag function centroid']
drawMessage[2:3] = ['Click the left mouse button to finish','']

if n_elements(resPtr) eq 0 then return,-1

area = parms[0]
cen = parms[1]
pd = [area,cen]

if n_elements(resPtr) ne 0 then begin
  ytest = (1.0*(*resPtr))
  yout = rmd_convol(x,resPtr,pseudo_delta = [[pd]], $
         reslimit = rlimit)
;  yout = rmd_convolution(x,resPtr,ytest,pseudo_delta = pd, $
;         reslimit = rlimit)
endif else begin
  yout = pan_gaussian(x,[area,cen,1d-1])
endelse

return,yout
end