; $Id: pan_dho.pro,v 1.2 2005/02/08 17:57:15 dimeo Exp $
;###############################################################################
;
; NAME:
;  PAN_DHO
;
; PURPOSE:
;  Damped harmonic oscillator for use in fitting data in PAN.
;
; CATEGORY:
;  DAVE, Data Analysis, PAN
;
; AUTHOR:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
;
;###############################################################################
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
function pan_dho, 		x,parms,$
                    	parmnames = parmnames, $
                    	canDraw = canDraw, $
                    	drawMessage = drawMessage, $
                    	_Extra = extra
if n_params() eq 0 then begin
  parmnames = ['amp','center','fwhm','temperature']
  return,-1
endif

drawMessage = strarr(4)
drawMessage[0:1] = ['Hold left mouse button down','and drag function centroid']
drawMessage[2:3] = ['Hold left mouse button down','and drag to change width']

amp = parms[0]
cen = abs(parms[1])
width = abs(parms[2])
gam = 0.5*width
temperature = parms[3]

npts = n_elements(x)
bose = dblarr(npts)
wherevalid = where(x ne 0.0,count)
if count eq npts then begin
  bose = 1./(1.-exp(-11.6*x/temperature))
endif else begin
  wherenotvalid = where(x eq 0.0,newcount)
  bose[wherevalid] = 1./(1.-exp(-11.6*x[wherevalid]/temperature))
  bose[wherenotvalid] = 1./(1.-exp(-11.6*x[wherenotvalid]/temperature))
endelse
y = dblarr(npts)
y = bose*amp*(gam/!pi)*((1.0/((x-cen)^2+gam^2))-(1.0/((x+cen)^2+gam^2)))

canDraw = 1
return,y
end