; $Id: is_ice.pro,v 1.2 2005/08/23 13:19:53 rdimeo Exp $
; ************************************ ;
; NAME:
;  IS_ICE
;
; PURPOSE:
;  This function determines if the file specified
;  by the argument FILENAME is a valid ICE file
;  as produced by the ICE data acquisition system
;  at the NCNR.
;
; CATEGORY:
;  DAVE, Data Reduction, TAS.
;
; AUTHOR:
;   Robert M. Dimeo, Ph.D.
;   NIST Center for Neutron Research
;   100 Bureau Drive
;   Gaithersburg, MD 20899
;   Phone: (301) 975-8135
;   E-mail: robert.dimeo@nist.gov
;   http://www.ncnr.nist.gov/staff/dimeo
;
; LICENSE:
;  The software in this file is written by an employee of
;  National Institute of Standards and Technology
;  as part of the DAVE software project.
;
;  The DAVE software package is not subject to copyright protection
;  and is in the public domain. It should be considered as an
;  experimental neutron scattering data reduction, visualization, and
;  analysis system. As such, the authors assume no responsibility
;  whatsoever for its use, and make no guarantees, expressed or
;  implied, about its quality, reliability, or any other
;  characteristic. The use of certain trade names or commercial
;  products does not imply any endorsement of a particular product,
;  nor does it imply that the named product is necessarily the best
;  product for the stated purpose. We would appreciate acknowledgment
;  if the DAVE software is used of if the code in this file is
;  included in another product.
; ************************************ ;
function is_ice,filename,errmsg = errmsg
errmsg = ''
catch,the_error
if the_error ne 0 then begin
   catch,/cancel
   if n_elements(lun) ne 0 then free_lun,lun,/force
   errmsg = !error_state.msg
   return,0B
endif
if ~file_test(filename) then begin
   errmsg = 'File '+filename+' not found'
   return,0B
endif
openr,lun,filename,/get_lun
dum = ''
readf,lun,dum
free_lun,lun,/force
split = strsplit(dum,/extract)
if strupcase(split[0]) ne '#ICE' then begin
   errmsg = 'Not a valid ICE file'
   return,0B
endif
catch,/cancel
return,1B
end
; ************************************ ;
pro test_is_ice
filename = sourceroot()+'fpx444.bt7.txt'
ret = is_ice(filename,errmsg = errmsg)
if ~ret then print,errmsg else print,filename+' is a valid ICE file'

end